#region Header
//
// .NET Sample
//
// Copyright (C) 2005-2007 by Autodesk, Inc.
//
// Permission to use, copy, modify, and distribute this software
// for any purpose and without fee is hereby granted, provided
// that the above copyright notice appears in all copies and
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS.
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC.
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//
#endregion // Header

#region Namespaces
using System.Reflection;

// AutoCAD
using Autodesk.AutoCAD.Runtime;
using Autodesk.AutoCAD.ApplicationServices;
using Autodesk.AutoCAD.DatabaseServices;
using Autodesk.AutoCAD.EditorInput;
using Autodesk.AutoCAD.Geometry;

// ACA (Aec)
using Autodesk.Aec.DatabaseServices;
using Autodesk.Aec.Arch.DatabaseServices;
using Autodesk.Aec.ApplicationServices;
using Autodesk.Aec.Arch.ApplicationServices;

// To resolve Object/Entity ambiguities
using AcObject = Autodesk.AutoCAD.DatabaseServices.DBObject;
using AecObject = Autodesk.Aec.DatabaseServices.DBObject;
using AcEntity = Autodesk.AutoCAD.DatabaseServices.Entity;
using AecEntity = Autodesk.Aec.DatabaseServices.Entity;

// To resolve Application ambiguities
using AcadApp = Autodesk.AutoCAD.ApplicationServices.Application;

// To resolve TransactionManager ambiguities
using DbTrMgr = Autodesk.AutoCAD.DatabaseServices.TransactionManager; //wraps AcDbTransactionManager
using AppTrMgr = Autodesk.AutoCAD.ApplicationServices.TransactionManager; //wraps AcTransactionManager

// To resolve ObjectId ambiguities
using ObjectId = Autodesk.AutoCAD.DatabaseServices.ObjectId;
using ObjectIdCollection = Autodesk.AutoCAD.DatabaseServices.ObjectIdCollection;

// To resolve Application ambiguities
using AcadNETApp = Autodesk.AutoCAD.ApplicationServices.Application;

using Autodesk.AutoCAD.Interop;
using Autodesk.AutoCAD.Interop.Common;
using Autodesk.AEC.Interop.Base;
using Autodesk.AEC.Interop.ArchBase;


#endregion // Namespaces

namespace AcaNetIntroCs
{
  public class Commands6
  {
    [CommandMethod( "LayerFromLayerKeyName" )]
    static public void LayerFromLayerKeyName()
    {
      Editor editor = AcadNETApp.DocumentManager.MdiActiveDocument.Editor; 
      Database db = HostApplicationServices.WorkingDatabase; 
      try
      {
        DictionaryLayerKeyStyle dictLKS = new DictionaryLayerKeyStyle( db ); 
        ObjectId idLKS = dictLKS.Records[0];
        using( Transaction tr = db.TransactionManager.StartTransaction() ) 
        {
          LayerKeyStyle lks = tr.GetObject( idLKS, OpenMode.ForRead ) as LayerKeyStyle; 
          editor.WriteMessage( "The name of the #1 LayerKeyStyle is: " + lks.Name + "\n" );
          //Autodesk.Aec.DatabaseServices.LayerKeyDefinition lkd;
          //lkd = lks.GetLayerKeyDefinitionAt( "WIND" ); 
          //editor.WriteMessage( "The description of the LayerKeyDefinition 'WIND' is: " + lkd.Description + "\n" ); 
        }
        ObjectId idLayer = dictLKS.GenerateLayer( "WIND" );
        editor.WriteMessage( "The Id of the layer created from the layer key 'WIND' is: " + idLayer.ToString() + "\n" );
      }
      catch 
      {
      }
    }
  }
}
