#Region "Header"
'      .NET Sample
'
'      Copyright (C) 2005-2007 by Autodesk, Inc.
'
'      Permission to use, copy, modify, and distribute this software
'      for any purpose and without fee is hereby granted, provided
'      that the above copyright notice appears in all copies and
'      that both that copyright notice and the limited warranty and
'      restricted rights notice below appear in all supporting
'      documentation.
'
'      AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS.
'      AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
'      MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC.
'      DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
'      UNINTERRUPTED OR ERROR FREE.
'
'      Use, duplication, or disclosure by the U.S. Government is subject to
'      restrictions set forth in FAR 52.227-19 (Commercial Computer
'      Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
'      (Rights in Technical Data and Computer Software), as applicable.
#End Region

#Region "Imports"

Imports System.Reflection
'Imports System.Windows.Forms
Imports System.Collections.Specialized
' AutoCAD
Imports Autodesk
Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.AutoCAD.ApplicationServices
Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.EditorInput
Imports Autodesk.AutoCAD.Geometry
' ACA (Aec)
Imports Autodesk.Aec.DatabaseServices
Imports Autodesk.Aec.ApplicationServices
Imports Autodesk.Aec.Arch.DatabaseServices
Imports Autodesk.Aec.Arch.ApplicationServices
Imports Autodesk.Aec.PropertyData
Imports Autodesk.Aec.PropertyData.DatabaseServices

' To resolve Object/Entity ambiguities
Imports AcObject = Autodesk.AutoCAD.DatabaseServices.DBObject
Imports AecObject = Autodesk.Aec.DatabaseServices.DBObject
Imports AcEntity = Autodesk.AutoCAD.DatabaseServices.Entity
Imports AecEntity = Autodesk.Aec.DatabaseServices.Entity

' To resolve Application ambiguities
'Imports AcadApp = Autodesk.AutoCAD.ApplicationServices.Application

' To resolve ObjectId ambiguities
Imports ObjectId = Autodesk.AutoCAD.DatabaseServices.ObjectId
Imports ObjectIdCollection = Autodesk.AutoCAD.DatabaseServices.ObjectIdCollection

#End Region

'A class containing commands
Public Class Commands3

    ' List all properties from all property sets attached on all entities  
    <CommandMethod("AcaNetIntroVb", "AcaNetListProperties", CommandFlags.Modal)> _
    Public Sub PropsList()

        Dim db As Database = HostApplicationServices.WorkingDatabase
        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor
        Dim tr As Transaction = db.TransactionManager.StartTransaction

        Try

            'Loop all Entities
            Dim bt As BlockTable = CType(tr.GetObject(db.BlockTableId, OpenMode.ForRead, False), BlockTable)
            Dim btr As BlockTableRecord = CType(tr.GetObject(bt(BlockTableRecord.ModelSpace), OpenMode.ForRead, False), BlockTableRecord)

            Dim idEnt As ObjectId
            For Each idEnt In btr

                ' get the property sets collection on this entity
                Dim ent As AcObject = tr.GetObject(idEnt, OpenMode.ForRead)
                Dim idsPropSets As ObjectIdCollection = PropertyDataServices.GetPropertySets(ent)

                ' how many property sets does it have?  
                ed.WriteMessage(vbCrLf + vbCrLf + ent.GetRXClass.Name + " (" + ent.Handle.ToString + ") has " + _
                 idsPropSets.Count.ToString + " PropSets:")

                ' Loop all PropertySets
                Dim idPropSet As ObjectId
                Dim iPropSet As Integer = 0
                For Each idPropSet In idsPropSets
                    iPropSet += 1

                    Dim propSet As PropertySet = CType(tr.GetObject(idPropSet, OpenMode.ForRead), PropertySet)
                    ' The trick to get the properties is to really loop the *definitions*!
                    '   Then we can evaluate automatic ones from the very PropSetDef, 
                    '   or get static ones from PropSet
                    Dim propSetDef As PropertySetDefinition = _
                     CType(tr.GetObject(propSet.PropertySetDefinition, OpenMode.ForRead), PropertySetDefinition)
                    Dim propDefs As PropertyDefinitionCollection = propSetDef.Definitions

                    ed.WriteMessage(vbCrLf + vbCrLf + "  PropSet " + iPropSet.ToString + "[" _
                     + propSetDef.Name + "] has " + propDefs.Count.ToString + " Props:")

                    ' Loop all properties in the propSetDef  
                    Dim propDef As PropertyDefinition
                    Dim iProp As Integer = 0
                    For Each propDef In propDefs
                        iProp += 1
                        Dim value As String

                        If (propDef.Automatic) Then                     'AUTOMATIC
                            Dim idsDummy As ObjectIdCollection = New ObjectIdCollection()
                            value = propSetDef.GetValue(propDef.Id, idEnt, idsDummy).ToString
                            ed.WriteMessage(vbCrLf + "    " + iProp.ToString + _
                             ".(AUTO) Name=" + propDef.Name + " Value=" + value)

                        Else                        ' STATIC
                            value = propSet.GetAt(propDef.Id).ToString
                            ed.WriteMessage(vbCrLf + "    " + iProp.ToString + _
                             ".(STATIC) Name=" + propDef.Name + " Value=" + value)

                        End If

                    Next propDef
                Next idPropSet
            Next idEnt

            ' NOTE: the following contain ONLY STATIC data!
            'Dim props As PropertySetDataCollection = propSet.PropertySetData
            'Dim prop As PropertySetData
            'For Each prop In props
            '...
            'Next prop

            tr.Commit()
        Catch
            tr.Abort()
        Finally
            tr.Dispose()
        End Try

    End Sub

End Class
