#Region "Header"
'      .NET Sample
'
'      Copyright (C) 2005-2007 by Autodesk, Inc.
'
'      Permission to use, copy, modify, and distribute this software
'      for any purpose and without fee is hereby granted, provided
'      that the above copyright notice appears in all copies and
'      that both that copyright notice and the limited warranty and
'      restricted rights notice below appear in all supporting
'      documentation.
'
'      AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS.
'      AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
'      MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC.
'      DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
'      UNINTERRUPTED OR ERROR FREE.
'
'      Use, duplication, or disclosure by the U.S. Government is subject to
'      restrictions set forth in FAR 52.227-19 (Commercial Computer
'      Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
'      (Rights in Technical Data and Computer Software), as applicable.
#End Region

#Region "Imports"

Imports System.Windows.Forms

' AutoCAD
Imports Autodesk
Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.AutoCAD.ApplicationServices
Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.EditorInput

' ACA (Aec)
Imports Autodesk.Aec.DatabaseServices
Imports Autodesk.AEC.ApplicationServices
Imports Autodesk.AEC.Arch.DatabaseServices
Imports Autodesk.AEC.Arch.ApplicationServices

' To resolve Object/Entity ambiguities
Imports AcObject = Autodesk.AutoCAD.DatabaseServices.DBObject
Imports AecObject = Autodesk.AEC.DatabaseServices.DBObject
Imports AcEntity = Autodesk.AutoCAD.DatabaseServices.Entity
Imports AecEntity = Autodesk.AEC.DatabaseServices.Entity

' AutoCAD Interop assemblies (Acad ActiveX API)
Imports Autodesk.AutoCAD.Interop
Imports Autodesk.AutoCAD.Interop.Common

' AEC Interop assemblies (ACA ActiveX API - only Aec Base needed for Display System objects)
Imports Autodesk.Aec.Interop.Base
Imports Autodesk.Aec.Interop.ArchBase
Imports Autodesk.Aec.Interop.UIBase

' To resolve Application ambiguities
Imports AcadNETApp = Autodesk.AutoCAD.ApplicationServices.Application

#End Region

'A class containing commands
Public Class Commands5

    ' Check which display properties a Wall uses for all current dispreps and list some display components
    ' This shows mixed usage of native .NET and COM Interop API (display system is exposed in the former as well since 2008)
    <CommandMethod("AcaNetIntroVb", "AcaComDispProps", CommandFlags.Modal)> _
    Public Sub DisplayOverrides()

        ' Minimum error checking!

        ' First use *native .NET* to select a wall and get the wall style Id
        Dim ed As Editor = AcadNETApp.DocumentManager.MdiActiveDocument.Editor
        Dim optEnt As New PromptEntityOptions(vbLf & "Select a Wall")
        optEnt.SetRejectMessage(vbLf & "Selected entity is NOT a Wall, try again...")
        optEnt.AddAllowedClass(GetType(Wall), True)

        Dim resEnt As PromptEntityResult = ed.GetEntity(optEnt)
        If resEnt.Status <> PromptStatus.OK Then
            ed.WriteMessage("Selection error - aborting")
            Exit Sub
        End If

        ' AutoCAD ActiveX App and Doc (for convenience name it ThisDrawing if porting VBA code)
        Dim ThisApp As AcadApplication = AcadNETApp.AcadApplication
        Dim ThisDrawing As AcadDocument = ThisApp.ActiveDocument

        ' Since we later need *ActiveX* Wall object, the following is the way to obtain one from ObjectId:
        Dim axWall As AecWall = CType(ThisDrawing.ObjectIdToObject(resEnt.ObjectId.OldIdPtr), AecWall)
        ' From this, we can also get Activex Style object
        Dim axWallStyle As AecWallStyle = axWall.Style

        MsgBox("Got ActiveX Wall and WallStyle objects: " & axWall.Handle & ", " & axWallStyle.Name)

        ' AEC ActiveX app and doc (enough for dispreps - we don't need ACA app and doc)
        Dim app As AecBaseApplication = ThisApp.GetInterfaceObject("AecX.AecBaseApplication.5.0")
        app.Init(ThisApp)
        Dim doc As AecBaseDocument = app.ActiveDocument

        ' get all DispReps used in the current viewport and loop them all
        'Dim dispset As AecDisplaySet = doc.DisplaySets.ActiveSet(ThisDrawing.ActiveViewport)
        'Dim dispreps As AecDisplayRepresentations = dispset.DisplayRepresentations
        Dim dispreps As AecDisplayRepresentations = doc.DisplaySets.ActiveSet(ThisDrawing.ActiveViewport).DisplayRepresentations

        Dim disprep As AecDisplayRepresentation
        For Each disprep In dispreps

            'We are interested ONLY in those drawing for walls
            If disprep.ClassName = "Wall" Then

                Dim msg As String = disprep.DisplayName & " DispRep (" & disprep.Name & ") uses "
                Dim wallDPs As AecDisplayProperties

                ' Get corresponding Display Properties object
                Try             ' check for OBJECT OVERRIDE
                    wallDPs = disprep.OverrideDisplayProperties(axWall)
                    msg += " OBJECT OVERRIDE:"
                Catch
                    Try                 ' check for STYLE OVERRIDE
                        wallDPs = disprep.OverrideDisplayProperties(axWallStyle)
                        msg += " STYLE OVERRIDE:"
                    Catch
                        Try                  ' must be DWG DEFAULT
                            wallDPs = disprep.DefaultDisplayProperties
                            msg += " DWG DEFAULT:"
                        Catch                    ' Something seriously wrong
                            MsgBox("No default DispReps??? - aborting...")
                            Exit Sub
                        End Try
                    End Try
                End Try

                ' List a few Display Components
                Dim dispcomps As AecDisplayComponents
                dispcomps = wallDPs.DisplayComponents

                Dim dispcomp As AecDisplayComponent
                For Each dispcomp In dispcomps
                    ' First must check the most-derived class (note: Hatch is derived from Entity)
                    If TypeOf dispcomp Is AecDisplayComponentHatch Then
                        Dim comphatch As AecDisplayComponentHatch
                        comphatch = dispcomp
                        ' NOTE: TrueColor.ColorIndex will fail if RGB
                        '       or .HatchPattern may fail if user hatch, but keep it simple here...
                        ' List only a few
                        msg += vbCrLf & "  HATCH component " & dispcomp.Name & _
                         ": Vis=" & comphatch.Visible & _
                         ", Col=" & comphatch.TrueColor.ColorIndex & _
                         ", Pat=" & comphatch.PatternName
                    ElseIf TypeOf dispcomp Is AecDisplayComponentEntity Then
                        Dim compent As AecDisplayComponentEntity
                        compent = dispcomp
                        ' NOTE: TrueColor.ColorIndex will fail if RGB
                        ' List only a few
                        msg += vbCrLf & "  CURVES component " & compent.Name & _
                         ": Vis=" & compent.Visible & _
                         ", Col=" & compent.TrueColor.ColorIndex
                    End If
                Next

                ' report result
                MsgBox(msg)

            End If
        Next

    End Sub

End Class
