﻿#Region "Copyright Note"
'' 
'' (C) Copyright 2009 by Autodesk, Inc.
'' 
'' Permission to use, copy, modify, and distribute this software in
'' object code form for any purpose and without fee is hereby granted,
'' provided that the above copyright notice appears in all copies and
'' that both that copyright notice and the limited warranty and
'' restricted rights notice below appear in all supporting
'' documentation.
''
'' AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS.
'' AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
'' MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC.
'' DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
'' UNINTERRUPTED OR ERROR FREE.
'' 
'' Use, duplication, or disclosure by the U.S. Government is subject to
'' restrictions set forth in FAR 52.227-19 (Commercial Computer
'' Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
'' (Rights in Technical Data and Computer Software), as applicable.
''
#End Region

#Region "Imports"
Imports System.Reflection
Imports System.Collections.Specialized '' need this for StringCollection
'Imports System.Windows.Forms
' AutoCAD
Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.AutoCAD.ApplicationServices
Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.EditorInput
' ACA (Aec)
Imports Autodesk.Aec.DatabaseServices
Imports Autodesk.Aec.Arch.DatabaseServices
Imports Autodesk.Aec.PropertyData.DatabaseServices '' we need this for prop set def 

' To resolve Object/Entity ambiguities
Imports AcObject = Autodesk.AutoCAD.DatabaseServices.DBObject
Imports AecObject = Autodesk.Aec.DatabaseServices.DBObject
Imports AcEntity = Autodesk.AutoCAD.DatabaseServices.Entity
Imports AecEntity = Autodesk.Aec.DatabaseServices.Entity

' To resolve Application ambiguities
Imports AcadApp2 = Autodesk.AutoCAD.ApplicationServices.Application
'Imports WinApp = System.Windows.Forms.Application

' To resolve TransactionManager ambiguities
Imports DbTrMgr = Autodesk.AutoCAD.DatabaseServices.TransactionManager 'wraps AcDbTransactionManager
Imports AppTrMgr = Autodesk.AutoCAD.ApplicationServices.TransactionManager 'wraps AcTransactionManager

' To resolve ObjectId ambiguities
Imports ObjectId = Autodesk.AutoCAD.DatabaseServices.ObjectId
Imports ObjectIdCollection = Autodesk.AutoCAD.DatabaseServices.ObjectIdCollection
#End Region

#Region "Description"
''
''  In a sequence of exercise labs, we will cover the following topics: 
'' '
''  1. define a property set definition.
''  2. attach property set definitions to all the doors 
''  3. anchor a tag to a door
''  4. create a schedule table style 
''  5. create a instance of schedule table 
''  
#End Region


Public Class ACANet_PropertySetDefinition
    ''
    ''  Create a minimum set of property set definition.
    ''
    ''  Minimum error checking for code readability. In practice, you may want to check 
    ''  its existace, for example.
    ''
    <CommandMethod("ACANetScheduleLabs", "AcaCreatePropSetDef", CommandFlags.Modal)> _
    Public Shared Sub CreatePropertySetDefinition()

        Dim db As Database = Application.DocumentManager.MdiActiveDocument.Database
        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor

        ''  the name of the property set def. Could be for objects or styles. 
        ''  hard coding for simplicity.
        ''  
        Dim propSetDefName As String = "ACADoorObjects" ''  this will be the key in the dictionary 
        'Dim propSetDefName As String = "ACANetDoorStyles" 

        Try

            ''  (1) create prop set def 
            ''
            Dim propSetDef As PropertySetDefinition = New PropertySetDefinition
            propSetDef.SetToStandard(db)
            propSetDef.SubSetDatabaseDefaults(db)

            ''  alternatively, you can use dictionary's NewEntry 
            'Dim dictPropSetDef = New DictionaryPropertySetDefinitions(db)
            'Dim propSetDef As PropertySetDefinition = dictPropSetDef.NewEntry()

            ''  General tab 
            ''
            propSetDef.Description = "Property Set Definition by ACA.NET"

            ''  Applies To tab
            '' 
            Dim isStyle As Boolean = False '' apply to objects or styles. True if style, False if objects
            Dim appliedTo = New StringCollection()
            appliedTo.Add("AecDbDoor")       '' to apply to a door object, use this.  
            'appliedTo.Add("AecDbDoorStyle") '' to apply to a door STYLE, use this instead. 
            'appliedTo.Add("AecDbWindow")    '' to apply to more than one object type, add more here. 
            propSetDef.SetAppliesToFilter(appliedTo, isStyle)

            ''  Definition tab 
            ''   
            ''  This is main part. 
            ''  In UI, there are eight types of properties you can add here: 
            ''  1 manual  2 automatic, 3 formula, 4 location, 5 classification, 
            ''  6 material, 7 project, 8 anchor, and 9 graphic. 
            '' 
            ''  3 formula is not possible to add with .NET API. 
            ''  DID 1213849  API wish: programmatically defining formula property through OMF/.NET 
            ''  How about others? - others are not possible, either. 
            '' 

            ''  (2) we can add a set of property definitions.  We first make a container to hold them.
            ''      This is the main part.  A property set definition can contain a set of property definition.
            '' 

            ''  (2.1) let's first add manual property. 
            ''  Here we use text type 
            Dim propDefManual = New PropertyDefinition()
            propDefManual.SetToStandard(db)
            propDefManual.SubSetDatabaseDefaults(db)
            propDefManual.Name = "ACAManualProp"
            propDefManual.Description = "Manual property by ACA.NET"
            propDefManual.DataType = Autodesk.Aec.PropertyData.DataType.Text
            propDefManual.DefaultData = "ACA Default"
            ''  add to the prop set def 
            propSetDef.Definitions.Add(propDefManual)

            ''  (2.2) let's add another one, automatic one this time
            ''
            Dim propDefAutomatic = New PropertyDefinition()
            propDefAutomatic.SetToStandard(db)
            propDefAutomatic.SubSetDatabaseDefaults(db)
            propDefAutomatic.Name = "ACAWidth"
            propDefAutomatic.Description = "Automatic property by ACA.NET"
            propDefAutomatic.SetAutomaticData("AecDbDoor", "Width")
            ''  add to the prop set def 
            propSetDef.Definitions.Add(propDefAutomatic)

            ''  similarly, add one with height 
            '' 
            propDefAutomatic = New PropertyDefinition()
            propDefAutomatic.SetToStandard(db)
            propDefAutomatic.SubSetDatabaseDefaults(db)
            propDefAutomatic.Name = "ACAHeight"
            propDefAutomatic.Description = "Automatic property by ACA.NET"
            propDefAutomatic.SetAutomaticData("AecDbDoor", "Height")
            ''  add to the prop set def 
            propSetDef.Definitions.Add(propDefAutomatic)

            ''  (3)  finally add the prop set def to the database 
            '' 
            Using tr As Transaction = db.TransactionManager.StartTransaction

                ''  check the name 
                ''
                Dim dictPropSetDef = New DictionaryPropertySetDefinitions(db)
                If dictPropSetDef.Has(propSetDefName, tr) Then
                    ed.WriteMessage("error - the property set defintion already exists: " + propSetDefName + vbCrLf)
                    Return
                End If

                dictPropSetDef.AddNewRecord(propSetDefName, propSetDef)
                tr.AddNewlyCreatedDBObject(propSetDef, True)
                tr.Commit()

            End Using

        Catch ex As Exception
            ed.WriteMessage("error in CreatePropSetDef: " + ex.ToString + vbCrLf)
            Return
        End Try

        ed.WriteMessage("property set definition " + propSetDefName + " is successfully created." + vbCrLf)

    End Sub

    ''
    ''  list automatic property source or a set of possible automatic properties  
    ''
    <CommandMethod("ACANetScheduleLabs", "AcaListAutomaticPropertySource", CommandFlags.Modal)> _
    Public Sub ListAutomaticPropertySource()

        Dim db As Database = Application.DocumentManager.MdiActiveDocument.Database
        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor

        ''  here is the main method
        ''  get the list of possible automatic properties (as an example) for a door object  
        ''  
        Dim propNames() As String = PropertyDataServices.FindAutomaticSourceNames("AecDbDoor", db)

        ''  show the list we just got 
        ''
        ed.WriteMessage(" -- automatic property source < AecDbDoor > -- " + vbCrLf + vbCrLf)

        For Each name As String In propNames
            ed.WriteMessage(name + vbCrLf)
        Next

    End Sub

    '' 
    ''  import a property set definition
    ''  DevNote TS88808  Import/Export styles from/to another drawing in .NET
    ''   
    <CommandMethod("ACANetScheduleLabs", "AcaImportPropSetDef", CommandFlags.Modal)> _
    Public Sub ImportPropertySetDefinition()

        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor

        Try
            ''  our destination file is the current db.  
            Dim dbDestination As Database = Application.DocumentManager.MdiActiveDocument.Database

            ''  our source file from which we import a prop set def 
            ''  the following is taken from:
            ''  C:\ProgramData\Autodesk\ACA 2010\enu\Styles\Metric\Schedule Tables (Metric).dwg
            '' 
            Dim sourcePath As String = "D:\ACA.NET Webcast\Demo\Schedule Tables (Metric).dwg"
            Dim dbSource As Database = New Database(False, True)
            dbSource.ReadDwgFile(sourcePath, System.IO.FileShare.Read, True, "")

            ''  get the source dictionary   
            Dim dictPropSetDef As New DictionaryPropertySetDefinitions(dbSource)

            ''  get the list of def ids that you want to import 
            ''  (1) if you want to import everything from the given dictionary, use this.
            'Dim objCollectionSrc As ObjectIdCollection = dictPropSetDef.Records

            ''  (2) if you want to import a specific style, use this. 
            Dim objCollectionSrc As New ObjectIdCollection
            objCollectionSrc.Add(dictPropSetDef.GetAt("DoorObjects"))  '' the name of def you want to import  

            ''  now use CloningHelper class to import styles.   

            ''  there are four options for merge type:
            ''    Normal     = 0, // no overwrite 
            ''    Overwrite  = 1, // this is default. 
            ''    Unique     = 2, // rename it if the same name exists.
            ''    Merge      = 3  // no overwrite + add overlapping ones as anonymous name (probably intended for behind the scenes further processing.) 
            Dim helper As New Autodesk.Aec.ApplicationServices.Utility.CloningHelper

            ''  uncomment one of these if you want to have a behavior other than default (i.e., overwrite). 
            ''helper.MergeType = Autodesk.Aec.ApplicationServices.Utility.DictionaryRecordMergeBehavior.Unique; // rename 
            ''helper.MergeType = Autodesk.Aec.ApplicationServices.Utility.DictionaryRecordMergeBehavior.Merge;  // no overrite + add overlapping ones as anonymous name (probably internal use only.)  
            ''helper.MergeType = Autodesk.Aec.ApplicationServices.Utility.DictionaryRecordMergeBehavior.Normal; // no overrite.  

            ''  finally call clone. 
            helper.Clone(dbSource, dbDestination, objCollectionSrc, dictPropSetDef.RecordType, True)

        Catch ex As Exception
            ed.WriteMessage("error in AcaImportPropSetDef" + ex.Message + vbCrLf)
            Return
        End Try

        ed.WriteMessage("Property set definition DoorObjects has been successfully imported" + vbCrLf)
    End Sub

End Class
