﻿#Region "Copyright Note"
'' 
'' (C) Copyright 2009 by Autodesk, Inc.
'' 
'' Permission to use, copy, modify, and distribute this software in
'' object code form for any purpose and without fee is hereby granted,
'' provided that the above copyright notice appears in all copies and
'' that both that copyright notice and the limited warranty and
'' restricted rights notice below appear in all supporting
'' documentation.
''
'' AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS.
'' AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
'' MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC.
'' DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
'' UNINTERRUPTED OR ERROR FREE.
'' 
'' Use, duplication, or disclosure by the U.S. Government is subject to
'' restrictions set forth in FAR 52.227-19 (Commercial Computer
'' Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
'' (Rights in Technical Data and Computer Software), as applicable.
''
#End Region

#Region "Imports"
Imports System.Reflection
Imports System.Collections.Specialized '' need this for StringCollection
'Imports System.Windows.Forms
' AutoCAD
Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.AutoCAD.ApplicationServices
Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.EditorInput
' ACA (Aec)
Imports Autodesk.Aec.DatabaseServices
Imports Autodesk.Aec.Arch.DatabaseServices
Imports Autodesk.Aec.PropertyData.DatabaseServices '' we need this for prop set def 

' To resolve Object/Entity ambiguities
Imports AcObject = Autodesk.AutoCAD.DatabaseServices.DBObject
Imports AecObject = Autodesk.Aec.DatabaseServices.DBObject
Imports AcEntity = Autodesk.AutoCAD.DatabaseServices.Entity
Imports AecEntity = Autodesk.Aec.DatabaseServices.Entity

' To resolve Application ambiguities
Imports AcadApp2 = Autodesk.AutoCAD.ApplicationServices.Application
'Imports WinApp = System.Windows.Forms.Application

' To resolve TransactionManager ambiguities
Imports DbTrMgr = Autodesk.AutoCAD.DatabaseServices.TransactionManager 'wraps AcDbTransactionManager
Imports AppTrMgr = Autodesk.AutoCAD.ApplicationServices.TransactionManager 'wraps AcTransactionManager

' To resolve ObjectId ambiguities
Imports ObjectId = Autodesk.AutoCAD.DatabaseServices.ObjectId
Imports ObjectIdCollection = Autodesk.AutoCAD.DatabaseServices.ObjectIdCollection
#End Region

'' Lab2 uses helper in Utils: findStyle and findObjects 

Public Class ACANet_AttachPropertySet
    ''
    ''  Attach a property set at objects
    ''
    ''  Minimum error checking for code readability. 
    ''  In practice, you may want to check if a specific prop set is already attached, for example.
    ''
    <CommandMethod("ACANetScheduleLabs", "AcaAttachPropertySet", CommandFlags.Modal)> _
    Public Sub AttachPropertySet()

        Dim db As Database = Application.DocumentManager.MdiActiveDocument.Database
        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor

        ''  (1) select an AEC object to attach a property set 
        '' 
        Dim optEnt As New PromptEntityOptions(vbLf & "Select an AEC object to attach a property set")
        optEnt.SetRejectMessage(vbLf & "Selected entity is NOT an AEC object, try again...")
        optEnt.AddAllowedClass(GetType(Geo), False)  ''  Geo is the base class for AEC object. use this if you want to apply to all the AEC objects. 
        'optEnt.AddAllowedClass(GetType(Door), False)  ''  if you are interested in only Door object, use this instead. 

        Dim resEnt As PromptEntityResult = ed.GetEntity(optEnt)
        If resEnt.Status <> PromptStatus.OK Then
            ed.WriteMessage("Selection error - aborting")
            Exit Sub
        End If

        Dim idTargetObj As ObjectId = resEnt.ObjectId

        ''  (2) ask for property set definition name 
        ''
        Dim optStr As New PromptStringOptions(vbLf + "Enter the name of property set definition you want to attach.")

        Dim resPropSetDef As PromptResult = ed.GetString(optStr)
        If resPropSetDef.Status <> PromptStatus.OK Then
            ed.WriteMessage("String input error - aborting")
            Exit Sub
        End If

        Dim propSetDefName As String = resPropSetDef.StringResult

        Dim dictPropSetDef = New DictionaryPropertySetDefinitions(db)
        Dim idPropSetDef As ObjectId = Utils.findStyle(dictPropSetDef, propSetDefName)

        If idPropSetDef = Nothing Then Return
        ''  if we come here, we have a prop set def id and an object id.  

        ''  (3)  attach the given property set to the given object. see the helper function below.  
        ''
        Try
            Using tr As Transaction = db.TransactionManager.StartTransaction

                Dim obj As AcObject = tr.GetObject(idTargetObj, OpenMode.ForWrite, False, False)

                ''  PropertyDataServices provide a convenient method to do the actual work.
                PropertyDataServices.AddPropertySet(obj, idPropSetDef)

                tr.Commit()

            End Using

        Catch ex As Exception
            ed.WriteMessage("error in AttachPropertySet: " + ex.ToString + vbCrLf)
        End Try

    End Sub

    ''
    ''  Attach a property set to all door objects 
    ''
    <CommandMethod("ACANetScheduleLabs", "AcaAttachPropertySetToAllDoors", CommandFlags.Modal)> _
    Public Sub AttachPropertySetToAllDoors()

        Dim db As Database = Application.DocumentManager.MdiActiveDocument.Database
        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor

        ''  (1) find all the door object in the model space 
        ''  
        Dim ids As ObjectIdCollection = Utils.findObjects(GetType(Door))

        ''  (2) find the property set definition with the given name.  
        ''
        Dim dictPropSetDef = New DictionaryPropertySetDefinitions(db)
        '' "ACADoorObjects" is what we defined in Lab1.   
        Dim idPropSetDef As ObjectId = Utils.findStyle(dictPropSetDef, "ACADoorObjects")

        If idPropSetDef = Nothing Then Return

        ''  (3)  attach the given property set to the given object. see the helper function below.  
        ''
        Try
            Using tr As Transaction = db.TransactionManager.StartTransaction
                For Each id As ObjectId In ids
                    Dim obj As AcObject = tr.GetObject(id, OpenMode.ForWrite, False, False)

                    ''  PropertyDataServices provide a convenient method to do the actual work.
                    PropertyDataServices.AddPropertySet(obj, idPropSetDef)
                Next
                tr.Commit()
            End Using

        Catch ex As Exception
            ed.WriteMessage("error in AttachPropertySetToAllDoors: " + ex.ToString + vbCrLf)
        End Try

        ed.WriteMessage("The property set was attached successfully." + vbCrLf)

    End Sub

End Class