﻿#Region "Copyright Note"
'' 
'' (C) Copyright 2009 by Autodesk, Inc.
'' 
'' Permission to use, copy, modify, and distribute this software in
'' object code form for any purpose and without fee is hereby granted,
'' provided that the above copyright notice appears in all copies and
'' that both that copyright notice and the limited warranty and
'' restricted rights notice below appear in all supporting
'' documentation.
''
'' AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS.
'' AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
'' MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC.
'' DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
'' UNINTERRUPTED OR ERROR FREE.
'' 
'' Use, duplication, or disclosure by the U.S. Government is subject to
'' restrictions set forth in FAR 52.227-19 (Commercial Computer
'' Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
'' (Rights in Technical Data and Computer Software), as applicable.
''
#End Region

#Region "Imports"
'' set here or set it in the project properties/references. (I'm setting the project here for VB.NET) 
''
Imports System.Reflection
Imports System.Collections.Specialized '' need this for StringCollection
'Imports System.Windows.Forms
' AutoCAD
Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.AutoCAD.ApplicationServices
Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.EditorInput
Imports Autodesk.AutoCAD.Geometry
' ACA (Aec)
Imports Autodesk.Aec.DatabaseServices
Imports Autodesk.Aec.Arch.DatabaseServices
Imports Autodesk.Aec.PropertyData.DatabaseServices '' we need this for prop set def 

' To resolve Object/Entity ambiguities
Imports AcObject = Autodesk.AutoCAD.DatabaseServices.DBObject
Imports AecObject = Autodesk.Aec.DatabaseServices.DBObject
Imports AcEntity = Autodesk.AutoCAD.DatabaseServices.Entity
Imports AecEntity = Autodesk.Aec.DatabaseServices.Entity

' To resolve Application ambiguities
Imports AcadApp2 = Autodesk.AutoCAD.ApplicationServices.Application
'Imports WinApp = System.Windows.Forms.Application

' To resolve TransactionManager ambiguities
Imports DbTrMgr = Autodesk.AutoCAD.DatabaseServices.TransactionManager 'wraps AcDbTransactionManager
Imports AppTrMgr = Autodesk.AutoCAD.ApplicationServices.TransactionManager 'wraps AcTransactionManager

' To resolve ObjectId ambiguities
Imports ObjectId = Autodesk.AutoCAD.DatabaseServices.ObjectId
Imports ObjectIdCollection = Autodesk.AutoCAD.DatabaseServices.ObjectIdCollection
#End Region

Public Class ACANet_AnchorTag
    ''
    ''  Attach a property set at objects
    ''
    <CommandMethod("ACANetScheduleLabs", "AcaAnchorTag", CommandFlags.Modal)> _
    Public Sub anchorExtendedTagToEnt()

        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor
        Dim db As Database = HostApplicationServices.WorkingDatabase

        ''  (1) select a Geo 
        '' 
        Dim optGeo As New PromptEntityOptions("Select a geo to anchor a tag")
        optGeo.SetRejectMessage("Not a geo")
        optGeo.AddAllowedClass(GetType(Geo), False) '' Geo is a base class for AEC objects. 

        Dim resGeo As PromptEntityResult = ed.GetEntity(optGeo)
        If resGeo.Status <> PromptStatus.OK Then
            ed.WriteMessage("failed to pick an geo")
            Exit Sub
        End If

        Dim idGeo As ObjectId = resGeo.ObjectId

        ''  (2) select a tag 
        '' 
        Dim optTag As New PromptEntityOptions("Select a tag ")
        optTag.SetRejectMessage("Not a tag")
        optTag.AddAllowedClass(GetType(MultiViewBlockReference), True)

        Dim resTag As PromptEntityResult = ed.GetEntity(optTag)
        If resTag.Status <> PromptStatus.OK Then
            ed.WriteMessage("failed to pick a tag")
            Exit Sub
        End If

        Dim idTag As ObjectId = resTag.ObjectId

        ''  (3) pick a location to place the tag
        '' 
        Dim resPt As PromptPointResult = ed.GetPoint("Pick a location of the tag ")
        If resPt.Status <> PromptStatus.OK Then
            ed.WriteMessage("failed to pick a location of the tag")
            Exit Sub
        End If

        ''  (4) add a tag to the geo using AnchorExtendedTagToEntity  
        '' 
        Dim trans As Transaction = db.TransactionManager.StartTransaction()

        Try
            ''  open the geo for read 
            '' 
            Dim obj As Object = trans.GetObject(idGeo, OpenMode.ForRead)
            Dim objGeo As Geo = CType(obj, Geo)
            Dim ecsEnt As Matrix3d = objGeo.Ecs()

            ''  open the tag for write 
            '' 
            obj = trans.GetObject(idTag, OpenMode.ForWrite)
            Dim objTag As MultiViewBlockReference = CType(obj, MultiViewBlockReference)

            ''  adjust the location of the tag 
            Dim pt As Point3d = resPt.Value
            objTag.Location = pt.TransformBy(ecsEnt.Inverse())

            ''  add an anchor between the tag and the geo 
            '' 
            Dim pAnchor As New AnchorExtendedTagToEntity
            pAnchor.SubSetDatabaseDefaults(db)
            pAnchor.SetToStandard(db)

            pAnchor.ForceHorizontal = False '' or True

            ''  here is the main part that attach the tag to the geo entity (e.g., door and window) 
            pAnchor.ReferencedEntityId = idGeo
            objTag.SetAnchor(pAnchor)

            trans.Commit()

        Catch ex As Exception
            trans.Abort()
        Finally
            trans.Dispose()
        End Try

    End Sub

    '' 
    ''  import a multi view definition
    ''  DevNote TS88808  Import/Export styles from/to another drawing in .NET
    ''   
    <CommandMethod("ACANetScheduleLabs", "AcaImportMvBlockDef", CommandFlags.Modal)> _
    Public Sub ImportMvBlockDefinition()

        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor

        Try
            ''  our destination file is the current db.  
            Dim dbDestination As Database = Application.DocumentManager.MdiActiveDocument.Database

            ''  our source file from which we import a prop set def 
            ''  the following is taken from:
            ''  C:\ProgramData\Autodesk\ACA 2010\enu\Styles\Metric\Schedule Tables (Metric).dwg
            '' 
            Dim sourcePath As String = "D:\ACA.NET Webcast\Demo\Schedule Tables (Metric).dwg"
            Dim dbSource As Database = New Database(False, True)
            dbSource.ReadDwgFile(sourcePath, System.IO.FileShare.Read, True, "")

            ''  get the source dictionary   
            Dim dictMvBlkDef As New DictionaryMultiViewBlockDefinition(dbSource)

            ''  get the list of def ids that you want to import 
            ''  (1) if you want to import everything from the given dictionary, use this.
            'Dim objCollectionSrc As ObjectIdCollection = dictMvBlkDef.Records

            ''  (2) if you want to import a specific style, use this. 
            Dim objCollectionSrc As New ObjectIdCollection
            objCollectionSrc.Add(dictMvBlkDef.GetAt("M_Aec6_Door_Tag"))  '' the name of def you want to import  

            ''  now use CloningHelper class to import styles.   

            Dim helper As New Autodesk.Aec.ApplicationServices.Utility.CloningHelper

            ''  finally call clone. 
            helper.Clone(dbSource, dbDestination, objCollectionSrc, dictMvBlkDef.RecordType, True)

        Catch ex As Exception
            ed.WriteMessage("error in AcaImportPropSetDef" + ex.Message + vbCrLf)
            Return
        End Try

        ed.WriteMessage("Property set definition DoorObjects has been successfully imported" + vbCrLf)
    End Sub

End Class