﻿#Region "Copyright Note"
'' 
'' (C) Copyright 2009 by Autodesk, Inc.
'' 
'' Permission to use, copy, modify, and distribute this software in
'' object code form for any purpose and without fee is hereby granted,
'' provided that the above copyright notice appears in all copies and
'' that both that copyright notice and the limited warranty and
'' restricted rights notice below appear in all supporting
'' documentation.
''
'' AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS.
'' AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
'' MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC.
'' DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
'' UNINTERRUPTED OR ERROR FREE.
'' 
'' Use, duplication, or disclosure by the U.S. Government is subject to
'' restrictions set forth in FAR 52.227-19 (Commercial Computer
'' Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
'' (Rights in Technical Data and Computer Software), as applicable.
''
#End Region

#Region "Imports"
'' set here or set it in the project properties/references. (I'm setting the project here for VB.NET) 
''
Imports System.Reflection
Imports System.Collections.Specialized '' need this for StringCollection
'Imports System.Windows.Forms
' AutoCAD
Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.AutoCAD.ApplicationServices
Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.EditorInput
Imports Autodesk.AutoCAD.Geometry
' ACA (Aec)
Imports Autodesk.Aec.DatabaseServices
Imports Autodesk.Aec.Arch.DatabaseServices
Imports Autodesk.Aec.PropertyData.DatabaseServices '' we need this for prop set def 

' To resolve Object/Entity ambiguities
Imports AcObject = Autodesk.AutoCAD.DatabaseServices.DBObject
Imports AecObject = Autodesk.Aec.DatabaseServices.DBObject
Imports AcEntity = Autodesk.AutoCAD.DatabaseServices.Entity
Imports AecEntity = Autodesk.Aec.DatabaseServices.Entity

' To resolve Application ambiguities
Imports AcadApp2 = Autodesk.AutoCAD.ApplicationServices.Application
'Imports WinApp = System.Windows.Forms.Application

' To resolve TransactionManager ambiguities
Imports DbTrMgr = Autodesk.AutoCAD.DatabaseServices.TransactionManager 'wraps AcDbTransactionManager
Imports AppTrMgr = Autodesk.AutoCAD.ApplicationServices.TransactionManager 'wraps AcTransactionManager

' To resolve ObjectId ambiguities
Imports ObjectId = Autodesk.AutoCAD.DatabaseServices.ObjectId
Imports ObjectIdCollection = Autodesk.AutoCAD.DatabaseServices.ObjectIdCollection

' specific to this lab 
Imports ScheduleLabs

#End Region

''  in this lab, we define a new schedule table style.  
'' 
''    +-------------------------------------+                             
''    |           ACA Table                 |                   O
''    +-------------------------------------+                  / \
''    |     Dimensions     |      Make      |                 O   O 
''    +--------------------+----------------+                / \   \
''    | Width   |  Height  |   ManualProp   |               O   O   O 
''    +---------+----------+----------------+               |   |   | 
''    |         |          |                |
''    | column1 | column2  |    column3     |
''    |         |          |                |
''    |         |          |                |
'' 


Public Class ACANet_ScheduleTableStyle
    ''
    ''  create schedule table style 
    ''  for clarity, we are making it minimum
    ''
    <CommandMethod("ACANetScheduleLabs", "AcaCreateScheduleTableStyle", CommandFlags.Modal)> _
    Public Shared Sub createScheduleTableStyle()

        Dim db As Database = HostApplicationServices.WorkingDatabase
        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor

        ''  the name of the schedule table style. 
        ''  hard coding for simplicity.
        ''  
        Dim scheduleTableStyleName As String = "ACAScheduleTableStyle" ''  this will be the key in the dictionary 

        Try
            ''  (1) create a new schedule table stye object 
            '' 
            Dim tableStyle As New ScheduleTableStyle()
            tableStyle.SetToStandard(db)
            tableStyle.SubSetDatabaseDefaults(db)

            ''  General tab
            '' 
            tableStyle.Description = "Schedule Table Style defined by ACA.NET"

            ''  Default Format 
            ''  set the properties under DefaultCellFormat if you want to change here. 
            '' 
            tableStyle.DefaultCellFormat.Alignment = AttachmentPoint.MiddleCenter

            ''  (2) Applies To tab 
            ''
            Dim appliedTo = New StringCollection()
            appliedTo.Add("AecDbDoor")       '' to apply to a door object, use this.  
            'appliedTo.Add("AecDbWindow")    '' to apply to more than one object type, add more here. 
            tableStyle.AppliesToFilter = appliedTo

            ''  (3) Columns tab 
            ''      this is the main one for formating table.  
            '' 
            ''      we are hard coding names of property definitions. adjust names for your needs. 
            ''      see the Lab1 for actual names used here. 
            ''   
            ''  in order to define columns, we need to know which properties we want to use.
            ''  let's assume we want to make a table using:
            ''      prop set def = "ACADoorObjects"
            ''      prop def     = "ACAWidth", "ACAHeight" and "ACAMAnualProp" 
            ''  
            ''  note: this is not most efficient way of creating columns, 
            ''  but clarify what kind of data we are trying to get. 

            ''  (3.1) add three columns using our "ACADoorObjects" prop set def defined in Lab1. 
            '' 
            ''    +--------------------+----------------+                
            ''    | Width   |  Height  |   ManualProp   |               
            ''    +---------+----------+----------------+               
            ''    |         |          |                |
            ''    | column1 | column2  |    column3     |
            ''    |         |          |                |
            ''    |         |          |                |
            '' 
            Dim column1 As ScheduleTableStyleColumn = createColumn("ACADoorObjects", "ACAWidth")
            Dim index1 As Integer = tableStyle.Columns.Add(column1)

            ''  add another one 
            Dim column2 As ScheduleTableStyleColumn = createColumn("ACADoorObjects", "ACAHeight")
            Dim index2 As Integer = tableStyle.Columns.Add(column2)

            ''  one more 
            Dim column3 As ScheduleTableStyleColumn = createColumn("ACADoorObjects", "ACAManualProp")
            Dim index3 As Integer = tableStyle.Columns.Add(column3)

            ''  MH: note - formula is not exposed yet.  

            ''  (3.2) add headers 
            ''        now, this is where tree structure comes in. see the illustration above.   
            ''  
            ''                                                             R 
            ''    +-------------------------------------+                / | \
            ''    | Width   |  Height  |   ManualProp   |               O  O  O 

            Dim tree As ScheduleTableStyleHeaderTree = tableStyle.Tree
            Dim nodes As List(Of ScheduleTableStyleHeaderNode) = New List(Of ScheduleTableStyleHeaderNode)()

            ''  we want to group Width and Height under "Dimension" 
            '' 
            ''                                                             R
            ''    +-------------------------------------+                 / \
            ''    |     Dimensions     |                |                N   O 
            ''    +--------------------|   ManualProp   |               / \   
            ''    | Width   |  Height  |                |              O   O   
            '' 
            nodes.Add(column1)
            nodes.Add(column2)
            Dim nodeDimension As ScheduleTableStyleHeaderNode = tree.InsertNode("Dimension", 1, tree.Root, nodes.ToArray)  '' this adds to the left. intended. 

            ''  put ManualProp under "Make"
            ''                                                              R
            ''    +-------------------------------------+                  / \
            ''    |     Dimensions     |      Make      |                 O   N 
            ''    +--------------------+----------------+                / \   \
            ''    | Width   |  Height  |   ManualProp   |               O   O   O 
            '' 
            nodes.Clear()
            nodes.Add(column3)
            Dim nodeMake As ScheduleTableStyleHeaderNode = tree.InsertNode("Make", 2, tree.Root, nodes.ToArray)

            ''  add top header above "Dimension" and "Make"  
            ''                                                              R
            ''    +-------------------------------------+                   |        
            ''    |           ACA Table                 |                   N
            ''    +-------------------------------------+                  / \
            ''    |     Dimensions     |      Make      |                 O   O 
            ''    +--------------------+----------------+                / \   \
            ''    | Width   |  Height  |   ManualProp   |               O   O   O 
            '' 
            nodes.Clear()
            nodes.Add(nodeDimension)
            nodes.Add(nodeMake)
            tree.InsertNode("ACA Table", 1, tree.Root, nodes.ToArray)

            ''  (4) other tabs are optional. 
            ''  
            ''  Sorting tab 
            ''  
            ''  e.g., if you want to add sort by ACAWidth in descending order, 
            ''  here is an example. 
            '' 
            Dim sortings As ScheduleTableSortingCollection = tableStyle.Sortings()

            Dim sorting As New ScheduleTableSorting()
            sorting.SetToStandard(db)
            sorting.SubSetDatabaseDefaults(db)


            Dim idPropSetDef As ObjectId = findPropertySetDef("ACADoorObjects")
            Dim idx As Integer = findPropertyDef(idPropSetDef, "ACAWidth")
            sorting.SetColumn(idPropSetDef, idx, False)
            sorting.SortOrder = SortOrder.Descending

            sortings.Add(sorting)

            ''  Layout tab 
            ''
            tableStyle.Title = "ACA Schedule Table"
            Dim titleCellFormat As ScheduleTableCellFormat = tableStyle.TitleFormat
            Dim h As Double = titleCellFormat.Height
            titleCellFormat.Height = h * 2

            ''  (4)  finally add the schedule table style to the database 
            '' 
            Using tr As Transaction = db.TransactionManager.StartTransaction

                ''  get the dictionary of schedule table style 
                '' 
                Dim dictScheduleTableStyle = New DictionaryScheduleTableStyle(db)

                ''  check the name.  do we have it already?  
                '' 
                If dictScheduleTableStyle.Has(scheduleTableStyleName, tr) Then
                    ed.WriteMessage("error - the schedule table style with the given name already exists: " + scheduleTableStyleName + vbCrLf)
                    Return
                End If

                ''  finally, add the newly created table style to the dictionary. 
                '' 
                dictScheduleTableStyle.AddNewRecord(scheduleTableStyleName, tableStyle)
                tr.AddNewlyCreatedDBObject(tableStyle, True)
                tr.Commit()

            End Using

        Catch ex As Exception
            ed.WriteMessage("error in CreateScheduleTableStyle: " + ex.ToString + vbCrLf)
            Return
        End Try

        ed.WriteMessage("schedule table style " + scheduleTableStyleName + " is successfully created." + vbCrLf)

    End Sub

    ''
    ''  Create a column 
    '' 
    ''  This is what [Add Column..] button does in Columns tab. 
    ''
    Private Shared Function createColumn(ByVal propSetDefName As String, ByVal propDefName As String) As ScheduleTableStyleColumn

        Dim db As Database = HostApplicationServices.WorkingDatabase
        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor

        Try
            ''  create a Schedule table style column instance 
            '' 
            Dim column As ScheduleTableStyleColumn = New ScheduleTableStyleColumn
            column.SetToStandard(db)
            column.SubSetDatabaseDefaults(db)

            ''  set the type of column. either normal, formula or quantity 
            ''  
            column.ColumnType = ScheduleTableStyleColumnType.Normal '' Normal/Formula/Quantity (note: Formula not fully exposed, yet.) 

            ''  we need to get the id of property set definition and the index of property. 
            ''  see the helper functions below 
            '' 
            Dim idPropSetDef As ObjectId = findPropertySetDef(propSetDefName)
            If idPropSetDef = Nothing Then
                Return Nothing
            End If

            Dim indexPropDef As Integer = findPropertyDef(idPropSetDef, propDefName)
            If indexPropDef < 0 Then
                Return Nothing
            End If

            ''  set them to the table column.
            column.PropertySetDefinitionId = idPropSetDef
            column.PropertyId = indexPropDef

            ''  for simplicity, we are setting the heading same as the name of the property definition for now. 
            '' 
            column.Heading = propDefName

            ''  MH: note: currently example of automatic property shows as "?".  

            Return column

        Catch ex As Exception
            ed.WriteMessage("error in CreateColumn: " + ex.ToString + vbCrLf)
            Return Nothing
        End Try

        Return Nothing

    End Function

    '' ============================================================================== 
    '' 
    ''   Helper function: findPropertySetDef() 
    '' 
    '' ==================================== 
    '' 
    ''  find a property set definition with the given name from the dictionary of property set definition,
    ''  and return its object id.  
    ''
    ''  note this is the same as one in the lab2. you can replace this withUtils.  
    '' 
    Private Shared Function findPropertySetDef(ByVal propSetDefName As String) As ObjectId

        Dim db As Database = HostApplicationServices.WorkingDatabase
        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor

        Dim idPropSetDef As ObjectId = Nothing

        Try
            ''  get the dictionary of property set definitions. 
            '' 
            Dim dictPropSetDef = New DictionaryPropertySetDefinitions(db)

            Using tr As Transaction = db.TransactionManager.StartTransaction

                ''  do we have a property set definition with the given name?  
                '' 
                If Not dictPropSetDef.Has(propSetDefName, tr) Then
                    ''  if not, return 
                    ed.WriteMessage("cannot find property set definition: " + propSetDefName + vbCrLf)
                    Return Nothing
                End If
                tr.Commit()
            End Using

            ''  get the id of property set definition from the name
            ''
            idPropSetDef = dictPropSetDef.GetAt(propSetDefName)

        Catch ex As Exception
            ed.WriteMessage("error in findPropertySetDef: " + ex.ToString + vbCrLf)
        End Try

        Return idPropSetDef

    End Function

    '' ============================================================================== 
    '' 
    ''   Helper function: findPropertyDef() 
    '' 
    '' ==================================== 
    ''
    ''  find a property definition with the given name in the given property set def (this is each property) 
    '' 
    Private Shared Function findPropertyDef(ByVal idPropSetDef As ObjectId, ByVal propDefName As String) As Integer

        Dim db As Database = HostApplicationServices.WorkingDatabase
        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor

        Dim indexPropDef As Integer = -1

        Try
            Using tr As Transaction = db.TransactionManager.StartTransaction

                ''  open the given property set definition 
                '' 
                Dim obj As Object = tr.GetObject(idPropSetDef, OpenMode.ForRead)
                Dim propSetDef As PropertySetDefinition = CType(obj, PropertySetDefinition)

                ''  get the list of difinitions in it 
                '' 
                Dim definitions As PropertyDefinitionCollection = propSetDef.Definitions

                ''  do we have the difinition with the given name?  
                ''
                indexPropDef = definitions.IndexOf(propDefName)
                If indexPropDef < 0 Then
                    ''  no, cannot find it. 
                    ed.WriteMessage("Cannot find the property: " + propDefName + vbCrLf)
                End If

                tr.Commit()

            End Using

        Catch ex As Exception
            ed.WriteMessage("Error in findPropertyDef: " + ex.ToString + vbCrLf)
        End Try

        ''  we have it. return the index.   
        Return indexPropDef

    End Function

End Class