﻿#Region "Copyright Note"
'' 
'' (C) Copyright 2009 by Autodesk, Inc.
'' 
'' Permission to use, copy, modify, and distribute this software in
'' object code form for any purpose and without fee is hereby granted,
'' provided that the above copyright notice appears in all copies and
'' that both that copyright notice and the limited warranty and
'' restricted rights notice below appear in all supporting
'' documentation.
''
'' AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS.
'' AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
'' MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC.
'' DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
'' UNINTERRUPTED OR ERROR FREE.
'' 
'' Use, duplication, or disclosure by the U.S. Government is subject to
'' restrictions set forth in FAR 52.227-19 (Commercial Computer
'' Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
'' (Rights in Technical Data and Computer Software), as applicable.
''
#End Region

#Region "Imports"
'' set here or set it in the project properties/references. (I'm setting the project here for VB.NET) 
''
Imports System.Reflection
Imports System.Collections.Specialized '' need this for StringCollection
'Imports System.Windows.Forms
' AutoCAD
Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.AutoCAD.ApplicationServices
Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.EditorInput
Imports Autodesk.AutoCAD.Geometry
' ACA (Aec)
Imports Autodesk.Aec.DatabaseServices
Imports Autodesk.Aec.Arch.DatabaseServices
Imports Autodesk.Aec.PropertyData.DatabaseServices '' we need this for prop set def 

' To resolve Object/Entity ambiguities
Imports AcObject = Autodesk.AutoCAD.DatabaseServices.DBObject
Imports AecObject = Autodesk.Aec.DatabaseServices.DBObject
Imports AcEntity = Autodesk.AutoCAD.DatabaseServices.Entity
Imports AecEntity = Autodesk.Aec.DatabaseServices.Entity

' To resolve Application ambiguities
Imports AcadApp2 = Autodesk.AutoCAD.ApplicationServices.Application
'Imports WinApp = System.Windows.Forms.Application

' To resolve TransactionManager ambiguities
Imports DbTrMgr = Autodesk.AutoCAD.DatabaseServices.TransactionManager 'wraps AcDbTransactionManager
Imports AppTrMgr = Autodesk.AutoCAD.ApplicationServices.TransactionManager 'wraps AcTransactionManager

' To resolve ObjectId ambiguities
Imports ObjectId = Autodesk.AutoCAD.DatabaseServices.ObjectId
Imports ObjectIdCollection = Autodesk.AutoCAD.DatabaseServices.ObjectIdCollection
#End Region

'' 
''  in this lab, we create an instance of schedule table.  
'' 
''  Lab5 uses helper in Utils: findStyle. 
'' 

Public Class ACANet_ScheduleTable
    ''
    ''  create schedule table 
    ''  for clarity, we are making it minimum
    ''
    <CommandMethod("ACANetScheduleLabs", "AcaCreateScheduleTable", CommandFlags.Modal)> _
    Public Sub createScheduleTable()

        Dim db As Database = HostApplicationServices.WorkingDatabase
        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor

        ''  (1) pick a location to place the tag
        '' 
        Dim resPt As PromptPointResult = ed.GetPoint("Pick a location of the tag ")
        If resPt.Status <> PromptStatus.OK Then
            ed.WriteMessage("failed to pick a location of the tag")
            Exit Sub
        End If
        Dim pt As Point3d = resPt.Value

        Try
            ''  (2) create a instance of schedule table 
            '' 
            Dim table As New ScheduleTable
            table.SetToStandard(db)
            table.SetDatabaseDefaults(db)
            table.SetDefaultLayer()

            Dim dictScheduleTableStyle = New DictionaryScheduleTableStyle(db)
            table.StyleId = Utils.findStyle(dictScheduleTableStyle, "ACAScheduleTableStyle")

            ''
            ''
            Using tr As Transaction = db.TransactionManager.StartTransaction
                Utils.AddToModelSpaceAndTransaction(table, db, tr)

                Dim ids As ObjectIdCollection = Utils.findObjects(GetType(Door))

                table.SetSelectionSet(ids, New ObjectIdCollection())

                table.AutomaticUpdate = True

                table.Location = pt
                table.Scale = 100.0

                tr.Commit()
            End Using

        Catch ex As Exception
            ed.WriteMessage("failed to create a schedule table." + vbCrLf)
        End Try

        ed.WriteMessage("schedule table is successfully created." + vbCrLf)

    End Sub

End Class