﻿#Region "Copyright Note"
'' 
'' (C) Copyright 2009 by Autodesk, Inc.
'' 
'' Permission to use, copy, modify, and distribute this software in
'' object code form for any purpose and without fee is hereby granted,
'' provided that the above copyright notice appears in all copies and
'' that both that copyright notice and the limited warranty and
'' restricted rights notice below appear in all supporting
'' documentation.
''
'' AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS.
'' AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
'' MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC.
'' DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
'' UNINTERRUPTED OR ERROR FREE.
'' 
'' Use, duplication, or disclosure by the U.S. Government is subject to
'' restrictions set forth in FAR 52.227-19 (Commercial Computer
'' Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
'' (Rights in Technical Data and Computer Software), as applicable.
''
#End Region

#Region "Imports"
'' set here or set it in the project properties/references. (I'm setting the project here for VB.NET) 
''
Imports System.Reflection
Imports System.Collections.Specialized '' need this for StringCollection
'Imports System.Windows.Forms
' AutoCAD
Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.AutoCAD.ApplicationServices
Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.EditorInput
' ACA (Aec)
Imports Autodesk.Aec.DatabaseServices
Imports Autodesk.Aec.Arch.DatabaseServices
Imports Autodesk.Aec.PropertyData.DatabaseServices '' we need this for prop set def 

' To resolve Object/Entity ambiguities
Imports AcObject = Autodesk.AutoCAD.DatabaseServices.DBObject
Imports AecObject = Autodesk.Aec.DatabaseServices.DBObject
Imports AcEntity = Autodesk.AutoCAD.DatabaseServices.Entity
Imports AecEntity = Autodesk.Aec.DatabaseServices.Entity

' To resolve Application ambiguities
Imports AcadApp2 = Autodesk.AutoCAD.ApplicationServices.Application
'Imports WinApp = System.Windows.Forms.Application

' To resolve TransactionManager ambiguities
Imports DbTrMgr = Autodesk.AutoCAD.DatabaseServices.TransactionManager 'wraps AcDbTransactionManager
Imports AppTrMgr = Autodesk.AutoCAD.ApplicationServices.TransactionManager 'wraps AcTransactionManager

' To resolve ObjectId ambiguities
Imports ObjectId = Autodesk.AutoCAD.DatabaseServices.ObjectId
Imports ObjectIdCollection = Autodesk.AutoCAD.DatabaseServices.ObjectIdCollection
#End Region

Public Class Utils

    '' ============================================================================== 
    '' 
    ''   Helper function: findStyle() 
    '' 
    '' ==================================== 
    '' 
    ''  find a style (or dictionary record) with the given name from the given dictionary, 
    ''  and return its object id.  
    '' 
    Public Shared Function findStyle(ByRef dict As Autodesk.Aec.DatabaseServices.Dictionary, ByVal key As String) As ObjectId

        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor
        Dim db As Database = HostApplicationServices.WorkingDatabase

        Dim id As ObjectId = Nothing  '' id of the style we are looking for. return value 

        Try
            Using tr As Transaction = db.TransactionManager.StartTransaction

                ''  do we have a property set definition with the given name?  
                '' 
                If Not dict.Has(key, tr) Then
                    ''  if not, return 
                    ed.WriteMessage("cannot find the style: " + key + vbCrLf)
                    Return Nothing
                End If
                tr.Commit()
            End Using

            ''  get the id of property set definition from the name
            ''
            id = dict.GetAt(key)

        Catch ex As Exception
            ed.WriteMessage("error in findPropertySetDef: " + ex.ToString + vbCrLf)
        End Try

        Return id

    End Function

    '' ============================================================================== 
    '' 
    ''   Helper function: findObjects
    '' 
    '' ====================================
    ''
    ''  collect ids of objects of the given type in the model space of the current drawings 
    '' 
    Public Shared Function findObjects(ByVal targetType As Type) As ObjectIdCollection

        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor
        Dim db As Database = Application.DocumentManager.MdiActiveDocument.Database

        Dim ids As New ObjectIdCollection '' return value. save ids here.

        Dim targetClass As RXClass = RXClass.GetClass(targetType) '' ex., RXClass = AcDbDoor <--> Type = Door 

        Try
            Using tr As Transaction = db.TransactionManager.StartTransaction

                ''  get the model space
                '' 
                Dim blkTbl As BlockTable = CType(tr.GetObject(db.BlockTableId, OpenMode.ForRead), BlockTable)
                Dim ms As BlockTableRecord = CType(tr.GetObject(blkTbl(BlockTableRecord.ModelSpace), OpenMode.ForRead), BlockTableRecord)

                '' iterate over model space and keeps the ids of indended type.
                '' 
                For Each id As ObjectId In ms
                    'ed.WriteMessage("Name = " + id.ObjectClass.Name + vbCrLf)

                    If id.ObjectClass.IsDerivedFrom(targetClass) Then
                        ids.Add(id)
                    End If
                Next

            End Using

        Catch ex As Exception
            ed.WriteMessage("error in findObjects: " + ex.ToString + vbCrLf)
        End Try

        Return ids

    End Function

    '' ============================================================================== 
    '' 
    ''   Helper Function: addToModelSpaceAndTransaction() 
    '' 
    '' ==================================== 
    ''
    ''  find a property definition with the given name in the given property set def (this is each property) 
    '' 

    ''  Generic AutoCAD.NET helper to add an entity to Db and Transaction
    '' 
    Public Shared Function AddToModelSpaceAndTransaction(ByRef ent As AcEntity, ByRef db As Database, ByRef tr As Transaction) As Boolean

        Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor

        Try
            Dim bt As BlockTable = CType(tr.GetObject(db.BlockTableId, OpenMode.ForRead), BlockTable)
            Dim btr As BlockTableRecord = _
             CType(tr.GetObject(bt(BlockTableRecord.ModelSpace), OpenMode.ForWrite), BlockTableRecord)
            btr.AppendEntity(ent)
            tr.AddNewlyCreatedDBObject(ent, True)
        Catch ex As Exception
            ed.WriteMessage("Exception in AddToModelSpaceAndTransaction: " + ex.Message)
            Return False
        End Try

        '' OK if we are here
        Return True

    End Function

End Class
