
// $Header: //Desktop/Source/TestTools/MgdDbgAec/App/App.cs#4 $
// $Author: ayyara $ $DateTime: 2008/03/11 16:09:16 $ $Change: 95152 $
//
// (C) Copyright 2004 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//


using System;
using System.IO;
using System.Collections;
using System.Diagnostics;
using System.Reflection;
using Autodesk.AutoCAD.Runtime;
using Autodesk.AutoCAD.ApplicationServices;


[assembly:ExtensionApplication(typeof(MgdDbgAec.App))]
[assembly:CommandClass(typeof(MgdDbgAec.TestCmds))]

namespace MgdDbgAec
{
	
	public class App : IExtensionApplication
	{
        private ArrayList m_tests = new ArrayList();

		public void
		Initialize()
		{
            MgdDbg.Utils.AcadUi.PrintToCmdLine("\nLoading MgdDbgAec...");
            
                // register any Snoop Collector Extension objects that we have
            Snoop.CollectorExts.ObjectBase snoopDBObject = new Snoop.CollectorExts.ObjectBase();
            Snoop.CollectorExts.EntityBase snoopDBEntity = new Snoop.CollectorExts.EntityBase();
            Snoop.CollectorExts.ImpObjectBase snoopImpObj = new Snoop.CollectorExts.ImpObjectBase();
            Snoop.CollectorExts.DictBase snoopDicts = new Snoop.CollectorExts.DictBase();
            Snoop.CollectorExts.DictRecBase snoopDictRecs = new Snoop.CollectorExts.DictRecBase();
            Snoop.CollectorExts.DictRecPropData snoopDictRecsPropData = new Snoop.CollectorExts.DictRecPropData();
            
            Snoop.CollectorExts.ObjectArch snoopDBObjectArch = new Snoop.CollectorExts.ObjectArch();
            Snoop.CollectorExts.EntityArch snoopDBEntityArch = new Snoop.CollectorExts.EntityArch();
            Snoop.CollectorExts.ImpObjectArch snoopImpObjArch = new Snoop.CollectorExts.ImpObjectArch();
            Snoop.CollectorExts.DictArch snoopDictsArch = new Snoop.CollectorExts.DictArch();
            Snoop.CollectorExts.Geometry snoopGeom = new Snoop.CollectorExts.Geometry();
            Snoop.CollectorExts.Modeler snoopModeler = new Snoop.CollectorExts.Modeler();
            Snoop.CollectorExts.MiscArch snoopMiscArch = new Snoop.CollectorExts.MiscArch();
            Snoop.CollectorExts.MiscDb snoopMiscDb = new Snoop.CollectorExts.MiscDb();
            Snoop.CollectorExts.RxObject snoopRxObject = new Snoop.CollectorExts.RxObject();

            Snoop.CollectorExts.EntityStructure snoopDBEntityStructure = new Snoop.CollectorExts.EntityStructure();
            Snoop.CollectorExts.DictStructure snoopStructure = new MgdDbgAec.Snoop.CollectorExts.DictStructure();
            Snoop.CollectorExts.ImpObjectStructure snoopImpObjStructure = new Snoop.CollectorExts.ImpObjectStructure();

            Snoop.CollectorExts.ImpObjectPropData snoopImpObjPropData = new Snoop.CollectorExts.ImpObjectPropData();

                // force our wrapper DLLs in so that our protocol extension is registered the first time.
                // TBD: check with Albert to see if there is a more formal means to accomplish this!  (jma - 08/31/04)
            Autodesk.Aec.DatabaseServices.MassElement fakeMe = new Autodesk.Aec.DatabaseServices.MassElement();
            fakeMe.Dispose();
            
            Autodesk.Aec.Arch.DatabaseServices.Door fakeDoor = new Autodesk.Aec.Arch.DatabaseServices.Door();
            fakeDoor.Dispose();

            Autodesk.Aec.PropertyData.DatabaseServices.PropertySet fakeSet = new Autodesk.Aec.PropertyData.DatabaseServices.PropertySet();
            fakeSet.Dispose();

            Autodesk.Aec.Structural.DatabaseServices.Member fakeMember = new Autodesk.Aec.Structural.DatabaseServices.Member();
            
                // add our App context menu
            MgdDbgAec.AppContextMenu.AddContextMenu();

            CreateAndAddTestFuncs();

            AddAssemblies();
		}
		
		public void
		Terminate()
		{
            MgdDbgAec.AppContextMenu.RemoveContextMenu();

            RemoveAndFreeTestFuncs();
        }

        /// <summary>
        /// 
        /// </summary>
        private void CreateAndAddTestFuncs ()
        {
            m_tests.Add(new MgdDbgAec.ObjTests.MakeEntsArch());
            m_tests.Add(new MgdDbgAec.ObjTests.MakeEntsBase());
            m_tests.Add(new MgdDbgAec.ObjTests.MakeEntsStructure());
            m_tests.Add(new MgdDbgAec.ObjTests.MakeStylesArch());
            m_tests.Add(new MgdDbgAec.ObjTests.MakeStylesBase());
            m_tests.Add(new MgdDbgAec.ObjTests.ModifyArch());
            m_tests.Add(new MgdDbgAec.ObjTests.ModifyBase());
            m_tests.Add(new MgdDbgAec.ObjTests.QueryArch());
            m_tests.Add(new MgdDbgAec.ObjTests.QueryBase());
            m_tests.Add(new MgdDbgAec.ObjTests.QueryStructure());
            m_tests.Add(new MgdDbgAec.ObjTests.HardwireHouse());
            m_tests.Add(new MgdDbgAec.ObjTests.HardwireBuilding());
            m_tests.Add(new MgdDbgAec.ObjTests.PropData());

            foreach (MgdDbg.Test.MgdDbgTestFuncs testFunc in m_tests) {
                MgdDbg.Test.MgdDbgTestFuncs.AddTestFuncsToFramework(testFunc);
            }
        }

        /// <summary>
        /// 
        /// </summary>
        private void RemoveAndFreeTestFuncs ()
        {
            foreach (MgdDbg.Test.MgdDbgTestFuncs testFunc in m_tests) {
                MgdDbg.Test.MgdDbgTestFuncs.RemoveTestFuncsFromFramework(testFunc);
            }
        }

        /// <summary>
        /// 
        /// </summary>
        private void AddAssemblies ()
        {
            ArrayList assembliesToLoad = new ArrayList();
            assembliesToLoad.Add("AecBaseMgd");
            assembliesToLoad.Add("AecArchMgd");
            assembliesToLoad.Add("AecPropDataMgd");
            assembliesToLoad.Add("AecStructureMgd");

            System.Reflection.AssemblyName[] assemblyNames = System.Reflection.Assembly.GetExecutingAssembly().GetReferencedAssemblies();
            foreach (System.Reflection.AssemblyName assemblyName in assemblyNames) {
                if (assembliesToLoad.Contains(assemblyName.Name)) {
                    MgdDbg.Snoop.Forms.Editor.assemblyNamesToLoad.Add(assemblyName.FullName);
                }
            }
        }

	}
}
