
// $Header: //Desktop/Source/TestTools/MgdDbgAec/ObjTests/HardwireHouse.cs#1 $
// $Author: fergusja $ $DateTime: 2007/03/16 11:08:08 $ $Change: 68854 $
//
// (C) Copyright 2006 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//


using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;

using Autodesk.AutoCAD.DatabaseServices;
using Autodesk.AutoCAD.Geometry;
using AcDb = Autodesk.AutoCAD.DatabaseServices;

using AecDb = Autodesk.Aec.DatabaseServices;
using Autodesk.Aec.Geometry;
using Autodesk.Aec.Arch.Geometry;
using Autodesk.Aec.DatabaseServices;
using Autodesk.Aec.Arch.DatabaseServices;

using AecAppSrvcs = Autodesk.Aec.ApplicationServices;
using ArchDb = Autodesk.Aec.Arch.DatabaseServices;
using AcadApp = Autodesk.AutoCAD.ApplicationServices.Application;

using MgdDbg;
using MgdDbg.Test;

namespace MgdDbgAec.ObjTests {

    class HardwireHouse : MgdDbgTestFuncs {

        private Database    m_db = null;

		public
		HardwireHouse()
		{		    
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Simple House 1", "Hardwire a simple house", "Hardwired Buildings", new MgdDbgTestFuncInfo.TestFunc(House1), MgdDbgTestFuncInfo.TestType.Create));
        }
        
        public void
        House1()
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            using (MgdDbg.CompBldrCurSpace compBldr = new MgdDbg.CompBldrCurSpace(m_db)) {
                compBldr.Start();
                compBldr.PushXform(MgdDbg.Utils.Db.GetUcsMatrix(m_db));

                Point2dCollection outlinePts = House1_OutlinePoints();

                AcDb.ObjectIdCollection extWallIds = CreateWallsFromPoints(compBldr, AcDb.ObjectId.Null, outlinePts);
                Debug.Assert(extWallIds.Count == 9);    // sanity check

                AcDb.ObjectIdCollection extDoorIds = House1_ExteriorDoors(compBldr, extWallIds);
                Debug.Assert(extDoorIds.Count == 2);

                AcDb.ObjectIdCollection extWindowIds = House1_ExteriorWindows(compBldr, extWallIds);
                Debug.Assert(extWindowIds.Count == 4);

                AcDb.ObjectIdCollection extWinAssmIds = House1_ExteriorWinAssembly(compBldr, extWallIds);
                Debug.Assert(extWinAssmIds.Count == 1);

                AcDb.ObjectId roofId = House1_Roof(compBldr, outlinePts);

                compBldr.Commit();
            }
        }

        private Point2dCollection
        House1_OutlinePoints()
        {
            Point2dCollection pts = new Point2dCollection();

            double x = 0.0;
            double y = 0.0;

            pts.Add(new Point2d(x, y));

            x += 480.0;
            pts.Add(new Point2d(x, y));

            y -= 120.0;
            pts.Add(new Point2d(x, y));

            x += 120.0;
            pts.Add(new Point2d(x, y));

            y += 240.0;
            pts.Add(new Point2d(x, y));

            x += 60.0;
            pts.Add(new Point2d(x, y));

            y += 240.0;
            pts.Add(new Point2d(x, y));

            x -= 480.0;
            pts.Add(new Point2d(x, y));

            x -= 180.0;  y -= 180.0;
            pts.Add(new Point2d(x, y));

            x = 0.0; y = 0.0;
            pts.Add(new Point2d(x, y));

            return pts;
        }

        private AcDb.ObjectIdCollection
        House1_ExteriorDoors(CompBldr compBldr, AcDb.ObjectIdCollection wallIds)
        {
            AcDb.ObjectIdCollection extDoorIds = new AcDb.ObjectIdCollection();

                // create first door centered in entry way
            Door door1 = new Door();
            door1.Width = 48.0;
            door1.Height = 84.0;
            compBldr.AddToDb(door1);

            AnchorOpeningBaseToWall anchor1 = new AnchorOpeningBaseToWall();
            anchor1.CurveId = wallIds[2];   // the front entry way wall
            anchor1.AnchorX.MeasureToType = CurveXMeasureToType.ToCenter;
            anchor1.AnchorX.OffsetType = CurveXOffsetType.FromMidpointOfCurve;
            anchor1.AnchorX.OffsetDistance = 0.0;

            door1.SetAnchor(anchor1);
            extDoorIds.Add(door1.ObjectId);

                // create second door 2' off back corner
            Door door2 = new ArchDb.Door();
            door2.Width = 36.0;
            door2.Height = 80.0;
            compBldr.AddToDb(door2);

            AnchorOpeningBaseToWall anchor2 = new AnchorOpeningBaseToWall();
            anchor2.CurveId = wallIds[6];   // the back wall
            anchor2.AnchorX.MeasureToType = CurveXMeasureToType.ToStartEdge;
            anchor2.AnchorX.OffsetType = CurveXOffsetType.FromStartOfCurve;
            anchor2.AnchorX.OffsetDistance = 24.0;

            door2.SetAnchor(anchor2);
            extDoorIds.Add(door1.ObjectId);

            return extDoorIds;
        }

        private AcDb.ObjectIdCollection
        House1_ExteriorWindows(CompBldr compBldr, AcDb.ObjectIdCollection wallIds)
        {
            AcDb.ObjectIdCollection extWindowIds = new AcDb.ObjectIdCollection();

                // create a row of windows in the front
            double spacing = 12.0;
            double xDist = spacing;     // start 12" over from corner of wall
            double yDist = spacing;     // start 12" down from top of wall
            for (int i = 0; i < 4; i++) {
                Window tmpWin = new Window();
                tmpWin.Width = 24.0;
                tmpWin.Height = 48.0;
                compBldr.AddToDb(tmpWin);

                AnchorOpeningBaseToWall anchor1 = new AnchorOpeningBaseToWall();
                anchor1.CurveId = wallIds[0];   // the front, left wall
                anchor1.AnchorX.MeasureToType = CurveXMeasureToType.ToStartEdge;
                anchor1.AnchorX.OffsetType = CurveXOffsetType.FromStartOfCurve;
                anchor1.AnchorX.OffsetDistance = xDist;

                anchor1.AnchorY.OffsetType = CurveYOffsetType.FromRightEdge;
                anchor1.AnchorY.MeasureToType = CurveYMeasureToType.ToFrontEdge;
                anchor1.AnchorY.OffsetDistance = 0.0;

                anchor1.AnchorZ.OffsetFromType = CurveZOffsetFromType.FromTop;
                anchor1.AnchorZ.OffsetToType = CurveZOffsetToType.ToTop;
                anchor1.AnchorZ.OffsetDistance = -yDist;      // make the windows 12" from top of window to top of wall

                tmpWin.SetAnchor(anchor1);
                extWindowIds.Add(tmpWin.ObjectId);

                xDist += (tmpWin.Width + spacing);
                yDist += spacing;   // move down 12" each time
            }

            return extWindowIds;
        }

        private AcDb.ObjectIdCollection
        House1_ExteriorWinAssembly(CompBldr compBldr, AcDb.ObjectIdCollection wallIds)
        {
            AcDb.ObjectIdCollection extWinAssmIds = new AcDb.ObjectIdCollection();

                // create first door centered in entry way
            WindowAssembly winAssm = new WindowAssembly();
            winAssm.Length = 144.0;
            winAssm.Height = 60.0;
            compBldr.AddToDb(winAssm);

            AnchorOpeningBaseToWall anchor1 = new AnchorOpeningBaseToWall();
            anchor1.CurveId = wallIds[7];   // the back left corner angled wall
            anchor1.AnchorX.MeasureToType = CurveXMeasureToType.ToCenter;
            anchor1.AnchorX.OffsetType = CurveXOffsetType.FromMidpointOfCurve;
            anchor1.AnchorX.OffsetDistance = 0.0;

            anchor1.AnchorZ.OffsetFromType = CurveZOffsetFromType.FromCenter;
            anchor1.AnchorZ.OffsetToType = CurveZOffsetToType.ToCenter;
            anchor1.AnchorZ.OffsetDistance = 0.0;

            winAssm.SetAnchor(anchor1);
            extWinAssmIds.Add(winAssm.ObjectId);

            return extWinAssmIds;
        }

        private AcDb.ObjectId
        House1_Roof(CompBldr compBldr, Point2dCollection pts)
        {
            Ring ring = new Ring();

            for (int i = 0; i < pts.Count - 1; i++) {
                RoofSegment tmpSeg = new RoofSegment(new LineSegment2d(pts[i], pts[i + 1]));

                tmpSeg.Init();
                tmpSeg.SetOverhang(18.0, Tolerance.Global.EqualPoint);
                tmpSeg.NumberOfFacets = 1;
                tmpSeg.SetSlope(0, MgdDbg.Utils.Ge.DegreesToRadians(25.0), Tolerance.Global.EqualPoint);
                tmpSeg.SetHeight(0, 0.0, Tolerance.Global.EqualPoint);

                ring.AddSegment(tmpSeg, true);
            }

            RoofProfile roofProf = new RoofProfile();
            roofProf.AddRing(ring);
            roofProf.AddHeightAll(120.0);

            Roof roof = new Roof();
            roof.RoofProfile = roofProf;
            roof.SetThickness(8.0, ThicknessDirection.Roof, Tolerance.Global.EqualPoint);
            roof.CalculateRoof();

            compBldr.AddToDb(roof);

            return roof.ObjectId;
        }

        private AcDb.ObjectIdCollection
        CreateWallsFromPoints(CompBldr compBldr, AcDb.ObjectId wallStyleId, Point2dCollection pts)
        {
            AcDb.ObjectIdCollection wallIds = new AcDb.ObjectIdCollection();

            int len = pts.Count;
            for (int i = 0; i < len - 1; i++) {
                ArchDb.Wall wall = new ArchDb.Wall();
                wall.BaseHeight = 120.0;
                wall.Set(new Point3d(pts[i].X, pts[i].Y, 0.0), new Point3d(pts[i + 1].X, pts[i + 1].Y, 0.0), Vector3d.ZAxis);

                if (wallStyleId != AcDb.ObjectId.Null)   // if its null, SetDatabaseDefaults will set it to default
                    wall.StyleId = wallStyleId;

                compBldr.AddToDb(wall);
                wallIds.Add(wall.ObjectId);
            }

            return wallIds;
        }


    }
}
