// $Header: //Desktop/Source/TestTools/MgdDbgAec/ObjTests/MakeEntsArch.cs#2 $
// $Author: ayyara $ $DateTime: 2008/02/27 15:33:22 $ $Change: 94137 $
//
// (C) Copyright 2004 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//

using System;
using System.Windows.Forms;
using Autodesk.AutoCAD.DatabaseServices;
using Autodesk.AutoCAD.Geometry;
using Autodesk.AutoCAD.EditorInput;
using MgdDbg;
using MgdDbg.Test;
using Autodesk.Aec.DatabaseServices;
using Autodesk.Aec.Geometry;
using Autodesk.Aec.Arch.DatabaseServices;
using Autodesk.Aec.Arch.Geometry;

using AcDb = Autodesk.AutoCAD.DatabaseServices;

namespace MgdDbgAec.ObjTests
{
	/// <summary>
	/// Summary description for MakeEntsArch.
	/// </summary>
    public class MakeEntsArch : MgdDbgTestFuncs
    {
        private Database    m_db = null;
	    
        public
        MakeEntsArch()
        {
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Simple Door", "Simple hardwired door", typeof(Door), new MgdDbgTestFuncInfo.TestFunc(DoorSimple), MgdDbgTestFuncInfo.TestType.Create));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Simple Window", "Simple hardwired window", typeof(Window), new MgdDbgTestFuncInfo.TestFunc(WindowSimple), MgdDbgTestFuncInfo.TestType.Create));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Simple Wall", "Simple hardwired wall", typeof(Wall), new MgdDbgTestFuncInfo.TestFunc(WallSimple), MgdDbgTestFuncInfo.TestType.Create));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Simple Stair", "Simple hardwired stair", typeof(Stair), new MgdDbgTestFuncInfo.TestFunc(StairSimple), MgdDbgTestFuncInfo.TestType.Create));          
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Hardwire instance for all Styles", "Iterate through all styles and create an instance of each", typeof(Window), new MgdDbgTestFuncInfo.TestFunc(HardwireWindowInstances), MgdDbgTestFuncInfo.TestType.Create));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Hardwire instance for all Styles", "Iterate through all styles and create an instance of each", typeof(Door), new MgdDbgTestFuncInfo.TestFunc(HardwireDoorInstances), MgdDbgTestFuncInfo.TestType.Create));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Hardwire instance for all Styles", "Iterate through all styles and create an instance of each", typeof(Wall), new MgdDbgTestFuncInfo.TestFunc(HardwireWallInstances), MgdDbgTestFuncInfo.TestType.Create));
                       
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Hardwire Wall, Door, and Window", "Anchor a Door and Window to a Wall", typeof(AnchorOpeningBaseToWall), new MgdDbgTestFuncInfo.TestFunc(AnchorDoorAndWindowToWall), MgdDbgTestFuncInfo.TestType.Create));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Hardwire Stair and Railing", "Anchor a railing to a Stair", typeof(Autodesk.Aec.Arch.DatabaseServices.AnchorRailingToStair), new MgdDbgTestFuncInfo.TestFunc(AnchorRailingToStair), MgdDbgTestFuncInfo.TestType.Create));

            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Space", "Space created from a profile", typeof(Space), new MgdDbgTestFuncInfo.TestFunc(SpaceFromProfile), MgdDbgTestFuncInfo.TestType.Create));

            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Curtain Wall Layout - linear", "Creates a hard-coded linear curtain wall layout using the MgdDbgAec_CurtainWallLayoutStyle style", typeof(CurtainWallLayout), new MgdDbgTestFuncInfo.TestFunc(CreateCurtainWallLayout), MgdDbgTestFuncInfo.TestType.Create));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Curtain Wall Layout - curved", "Creates a hard-coded curved curtain wall layout using the MgdDbgAec_CurtainWallLayoutStyle style", typeof(CurtainWallLayout), new MgdDbgTestFuncInfo.TestFunc(CreateCurvedCurtainWallLayout), MgdDbgTestFuncInfo.TestType.Create));
            
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Curtain Wall Unit",   "Creates a hard-coded curtain wall unit using the MgdDbgAec_CurtainWallUnitStyle style", typeof(CurtainWallUnit), new MgdDbgTestFuncInfo.TestFunc(CreateCurtainWallUnit), MgdDbgTestFuncInfo.TestType.Create));

            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Ceiling Grid based on a custom grid",   "Creates a hard-coded ceiling grid using a custom grid", typeof(CeilingGrid), new MgdDbgTestFuncInfo.TestFunc(CreateCeilingGrid), MgdDbgTestFuncInfo.TestType.Create));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Ceiling Grid converted from a LayoutGrid2d",   "Creates a ceiling grid from a selected LayoutGrid2d", typeof(CeilingGrid), new MgdDbgTestFuncInfo.TestFunc(CreateCeilingGridFromLayoutGrid), MgdDbgTestFuncInfo.TestType.Create));

            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Hardwired Window Assembly",   "Creates a hard-coded Window Assembly with the style given by the user.", typeof(WindowAssembly), new MgdDbgTestFuncInfo.TestFunc(MakeWindowAssembly), MgdDbgTestFuncInfo.TestType.Create));

            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Hardwired Roof",   "Creates a hard-coded roof", typeof(Roof), new MgdDbgTestFuncInfo.TestFunc(MakeRoof), MgdDbgTestFuncInfo.TestType.Create));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Hardwired Roof using polyline",   "Creates a hard-coded roof using a polyline to define its contour", typeof(Roof), new MgdDbgTestFuncInfo.TestFunc(MakeRoofWithPline), MgdDbgTestFuncInfo.TestType.Create));            
        }
		
        public void
        DoorSimple()
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            Door door = new Door();
            door.ColorIndex = 1;
            door.Location = new Point3d(10.0, 10.0, 0.0);
            door.Width = 48.0;
            door.Height = 90.0;
            door.OpenPercent = 15.0;
            door.Description = "This door was created from MgdDbgAec!";
            
            MgdDbg.Utils.Db.TransformToWcs(door, m_db);
            MgdDbg.Utils.SymTbl.AddToCurrentSpaceAndClose(door, m_db);
        }
        
        public void
        WindowSimple()
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            Window win = new Window();
            win.ColorIndex = 2;
            win.Location = new Point3d(20.0, 20.0, 0.0);
            win.Width = 36.0;
            win.Height = 60.0;
            win.OpenPercent = 15.0;
            win.Description = "This window was created from MgdDbgAec!";
            
            MgdDbg.Utils.Db.TransformToWcs(win, m_db);
            MgdDbg.Utils.SymTbl.AddToCurrentSpaceAndClose(win, m_db);
        }
        
        public void
        WallSimple()
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            Wall wall = new Wall();
            wall.ColorIndex = 4;
            wall.BaseHeight = 90.0;
            wall.Set(new Point3d(0.0, 0.0, 0.0), new Point3d(200.0, 0.0, 0.0), new Vector3d(0.0, 0.0, 1.0));
            wall.InstanceWidth = 8.0;
            
            MgdDbg.Utils.Db.TransformToWcs(wall, m_db);
            MgdDbg.Utils.SymTbl.AddToCurrentSpaceAndClose(wall, m_db);
        }

        public void
        StairSimple()
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            Stair stair = new Stair();
            stair.Location = new Point3d(10.0, 10.0, 0.0);
            stair.CalculatorData.Rise = 25.0;           
            stair.CalculatorData.RiserCount = 18;            
            stair.CalculatorData.StairHeight = 120.0;           
            stair.CalculatorData.StoreyHeight = 70.0;                    
            stair.CalculatorData.Tread = 11.0;            
            stair.CalculatorData.TreadThickness = 10.0;
            stair.Description = "This stair was created from MgdDbgAec!";

            MgdDbg.Utils.Db.TransformToWcs(stair, m_db);
            MgdDbg.Utils.SymTbl.AddToCurrentSpaceAndClose(stair, m_db);
        }        
                 
        public void
        HardwireWindowInstances()
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            Point3d insPt = new Point3d();
            int maxPerRow = 20;
            double xDist = 6.0;
            double yDist = 24.0;
            if (!GetInsertParamsOpening(ref insPt, ref maxPerRow, ref xDist, ref yDist))
                return;
                
            using (MgdDbg.CompBldrCurSpace compBldr = new MgdDbg.CompBldrCurSpace(m_db)) {
                compBldr.Start();
                compBldr.PushXform(MgdDbg.Utils.Db.GetUcsMatrix(m_db));

                using (DictionaryWindowStyle dict = new DictionaryWindowStyle(m_db)) {
                    int perRowCount = 0;
                    double width;
                    Point3d tmpPt = insPt;

                    AcDb.ObjectIdCollection styles = dict.Records;
                    foreach (AcDb.ObjectId styleId in styles) {        
                        Window newWin = new Window();
                        newWin.SetToStandard(m_db);
                        newWin.Location = tmpPt;
                        newWin.StyleId = styleId;
                                                
                        width = newWin.Width;
                        compBldr.AddToDb(newWin);
                        
                        tmpPt += (MgdDbg.Utils.Ge.kXAxis * (width + xDist));
                        perRowCount++;
                        if (perRowCount == maxPerRow) {
                            perRowCount = 0;
                            tmpPt = new Point3d(insPt.X, tmpPt.Y, tmpPt.Z);
                            tmpPt = tmpPt - (MgdDbg.Utils.Ge.kYAxis * yDist);
                        }
                    }
                }
                compBldr.Commit();
            }
        }
        
        public void
        HardwireDoorInstances()
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            Point3d insPt = new Point3d();
            int maxPerRow = 20;
            double xDist = 6.0;
            double yDist = 24.0;
            if (!GetInsertParamsOpening(ref insPt, ref maxPerRow, ref xDist, ref yDist))
                return;
                
            using (MgdDbg.CompBldrCurSpace compBldr = new MgdDbg.CompBldrCurSpace(m_db)) {
                compBldr.Start();
                compBldr.PushXform(MgdDbg.Utils.Db.GetUcsMatrix(m_db));

                using (DictionaryDoorStyle dict = new DictionaryDoorStyle(m_db)) {
                    int perRowCount = 0;
                    double width;
                    Point3d tmpPt = insPt;

                    AcDb.ObjectIdCollection styles = dict.Records;
                    foreach (AcDb.ObjectId styleId in styles) {        
                        Door newDoor = new Door();
                        newDoor.SetToStandard(m_db);
                        newDoor.Location = tmpPt;
                        newDoor.StyleId = styleId;
                                                
                        width = newDoor.Width;
                        compBldr.AddToDb(newDoor);
                        
                        tmpPt += (MgdDbg.Utils.Ge.kXAxis * (width + xDist));
                        perRowCount++;
                        if (perRowCount == maxPerRow) {
                            perRowCount = 0;
                            tmpPt = new Point3d(insPt.X, tmpPt.Y, tmpPt.Z);
                            tmpPt = tmpPt - (MgdDbg.Utils.Ge.kYAxis * yDist);
                        }
                    }
                }
                compBldr.Commit();
            }
        }
        
        public void
        HardwireWallInstances()
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            Point3d insPt = new Point3d();
            int maxPerRow = 20;
            double xDist = 6.0;
            double yDist = 24.0;
            if (!GetInsertParamsOpening(ref insPt, ref maxPerRow, ref xDist, ref yDist))
                return;
                
            using (MgdDbg.CompBldrCurSpace compBldr = new MgdDbg.CompBldrCurSpace(m_db)) {
                compBldr.Start();
                compBldr.PushXform(MgdDbg.Utils.Db.GetUcsMatrix(m_db));

                using (DictionaryWallStyle dict = new DictionaryWallStyle(m_db)) {
                    int perRowCount = 0;
                    double width;
                    Point3d tmpPt = insPt;
                    Vector3d wallLength = new Vector3d(0.0, 100.0, 0.0);

                    AcDb.ObjectIdCollection styles = dict.Records;
                    foreach (AcDb.ObjectId styleId in styles) {        
                        Wall wall = new Wall();
                        wall.SetToStandard(m_db);
                        wall.Set(tmpPt, tmpPt + wallLength, MgdDbg.Utils.Ge.kZAxis);
                        wall.StyleId = styleId;
                                                
                        width = wall.Width;
                        compBldr.AddToDb(wall);
                        
                        tmpPt += (MgdDbg.Utils.Ge.kXAxis * (width + xDist));
                        perRowCount++;
                        if (perRowCount == maxPerRow) {
                            perRowCount = 0;
                            tmpPt = new Point3d(insPt.X, tmpPt.Y, tmpPt.Z);
                            tmpPt = tmpPt - (MgdDbg.Utils.Ge.kYAxis * yDist);
                        }
                    }
                }
                compBldr.Commit();
            }
        }   
        
        bool
        GetInsertParamsOpening(ref Point3d insPt, ref int maxPerRow, ref double xDist, ref double yDist)
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;

            PromptPointOptions prPtOpt = new PromptPointOptions("\nInsertion point <0,0,0>: ");
            prPtOpt.AllowNone = true;

            PromptPointResult prPtRes = ed.GetPoint(prPtOpt);
            if (prPtRes.Status == PromptStatus.None)            
                insPt = MgdDbg.Utils.Ge.kOrigin;
            else if (prPtRes.Status == PromptStatus.OK)
                insPt = prPtRes.Value;
            else
                return false;
                
            PromptIntegerOptions prOptMaxPerRow = new PromptIntegerOptions("\nMax number of objects per row");
            prOptMaxPerRow.AllowNegative = false;
            prOptMaxPerRow.AllowZero = false;
            prOptMaxPerRow.AllowNone = true;
            prOptMaxPerRow.DefaultValue = maxPerRow;
            
            PromptIntegerResult prResMaxPerRow = ed.GetInteger(prOptMaxPerRow);
            if (prResMaxPerRow.Status == PromptStatus.None)
                maxPerRow = prOptMaxPerRow.DefaultValue;
            else if (prResMaxPerRow.Status == PromptStatus.OK)
                maxPerRow = prResMaxPerRow.Value;
            else
                return false;
               
            PromptDistanceOptions prOptDist = new PromptDistanceOptions("\nSpacing between object in X-dir");
            prOptDist.AllowNegative = false;
            prOptDist.AllowZero = false;
            prOptDist.AllowNone = true;
            prOptDist.DefaultValue = xDist;
         
            PromptDoubleResult prResDistX = ed.GetDistance(prOptDist);
            if (prResDistX.Status == PromptStatus.None)
                xDist = prOptDist.DefaultValue;
            else if (prResDistX.Status == PromptStatus.OK)
                xDist = prResDistX.Value;
            else
                return false;
            
            prOptDist.Message = "\nSpacing between object in Y-dir";
            prOptDist.DefaultValue = yDist;
            
            PromptDoubleResult prResDistY = ed.GetDistance(prOptDist);
            if (prResDistY.Status == PromptStatus.None)
                yDist = prOptDist.DefaultValue;
            else if (prResDistY.Status == PromptStatus.OK)
                yDist = prResDistY.Value;
            else
                return false;

            return true;
        }
        
        public void
        AnchorDoorAndWindowToWall()
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            using (MgdDbg.CompBldrCurSpace compBldr = new MgdDbg.CompBldrCurSpace(m_db)) {
                compBldr.Start();
                compBldr.PushXform(MgdDbg.Utils.Db.GetUcsMatrix(m_db));
                
                Wall wall = new Wall();
                wall.ColorIndex = 2;
                wall.SetToStandard(m_db);
                wall.Set(new Point3d(0.0, 0.0, 0.0), new Point3d(200.0, 0.0, 0.0), new Vector3d(0.0, 0.0, 1.0));
                compBldr.AddToDb(wall);
                
                Door door = new Door();
                door.ColorIndex = 3;
                door.SetToStandard(m_db);
                door.StoringRoughDimensions = true; // make it measure to outside of frame
                compBldr.AddToDb(door);
                
                    // anchor the door 6" from the start of the wall.
                AnchorOpeningBaseToWall anchorDoor = new AnchorOpeningBaseToWall();
                anchorDoor.CurveId = wall.ObjectId;
                anchorDoor.AnchorX.MeasureToType = CurveXMeasureToType.ToStartEdge;
                anchorDoor.AnchorX.OffsetType = CurveXOffsetType.FromStartOfCurve;
                anchorDoor.AnchorX.OffsetDistance = 6.0;
                
                door.SetAnchor(anchorDoor);
                
                Window win = new Window();
                win.ColorIndex = 4;
                win.SetToStandard(m_db);
                compBldr.AddToDb(win);
                
                    // anchor the window in the middle of the wall (in X),
                    // and 6'8" to the top of the window
                AnchorOpeningBaseToWall anchorWin = new AnchorOpeningBaseToWall();
                anchorWin.CurveId = wall.ObjectId;
                anchorWin.AnchorX.MeasureToType = CurveXMeasureToType.ToCenter;
                anchorWin.AnchorX.OffsetType = CurveXOffsetType.FromMidpointOfCurve;
                anchorWin.AnchorZ.OffsetFromType = CurveZOffsetFromType.FromBottom;
                anchorWin.AnchorZ.OffsetToType = CurveZOffsetToType.ToTop;
                anchorWin.AnchorZ.OffsetDistance = 80.0;
                
                win.SetAnchor(anchorWin);
                
                compBldr.Commit();
            }            
        }

        public void
        AnchorRailingToStair()
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            using (MgdDbg.CompBldrCurSpace compBldr = new MgdDbg.CompBldrCurSpace(m_db)) {
                compBldr.Start();
                compBldr.PushXform(MgdDbg.Utils.Db.GetUcsMatrix(m_db));

                Railing rail = new Railing();
                rail.SetToStandard(m_db);
                compBldr.AddToDb(rail);

                Stair stair = new Stair();
                stair.Location = new Point3d(10.0, 10.0, 0.0);
                stair.CalculatorData.Rise = 25.0;
                stair.CalculatorData.RiserCount = 18;
                stair.CalculatorData.StairHeight = 120.0;
                stair.CalculatorData.StoreyHeight = 70.0;
                stair.CalculatorData.Tread = 11.0;
                stair.CalculatorData.TreadThickness = 10.0;
                stair.Description = "This stair was created from MgdDbgAec!";
                compBldr.AddToDb(stair);                

                // anchor single railing to the stair.
                Autodesk.Aec.Arch.DatabaseServices.AnchorRailingToStair anchRail = new AnchorRailingToStair();
                anchRail.StairId = stair.ObjectId;
                anchRail.IsTopToBottom = true;

                rail.SetAnchor(anchRail);

                compBldr.Commit();
            }                  
        }


        public void
        SpaceFromProfile()
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            using (MgdDbg.CompBldrCurSpace compBldr = new MgdDbg.CompBldrCurSpace(m_db)) 
            {
                compBldr.Start();
                compBldr.PushXform(MgdDbg.Utils.Db.GetUcsMatrix(m_db));
                
                // create ring
                Point2dCollection pts = new Point2dCollection();
                pts.Add(new Point2d(0.0, 0.0)); 
                pts.Add(new Point2d(120.0, 0.0)); 
                pts.Add(new Point2d(120.0, 120.0));
                pts.Add(new Point2d(0.0, 120.0)); 

                Ring ri = new Ring(pts);

                // create profile from ring
                Profile prof = new Profile();
                prof.AddRing(ri);

                // create space using profile
                Space space = new Space();
                space.SetToStandard(m_db);
                space.BaseProfile = prof;
                space.SetBaseProfileDirty();

                space.ColorIndex = 1;
                space.Location = new Point3d(10.0, 10.0, 0.0);

                space.CeilingHeight = 120.0;
                space.CeilingThickness = 12.0;
                space.DrawHatch = true;
                space.FloorThickness = 24.0;
                space.SpaceAboveCeiling = 36.0;
                space.StyleId = DictionarySpaceStyle.GetStandardStyle(m_db);

                compBldr.AddToDb(space);
                compBldr.Commit();
            } // end using
        }

        public void
        CreateCurtainWallLayout()
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            // find style to be used
            string symName = "MgdDbgAec_CurtainWallLayoutStyle";

            using (MgdDbg.CompBldrCurSpace compBldr = new MgdDbg.CompBldrCurSpace(m_db)) 
            {
                compBldr.Start();
                compBldr.PushXform(MgdDbg.Utils.Db.GetUcsMatrix(m_db));

                using (DictionaryCurtainWallLayoutStyle  dict = new DictionaryCurtainWallLayoutStyle (m_db)) 
                {
                    if(!dict.Has(symName, compBldr.Transaction)) 
                    {
                        MgdDbg.Utils.AcadUi.PrintToCmdLine(string.Format("\nCurtain Wall Layout Style \"{0}\" does not exist. Run AecMakeObjs test that creates it before running this test.", symName));
                        compBldr.Abort();
                        return;
                    }

                    // create a curtain wall layout that uses that style.
                    AcDb.ObjectId styleId = dict.GetAt(symName);

                    CurtainWallLayout layout = new CurtainWallLayout();
                    layout.SetToStandard(m_db);
                    layout.SetDatabaseDefaults(m_db);
                    layout.StyleId = styleId;
                    Point3d startPt = new Point3d(0.0, 0.0, 0.0);
                    Point3d endPt = new Point3d(400.0, 0.0, 0.0);
                    Vector3d normal = new Vector3d(0.0, 0.0, 1.0);

                    layout.SetLinearBaseCurve( startPt, endPt, normal );
                    layout.BaseHeight = 100.0;

                    bool styleBased = true;
                    if (MgdDbg.Utils.AcadUi.PromptYesNo("\nCreate the curtain wall layout as style based", styleBased, out styleBased) != PromptStatus.OK)
                        return;

                    if(styleBased)
                        layout.GridDefinitionLocation = GridDefinitionLocationType.StyleBased;
                    else
                        layout.GridDefinitionLocation = GridDefinitionLocationType.InstanceBased;

                    compBldr.AddToDb(layout);
                    compBldr.Commit();
                    MgdDbg.Utils.AcadUi.PrintToCmdLine(string.Format("\nCreated hard-coded Curtain Wall Layout using Style \"{0}\".", symName));
                }

            }
        }

        public void
        CreateCurvedCurtainWallLayout()
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            // find style to be used
            string symName = "MgdDbgAec_CurtainWallLayoutStyle";

            using (MgdDbg.CompBldrCurSpace compBldr = new MgdDbg.CompBldrCurSpace(m_db)) 
            {
                compBldr.Start();
                compBldr.PushXform(MgdDbg.Utils.Db.GetUcsMatrix(m_db));

                using (DictionaryCurtainWallLayoutStyle  dict = new DictionaryCurtainWallLayoutStyle (m_db)) 
                {
                    if(!dict.Has(symName, compBldr.Transaction)) 
                    {
                        MgdDbg.Utils.AcadUi.PrintToCmdLine(string.Format("\nCurtain Wall Layout Style \"{0}\" does not exist. Run AecMakeObjs test that creates it before running this test.", symName));
                        compBldr.Abort();
                        return;
                    }

                    // create a curtain wall layout that uses that style.
                    AcDb.ObjectId styleId = dict.GetAt(symName);

                    CurtainWallLayout layout = new CurtainWallLayout();
                    layout.SetToStandard(m_db);
                    layout.SetDatabaseDefaults(m_db);
                    layout.StyleId = styleId;
                    Point3d startPt = new Point3d(0.0, 0.0, 0.0);
                    Point3d ptOnArc = new Point3d(400.0, 400.0, 0.0);
                    Point3d endPt = new Point3d(800.0, 0.0, 0.0);
                    Vector3d normal = new Vector3d(0.0, 0.0, 1.0);
                    CircularArc3d circArc = new CircularArc3d(startPt, ptOnArc, endPt);

                    layout.SetArcBaseCurve( circArc, normal );
                    layout.BaseHeight = 100.0;

                    bool styleBased = true;
                    if (MgdDbg.Utils.AcadUi.PromptYesNo("\nCreate the curtain wall layout as style based", styleBased, out styleBased) != PromptStatus.OK)
                        return;

                    if(styleBased)
                        layout.GridDefinitionLocation = GridDefinitionLocationType.StyleBased;
                    else
                        layout.GridDefinitionLocation = GridDefinitionLocationType.InstanceBased;

                    compBldr.AddToDb(layout);
                    compBldr.Commit();
                    MgdDbg.Utils.AcadUi.PrintToCmdLine(string.Format("\nCreated hard-coded Curtain Wall Layout using Style \"{0}\".", symName));
                }

            }
        }

        public void
        CreateCurtainWallUnit()
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            // find style to be used
            string symName = "MgdDbgAec_CurtainWallUnitStyle";

            using (MgdDbg.CompBldrCurSpace compBldr = new MgdDbg.CompBldrCurSpace(m_db)) 
            {
                compBldr.Start();
                compBldr.PushXform(MgdDbg.Utils.Db.GetUcsMatrix(m_db));

                using (DictionaryCurtainWallUnitStyle  dict = new DictionaryCurtainWallUnitStyle (m_db)) 
                {
                    if(!dict.Has(symName, compBldr.Transaction)) 
                    {
                        MgdDbg.Utils.AcadUi.PrintToCmdLine(string.Format("\nCurtain Wall Unit Style \"{0}\" does not exist. Run AecMakeObjs test that creates it before running this test.", symName));
                        compBldr.Abort();
                        return;
                    }

                    // create a curtain wall unit that uses that style.
                    AcDb.ObjectId styleId = dict.GetAt(symName);

                    CurtainWallUnit unit = new CurtainWallUnit();
                    unit.SetToStandard(m_db);
                    unit.SetDatabaseDefaults(m_db);
                    unit.StyleId = styleId;
                    Point3d startPt = new Point3d(0.0, 0.0, 0.0);
                    Point3d endPt = new Point3d(400.0, 0.0, 0.0);
                    Vector3d normal = new Vector3d(0.0, 0.0, 1.0);

                    unit.Length = 400.0;
                    unit.Height = 100.0;
                    unit.GeoEcs = MgdDbg.Utils.Db.GetUcsMatrix(m_db);

                    compBldr.AddToDb(unit);
                    compBldr.Commit();
                    MgdDbg.Utils.AcadUi.PrintToCmdLine(string.Format("\nCreated hard-coded Curtain Wall Unit using Style \"{0}\".", symName));
                }

            }
        }


        public void
        CreateCeilingGrid()
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            using (MgdDbg.CompBldrCurSpace compBldr = new MgdDbg.CompBldrCurSpace(m_db)) 
            {
                compBldr.Start();
                compBldr.PushXform(MgdDbg.Utils.Db.GetUcsMatrix(m_db));

                using (DictionaryCurtainWallUnitStyle  dict = new DictionaryCurtainWallUnitStyle (m_db)) 
                {
                    if(!dict.HasStandardEntries)
                        return;

                    // create segments
                    Point2d startPt1 = new Point2d(0.0,    0.0);
                    Point2d endPt1   = new Point2d(1000.0, 0.0);
                    LineSegment2d line1 = new LineSegment2d(startPt1, endPt1);
                    Point2d startPt2 = new Point2d(1000.0, 0.0);
                    Point2d endPt2   = new Point2d(1000.0, 1000.0);
                    LineSegment2d line2 = new LineSegment2d(startPt2, endPt2);
                    Point2d startPt3 = new Point2d(1000.0, 1000.0);
                    Point2d endPt3   = new Point2d(0.0,    1000.0);
                    LineSegment2d line3 = new LineSegment2d(startPt3, endPt3);
                    Point2d startPt4 = new Point2d(0.0, 1000.0);
                    Point2d endPt4   = new Point2d(0.0, 0.0);
                    LineSegment2d line4 = new LineSegment2d(startPt4, endPt4);
                    Point2d startPt5 = new Point2d(0.0,    500.0);
                    Point2d endPt5   = new Point2d(1000.0, 500.0);
                    LineSegment2d line5 = new LineSegment2d(startPt5, endPt5);
                    Point2d startPt6 = new Point2d(500.0, 0.0);
                    Point2d endPt6   = new Point2d(500.0, 1000.0);
                    LineSegment2d line6 = new LineSegment2d(startPt6, endPt6);

                    GridCustomSegment seg1 = new GridCustomSegment(line1);
                    GridCustomSegment seg2 = new GridCustomSegment(line2);
                    GridCustomSegment seg3 = new GridCustomSegment(line3);
                    GridCustomSegment seg4 = new GridCustomSegment(line4);
                    GridCustomSegment seg5 = new GridCustomSegment(line5);
                    GridCustomSegment seg6 = new GridCustomSegment(line6);

                    // create custom grid
                    LayoutGrid2d ceilingGrid = new CeilingGrid();
                    ceilingGrid.SetDatabaseDefaults(m_db);
                    ceilingGrid.GridType = GridType.UVCustom;

                    GridCustom customGrid = ceilingGrid.CustomGrid;
                    if(customGrid == null)
                        return;

                    customGrid.PersistentSolution = false;

#if !ACA2009
                    customGrid.AppendSegment(seg1);
                    customGrid.AppendSegment(seg2);
                    customGrid.AppendSegment(seg3);
                    customGrid.AppendSegment(seg4);
                    customGrid.AppendSegment(seg5);
                    customGrid.AppendSegment(seg6);
#endif

#if ACA2009
                    customGrid.Segments.Add(seg1);
                    customGrid.Segments.Add(seg2);
                    customGrid.Segments.Add(seg3);
                    customGrid.Segments.Add(seg4);
                    customGrid.Segments.Add(seg5);
                    customGrid.Segments.Add(seg6);
#endif


                    customGrid.UpdateParameters();
                    customGrid.DeleteTransientData();

                    ceilingGrid.Location = new Point3d(0.0, 0.0, 0.0);
                    ceilingGrid.GeoEcs = MgdDbg.Utils.Db.GetUcsMatrix(m_db);


                    compBldr.AddToDb(ceilingGrid);
                    compBldr.Commit();

                    MgdDbg.Utils.AcadUi.PrintToCmdLine(string.Format("\nCreated hard-coded Curtain Wall layout using custom grid."));
                }

            }

        }

        private void
        CreateCeilingGridFromLayoutGrid()
        {
            m_db      = MgdDbg.Utils.Db.GetCurDwg();

            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;

            // select grid
            PromptEntityOptions prLayoutGridOpts = new PromptEntityOptions("\nSelect a LayoutGrid2d");
            prLayoutGridOpts.SetRejectMessage("\nSelected entity must be of type LayoutGrid2d.");
            prLayoutGridOpts.AddAllowedClass(typeof(LayoutGrid2d), false);
            
            PromptEntityResult prLayoutGridRes = ed.GetEntity(prLayoutGridOpts);
            if (prLayoutGridRes.Status != PromptStatus.OK)
                return;
                 
            using (MgdDbg.CompBldrCurSpace compBldr = new MgdDbg.CompBldrCurSpace(m_db)) 
            {
                compBldr.Start();
                compBldr.PushXform(MgdDbg.Utils.Db.GetUcsMatrix(m_db));
                
                LayoutGrid2d layoutGrid = (LayoutGrid2d)compBldr.Transaction.GetObject(prLayoutGridRes.ObjectId, OpenMode.ForRead);
                if(layoutGrid == null)
                {
                    MgdDbg.Utils.AcadUi.PrintToCmdLine("\nLayoutGrid was null");
                    compBldr.Abort();
                    return;
                }

                CeilingGrid ceilingGrid = new CeilingGrid();
                GridType gridType = layoutGrid.GridType;
                if (gridType == GridType.UVRectangular )
                    ceilingGrid.GridType = GridType.UVRectangular;
                else if (gridType == GridType.UVRadial )
                    ceilingGrid.GridType = GridType.UVRadial;
                else
                {
                    MgdDbg.Utils.AcadUi.PrintToCmdLine("\nCannot convert this LayoutGrid2d");
                    compBldr.Abort();
                    return;
                }

                ceilingGrid.ConvertFrom(layoutGrid);

                if(ceilingGrid == null)
                {
                    MgdDbg.Utils.AcadUi.PrintToCmdLine("\nConversion failed");
                    compBldr.Abort();
                    return;
                }

                compBldr.AddToDb(ceilingGrid);
                compBldr.Commit();
            }
        }

        
///////////////////////////////////////////////////////////
        private void
        MakeWindowAssembly()
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            // ask the user to select the desired style
            AcDb.ObjectId styleId = Utils.Ui.SelectStyle.GetWindowAssemblyStyleId(m_db);
            if (styleId.IsNull)
                return;

            using (MgdDbg.CompBldrCurSpace compBldr = new MgdDbg.CompBldrCurSpace(m_db)) 
            {
                compBldr.Start();
                compBldr.PushXform(MgdDbg.Utils.Db.GetUcsMatrix(m_db));

                WindowAssembly winAssembly = new WindowAssembly();
                winAssembly.SetToStandard(m_db);
                winAssembly.SetDatabaseDefaults(m_db);
                winAssembly.StyleId = styleId;
                winAssembly.Length = 600.0;
                winAssembly.Height = 100.0;

                winAssembly.GeoEcs = MgdDbg.Utils.Db.GetUcsMatrix(m_db);

                compBldr.AddToDb(winAssembly);
                compBldr.Commit();

            } //end using
                
            MgdDbg.Utils.AcadUi.PrintToCmdLine("\nCreated a window assembly");
        }



        private void SetRoofSegmentData(ref RoofSegment seg) 
        {
            if (seg != null) 
            {
                seg.Init();
                // MH: modified for 2010 
                //seg.Gable = 100.0; // has to be set before the others!
                seg.SetGable(100.0); // has to be set before the others!

                seg.SetOverhang (20.0, Tolerance.Global.EqualPoint);   // in inches (by default)

                seg.NumberOfFacets = 1;
                seg.SetSlope(0, 0.7853981634, Tolerance.Global.EqualPoint);  // 0.7853981634 radians == 45 deg
                seg.SetHeight(0, 0.0, Tolerance.Global.EqualPoint);
            }
        }

        private void
        MakeRoof()
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            using (MgdDbg.CompBldrCurSpace compBldr = new MgdDbg.CompBldrCurSpace(m_db)) 
            {
                compBldr.Start();
                compBldr.PushXform(MgdDbg.Utils.Db.GetUcsMatrix(m_db));

                try
                {
                    // roof
                    Autodesk.Aec.Arch.DatabaseServices.Roof roof = new Roof();
                    roof.SetDatabaseDefaults(m_db);
                    roof.SetToStandard(m_db);

                    //Autodesk.Aec.Arch.Geometry.RoofProfile oldProf = roof.RoofProfile;
                    //oldProf.RemoveAllRings();
                    //roof.RoofProfile = oldProf;

                    //RoofProfile roofProf = new RoofProfile();
                    //roofProf.SetToStandard(m_db);
                    //roofProf.RemoveAllRings();

                    //int rc = roofProf.RingCount;

                    //Ring ring = new Ring();
                    //if (ring != null) 
                    //{
                    //    LineSegment2d  geLine1 = new LineSegment2d(new Point2d(0.0, 0.0), new Point2d(1000.0, 0.0) );
                    //    LineSegment2d  geLine2 = new LineSegment2d(new Point2d(1000.0, 0.0), new Point2d(1000.0, 500.0) );
                    //    LineSegment2d  geLine3 = new LineSegment2d(new Point2d(1000.0, 500.0), new Point2d(0.0, 500.0) );
                    //    LineSegment2d  geLine4 = new LineSegment2d(new Point2d(0.0, 500.0), new Point2d(0.0, 0.0) );

                    //    RoofSegment seg1 = new RoofSegment(geLine1);
                    //    RoofSegment seg2 = new RoofSegment(geLine2);
                    //    RoofSegment seg3 = new RoofSegment(geLine3);
                    //    RoofSegment seg4 = new RoofSegment(geLine4);


                    //    SetRoofSegmentData(ref seg1);
                    //    SetRoofSegmentData(ref seg2);
                    //    SetRoofSegmentData(ref seg3);
                    //    SetRoofSegmentData(ref seg4);

                    //    ring.AddSegment(seg1, true);
                    //    ring.AddSegment(seg2, true);
                    //    ring.AddSegment(seg3, true);
                    //    ring.AddSegment(seg4, true);

                    //    ring.IsVoid = false;
                    //    roofProf.AddRing(ring);

                    //    bool b = roofProf.IsValid(true, Tolerance.Global);
                    //    rc = roofProf.RingCount;

                    //    roof.RoofProfile = roofProf;
                    //    roof.SetLocation(new Point3d(0.0,0.0,0.0), new Point3d(0.0,0.0,0.0), Vector3d.ZAxis, Tolerance.Global);
                    //    roof.SetThickness(10.0, ThicknessDirection.Roof, Tolerance.Global.EqualPoint);

                    //    Autodesk.Aec.DatabaseServices.DictionaryLayerKeyStyle layerStyle = new DictionaryLayerKeyStyle(m_db);
                    //    // TBD: if layer doesn't exist, create it first!
                    //    AcDb.ObjectId layerId = layerStyle.GenerateLayer(roof.LayerKey);
                    //    roof.LayerId = layerId;

                        roof.CalculateRoof();


                        compBldr.AddToDb(roof);
                        compBldr.Commit();
                
                        MgdDbg.Utils.AcadUi.PrintToCmdLine("\nCreated a roof");
                    }
               // }
                catch
                {
                    compBldr.Abort();
                }
                finally
                {
                    compBldr.Dispose();

                    // force a GC so that we can catch any Dispose problems right away
                    GC.Collect();
                    GC.WaitForPendingFinalizers();
                }

            } //end using
        }


        private void
        MakeRoofWithPline()
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            using (MgdDbg.CompBldrCurSpace compBldr = new MgdDbg.CompBldrCurSpace(m_db)) 
            {
                compBldr.Start();
                compBldr.PushXform(MgdDbg.Utils.Db.GetUcsMatrix(m_db));

                try
                {
                    // roof
                    Autodesk.Aec.Arch.DatabaseServices.Roof roof = new Roof();
                    roof.SetDatabaseDefaults(m_db);
                    roof.SetToStandard(m_db);

                    RoofProfile roofProf = roof.RoofProfile; //new RoofProfile();

                    int rc = roofProf.RingCount;

                    Point2d ip1 = new Point2d(0.0,  0.0);
                    Point2d ip2 = new Point2d(240.0, 0.0);
                    Point2d ip3 = new Point2d(240.0, 120.0);
                    Point2d ip4 = new Point2d(0.0,  120.0);
                    Polyline ipline = new Polyline(4);
                    ipline.AddVertexAt(0, ip1, 0, 1, 1);
                    ipline.AddVertexAt(1, ip2, 0, 1, 1);
                    ipline.AddVertexAt(2, ip3, 0, 1, 1);
                    ipline.AddVertexAt(3, ip4, 0, 1, 1);
                    ipline.Closed = true;

                    roofProf.ConvertPolylineToRoofContour( ipline, Tolerance.Global );

                    bool b = roofProf.IsValid(true, Tolerance.Global);
                    rc = roofProf.RingCount;

                    roof.RoofProfile = roofProf;
                    roof.SetLocation(new Point3d(0.0,0.0,0.0), new Point3d(0.0,0.0,0.0), Vector3d.ZAxis, Tolerance.Global);
                    roof.CalculateRoof();

                    compBldr.AddToDb(roof);
                    compBldr.Commit();
                
                    MgdDbg.Utils.AcadUi.PrintToCmdLine("\nCreated a roof using a polyline");
                }
                catch
                {
                    compBldr.Abort();
                }
                finally
                {
                    compBldr.Dispose();

                    // force a GC so that we can catch any Dispose problems right away
                    GC.Collect();
                    GC.WaitForPendingFinalizers();
                }

            } //end using
        }

        //private void
        //MakeStair()
        //{
        //    using (MgdDbg.CompBldrCurSpace compBldr = new MgdDbg.CompBldrCurSpace(m_db)) {
        //        compBldr.Start();
        //        compBldr.PushXform(MgdDbg.Utils.Db.GetUcsMatrix(m_db));

        //        try {
        //            // stair
        //            Autodesk.Aec.Arch.DatabaseServices.Stair stair = new Stair();
        //            roof.SetDatabaseDefaults(m_db);
        //            roof.SetToStandard(m_db);

        //            stair.CalculatorData.se
        //            Autodesk.Aec.Arch.Geometry.RoofProfile oldProf = roof.RoofProfile;
        //            oldProf.RemoveAllRings();
        //            roof.RoofProfile = oldProf;

        //            RoofProfile roofProf = new RoofProfile();
        //            roofProf.SetToStandard(m_db);
        //            roofProf.RemoveAllRings();

        //            int rc = roofProf.RingCount;

        //            Ring ring = new Ring();
        //            if (ring != null) {
        //                LineSegment2d geLine1 = new LineSegment2d(new Point2d(0.0, 0.0), new Point2d(1000.0, 0.0));
        //                LineSegment2d geLine2 = new LineSegment2d(new Point2d(1000.0, 0.0), new Point2d(1000.0, 500.0));
        //                LineSegment2d geLine3 = new LineSegment2d(new Point2d(1000.0, 500.0), new Point2d(0.0, 500.0));
        //                LineSegment2d geLine4 = new LineSegment2d(new Point2d(0.0, 500.0), new Point2d(0.0, 0.0));

        //                RoofSegment seg1 = new RoofSegment(geLine1);
        //                RoofSegment seg2 = new RoofSegment(geLine2);
        //                RoofSegment seg3 = new RoofSegment(geLine3);
        //                RoofSegment seg4 = new RoofSegment(geLine4);


        //                SetRoofSegmentData(ref seg1);
        //                SetRoofSegmentData(ref seg2);
        //                SetRoofSegmentData(ref seg3);
        //                SetRoofSegmentData(ref seg4);

        //                ring.AddSegment(seg1, true);
        //                ring.AddSegment(seg2, true);
        //                ring.AddSegment(seg3, true);
        //                ring.AddSegment(seg4, true);

        //                ring.IsVoid = false;
        //                roofProf.AddRing(ring);

        //                bool b = roofProf.IsValid(true, Tolerance.Global);
        //                rc = roofProf.RingCount;

        //                roof.RoofProfile = roofProf;
        //                roof.SetLocation(new Point3d(0.0, 0.0, 0.0), new Point3d(0.0, 0.0, 0.0), Vector3d.ZAxis, Tolerance.Global);
        //                roof.SetThickness(10.0, ThicknessDirection.Roof, Tolerance.Global.EqualPoint);

        //                Autodesk.Aec.DatabaseServices.DictionaryLayerKeyStyle layerStyle = new DictionaryLayerKeyStyle(m_db);
        //                // TBD: if layer doesn't exist, create it first!
        //                AcDb.ObjectId layerId = layerStyle.GenerateLayer(roof.LayerKey);
        //                roof.LayerId = layerId;

        //                roof.CalculateRoof();


        //                compBldr.AddToDb(roof);
        //                compBldr.Commit();

        //                MgdDbg.Utils.AcadUi.PrintToCmdLine("\nCreated a roof");
        //            }
        //        }
        //        catch {
        //            compBldr.Abort();
        //        }
        //        finally {
        //            compBldr.Dispose();

        //            // force a GC so that we can catch any Dispose problems right away
        //            GC.Collect();
        //            GC.WaitForPendingFinalizers();
        //        }

        //    } //end using
        //}


    }
}
