
// $Header: //Desktop/Source/TestTools/MgdDbgAec/ObjTests/MakeEntsBase.cs#1 $
// $Author: fergusja $ $DateTime: 2007/03/16 11:08:08 $ $Change: 68854 $
//
// (C) Copyright 2004 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//

using System;
using System.Windows.Forms;
using Autodesk.AutoCAD.DatabaseServices;
using Autodesk.AutoCAD.Geometry;
using Autodesk.AutoCAD.EditorInput;
using MgdDbg;
using MgdDbg.Test;
using Autodesk.Aec.DatabaseServices;
using Autodesk.Aec.Geometry;
using Autodesk.Aec.Arch.DatabaseServices;

using AcDb = Autodesk.AutoCAD.DatabaseServices;

namespace MgdDbgAec.ObjTests
{
	/// <summary>
	/// Summary description for MakeEntsBase.
	/// </summary>
    public class MakeEntsBase : MgdDbgTestFuncs
    {
        private Database    m_db = null;
	    
        public
        MakeEntsBase()
        {            		    
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Mass Element (Pyramid)", "Hardwired Mass Element Pyramid", typeof(MassElement), new MgdDbgTestFuncInfo.TestFunc(MassElementPyramid), MgdDbgTestFuncInfo.TestType.Create));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Mass Element (Sphere)", "Hardwired Mass Element Sphere", typeof(MassElement), new MgdDbgTestFuncInfo.TestFunc(MassElementSphere), MgdDbgTestFuncInfo.TestType.Create));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Mass Elements (All)", "Hardwired Mass Elements", typeof(MassElement), new MgdDbgTestFuncInfo.TestFunc(MassElementsAll), MgdDbgTestFuncInfo.TestType.Create));

            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Polygon (Set Base Profile)", "Hardwired Polygon, using profile", typeof(Polygon), new MgdDbgTestFuncInfo.TestFunc(PolygonProfile), MgdDbgTestFuncInfo.TestType.Create));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Polygon (Set ECS Vertices)", "Hardwired Polygon, using vertices", typeof(Polygon), new MgdDbgTestFuncInfo.TestFunc(PolygonEcsVertices), MgdDbgTestFuncInfo.TestType.Create));

            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("MvBlock (Simple)", "Hardwired MvBlock", typeof(MultiViewBlockReference), new MgdDbgTestFuncInfo.TestFunc(MvBlockReference), MgdDbgTestFuncInfo.TestType.Create));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("MvBlock (Atts, Offsets)", "Hardwired MvBlock with attributes and offets", typeof(MultiViewBlockReference), new MgdDbgTestFuncInfo.TestFunc(MvBlockReferenceAtts), MgdDbgTestFuncInfo.TestType.Create));

            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Mass Group (Un-Anchored)", "Hardwired MassGroup", typeof(MassGroup), new MgdDbgTestFuncInfo.TestFunc(MassGroupUnAnchored), MgdDbgTestFuncInfo.TestType.Create));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Mass Group (Anchored)", "Hardwired MassGroup", typeof(MassGroup), new MgdDbgTestFuncInfo.TestFunc(MassGroupAnchored), MgdDbgTestFuncInfo.TestType.Create));

            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Layout Curve (Manual)", "Hardwired Layout Curve with nodes manually assigned", typeof(LayoutCurve), new MgdDbgTestFuncInfo.TestFunc(LayoutCurveManual), MgdDbgTestFuncInfo.TestType.Create));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Layout Curve (Space Evenly)", "Hardwired Layout Curve with nodes spaced evenly", typeof(LayoutCurve), new MgdDbgTestFuncInfo.TestFunc(LayoutCurveSpaceEvenly), MgdDbgTestFuncInfo.TestType.Create));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Layout Curve (Repeat)", "Hardwired Layout Curve with nodes repeating every X distance", typeof(LayoutCurve), new MgdDbgTestFuncInfo.TestFunc(LayoutCurveRepeat), MgdDbgTestFuncInfo.TestType.Create));
            //m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Layout Curve (Massing)", "Hardwired Layout Curve", typeof(LayoutCurve), new MgdDbgTestFuncInfo.TestFunc(LayoutCurveMassGroup), MgdDbgTestFuncInfo.TestType.Create));

            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Anchor to Curve Midpoint", "Select a curve and anchor an object", typeof(AnchorEntityToCurve), new MgdDbgTestFuncInfo.TestFunc(AnchorToCurveMidpoint), MgdDbgTestFuncInfo.TestType.Create));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Anchor to Entity", "Select an entity and anchor an object", typeof(AnchorEntityToEntity), new MgdDbgTestFuncInfo.TestFunc(AnchorToEntity), MgdDbgTestFuncInfo.TestType.Create));

            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Layout Grid 2d", "Hardwired Layout Grid 2ds, one of each type", typeof(LayoutGrid2d), new MgdDbgTestFuncInfo.TestFunc(LayoutGrid2dGrids), MgdDbgTestFuncInfo.TestType.Create));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Layout Grid 3d", "Hardwired Layout Grid 3ds", typeof(LayoutGrid3d), new MgdDbgTestFuncInfo.TestFunc(LayoutGrid3dGrids), MgdDbgTestFuncInfo.TestType.Create));
        }
		
		        
        public void
        MassElementPyramid()
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            MassElement massElem = MassElement.Create(ShapeType.Pyramid);
            massElem.ColorIndex = 3;
            massElem.Location = new Point3d(20.0, 20.0, 0.0);
            massElem.Width = 20.0;
            massElem.Depth = 30.0;
            massElem.Height = 100.0;
            massElem.Description = "This MassElement was created from MgdDbgAec!";
            
            MgdDbg.Utils.Db.TransformToWcs(massElem, m_db);
            MgdDbg.Utils.SymTbl.AddToCurrentSpaceAndClose(massElem, m_db);
        }
        
        public void
        MassElementSphere()
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            MassElement massElem = MassElement.Create(ShapeType.Sphere);
            massElem.ColorIndex = 3;
            massElem.Location = new Point3d(20.0, 20.0, 0.0);
            massElem.Radius = 10.0;
            massElem.Description = "This MassElement was created from MgdDbgAec!";
            
            MgdDbg.Utils.Db.TransformToWcs(massElem, m_db);
            MgdDbg.Utils.SymTbl.AddToCurrentSpaceAndClose(massElem, m_db);
        }
        
        public void
        MassElementsAll()
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            using (MgdDbg.CompBldrCurSpace compBldr = new MgdDbg.CompBldrCurSpace(m_db)) {
                compBldr.Start();
                compBldr.PushXform(MgdDbg.Utils.Db.GetUcsMatrix(m_db));
                
                MassElement massElem = MassElement.Create(ShapeType.Pyramid);
                massElem.ColorIndex = 1;
                massElem.Location = new Point3d(0.0, 0.0, 0.0);
                massElem.Width = 10.0;
                massElem.Depth = 20.0;
                massElem.Height = 50.0;
                massElem.Description = "This MassElement was created from MgdDbgAec!";
                compBldr.AddToDb(massElem);
                
                MassElement massElem2 = MassElement.Create(ShapeType.Sphere);
                massElem2.ColorIndex = 2;
                massElem2.Location = new Point3d(20.0, 0.0, 0.0);
                massElem2.Radius = 10.0;
                compBldr.AddToDb(massElem2);
                
                MassElement massElem3 = MassElement.Create(ShapeType.Dome);
                massElem3.ColorIndex = 3;
                massElem3.Location = new Point3d(40.0, 0.0, 0.0);
                massElem3.Radius = 10.0;
                compBldr.AddToDb(massElem3);
                
                MassElement massElem4 = MassElement.Create(ShapeType.Box);
                massElem4.ColorIndex = 4;
                massElem4.Location = new Point3d(60.0, 0.0, 0.0);
                massElem4.Width = 10.0;
                massElem4.Depth = 20.0;
                massElem4.Height = 50.0;
                compBldr.AddToDb(massElem4);
                
                MassElement massElem5 = MassElement.Create(ShapeType.Arch);
                massElem5.ColorIndex = 5;
                massElem5.Location = new Point3d(110.0, 0.0, 0.0);
                massElem5.Width = 40.0;
                massElem5.Depth = 20.0;
                massElem5.Height = 50.0;
                massElem5.Radius = 10.0;
                compBldr.AddToDb(massElem5);
                
                MassElement massElem6 = MassElement.Create(ShapeType.BarrelVault);
                massElem6.ColorIndex = 6;
                massElem6.Location = new Point3d(145.0, 0.0, 0.0);
                massElem6.Width = 20.0;
                massElem6.Radius = 10.0;
                compBldr.AddToDb(massElem6);
                
                MassElement massElem7 = MassElement.Create(ShapeType.Cone);
                massElem7.ColorIndex = 7;
                massElem7.Location = new Point3d(170.0, 0.0, 0.0);
                massElem7.Height = 50.0;
                massElem7.Radius = 10.0;
                compBldr.AddToDb(massElem7);

                MassElement massElem8 = MassElement.Create(ShapeType.Cylinder);
                massElem8.ColorIndex = 8;
                massElem8.Location = new Point3d(200.0, 0.0, 0.0);
                massElem8.Height = 50.0;
                massElem8.Radius = 10.0;
                compBldr.AddToDb(massElem8);
                
                MassElement massElem9 = MassElement.Create(ShapeType.Doric);
                massElem9.ColorIndex = 9;
                massElem9.Location = new Point3d(230.0, 0.0, 0.0);
                massElem9.Height = 50.0;
                massElem9.Radius = 10.0;
                compBldr.AddToDb(massElem9);
                
                MassElement massElem10 = MassElement.Create(ShapeType.Gable);
                massElem10.ColorIndex = 10;
                massElem10.Location = new Point3d(0.0, 30.0, 0.0);
                massElem10.Width = 40.0;
                massElem10.Depth = 20.0;
                massElem10.Height = 50.0;
                massElem10.Rise = 10.0;
                compBldr.AddToDb(massElem10);
                
                MassElement massElem11 = MassElement.Create(ShapeType.IsoscelesTriangle);
                massElem11.ColorIndex = 11;
                massElem11.Location = new Point3d(30.0, 30.0, 0.0);
                massElem11.Width = 40.0;
                massElem11.Depth = 20.0;
                massElem11.Height = 50.0;
                compBldr.AddToDb(massElem11);
                
                MassElement massElem12 = MassElement.Create(ShapeType.RightTriangle);
                massElem12.ColorIndex = 12;
                massElem12.Location = new Point3d(60.0, 30.0, 0.0);
                massElem12.Width = 40.0;
                massElem12.Depth = 20.0;
                massElem12.Height = 50.0;
                compBldr.AddToDb(massElem12);
    
                compBldr.Commit();
            }
        }
        
        public void
        MassGroupUnAnchored()
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            using (MgdDbg.CompBldrCurSpace compBldr = new MgdDbg.CompBldrCurSpace(m_db)) {
                compBldr.Start();
                compBldr.PushXform(MgdDbg.Utils.Db.GetUcsMatrix(m_db));
                
                MassElement massElem1 = MassElement.Create(ShapeType.Pyramid);
                massElem1.ColorIndex = 1;
                massElem1.Location = new Point3d(0.0, 0.0, 0.0);
                massElem1.Width = 20.0;
                massElem1.Depth = 20.0;
                massElem1.Height = 70.0;
                compBldr.AddToDb(massElem1);
                
                MassElement massElem2 = MassElement.Create(ShapeType.Cylinder);
                massElem2.ColorIndex = 2;
                massElem2.Location = new Point3d(0.0, 0.0, 0.0);
                massElem2.Radius = 10.0;
                massElem2.Height = 40.0;
                compBldr.AddToDb(massElem2);
                
                MassElement massElem3 = MassElement.Create(ShapeType.Box);
                massElem3.ColorIndex = 3;
                massElem3.Location = new Point3d(10.0, 10.0, 0.0);
                massElem3.Width = 15.0;
                massElem3.Depth = 15.0;
                massElem3.Height = 30.0;
                compBldr.AddToDb(massElem3);
                
                MassGroup massGroup = new MassGroup();
                massGroup.ColorIndex = 4;
                massGroup.Location = new Point3d(-20.0, 0.0, 0.0);
                compBldr.AddToDb(massGroup);
                massGroup.AddNode(massElem1, Operation.Additive);
                massGroup.AddNode(massElem2, Operation.Additive);
                massGroup.AddNode(massElem3, Operation.Subtractive);
    
                compBldr.Commit();
            }
        }
        
        public void
        MassGroupAnchored()
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            using (MgdDbg.CompBldrCurSpace compBldr = new MgdDbg.CompBldrCurSpace(m_db)) {
                compBldr.Start();
                compBldr.PushXform(MgdDbg.Utils.Db.GetUcsMatrix(m_db));
                
                MassElement massElem1 = MassElement.Create(ShapeType.Box);
                massElem1.ColorIndex = 1;
                massElem1.Location = new Point3d(0.0, 0.0, 0.0);
                massElem1.Width = 20.0;
                massElem1.Depth = 20.0;
                massElem1.Height = 20.0;
                compBldr.AddToDb(massElem1);
                
                MassElement massElem2 = MassElement.Create(ShapeType.Cylinder);
                massElem2.ColorIndex = 2;
                massElem2.Location = new Point3d(0.0, 0.0, -10.0);
                massElem2.Radius = 5.0;
                massElem2.Height = 40.0;
                compBldr.AddToDb(massElem2);
                                
                MassGroup massGroup = new MassGroup();
                massGroup.ColorIndex = 4;
                massGroup.Location = new Point3d(50.0, 0.0, 0.0);
                    // need to add to DB before adding nodes to it
                compBldr.AddToDb(massGroup);
                    // tell this MassGroup to anchor all of its nodes
                MassGroup.SetSupportsAnchoring(massGroup.ObjectId, true);
                massGroup.AddNode(massElem1, Operation.Additive);
                massGroup.AddNode(massElem2, Operation.Subtractive);
    
                compBldr.Commit();
            }
        }
        
		public void
		PolygonProfile()
		{
            m_db = MgdDbg.Utils.Db.GetCurDwg();

			using (MgdDbg.CompBldrCurSpace compBldr = new MgdDbg.CompBldrCurSpace(m_db)) {
				compBldr.Start();
				compBldr.PushXform(MgdDbg.Utils.Db.GetUcsMatrix(m_db));

				    // create ring
				Point2dCollection pts = new Point2dCollection();
				pts.Add(new Point2d(0.0, 0.0)); 
				pts.Add(new Point2d(120.0, 0.0)); 
				pts.Add(new Point2d(120.0, 120.0));
				pts.Add(new Point2d(0.0, 180.0)); 

				Ring ri = new Ring(pts);

				    // create profile from ring
				Profile prof = new Profile();
				prof.AddRing(ri);

				    // create polygon from profile
				Polygon poly = new Polygon();
				poly.SetBaseProfile(prof, Matrix3d.Identity);

				poly.ColorIndex = 3;
				poly.Location = new Point3d(20.0, 20.0, 0.0);
				poly.Description = "This Polygon was created from MgdDbgAec!";
				compBldr.AddToDb(poly);
				compBldr.Commit();
			}
		}
		
	    public void
		PolygonEcsVertices()
		{
            m_db = MgdDbg.Utils.Db.GetCurDwg();

		        // ask the user to select the desired style
            AcDb.ObjectId styleId = Utils.Ui.SelectStyle.GetPolygonStyleId(m_db);
            if (styleId.IsNull)
                return;
                		    
			using (MgdDbg.CompBldrCurSpace compBldr = new MgdDbg.CompBldrCurSpace(m_db)) {
				compBldr.Start();
				compBldr.PushXform(MgdDbg.Utils.Db.GetUcsMatrix(m_db));

				    // create ring
				Point2dCollection pts = new Point2dCollection();
				pts.Add(new Point2d(0.0, 0.0)); 
				pts.Add(new Point2d(140.0, 0.0)); 
				pts.Add(new Point2d(140.0, 140.0));
				pts.Add(new Point2d(0.0, 200.0));
				pts.Add(pts[0]);    // must explicitly close the loop

				    // create polygon from profile
				Polygon poly = new Polygon();
				poly.StyleId = styleId;
				poly.SetEcsVertices(pts);

				poly.ColorIndex = 4;
				poly.Location = new Point3d(10.0, 10.0, 0.0);
				compBldr.AddToDb(poly);
				compBldr.Commit();
			}
		}
		
		public void
		MvBlockReference()
		{
            m_db = MgdDbg.Utils.Db.GetCurDwg();

		        // ask the user to select the desired style
            AcDb.ObjectId styleId = Utils.Ui.SelectStyle.GetMvBlockDefinitionId(m_db);
            if (styleId.IsNull)
                return;

			using (MgdDbg.CompBldrCurSpace compBldr = new MgdDbg.CompBldrCurSpace(m_db)) {
				compBldr.Start();
				compBldr.PushXform(MgdDbg.Utils.Db.GetUcsMatrix(m_db));
				
				    // NOTE: the default values of attributes (if there are any)
				    // will be filled in automatically when added to the database

				MultiViewBlockReference mvBlkRef = new MultiViewBlockReference();
				mvBlkRef.BlockDefId = styleId;
				mvBlkRef.Scale = new Scale3d(2.0, 2.0, 2.0);
				mvBlkRef.Location = new Point3d(10.0, 10.0, 0.0);
				compBldr.AddToDb(mvBlkRef);
				
                mvBlkRef = new MultiViewBlockReference();
				mvBlkRef.BlockDefId = styleId;
				mvBlkRef.Scale = new Scale3d(1.0, 2.0, 1.0);
				mvBlkRef.Location = new Point3d(20.0, 20.0, 0.0);
				mvBlkRef.Rotation = MgdDbg.Utils.Ge.kRad45;
				compBldr.AddToDb(mvBlkRef);

				compBldr.Commit();
			}
		}
		
		public void
		MvBlockReferenceAtts()
		{
            m_db = MgdDbg.Utils.Db.GetCurDwg();

		        // ask the user to select the desired style
            AcDb.ObjectId styleId = Utils.Ui.SelectStyle.GetMvBlockDefinitionId(m_db);
            if (styleId.IsNull)
                return;

			using (MgdDbg.CompBldrCurSpace compBldr = new MgdDbg.CompBldrCurSpace(m_db)) {
				compBldr.Start();
				compBldr.PushXform(MgdDbg.Utils.Db.GetUcsMatrix(m_db));

				MultiViewBlockReference mvBlkRef = new MultiViewBlockReference();
				mvBlkRef.BlockDefId = styleId;
				mvBlkRef.Scale = new Scale3d(2.0, 2.0, 2.0);
				mvBlkRef.Location = new Point3d(10.0, 10.0, 0.0);
				compBldr.AddToDb(mvBlkRef);
				
				    // walk the view blocks and fill in different values for
				    // the offsets and the attributes.
                foreach (MultiViewBlockViewInstance viewInst in mvBlkRef.ViewInstances) {
                    
                    int j = 1;
                    double k = 10.0;
                    
                        // walk through the attributes and give them an arbitrary value.  Normally
                        // you would prompt the user for the values.  Also, we'll arbitrarily set the
                        // colorIndex.  You would normally let it take on the properties of the AttDef
                        // (contained within the referenced BlockId), but we'll set it just to prove
                        // we can.  You can make it match the def again by using the command
                        // "MVBLOCK,Update" or the function MutliViewBlockViewInstance.EnsureConsistentAttributesAndUpdateExisting(true);
                    foreach (MultiViewBlockAttributeReference att in viewInst.AttributeReferences) {
                        att.Attribute.TextString = string.Format("Att value {0:d}", j++);
                        att.Attribute.ColorIndex = j;
                        att.Attribute.AdjustAlignment(m_db);
                    }
                    
                    viewInst.Offset = new Vector3d(k, k, k);    // arbitrary offset just to demonstrate
                    k += 10.0;
                }
				compBldr.Commit();
			}
		}

		
        public void
        LayoutCurveManual()
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            using (MgdDbg.CompBldrCurSpace compBldr = new MgdDbg.CompBldrCurSpace(m_db)) 
            {
                compBldr.Start();
                compBldr.PushXform(MgdDbg.Utils.Db.GetUcsMatrix(m_db));
                                
                Circle circ1 = new Circle();
                circ1.ColorIndex = 4;
                circ1.Radius = 100.0;
                compBldr.AddToDb(circ1);
                
                LayoutCurve layoutCurve = new LayoutCurve();
                layoutCurve.CurveId = circ1.ObjectId;
                layoutCurve.LayoutMode = LayoutModeType.Manual;
                
                // create just 4 nodes spaced through the cirle
                for (int i=0; i<4; i++)
                    layoutCurve.AppendLayoutNode(20.0 * i);
                    
                compBldr.AddToDb(layoutCurve);    
                compBldr.Commit();
            }
        }
        
        public void
        LayoutCurveSpaceEvenly()
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            using (MgdDbg.CompBldrCurSpace compBldr = new MgdDbg.CompBldrCurSpace(m_db)) {
                compBldr.Start();
                compBldr.PushXform(MgdDbg.Utils.Db.GetUcsMatrix(m_db));
                                
                Circle circ1 = new Circle();
                circ1.ColorIndex = 4;
                circ1.Radius = 100.0;
                compBldr.AddToDb(circ1);
                
                LayoutCurve layoutCurve = new LayoutCurve();
                layoutCurve.CurveId = circ1.ObjectId;
                layoutCurve.LayoutMode = LayoutModeType.SpaceOut;
                
                    // we just need to add the number of nodes.  The layout mode
                    // will adjust them to the proper location based on the length
                    // of the curve.
                for (int i=0; i<8; i++)
                    layoutCurve.AppendLayoutNode(0.0);
                    
                compBldr.AddToDb(layoutCurve);    
                compBldr.Commit();
            }
        }
        
        public void
        LayoutCurveRepeat()
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            using (MgdDbg.CompBldrCurSpace compBldr = new MgdDbg.CompBldrCurSpace(m_db)) {
                compBldr.Start();
                compBldr.PushXform(MgdDbg.Utils.Db.GetUcsMatrix(m_db));
                                
                Arc arc1 = new Arc();
                arc1.ColorIndex = 2;
                arc1.Center = new Point3d(10.0, 10.0, 0.0);
                arc1.Radius = 100.0;
                arc1.StartAngle = MgdDbg.Utils.Ge.kRad0;
                arc1.EndAngle = MgdDbg.Utils.Ge.kRad270;
                compBldr.AddToDb(arc1);
                
                LayoutCurve layoutCurve = new LayoutCurve();
                layoutCurve.CurveId = arc1.ObjectId;
                layoutCurve.LayoutMode = LayoutModeType.Repeat;
                layoutCurve.StartOffset = 5.0;
                layoutCurve.EndOffset = 5.0;
                layoutCurve.Spacing = 20.0;
                compBldr.AddToDb(layoutCurve);
    
                compBldr.Commit();
            }
        }

		/*public void
        LayoutCurveMassGroup()
        {
            using (MgdDbg.CompBldrCurSpace compBldr = new MgdDbg.CompBldrCurSpace(m_db)) {
                compBldr.Start();
                compBldr.PushXform(MgdDbg.Utils.Db.GetUcsMatrix(m_db));
                                
                MassElement massElem1 = MassElement.Create(ShapeType.Cylinder);
                massElem1.ColorIndex = 4;
                massElem1.Location = new Point3d(0.0, 0.0, 0.0);
                massElem1.Radius = 100.0;
                massElem1.Height = 400.0;
                compBldr.AddToDb(massElem1);
                
                LayoutCurve layoutCurve = new LayoutCurve();
                layoutCurve.CurveId = massElem1.ObjectId;
                layoutCurve.LayoutMode = LayoutModeType.SpaceOut;
                //layoutCurve.Spacing = 40.0;
                for (int i=0; i<8; i++)
                    layoutCurve.AppendLayoutNode(0.0);
                compBldr.AddToDb(layoutCurve);
                                
                MassGroup massGroup = new MassGroup();
                massGroup.ColorIndex = 3;
                massGroup.Location = new Point3d(50.0, 0.0, 0.0);
                    // need to add to DB before adding nodes to it
                compBldr.AddToDb(massGroup);
                    // tell this MassGroup to anchor all of its nodes
                MassGroup.SetMassGroupSupportsAnchoring(massGroup.ObjectId, true);
                massGroup.AddNode(massElem1, Operation.Additive);
                massGroup.AddNode(massElem2, Operation.Subtractive);
    
                compBldr.Commit();
            }
        }*/
        
        public void
        AnchorToCurveMidpoint()
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            AcDb.ObjectId curveId, anchoredObjId;
            Point3d pickedPt;
            if (GetAnchorToCurveObjects(out curveId, out anchoredObjId, out pickedPt) == false)
                return;

            using (MgdDbg.TransactionHelper trHlpr = new MgdDbg.TransactionHelper(m_db)) {
                trHlpr.Start();
                
                Geo geoObj = (Geo)trHlpr.Transaction.GetObject(anchoredObjId, OpenMode.ForWrite);
                    
                    // anchor to the midpoint of the selected curve
                AnchorEntityToCurve anchor = new AnchorEntityToCurve();
                anchor.CurveId = curveId;
                anchor.AnchorX.MeasureToType = CurveXMeasureToType.ToCenter;
                anchor.AnchorX.OffsetType = CurveXOffsetType.FromMidpointOfCurve;
                anchor.AnchorX.OffsetDistance = 0.0;
                
                geoObj.SetAnchor(anchor);
                //trHlpr.Transaction.AddNewlyCreatedDBObject(anchor, true); // TBD: ?? how to handle
                
                trHlpr.Commit();
            }
        }
        
        public void
        AnchorToEntity()
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            AcDb.ObjectId entId, anchoredObjId;
            if (GetAnchorToEntityObjects(out entId, out anchoredObjId) == false)
                return;

            using (MgdDbg.TransactionHelper trHlpr = new MgdDbg.TransactionHelper(m_db)) {
                trHlpr.Start();
                
                Geo geoObj = (Geo)trHlpr.Transaction.GetObject(anchoredObjId, OpenMode.ForWrite);
                    
                AnchorEntityToEntity anchor = new AnchorEntityToEntity();
                anchor.ReferencedEntityId = entId;
                //anchor.SetReferencedEntityOldEcs(geoObj.Ecs);     // TBD: this isn't quite working right (at least it seems)
                
                geoObj.SetAnchor(anchor);
                //trHlpr.Transaction.AddNewlyCreatedDBObject(anchor, true); // TBD: ?? how to handle
                
                trHlpr.Commit();
            }
        }
        
        /// <summary>
        /// Allow the user to pick a curve and a Geo entity to anchor to the curve
        /// </summary>
        /// <param name="curveId">The curve picked</param>
        /// <param name="anchoredObjId">The entity to anchor</param>
        /// <returns>true if picked, false if canceled</returns>
        
        private bool
        GetAnchorToCurveObjects(out AcDb.ObjectId curveId, out AcDb.ObjectId anchoredObjId, out Point3d pickPt)
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            curveId = new AcDb.ObjectId();
            anchoredObjId = new AcDb.ObjectId();
            pickPt = new Point3d();
            
            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;
            Autodesk.AutoCAD.DatabaseServices.TransactionManager tm = m_db.TransactionManager;

                // prompt for the Anchored to Object
            PromptEntityOptions prAnchoredOpts = new PromptEntityOptions("\nSelect object to anchor");
            prAnchoredOpts.SetRejectMessage("\nSelected entity must be of type Geo.");
            prAnchoredOpts.AddAllowedClass(typeof(Geo), false);
            
            PromptEntityResult prAnchoredToRes = ed.GetEntity(prAnchoredOpts);
            if (prAnchoredToRes.Status != PromptStatus.OK)
                return false;
                
                // prompt for the Curve entity
            PromptEntityOptions prCurveOpts = new PromptEntityOptions("\nSelect a curve to anchor to");
            prCurveOpts.SetRejectMessage("\nSelected entity must be of type Curve.");
            prCurveOpts.AddAllowedClass(typeof(Curve), false);
            prAnchoredOpts.AllowObjectOnLockedLayer = false;
            
            PromptEntityResult prCurveRes = ed.GetEntity(prCurveOpts);
            if (prCurveRes.Status != PromptStatus.OK)
                return false;
                
            if (prAnchoredToRes.ObjectId == prCurveRes.ObjectId) {
                MgdDbg.Utils.AcadUi.PrintToCmdLine("\nCannot anchor an object to itself!");
                return false;
            }
            
            curveId = prCurveRes.ObjectId;
            anchoredObjId = prAnchoredToRes.ObjectId;
            pickPt = prCurveRes.PickedPoint;
            
            return true;
        }


        /// <summary>
        /// Allow the user to pick a curve and a Geo entity to anchor to the entity
        /// </summary>
        /// <param name="entId">The entity picked</param>
        /// <param name="anchoredObjId">The entity to anchor</param>
        /// <returns>true if picked, false if canceled</returns>
        
        private bool
        GetAnchorToEntityObjects(out AcDb.ObjectId entId, out AcDb.ObjectId anchoredObjId)
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            entId = new AcDb.ObjectId();
            anchoredObjId = new AcDb.ObjectId();
            
            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;
            Autodesk.AutoCAD.DatabaseServices.TransactionManager tm = m_db.TransactionManager;

                // prompt for the Curve entity
            PromptEntityOptions prEntOpts = new PromptEntityOptions("\nSelect an entity to anchor to");            
            PromptEntityResult prEntRes = ed.GetEntity(prEntOpts);
            if (prEntRes.Status != PromptStatus.OK)
                return false;
                
                // prompt for the Anchored to Object
            PromptEntityOptions prAnchoredOpts = new PromptEntityOptions("\nSelect object to anchor");
            prAnchoredOpts.SetRejectMessage("\nSelected entity must be of type Geo.");
            prAnchoredOpts.AddAllowedClass(typeof(Geo), false);
            prAnchoredOpts.AllowObjectOnLockedLayer = false;
            
            PromptEntityResult prAnchoredToRes = ed.GetEntity(prAnchoredOpts);
            if (prAnchoredToRes.Status != PromptStatus.OK)
                return false;
                
            if (prAnchoredToRes.ObjectId == prEntRes.ObjectId) {
                MgdDbg.Utils.AcadUi.PrintToCmdLine("\nCannot anchor an object to itself!");
                return false;
            }
            
            entId = prEntRes.ObjectId;
            anchoredObjId = prAnchoredToRes.ObjectId;
            
            return true;
        }

        private void
        LayoutGrid2dGrids()
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            using (MgdDbg.CompBldrCurSpace compBldr = new MgdDbg.CompBldrCurSpace(m_db)) 
            {
                compBldr.Start();
                compBldr.PushXform(MgdDbg.Utils.Db.GetUcsMatrix(m_db));

                double uLength = 360.0;
                double vLength = 480.0;

                //////////////////////////////////////////////////////////////////////////
                // rectangular grid, repeat spacing
                LayoutGrid2d layoutGridRect = new LayoutGrid2d();
                layoutGridRect.SetToStandard(m_db);
                layoutGridRect.GridType = GridType.UVRectangular;

                  // set the grid
                GridUv grid = layoutGridRect.BaseGrid;

                grid.UAxisLength = uLength;
                grid.VAxisLength = vLength;

                grid.ULayoutMode = UVLayoutModeType.Repeat;
                grid.USpacing = uLength / 10.0; // 10 divisions

                grid.VLayoutMode = UVLayoutModeType.Repeat;
                grid.VSpacing = vLength / 5.0; // 5 divisions

                grid.UpdateParameters();  // Rebuild the grid...
                layoutGridRect.Location = new Point3d(0.0, 0.0, 0.0);
                
                compBldr.AddToDb(layoutGridRect);    

                //////////////////////////////////////////////////////////////////////////
                // radial grid, repeat spacing
                LayoutGrid2d layoutGridRad = new LayoutGrid2d();
                layoutGridRad.SetToStandard(m_db);
                layoutGridRad.GridType = GridType.UVRadial;

                // set the grid
                GridUv gridRad = layoutGridRad.BaseGrid;

                gridRad.UAxisLength = uLength;
                double vLengthRad = 1.5707963268; // 90 degrees (in radians)
                gridRad.VAxisLength =  vLengthRad;

                gridRad.ULayoutMode = UVLayoutModeType.Repeat;
                gridRad.USpacing = uLength / 10.0; // 10 divisions

                gridRad.VLayoutMode = UVLayoutModeType.Repeat;
                gridRad.VSpacing = vLengthRad / 5.0; // 5 divisions

                gridRad.UpdateParameters();  // Rebuild the gridRad...
                layoutGridRad.Location = new Point3d(500.0, 0.0, 0.0);
                
                compBldr.AddToDb(layoutGridRad);    

                //////////////////////////////////////////////////////////////////////////
                // rectangular grid, space evenly spacing, with offsets

                LayoutGrid2d layoutGridRectSE = new LayoutGrid2d();
                layoutGridRectSE.SetToStandard(m_db);
                layoutGridRectSE.GridType = GridType.UVRectangular;

                // set the grid
                GridUv gridSE = layoutGridRectSE.BaseGrid;

                gridSE.UAxisLength = uLength;
                gridSE.VAxisLength = vLength;

                gridSE.ULayoutMode = UVLayoutModeType.SpaceEvenly;
                gridSE.UStartOffset = 1.0;
                gridSE.UEndOffset = 1.0;
                for ( int i = 0; i < 8; i++ ) 
                    gridSE.AppendGridColumn(i);

                gridSE.VLayoutMode = UVLayoutModeType.SpaceEvenly;
                gridSE.VStartOffset = 2.0;
                gridSE.VEndOffset = 2.0;
                for ( int j = 0; j < 8; j++ ) 
                    gridSE.AppendGridRow(j);

                gridSE.UpdateParameters();  // Rebuild the grid...
                layoutGridRectSE.Location = new Point3d(1000.0, 0.0, 0.0);
                
                compBldr.AddToDb(layoutGridRectSE);    


                compBldr.Commit();
            } //end using
        }


        private void
        LayoutGrid3dGrids()
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            using (MgdDbg.CompBldrCurSpace compBldr = new MgdDbg.CompBldrCurSpace(m_db)) 
            {
                compBldr.Start();
                compBldr.PushXform(MgdDbg.Utils.Db.GetUcsMatrix(m_db));

                double uLength = 360.0;
                double vLength = 480.0;
                double wLength = 240.0;

                //////////////////////////////////////////////////////////////////////////
                // rectangular 3d grid, repeat spacing
                LayoutGrid3d layoutGridRect = new LayoutGrid3d();
                layoutGridRect.SetToStandard(m_db);
                layoutGridRect.GridType = GridType.UVWRectangular;

                // set the grid
                GridUvw grid = layoutGridRect.BaseGrid;

                grid.UAxisLength = uLength;
                grid.VAxisLength = vLength;
                grid.WAxisLength = wLength;

                grid.ULayoutMode = UVLayoutModeType.Repeat;
                grid.USpacing = uLength / 10.0; // 10 divisions

                grid.VLayoutMode = UVLayoutModeType.Repeat;
                grid.VSpacing = vLength / 5.0; // 5 divisions

                grid.WLayoutMode = UVLayoutModeType.Repeat;
                grid.WSpacing = wLength / 5.0; // 5 divisions

                grid.UpdateParameters();  // Rebuild the grid...
                layoutGridRect.Location = new Point3d(0.0, 0.0, 0.0);
                
                compBldr.AddToDb(layoutGridRect);    

                compBldr.Commit();
            } //end using
        }

    }
}
