// $Header: //Desktop/Source/TestTools/MgdDbgAec/ObjTests/MakeEntsStructure.cs#1 $
// $Author: fergusja $ $DateTime: 2008/03/04 16:12:35 $ $Change: 94604 $
//
// (C) Copyright 2008 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//

using System;
using System.Windows.Forms;
using Autodesk.AutoCAD.DatabaseServices;
using Autodesk.AutoCAD.Geometry;
using Autodesk.AutoCAD.EditorInput;
using MgdDbg;
using MgdDbg.Test;
using Autodesk.Aec.DatabaseServices;
using Autodesk.Aec.Geometry;
using Autodesk.Aec.Structural.DatabaseServices;

using AcDb = Autodesk.AutoCAD.DatabaseServices;
using AcGe = Autodesk.AutoCAD.Geometry;

namespace MgdDbgAec.ObjTests
{
    /// <summary>
    /// Summary description for MakeEntsStructure.
    /// </summary>
    public class MakeEntsStructure : MgdDbgTestFuncs
    {
        private Database m_db = null;

        public
        MakeEntsStructure()
        {
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Beam", "Simple hardwired Beam", typeof(Member), new MgdDbgTestFuncInfo.TestFunc(BeamSimple), MgdDbgTestFuncInfo.TestType.Create));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Brace", "Simple hardwired Brace", typeof(Member), new MgdDbgTestFuncInfo.TestFunc(BraceSimple), MgdDbgTestFuncInfo.TestType.Create));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Column", "Simple hardwired Column", typeof(Member), new MgdDbgTestFuncInfo.TestFunc(ColumnSimple), MgdDbgTestFuncInfo.TestType.Create));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("ColumnGrid", "Simple hardwired ColumnGrid", typeof(ColumnGrid), new MgdDbgTestFuncInfo.TestFunc(ColumnGridSimple), MgdDbgTestFuncInfo.TestType.Create));
        }

        public void
        BeamSimple()
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            Member beam = new Member();
            beam.MemberType = MemberType.Beam;

            beam.Set(new Point3d(0.0, 0.0, 0.0), new Point3d(200.0, 0.0, 0.0), new Vector3d(0.0, 0.0, 1.0));
            beam.SetDefaultLayer();
            beam.Description = "This beam was created by MgdDbgAec!";

            MgdDbg.Utils.Db.TransformToWcs(beam, m_db);
            MgdDbg.Utils.SymTbl.AddToCurrentSpaceAndClose(beam, m_db);
        }

        public void
        BraceSimple()
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            Member brace = new Member();
            brace.MemberType = MemberType.Brace;

            brace.Set(new Point3d(0.0, 100.0, 0.0), new Point3d(250.0, 100.0, 0.0), new Vector3d(0.0, 0.0, 1.0));
            brace.SetDefaultLayer();
            brace.Description = "This brace was created by MgdDbgAec!";

            MgdDbg.Utils.Db.TransformToWcs(brace, m_db);
            MgdDbg.Utils.SymTbl.AddToCurrentSpaceAndClose(brace, m_db);
        }

        public void
        ColumnSimple()
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            Member column = new Member();
            column.MemberType = MemberType.Column;

            AcGe.Vector3d zNormal = MgdDbg.Utils.Db.GetUcsZAxis(m_db);
            AcGe.LineSegment3d curve = new AcGe.LineSegment3d(new Point3d(0.0, 200.0, 0.0), zNormal.MultiplyBy(144.0));
            AcGe.Vector3d yNormal = MgdDbg.Utils.Db.GetUcsYAxis(m_db);
            column.SetBaseCurve(curve, yNormal);
            column.SetDefaultLayer();
            column.Roll = MgdDbg.Utils.Ge.DegreesToRadians(30.0);
            column.Description = "This column was created by MgdDbgAec!";

            MgdDbg.Utils.Db.TransformToWcs(column, m_db);
            MgdDbg.Utils.SymTbl.AddToCurrentSpaceAndClose(column, m_db);
        }

        public void
        ColumnGridSimple()
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            ColumnGrid cg = new ColumnGrid();
            IntegerCollection idNodes = cg.LayoutNodeIds;

            AnchorEntityToLayoutNode anchor = new AnchorEntityToLayoutNode();
            anchor.LayoutNodeId = idNodes[0];
            cg.Description = "This column grid was created by MgdDbgAec!";

            Member column = new Member();
            column.MemberType = MemberType.Column;

            AcGe.Vector3d zNormal = MgdDbg.Utils.Db.GetUcsZAxis(m_db);
            AcGe.LineSegment3d curve = new AcGe.LineSegment3d(new Point3d(0.0, 200.0, 0.0), zNormal.MultiplyBy(144.0));
            AcGe.Vector3d yNormal = MgdDbg.Utils.Db.GetUcsYAxis(m_db);
            column.SetBaseCurve(curve, yNormal);

            column.SetAnchor(anchor);

            MgdDbg.Utils.SymTbl.AddToCurrentSpaceAndClose(column, m_db);
            MgdDbg.Utils.Db.TransformToWcs(cg, m_db);
            MgdDbg.Utils.SymTbl.AddToCurrentSpaceAndClose(cg, m_db);
        }

    }
}
