
// $Header: //Desktop/Source/TestTools/MgdDbgAec/ObjTests/MakeStylesArch.cs#1 $
// $Author: fergusja $ $DateTime: 2007/03/16 11:08:08 $ $Change: 68854 $
//
// (C) Copyright 2004 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//

using System;
using System.Windows.Forms;
using Autodesk.AutoCAD.DatabaseServices;
using Autodesk.AutoCAD.Geometry;
using MgdDbg;
using MgdDbg.Test;
using Autodesk.Aec.DatabaseServices;
using Autodesk.Aec.Arch.DatabaseServices;

namespace MgdDbgAec.ObjTests
{
	/// <summary>
	/// Summary description for MakeStylesArch.
	/// </summary>
    public class MakeStylesArch : MgdDbgTestFuncs
    {
        private Database    m_db = null;
	    
        public
        MakeStylesArch()
        {           
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("All Window Styles", "Hardwired all Type/Shape combos", typeof(WindowStyle), new MgdDbgTestFuncInfo.TestFunc(WindowStyles), MgdDbgTestFuncInfo.TestType.Create));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("All Door Styles", "Hardwired all Type/Shape combos", typeof(DoorStyle), new MgdDbgTestFuncInfo.TestFunc(DoorStyles), MgdDbgTestFuncInfo.TestType.Create));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Wall Style (Cavity)", "Hardwired Cavity Wall Style", typeof(WallStyle), new MgdDbgTestFuncInfo.TestFunc(WallStyleCavity), MgdDbgTestFuncInfo.TestType.Create));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Wall Style (Cement Bottom)", "Hardwired Cement Bottom Wall Style", typeof(WallStyle), new MgdDbgTestFuncInfo.TestFunc(WallStyleCementBottom), MgdDbgTestFuncInfo.TestType.Create));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Wall Style (Bottom Trim)", "Hardwired Wall Style, with trim at bottom", typeof(WallStyle), new MgdDbgTestFuncInfo.TestFunc(WallStyleBottomTrim), MgdDbgTestFuncInfo.TestType.Create));

            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Curtain wall layout style", "Hardwired Curtain wall layout style", typeof(CurtainWallLayoutStyle), new MgdDbgTestFuncInfo.TestFunc(MakeCurtainWallLayoutStyle), MgdDbgTestFuncInfo.TestType.Create));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Curtain wall unit style", "Hardwired Curtain wall unit style", typeof(CurtainWallUnitStyle), new MgdDbgTestFuncInfo.TestFunc(MakeCurtainWallUnitStyle), MgdDbgTestFuncInfo.TestType.Create));
        }
		
		/// <summary>
		/// Iterate through all Type and Shape combos (except Custom, which requires a Profile ID)
		/// and create a new style with that combo.
		/// </summary>
		
        public void
        WindowStyles()
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            Autodesk.AutoCAD.DatabaseServices.TransactionManager tm = m_db.TransactionManager;
            using (Autodesk.AutoCAD.DatabaseServices.Transaction tr = tm.StartTransaction()) {

                using (DictionaryWindowStyle dict = new DictionaryWindowStyle(m_db)) {
                    foreach (WindowType wt in System.Enum.GetValues(typeof(WindowType))) {
                        foreach (OpenShapeType st in System.Enum.GetValues(typeof(OpenShapeType))) {
                            if ((wt != WindowType.Custom) && (st != OpenShapeType.Custom) && (WindowStyle.WorksWith(wt, st))) {
                                string styleName = string.Format("{0}_{1}", wt.ToString(), st.ToString());
                                if (dict.Has(styleName, tr) == false) {
                                    WindowStyle winStyle = new WindowStyle();
                                    winStyle.SetToStandard(m_db);
                                    
                                    winStyle.WindowType = wt;
                                    winStyle.WindowShape = st;
                                    
                                    dict.AddNewRecord(styleName, winStyle);
                                    tr.AddNewlyCreatedDBObject(winStyle, true);
                                    
                                    string msg = string.Format("\nCreated style {0}.", styleName);
                                    MgdDbg.Utils.AcadUi.PrintToCmdLine(msg);
                                }
                                else {
                                    string msg = string.Format("\nStyle {0} already exists.", styleName);
                                    MgdDbg.Utils.AcadUi.PrintToCmdLine(msg);
                                }
                            }
                        }
                    }
                }
                
                tr.Commit();
            }
        }
        
        /// <summary>
		/// Iterate through all Type and Shape combos (except Custom, which requires a Profile ID)
		/// and create a new style with that combo.
		/// </summary>
		
        public void
        DoorStyles()
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            Autodesk.AutoCAD.DatabaseServices.TransactionManager tm = m_db.TransactionManager;
            using (Autodesk.AutoCAD.DatabaseServices.Transaction tr = tm.StartTransaction()) {

                using (DictionaryDoorStyle dict = new DictionaryDoorStyle(m_db)) {
                    foreach (DoorType dt in System.Enum.GetValues(typeof(DoorType))) {
                        foreach (OpenShapeType st in System.Enum.GetValues(typeof(OpenShapeType))) {
                            if ((dt != DoorType.Custom) && (st != OpenShapeType.Custom) && (DoorStyle.WorksWith(dt, st))) {
                                string styleName = string.Format("{0}_{1}", dt.ToString(), st.ToString());
                                if (dict.Has(styleName, tr) == false) {
                                    DoorStyle doorStyle = new DoorStyle();
                                    doorStyle.SetToStandard(m_db);
                                    
                                    doorStyle.DoorType = dt;
                                    doorStyle.DoorShape = st;
                                    
                                    dict.AddNewRecord(styleName, doorStyle);
                                    tr.AddNewlyCreatedDBObject(doorStyle, true);
                                    
                                    string msg = string.Format("\nCreated style {0}.", styleName);
                                    MgdDbg.Utils.AcadUi.PrintToCmdLine(msg);
                                }
                                else {
                                    string msg = string.Format("\nStyle {0} already exists.", styleName);
                                    MgdDbg.Utils.AcadUi.PrintToCmdLine(msg);
                                }
                            }
                        }
                    }
                }
                
                tr.Commit();
            }
        }
        
        public void
        WallStyleCavity()
        {
            m_db           = MgdDbg.Utils.Db.GetCurDwg();
            string symName = "MgdDbgAec_Cavity";
            
            using (TransactionHelper tr = new TransactionHelper(m_db)) {
                tr.Start();
                
                using (DictionaryWallStyle dict = new DictionaryWallStyle(m_db)) {
                    if (dict.Has(symName, tr.Transaction)) {
                        MgdDbg.Utils.AcadUi.PrintToCmdLine(string.Format("\nWall Style \"{0}\" already exists.", symName));
                        return;
                    }
                    
                    WallStyle ws = new WallStyle();
                    WallStyleComponentCollection wsCompColl = ws.Components;
                    
                        // simple cavity wall with two fixed size components
                    WallStyleComponent comp = new WallStyleComponent();
                    comp.SubSetDatabaseDefaults(m_db);
                    comp.Name = "facer";
                    comp.Position.SetFixedEdgeOffset(-6.0);
                    comp.Position.SetFixedWidth(4.0);
                    comp.Priority = 1;
                    wsCompColl.Add(comp);

                    comp = new WallStyleComponent();
                    comp.SubSetDatabaseDefaults(m_db);
                    comp.Name = "facerl";
                    comp.Position.SetFixedEdgeOffset(2.0);
                    comp.Position.SetFixedWidth(4.0);
                    comp.Priority = 8;
                    wsCompColl.Add(comp);
                    
                    dict.AddNewRecord(symName, ws);
                    tr.Transaction.AddNewlyCreatedDBObject(ws, true);
                }
                
                tr.Commit();
                MgdDbg.Utils.AcadUi.PrintToCmdLine(string.Format("\nCreated Wall Style \"{0}\".", symName));
            }
        }
        
        public void
        WallStyleCementBottom()
        {
            m_db           = MgdDbg.Utils.Db.GetCurDwg();
            string symName = "MgdDbgAec_CementBottom";
            
            using (TransactionHelper tr = new TransactionHelper(m_db)) {
                tr.Start();
                
                using (DictionaryWallStyle dict = new DictionaryWallStyle(m_db)) {
                    if (dict.Has(symName, tr.Transaction)) {
                        MgdDbg.Utils.AcadUi.PrintToCmdLine(string.Format("\nWall Style \"{0}\" already exists.", symName));
                        return;
                    }
                    
                    WallStyle ws = new WallStyle();
                    WallStyleComponentCollection wsCompColl = ws.Components;

                        // this wall style has 3 components.  The first is the full width of the wall
                        // and only goes up 36 inches.  The other two components sit on top of this bottom
                        // component and stretch to the top of the wall (whatever height that might be)
                    WallStyleComponent comp = new WallStyleComponent();
                    comp.SubSetDatabaseDefaults(m_db);
                    comp.Name = "comp1";
                    comp.Position.SetFixedEdgeOffset(-4.0);
                    comp.Position.SetFixedWidth(8.0);
                    comp.Position.StartElevation.ElevationType = WallElevationType.FromWallBottom;
                    comp.Position.EndElevation.ElevationType = WallElevationType.FromBaseline;
                    comp.Position.StartElevation.ElevationOffset = 0.0;
                    comp.Position.EndElevation.ElevationOffset = 36.0;
                    comp.Priority = 1;
                    wsCompColl.Add(comp);

                    comp = new WallStyleComponent();
                    comp.SubSetDatabaseDefaults(m_db);
                    comp.Name = "comp2";
                    comp.Position.SetFixedEdgeOffset(-4.0);
                    comp.Position.SetFixedWidth(2.0);
                    comp.Position.StartElevation.ElevationType = WallElevationType.FromBaseline;
                    comp.Position.EndElevation.ElevationType = WallElevationType.FromWallTop;
                    comp.Position.StartElevation.ElevationOffset = 36.0;
                    comp.Position.EndElevation.ElevationOffset = 0.0;
                    comp.Priority = 12;
                    wsCompColl.Add(comp);
                    
                    comp = new WallStyleComponent();
                    comp.SubSetDatabaseDefaults(m_db);
                    comp.Name = "comp3";
                    comp.Position.SetFixedEdgeOffset(1.0);
                    comp.Position.SetFixedWidth(3.0);
                    comp.Position.StartElevation.ElevationType = WallElevationType.FromBaseline;
                    comp.Position.EndElevation.ElevationType = WallElevationType.FromBaseHeight;
                    comp.Position.StartElevation.ElevationOffset = 36.0;
                    comp.Position.EndElevation.ElevationOffset = 0.0;
                    comp.Priority = 15;
                    wsCompColl.Add(comp);

                    dict.AddNewRecord(symName, ws);
                    tr.Transaction.AddNewlyCreatedDBObject(ws, true);
                }
                
                tr.Commit();
                MgdDbg.Utils.AcadUi.PrintToCmdLine(string.Format("\nCreated Wall Style \"{0}\".", symName));
            }
        }
        
        public void
        WallStyleBottomTrim()
        {
            m_db           = MgdDbg.Utils.Db.GetCurDwg();
            string symName = "MgdDbgAec_BottomTrim";
            
            using (TransactionHelper tr = new TransactionHelper(m_db)) {
                tr.Start();
                
                using (DictionaryWallStyle dict = new DictionaryWallStyle(m_db)) {
                    if (dict.Has(symName, tr.Transaction)) {
                        MgdDbg.Utils.AcadUi.PrintToCmdLine(string.Format("\nWall Style \"{0}\" already exists.", symName));
                        return;
                    }
                    
                    WallStyle ws = new WallStyle();
                    WallStyleComponentCollection wsCompColl = ws.Components;

                    
                        // we want the main component to be instance based (so it can vary for
                        // each wall of this style).
                    WallStyleComponent comp = new WallStyleComponent();
                    comp.SubSetDatabaseDefaults(m_db);
                    comp.Name = "main";
                    comp.Position.InstanceEdgeOffset.UseInstanceValue = true;
                    comp.Position.InstanceEdgeOffset.Operand = -2.0;
                    comp.Position.InstanceEdgeOffset.OperatorType = InstanceBasedValueOperatorType.Divide;
                    comp.Position.InstanceWidth.UseInstanceValue = true;
                    comp.Priority = 1;
                    wsCompColl.Add(comp);

                        // the trim component we want to follow the width of the main component.
                        // So, the edgeOffset is variable (InstanceBased), but the width is fixed.
                    comp = new WallStyleComponent();
                    comp.SubSetDatabaseDefaults(m_db);
                    comp.Name = "trim";
                    comp.Position.InstanceEdgeOffset.UseInstanceValue = true;
                    comp.Position.InstanceEdgeOffset.OperatorType = InstanceBasedValueOperatorType.Divide;
                    comp.Position.InstanceEdgeOffset.Operand = 2.0;
                    comp.Position.SetFixedWidth(2.0);
                    comp.Position.StartElevation.ElevationType = WallElevationType.FromWallBottom;
                    comp.Position.EndElevation.ElevationType = WallElevationType.FromWallBottom;
                    comp.Position.StartElevation.ElevationOffset = 0.0;
                    comp.Position.EndElevation.ElevationOffset = 6.0;
                    comp.Priority = 12;
                    wsCompColl.Add(comp);
                    
                    dict.AddNewRecord(symName, ws);
                    tr.Transaction.AddNewlyCreatedDBObject(ws, true);
                }
                
                tr.Commit();
                MgdDbg.Utils.AcadUi.PrintToCmdLine(string.Format("\nCreated Wall Style \"{0}\".", symName));
            }
        }

        
        private void
        MakeCurtainWallLayoutStyle()
        {
            m_db           = MgdDbg.Utils.Db.GetCurDwg();
            string symName = "MgdDbgAec_CurtainWallLayoutStyle";
            
            using (TransactionHelper tr = new TransactionHelper(m_db)) 
            {
                tr.Start();
                
                using (DictionaryCurtainWallLayoutStyle dict = new DictionaryCurtainWallLayoutStyle(m_db)) 
                {
                    if (dict.Has(symName, tr.Transaction)) 
                    {
                        MgdDbg.Utils.AcadUi.PrintToCmdLine(string.Format("\nCurtain Wall Layout Style \"{0}\" already exists.", symName));
                        return;
                    }

                    double defBaseHt;
                    double defRoof;
                    double defFloor;
                    double defCellWidth;
                    double defCellHeight;
                    double defInfillDepth;
                    double defBoundaryDepth;
                    double defBoundaryWidth;
                    double defInternalWidthA;
                    double defInternalDepthA;
                    
                    // I'm assuming Imperial units (inches). Modify these values if you are in metric 
                    defBaseHt = 468.0; // 39'
                    defRoof = 0.0;
                    defFloor = -24.0;
                    defCellWidth = 36;
                    defCellHeight = 156;
                    defInfillDepth = 1;
                    defBoundaryDepth = 3;
                    defBoundaryWidth = 3;
                    defInternalWidthA = 1;
                    defInternalDepthA = 3;


                    CurtainWallLayoutStyle curtainWallStyle = new CurtainWallLayoutStyle();

                    curtainWallStyle.DefaultBaseHeight = defBaseHt;
                    curtainWallStyle.DefaultRooflineOffset = defRoof;
                    curtainWallStyle.DefaultFloorlineOffset = defFloor;

                    /////////////// Create and add default components.../////////////

                    GridAssemblyComponentSet compSet = new GridAssemblyComponentSet();

                    // Create a default grid division 1
                    NestedGridDivisionRepeat gridDivRepeatWidth = new NestedGridDivisionRepeat();
                    gridDivRepeatWidth.CellSize = defCellWidth;
                    gridDivRepeatWidth.DistributeSlack = true;
                    gridDivRepeatWidth.DistributionFlags = DistributionFlags.DistributeToFirst | DistributionFlags.DistributeToLast
                        | DistributionFlags.NeverExceedCellSize
                        | DistributionFlags.MaintainAtLeastHalfCellSize;

                    gridDivRepeatWidth.DivisionDirection = DivisionDirection.AlongXAxis;
                    gridDivRepeatWidth.Name = "MgdDbgAec Vertical Division";

                    compSet.DivisionComponents.Add(gridDivRepeatWidth);

                    // Create a default grid division 2
                    NestedGridDivisionRepeat gridDivRepeatHeight = new NestedGridDivisionRepeat();
                    gridDivRepeatHeight.CellSize = defCellHeight;
                    gridDivRepeatHeight.DistributeSlack = true;
                    gridDivRepeatHeight.DistributionFlags = DistributionFlags.DistributeToLast | DistributionFlags.NeverExceedCellSize;
                    gridDivRepeatHeight.DivisionDirection = DivisionDirection.AlongYAxis;
                    gridDivRepeatHeight.Name = "MgdDbgAec Horizontal Division";

                    compSet.DivisionComponents.Add(gridDivRepeatHeight);

                    // Create an infill component...

                    NestedGridInfillArch defaultCellInfill = new NestedGridInfillArch();
                    defaultCellInfill.YDepth = defInfillDepth;
                    defaultCellInfill.Name = "MgdDbgAec Default Infill";

                    compSet.InfillComponents.Add(defaultCellInfill);

                    // Create a boundary edge component

                    NestedGridEdgeDefinition boundaryEdgeDef = new NestedGridEdgeDefinition();
                    boundaryEdgeDef.SetToStandard(m_db);
                    boundaryEdgeDef.EdgeWidth = defBoundaryWidth;
                    boundaryEdgeDef.EdgeDepth = defBoundaryDepth;
                    boundaryEdgeDef.Name = "MgdDbgAec Default Boundary Edge";

                    compSet.BoundaryEdgeComponents.Add(boundaryEdgeDef);

                    // Create an internal edge component

                    NestedGridEdgeDefinition gridLineEdgeDefA = new NestedGridEdgeDefinition();
                    gridLineEdgeDefA.SetToStandard(m_db);
                    gridLineEdgeDefA.EdgeWidth = defInternalWidthA;
                    gridLineEdgeDefA.EdgeDepth = defInternalDepthA;
                    gridLineEdgeDefA.Name = "MgdDbgAec Default Interior Edge";
                    compSet.InteriorEdgeComponents.Add(gridLineEdgeDefA);

                    ////////////////// Set up the grid definition //////////////////

                    NestedGridDefinition mainGridDef = new NestedGridDefinition();
                    mainGridDef.GridDivisionId = gridDivRepeatHeight.Id;
                    mainGridDef.Name = "MgdDbgAec Primary Grid";

                    NestedGridDefinition nestedGridDef = new NestedGridDefinition();
                    nestedGridDef.GridDivisionId = gridDivRepeatWidth.Id;
                    nestedGridDef.Name = "MgdDbgAec Secondary Grid";

                    // Set up the default cell assignment...

                    NestedGridCellAssignment defaultCellAsnA = new NestedGridCellAssignment();
                    defaultCellAsnA.Id = curtainWallStyle.IncrementCellAssignmentId();
                    defaultCellAsnA.NestedGridDefinition = nestedGridDef;
                    defaultCellAsnA.Name = "MgdDbgAec Secondary Grid Assignment";
                    mainGridDef.SpecificCellAssignments.Default = defaultCellAsnA;

                    NestedGridCellAssignment defaultCellAsnB = new NestedGridCellAssignment();
                    defaultCellAsnB.Id = curtainWallStyle.IncrementCellAssignmentId();
                    defaultCellAsnB.InfillDefinitionId = defaultCellInfill.Id;
                    defaultCellAsnB.Name = "MgdDbgAec Default Infill Assignment";
                    nestedGridDef.SpecificCellAssignments.Default = defaultCellAsnB;

                    //// set up Specific Cell assignment

                    IndexSpecifierLocation specCellAsn = new IndexSpecifierLocation();
                    specCellAsn.AppliesToStartIndex = true;
                    specCellAsn.AppliesToMiddleIndex = true;
                    specCellAsn.AppliesToEndIndex = true;

                    NestedGridCellAssignment specNestedGridCellAsn = new NestedGridCellAssignment();
                    specNestedGridCellAsn.SetToStandard(m_db);
                    specNestedGridCellAsn.InfillDefinitionId = defaultCellInfill.Id;  // this or defaultCellInfill?
                    specNestedGridCellAsn.Id = curtainWallStyle.IncrementCellAssignmentId();
                    specNestedGridCellAsn.CellSpecifier = specCellAsn;
                    specNestedGridCellAsn.Name = "MgdDbgAec New Specific Cell Assignment";

                    nestedGridDef.SpecificCellAssignments.Add(specNestedGridCellAsn);

                    // Set up the default boundary edge assignments...

                    IndexSpecifierBoundaryEdge boundSpecA = new IndexSpecifierBoundaryEdge();
                    BoundaryFlags flagsA = (BoundaryFlags.Left & BoundaryFlags.Right &
                        BoundaryFlags.Top & BoundaryFlags.Bottom);
                    boundSpecA.BoundaryFlags = flagsA;

                    NestedGridEdgeAssignment boundaryEdgeAsnA = new NestedGridEdgeAssignment();

                    boundaryEdgeAsnA.EdgeSpecifier = boundSpecA;
                    boundaryEdgeAsnA.Id = curtainWallStyle.IncrementEdgeAssignmentId();
                    boundaryEdgeAsnA.EdgeDefinitionId = boundaryEdgeDef.Id;
                    boundaryEdgeAsnA.Name = "MgdDbgAec Default Boundary Edge Assignment";

                    mainGridDef.SpecificBoundaryEdgeAssignments.Default = boundaryEdgeAsnA;

                    IndexSpecifierBoundaryEdge boundSpecB = new IndexSpecifierBoundaryEdge();
                    BoundaryFlags flagsB = 0;
                    boundSpecB.BoundaryFlags = flagsB;

                    NestedGridEdgeAssignment boundaryEdgeAsnB = new NestedGridEdgeAssignment();

                    boundaryEdgeAsnB.EdgeSpecifier = boundSpecB;
                    boundaryEdgeAsnB.Id = curtainWallStyle.IncrementEdgeAssignmentId();
                    boundaryEdgeAsnB.EdgeDefinitionId = boundaryEdgeDef.Id;
                    boundaryEdgeAsnB.Name = "MgdDbgAec Default Boundary Edge Assignment";

                    nestedGridDef.SpecificBoundaryEdgeAssignments.Default = boundaryEdgeAsnB;

                    //// set up specific boundary edge assignment

                    IndexSpecifierBoundaryEdge specBoundaryEdgeAsn = new IndexSpecifierBoundaryEdge();
                    specBoundaryEdgeAsn.BoundaryFlags = BoundaryFlags.Bottom | BoundaryFlags.Top | BoundaryFlags.Left | BoundaryFlags.Right;

                    NestedGridEdgeAssignment specNestedGridEdgeAsn = new NestedGridEdgeAssignment();
                    specNestedGridEdgeAsn.SetToStandard(m_db);
                    specNestedGridEdgeAsn.EdgeDefinitionId = boundaryEdgeDef.Id;
                    specNestedGridEdgeAsn.Id = curtainWallStyle.IncrementEdgeAssignmentId();
                    specNestedGridEdgeAsn.EdgeSpecifier = specBoundaryEdgeAsn;
                    specNestedGridEdgeAsn.Name = "MgdDbgAec New Specific Frame Assignment";

                    nestedGridDef.SpecificBoundaryEdgeAssignments.Add(specNestedGridEdgeAsn);


                    // Set up the default internal edge assignment...

                    NestedGridEdgeAssignment internalEdgeAsnA = new NestedGridEdgeAssignment();

                    internalEdgeAsnA.Id = curtainWallStyle.IncrementEdgeAssignmentId();
                    internalEdgeAsnA.EdgeDefinitionId = gridLineEdgeDefA.Id;
                    internalEdgeAsnA.Name = "MgdDbgAec Default Internal Edge Assignment";

                    mainGridDef.SpecificInteriorEdgeAssignments.Default = internalEdgeAsnA;

                    NestedGridEdgeAssignment internalEdgeAsnB = new NestedGridEdgeAssignment();

                    internalEdgeAsnB.Id = curtainWallStyle.IncrementEdgeAssignmentId();
                    internalEdgeAsnB.EdgeDefinitionId = gridLineEdgeDefA.Id;
                    internalEdgeAsnB.Name = "MgdDbgAec Default Internal Edge Assignment";

                    nestedGridDef.SpecificInteriorEdgeAssignments.Default = internalEdgeAsnB;

                    //// Set up the specific internal edge assignment

                    IndexSpecifierLocation specInternalEdgeAsn = new IndexSpecifierLocation();
                    specInternalEdgeAsn.AppliesToStartIndex = true;
                    specInternalEdgeAsn.AppliesToMiddleIndex = true;
                    specInternalEdgeAsn.AppliesToEndIndex = true;

                    NestedGridEdgeAssignment specInternalNestedGridEdgeAsn = new NestedGridEdgeAssignment();
                    specInternalNestedGridEdgeAsn.SetToStandard(m_db);
                    specInternalNestedGridEdgeAsn.EdgeDefinitionId = gridLineEdgeDefA.Id;
                    specInternalNestedGridEdgeAsn.Id = curtainWallStyle.IncrementEdgeAssignmentId();
                    specInternalNestedGridEdgeAsn.EdgeSpecifier = specInternalEdgeAsn;
                    specInternalNestedGridEdgeAsn.Name = "MgdDbgAec New Specific Mullion Assignment";

                    nestedGridDef.SpecificInteriorEdgeAssignments.Add(specInternalNestedGridEdgeAsn);

                    // set the GridAssemblyDefinition, and then the ComponentSet
                    curtainWallStyle.GridAssemblyDefinition = mainGridDef;
                    curtainWallStyle.ComponentSet = compSet;

                    // add it to dictionary!
                    dict.AddNewRecord(symName, curtainWallStyle);
                    tr.Transaction.AddNewlyCreatedDBObject(curtainWallStyle, true);

                } //end using
                
                tr.Commit();
                MgdDbg.Utils.AcadUi.PrintToCmdLine(string.Format("\nCreated Curtain Wall Layout Style \"{0}\".", symName));
            }
        }

        
        private void
        MakeCurtainWallUnitStyle()
        {
            m_db           = MgdDbg.Utils.Db.GetCurDwg();
            string symName = "MgdDbgAec_CurtainWallUnitStyle";
            
            using (TransactionHelper tr = new TransactionHelper(m_db)) 
            {
                tr.Start();
                
                using (DictionaryCurtainWallUnitStyle dict = new DictionaryCurtainWallUnitStyle(m_db)) 
                {
                    if (dict.Has(symName, tr.Transaction)) 
                    {
                        MgdDbg.Utils.AcadUi.PrintToCmdLine(string.Format("\nCurtain Wall Unit Style \"{0}\" already exists.", symName));
                        return;
                    }

                    double defCellWidth;
                    double defCellHeight;
                    double defInfillDepth;
                    double defBoundaryDepth;
                    double defBoundaryWidth;
                    double defInternalWidthA;
                    double defInternalDepthA;
                    
                    // I'm assuming Imperial units (inches). Modify these values if you are in metric 
                    defCellWidth = 36;
                    defCellHeight = 156;
                    defInfillDepth = 1;
                    defBoundaryDepth = 3;
                    defBoundaryWidth = 3;
                    defInternalWidthA = 1;
                    defInternalDepthA = 3;


                    CurtainWallUnitStyle curtainWallUnitStyle = new CurtainWallUnitStyle();


                    /////////////// Create and add default components.../////////////

                    GridAssemblyComponentSet compSet = new GridAssemblyComponentSet();

                    // Create a default grid division 1
                    NestedGridDivisionRepeat gridDivRepeatWidth = new NestedGridDivisionRepeat();
                    gridDivRepeatWidth.CellSize = defCellWidth;
                    gridDivRepeatWidth.DistributeSlack = true;
                    gridDivRepeatWidth.DistributionFlags = DistributionFlags.DistributeToFirst | DistributionFlags.DistributeToLast
                        | DistributionFlags.NeverExceedCellSize
                        | DistributionFlags.MaintainAtLeastHalfCellSize;

                    gridDivRepeatWidth.DivisionDirection = DivisionDirection.AlongXAxis;
                    gridDivRepeatWidth.Name = "MgdDbgAec Vertical Division";

                    compSet.DivisionComponents.Add(gridDivRepeatWidth);

                    // Create a default grid division 2
                    NestedGridDivisionRepeat gridDivRepeatHeight = new NestedGridDivisionRepeat();
                    gridDivRepeatHeight.CellSize = defCellHeight;
                    gridDivRepeatHeight.DistributeSlack = true;
                    gridDivRepeatHeight.DistributionFlags = DistributionFlags.DistributeToLast | DistributionFlags.NeverExceedCellSize;
                    gridDivRepeatHeight.DivisionDirection = DivisionDirection.AlongYAxis;
                    gridDivRepeatHeight.Name = "MgdDbgAec Horizontal Division";

                    compSet.DivisionComponents.Add(gridDivRepeatHeight);

                    // Create an infill component...

                    NestedGridInfillArch defaultCellInfill = new NestedGridInfillArch();
                    defaultCellInfill.YDepth = defInfillDepth;
                    defaultCellInfill.Name = "MgdDbgAec Default Infill";

                    compSet.InfillComponents.Add(defaultCellInfill);

                    // Create a boundary edge component

                    NestedGridEdgeDefinition boundaryEdgeDef = new NestedGridEdgeDefinition();
                    boundaryEdgeDef.SetToStandard(m_db);
                    boundaryEdgeDef.EdgeWidth = defBoundaryWidth;
                    boundaryEdgeDef.EdgeDepth = defBoundaryDepth;
                    boundaryEdgeDef.Name = "MgdDbgAec Default Boundary Edge";

                    compSet.BoundaryEdgeComponents.Add(boundaryEdgeDef);

                    // Create an internal edge component

                    NestedGridEdgeDefinition gridLineEdgeDefA = new NestedGridEdgeDefinition();
                    gridLineEdgeDefA.SetToStandard(m_db);
                    gridLineEdgeDefA.EdgeWidth = defInternalWidthA;
                    gridLineEdgeDefA.EdgeDepth = defInternalDepthA;
                    gridLineEdgeDefA.Name = "MgdDbgAec Default Interior Edge";
                    compSet.InteriorEdgeComponents.Add(gridLineEdgeDefA);

                    ////////////////// Set up the grid definition //////////////////

                    NestedGridDefinition mainGridDef = new NestedGridDefinition();
                    mainGridDef.GridDivisionId = gridDivRepeatHeight.Id;
                    mainGridDef.Name = "MgdDbgAec Primary Grid";

                    NestedGridDefinition nestedGridDef = new NestedGridDefinition();
                    nestedGridDef.GridDivisionId = gridDivRepeatWidth.Id;
                    nestedGridDef.Name = "MgdDbgAec Secondary Grid";

                    // Set up the default cell assignment...

                    NestedGridCellAssignment defaultCellAsnA = new NestedGridCellAssignment();
                    defaultCellAsnA.Id = curtainWallUnitStyle.IncrementCellAssignmentId();
                    defaultCellAsnA.NestedGridDefinition = nestedGridDef;
                    defaultCellAsnA.Name = "MgdDbgAec Secondary Grid Assignment";
                    mainGridDef.SpecificCellAssignments.Default = defaultCellAsnA;

                    NestedGridCellAssignment defaultCellAsnB = new NestedGridCellAssignment();
                    defaultCellAsnB.Id = curtainWallUnitStyle.IncrementCellAssignmentId();
                    defaultCellAsnB.InfillDefinitionId = defaultCellInfill.Id;
                    defaultCellAsnB.Name = "MgdDbgAec Default Infill Assignment";
                    nestedGridDef.SpecificCellAssignments.Default = defaultCellAsnB;

                    // Set up the default boundary edge assignments...

                    IndexSpecifierBoundaryEdge boundSpecA = new IndexSpecifierBoundaryEdge();
                    BoundaryFlags flagsA = (BoundaryFlags.Left & BoundaryFlags.Right &
                        BoundaryFlags.Top & BoundaryFlags.Bottom);
                    boundSpecA.BoundaryFlags = flagsA;

                    NestedGridEdgeAssignment boundaryEdgeAsnA = new NestedGridEdgeAssignment();

                    boundaryEdgeAsnA.EdgeSpecifier = boundSpecA;
                    boundaryEdgeAsnA.Id = curtainWallUnitStyle.IncrementEdgeAssignmentId();
                    boundaryEdgeAsnA.EdgeDefinitionId = boundaryEdgeDef.Id;
                    boundaryEdgeAsnA.Name = "MgdDbgAec Default Boundary Edge Assignment";

                    mainGridDef.SpecificBoundaryEdgeAssignments.Default = boundaryEdgeAsnA;

                    IndexSpecifierBoundaryEdge boundSpecB = new IndexSpecifierBoundaryEdge();
                    BoundaryFlags flagsB = 0;
                    boundSpecB.BoundaryFlags = flagsB;

                    NestedGridEdgeAssignment boundaryEdgeAsnB = new NestedGridEdgeAssignment();

                    boundaryEdgeAsnB.EdgeSpecifier = boundSpecB;
                    boundaryEdgeAsnB.Id = curtainWallUnitStyle.IncrementEdgeAssignmentId();
                    boundaryEdgeAsnB.EdgeDefinitionId = boundaryEdgeDef.Id;
                    boundaryEdgeAsnB.Name = "MgdDbgAec Default Boundary Edge Assignment";

                    nestedGridDef.SpecificBoundaryEdgeAssignments.Default = boundaryEdgeAsnB;

                    // Set up the default internal edge assignment...

                    NestedGridEdgeAssignment internalEdgeAsnA = new NestedGridEdgeAssignment();

                    internalEdgeAsnA.Id = curtainWallUnitStyle.IncrementEdgeAssignmentId();
                    internalEdgeAsnA.EdgeDefinitionId = gridLineEdgeDefA.Id;
                    internalEdgeAsnA.Name = "MgdDbgAec Default Internal Edge Assignment";

                    mainGridDef.SpecificInteriorEdgeAssignments.Default = internalEdgeAsnA;

                    NestedGridEdgeAssignment internalEdgeAsnB = new NestedGridEdgeAssignment();

                    internalEdgeAsnB.Id = curtainWallUnitStyle.IncrementEdgeAssignmentId();
                    internalEdgeAsnB.EdgeDefinitionId = gridLineEdgeDefA.Id;
                    internalEdgeAsnB.Name = "MgdDbgAec Default Internal Edge Assignment";

                    nestedGridDef.SpecificInteriorEdgeAssignments.Default = internalEdgeAsnB;

                    curtainWallUnitStyle.GridAssemblyDefinition = mainGridDef;
                    curtainWallUnitStyle.ComponentSet = compSet;

                    // add it to dictionary!
                    dict.AddNewRecord(symName, curtainWallUnitStyle);
                    tr.Transaction.AddNewlyCreatedDBObject(curtainWallUnitStyle, true);

                } //end using
                
                tr.Commit();
                MgdDbg.Utils.AcadUi.PrintToCmdLine(string.Format("\nCreated Curtain Wall Unit Style \"{0}\".", symName));
            }
        }

    }
}
