
// $Header: //Desktop/Source/TestTools/MgdDbgAec/ObjTests/MakeStylesBase.cs#2 $
// $Author: ayyara $ $DateTime: 2007/04/13 17:09:53 $ $Change: 70520 $
//
// (C) Copyright 2004 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//

using System;
using System.Windows.Forms;
using Autodesk.AutoCAD.DatabaseServices;
using Autodesk.AutoCAD.Geometry;
using MgdDbg;
using MgdDbg.Test;
using Autodesk.Aec.DatabaseServices;
using Autodesk.Aec.Geometry;
using Autodesk.Aec.Arch.DatabaseServices;

namespace MgdDbgAec.ObjTests
{
	/// <summary>
	/// Summary description for MakeStyles.
	/// </summary>
    public class MakeStylesBase : MgdDbgTestFuncs
    {
        private Database    m_db = null;
	    
        public
        MakeStylesBase()
        {           		    
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Mass Element Style", "Hardwired Mass Element Style", typeof(MassElementStyle), new MgdDbgTestFuncInfo.TestFunc(MassElementStyle), MgdDbgTestFuncInfo.TestType.Create));
			m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Polygon Style", "Hardwired Polygon Style", typeof(PolygonStyle), new MgdDbgTestFuncInfo.TestFunc(PolygonStyle), MgdDbgTestFuncInfo.TestType.Create));
			//m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Mutli-view Block Def", "Hardwired Multi-view Block Def", typeof(MultiViewBlockDefinition), new MgdDbgTestFuncInfo.TestFunc(MvBlockDef), MgdDbgTestFuncInfo.TestType.Create));
		}
        
        public void
        MassElementStyle()
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            using (TransactionHelper trHlp = new TransactionHelper(m_db)) {
                trHlp.Start();

                string styleName = "Jimbo";
                
                using (DictionaryMassElementStyle dict = new DictionaryMassElementStyle(m_db)) {
                    if (dict.Has(styleName, trHlp.Transaction) == false) {
                        MassElementStyle massStyle = new MassElementStyle();
                        massStyle.SetToStandard(m_db);
                        massStyle.Description = "This style created by MgdDbgAec";
                        dict.AddNewRecord(styleName, massStyle);
                        trHlp.Transaction.AddNewlyCreatedDBObject(massStyle, true);
                        MgdDbg.Utils.AcadUi.PrintToCmdLine(string.Format("\nCreated Mass Element Style \"{0}\".", styleName));
                    }
                    else {
                        MessageBox.Show(string.Format("Style \"{0}\" already exists.", styleName));
                    }
                }
                
                trHlp.Commit();
            }
        }
        
        public void
        PolygonStyle()
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            using (TransactionHelper trHlp = new TransactionHelper(m_db)) {
                trHlp.Start();
                
                string styleName = "Roberto";

                using (DictionaryPolygonStyle dict = new DictionaryPolygonStyle(m_db)) {
                    if (dict.Has(styleName, trHlp.Transaction) == false) {
                        PolygonStyle polyStyle = new PolygonStyle();
                        polyStyle.SetToStandard(m_db);
                        
                        polyStyle.EdgeWidth = 2.0;
                        polyStyle.EdgeWidthJustifyType = PolygonStyleEdgeWidthJustify.Center;
                        
                        dict.AddNewRecord(styleName, polyStyle);
                        trHlp.Transaction.AddNewlyCreatedDBObject(polyStyle, true);
                        MgdDbg.Utils.AcadUi.PrintToCmdLine(string.Format("\nCreated Polygon style \"{0}\".", styleName));
                    }
                    else {
                        MessageBox.Show(string.Format("Style \"{0}\" already exists.", styleName));
                    }
                }
                
                trHlp.Commit();
            }
        }
        
            // TBD: can't finish MvBlockDef until we have a way to look up the DisplayRep IDs
       /* public void
        MvBlockDef()
        {
            using (TransactionHelper trHlp = new TransactionHelper(m_db)) {
                trHlp.Start();
            
                string styleName = "Susanne";

                using (DictionaryMultiViewBlockDefinition dict = new DictionaryMultiViewBlockDefinition(m_db)) {
                    if (dict.Has(styleName, trHlp.Transaction) == false) {
                        MultiViewBlockDefinition mvBlkDef = new MultiViewBlockDefinition();
                        
                        BlockTable blkTbl = (BlockTable)trHlp.Transaction.GetObject(m_db.BlockTableId, OpenMode.ForWrite);
                        
                        MultiViewBlockViewDefinition viewBlkDef = new MultiViewBlockViewDefinition();
                        viewBlkDef.BlockId = HardwireBlockDefTop(blkTbl);
                        viewBlkDef.SetViewOn(ViewDirection.Top);
                        
                        MultiViewBlockDisplayRepresentationDefinition dispRepTop = new MultiViewBlockDisplayRepresentationDefinition();
                        dispRepTop.AddViewBlock(viewBlkDef);
                        dispRepTop.DisplayRep(
                        mvBlkDef.AddDispRepDef(dispRepTop);
                                                
                        dict.AddNewRecord(styleName, mvBlkDef, trHlp.Transaction);
                    }
                    else {
                        MessageBox.Show(string.Format("Style \"{0}\" already exists.", styleName));
                    }
                }
                
                trHlp.Commit();
            }
        }
        
        private ObjectId
        HardwireBlockDefTop(BlockTable blkTbl)
        {
            string blkName = "MgdDbgAec_Test_Top";
            if (blkTbl.Has(blkName)) {
                return blkTbl[blkName];
            }

            using (CompBldrNamedBlkDef compBldr = new CompBldrNamedBlkDef(blkTbl.Database, blkName)) {
                compBldr.Start();
                
                Point2d pt1 = new Point2d(-0.5, -0.5);
                Point2d pt2 = new Point2d(0.5, -0.5);
                Point2d pt3 = new Point2d(0.5, 0.5);
                Point2d pt4 = new Point2d(-0.5, 0.5);

                Polyline pline = new Polyline();
                pline.AddVertexAt(0, pt1);
                pline.AddVertexAt(1, pt2);
                pline.AddVertexAt(2, pt3);
                pline.AddVertexAt(3, pt4);
                pline.Closed = true;
                pline.ColorIndex = 1;
                compBldr.AddToDb(pline);

                pt1.Set(-0.375, -0.375);
                pt2.Set(0.375, -0.375);
                pt3.Set(0.375, 0.375);
                pt4.Set(-0.375, 0.375);

                pline = new Polyline();
                pline.AddVertexAt(0, pt1);
                pline.AddVertexAt(1, pt2);
                pline.AddVertexAt(2, pt3);
                pline.AddVertexAt(3, pt4);
                pline.Closed = true;
                pline.ColorIndex = 4;
                compBldr.AddToDb(pline);
                
                compBldr.Commit();
                
                return compBldr.BlockDefId;
            }
        } */
        
    }
}
