
// $Header: //Desktop/Source/TestTools/MgdDbgAec/ObjTests/ModifyBase.cs#2 $
// $Author: ayyara $ $DateTime: 2008/02/28 10:13:37 $ $Change: 94226 $
//
// (C) Copyright 2004 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//

using System;
using System.Diagnostics;
using System.Collections;
using System.Windows.Forms;
using Autodesk.AutoCAD.DatabaseServices;
using Autodesk.AutoCAD.Geometry;
using Autodesk.AutoCAD.Colors;
using Autodesk.AutoCAD.ApplicationServices;
using Autodesk.AutoCAD.EditorInput;
using Autodesk.AutoCAD.Runtime;
using MgdDbg;
using MgdDbg.Test;

using AcDb = Autodesk.AutoCAD.DatabaseServices;
using AcGe = Autodesk.AutoCAD.Geometry;
using AecDb = Autodesk.Aec.DatabaseServices;
using AecGe = Autodesk.Aec.Geometry;
using AecDbArch = Autodesk.Aec.Arch.DatabaseServices;

namespace MgdDbgAec.ObjTests
{
	/// <summary>
	/// Summary description for ModifyEntTests.
	/// </summary>
	public class ModifyBase : MgdDbgTestFuncs
	{
        private Database    m_db = null;

		public
		ModifyBase()
		{		    
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Set Location", "Change location point", typeof(AecDb.Geo), new MgdDbgTestFuncInfo.TestFunc(SetLocation), MgdDbgTestFuncInfo.TestType.Modify));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Set Direction", "Change X direction", typeof(AecDb.Geo), new MgdDbgTestFuncInfo.TestFunc(SetDirection), MgdDbgTestFuncInfo.TestType.Modify));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Set Rotation", "Change rotation", typeof(AecDb.Geo), new MgdDbgTestFuncInfo.TestFunc(SetRotation), MgdDbgTestFuncInfo.TestType.Modify));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Set Normal", "Change normal direction", typeof(AecDb.Geo), new MgdDbgTestFuncInfo.TestFunc(SetNormal), MgdDbgTestFuncInfo.TestType.Modify));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Set ECS", "Set the ECS based on another entity", typeof(AecDb.Geo), new MgdDbgTestFuncInfo.TestFunc(SetEcs), MgdDbgTestFuncInfo.TestType.Modify));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Translate By", "Move by vector", typeof(AecDb.Geo), new MgdDbgTestFuncInfo.TestFunc(TranslateBy), MgdDbgTestFuncInfo.TestType.Modify));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Move in XY", "Move by vector in XY plane", typeof(AecDb.Geo), new MgdDbgTestFuncInfo.TestFunc(MoveInXY), MgdDbgTestFuncInfo.TestType.Modify));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Move in Z", "Move by distance in Z", typeof(AecDb.Geo), new MgdDbgTestFuncInfo.TestFunc(MoveInZ), MgdDbgTestFuncInfo.TestType.Modify));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Release Anchor", "Relase the anchor (if any)", typeof(AecDb.Geo), new MgdDbgTestFuncInfo.TestFunc(ReleaseAnchor), MgdDbgTestFuncInfo.TestType.Modify));

            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Attach", "Attach a node", typeof(AecDb.MassGroup), new MgdDbgTestFuncInfo.TestFunc(MassGroupAttach), MgdDbgTestFuncInfo.TestType.Modify));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Remove All", "Remove all nodes", typeof(AecDb.MassGroup), new MgdDbgTestFuncInfo.TestFunc(MassGroupRemoveAll), MgdDbgTestFuncInfo.TestType.Modify));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Remove Odd", "Remove odd numbered nodes", typeof(AecDb.MassGroup), new MgdDbgTestFuncInfo.TestFunc(MassGroupRemoveOdd), MgdDbgTestFuncInfo.TestType.Modify));

            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Set description", "Change the description of a selected entity", typeof(AecDb.Entity), new MgdDbgTestFuncInfo.TestFunc(SetDescription), MgdDbgTestFuncInfo.TestType.Modify));

            // LayoutCurve tests
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Move Layout Node", "Moves one of the layout nodes on a layout curve", typeof(AecDb.LayoutCurve), new MgdDbgTestFuncInfo.TestFunc(MoveLayoutNode), MgdDbgTestFuncInfo.TestType.Modify));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Remove Layout Node", "Removes the layout nodes on a layout curve", typeof(AecDb.LayoutCurve), new MgdDbgTestFuncInfo.TestFunc(RemoveLayoutNode), MgdDbgTestFuncInfo.TestType.Modify));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Remove Layout Node At", "Removes the first and last layout nodes on a layout curve", typeof(AecDb.LayoutCurve), new MgdDbgTestFuncInfo.TestFunc(RemoveLayoutNodeAt), MgdDbgTestFuncInfo.TestType.Modify));
            
            // LayoutGrid2d tests
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Set boundary", "Set a boundary mask for the layout grid", typeof(AecDb.LayoutGrid2d), new MgdDbgTestFuncInfo.TestFunc(SetBoundary), MgdDbgTestFuncInfo.TestType.Modify));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Add holes", "Add holes to the layout grid", typeof(AecDb.LayoutGrid2d), new MgdDbgTestFuncInfo.TestFunc(AddHoles), MgdDbgTestFuncInfo.TestType.Modify));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Remove holes", "Removes all holes from the layout grid", typeof(AecDb.LayoutGrid2d), new MgdDbgTestFuncInfo.TestFunc(RemoveHoles), MgdDbgTestFuncInfo.TestType.Modify));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Remove last grid column", "Removes the last grid column from the layout grid", typeof(AecDb.LayoutGrid2d), new MgdDbgTestFuncInfo.TestFunc(RemoveLastGridColumn), MgdDbgTestFuncInfo.TestType.Modify));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Remove last grid row", "Removes the last grid row from the layout grid", typeof(AecDb.LayoutGrid2d), new MgdDbgTestFuncInfo.TestFunc(RemoveLastGridRow), MgdDbgTestFuncInfo.TestType.Modify));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Move last grid column or row", "Moves the last grid column or row on a layout grid by .5 of the distance between it and the last segment.", typeof(AecDb.LayoutGrid2d), new MgdDbgTestFuncInfo.TestFunc(MoveColumnOrRow), MgdDbgTestFuncInfo.TestType.Modify));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Add Node", "Adds a node to the layout grid, on the last column/row, thus creating a new column/row.", typeof(AecDb.LayoutGrid2d), new MgdDbgTestFuncInfo.TestFunc(AddNode), MgdDbgTestFuncInfo.TestType.Modify));

            // LayoutGrid3d tests
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Add Node", "Adds a node to the layout grid 3d, on the last column/row/level, thus creating a new column/row/level.", typeof(AecDb.LayoutGrid3d), new MgdDbgTestFuncInfo.TestFunc(AddNode3d), MgdDbgTestFuncInfo.TestType.Modify));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Append Grid Level", "Adds a level to the layout grid 3d", typeof(AecDb.LayoutGrid3d), new MgdDbgTestFuncInfo.TestFunc(AppendGridLevel), MgdDbgTestFuncInfo.TestType.Modify));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Move Grid Level", "Moves the last level upwards by half the last level height.", typeof(AecDb.LayoutGrid3d), new MgdDbgTestFuncInfo.TestFunc(MoveGridLevel), MgdDbgTestFuncInfo.TestType.Modify));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Remove Last Grid Level", "Removes the last grid level of the layout grid 3d", typeof(AecDb.LayoutGrid3d), new MgdDbgTestFuncInfo.TestFunc(RemoveLastGridLevel), MgdDbgTestFuncInfo.TestType.Modify));

            // GridAssembly tests
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Add interference objects", "Adds interfering objects to a Grid Assembly", typeof(AecDb.GridAssembly), new MgdDbgTestFuncInfo.TestFunc(AddInterferences), MgdDbgTestFuncInfo.TestType.Modify));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Remove all interferences", "Removes interferences from a Grid Assembly", typeof(AecDb.GridAssembly), new MgdDbgTestFuncInfo.TestFunc(RemoveAllInterferences), MgdDbgTestFuncInfo.TestType.Modify));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Remove interferences", "Removes selected interferences from a Grid Assembly", typeof(AecDb.GridAssembly), new MgdDbgTestFuncInfo.TestFunc(RemoveInterferences), MgdDbgTestFuncInfo.TestType.Modify));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Set rectangle clipping boundary", "Sets a rectangle clipping boundary on a Grid Assembly", typeof(AecDb.GridAssembly), new MgdDbgTestFuncInfo.TestFunc(RectangleClippingBoundary), MgdDbgTestFuncInfo.TestType.Modify));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Remove clipping boundary", "Removes the clipping boundary from a Grid Assembly", typeof(AecDb.GridAssembly), new MgdDbgTestFuncInfo.TestFunc(RemoveClippingBoundary), MgdDbgTestFuncInfo.TestType.Modify));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Set miter angles", "Sets the start and end miter angles to 30 degrees", typeof(AecDb.GridAssembly), new MgdDbgTestFuncInfo.TestFunc(SetMiterAngles), MgdDbgTestFuncInfo.TestType.Modify));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Edit depth", "Sets the edit depth of the GridAssembly", typeof(AecDb.GridAssembly), new MgdDbgTestFuncInfo.TestFunc(SetEditDepth), MgdDbgTestFuncInfo.TestType.Modify));
                // GridAssemblyComponentSet
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Remove First Component from a component set of a GridAssembly", "Removes the first Component from a component set of a GridAssembly", typeof(AecDb.GridAssemblyComponentSet), new MgdDbgTestFuncInfo.TestFunc(RemoveFirstComponent), MgdDbgTestFuncInfo.TestType.Modify));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Replace First Division Component of a GridAssembly", "Replaces the first division component of a GridAssembly with one that has double its values", typeof(AecDb.GridAssemblyComponentSet), new MgdDbgTestFuncInfo.TestFunc(ReplaceFirstDivisionComponentTest), MgdDbgTestFuncInfo.TestType.Modify));

            // GridCustom tests
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Inserts a segment on a custom grid", "Inserts a segment on a custom grid", typeof(AecDb.GridCustom), new MgdDbgTestFuncInfo.TestFunc(InsertSegmentAt), MgdDbgTestFuncInfo.TestType.Modify));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Removes the last segment a custom grid", "Removes the last segment of a custom grid", typeof(AecDb.GridCustom), new MgdDbgTestFuncInfo.TestFunc(RemoveLastSegment), MgdDbgTestFuncInfo.TestType.Modify));

            // GridAssemblyDefinition tests
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Remove first specific Boundary edge/Internal edge/ or cell assignments", "Removes the first specific Boundary edge/Internal edge/ or cell assignments of a layout grid", typeof(AecDb.GridAssemblyDefinition), new MgdDbgTestFuncInfo.TestFunc(RemoveSpecificTest), MgdDbgTestFuncInfo.TestType.Modify));

        }
        
#region Ent Tests

        public void
        SetDescription()
        {
            m_db      = MgdDbg.Utils.Db.GetCurDwg();

            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;
            Autodesk.AutoCAD.DatabaseServices.TransactionManager tm = m_db.TransactionManager;

            PromptEntityOptions prEntOpts = new PromptEntityOptions("\nSelect AEC entity");
            prEntOpts.SetRejectMessage("\nSelected entity must be of type AEC entity.");
            prEntOpts.AddAllowedClass(typeof(AecDb.Entity), false);
            
            PromptStringOptions prDescOpts = new PromptStringOptions("\nEnter new description");
            prDescOpts.AllowSpaces = true;

            while (true) {
                PromptEntityResult prEntRes = ed.GetEntity(prEntOpts);
                if (prEntRes.Status != PromptStatus.OK)
                    return;
                    
                PromptResult prDescRes = ed.GetString(prDescOpts);
                if (prDescRes.Status != PromptStatus.OK)
                    return;

                using (Autodesk.AutoCAD.DatabaseServices.Transaction tr = tm.StartTransaction()) {
                    AecDb.Entity ent = (AecDb.Entity)tr.GetObject(prEntRes.ObjectId, OpenMode.ForWrite);
                    
                    ent.Description = prDescRes.StringResult;
                    tr.Commit();
                }
            }
        }

#endregion

#region Geo Tests

        public void
        SetLocation()
        {
            m_db      = MgdDbg.Utils.Db.GetCurDwg();

            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;
            Autodesk.AutoCAD.DatabaseServices.TransactionManager tm = m_db.TransactionManager;

            PromptEntityOptions prEntOpts = new PromptEntityOptions("\nSelect entity");
            prEntOpts.SetRejectMessage("\nSelected entity must be of type Geo.");
            prEntOpts.AddAllowedClass(typeof(AecDb.Geo), false);
            
            PromptPointOptions prInsPtOpts = new PromptPointOptions("\nNew location point");
            
            while (true) {
                PromptEntityResult prEntRes = ed.GetEntity(prEntOpts);
                if (prEntRes.Status != PromptStatus.OK)
                    return;
                    
                PromptPointResult prInsPtRes = ed.GetPoint(prInsPtOpts);
                if (prInsPtRes.Status != PromptStatus.OK)
                    return;

                using (Autodesk.AutoCAD.DatabaseServices.Transaction tr = tm.StartTransaction()) {
                    AecDb.Geo ent = (AecDb.Geo)tr.GetObject(prEntRes.ObjectId, OpenMode.ForWrite);
                    
                    ent.Location = MgdDbg.Utils.Db.UcsToWcs(prInsPtRes.Value);
                    tr.Commit();
                }
            }
        }
        
        public void
        SetDirection()
        {
            DoSetVectorGuts(0);
        }
        
        public void
        SetNormal()
        {
            DoSetVectorGuts(1);
        }
        
        public void
        TranslateBy()
        {
            DoSetVectorGuts(2);
        }
        
        public void
        MoveInXY()
        {
            DoSetVectorGuts(3);
        }
        
        private void
        DoSetVectorGuts(int funcId)
        {
            m_db      = MgdDbg.Utils.Db.GetCurDwg();

            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;

            PromptEntityOptions prEntOpts = new PromptEntityOptions("\nSelect entity");
            prEntOpts.SetRejectMessage("\nSelected entity must be of type Geo.");
            prEntOpts.AddAllowedClass(typeof(AecDb.Geo), false);
            
            PromptPointOptions prPt1Opts = new PromptPointOptions("\nFirst point on vector");
            PromptPointOptions prPt2Opts = new PromptPointOptions("\nSecond point on vector");
            
            Autodesk.AutoCAD.DatabaseServices.TransactionManager tm = m_db.TransactionManager;
            
            while (true) {
                PromptEntityResult prEntRes = ed.GetEntity(prEntOpts);
                if (prEntRes.Status != PromptStatus.OK)
                    return;
                
                PromptPointResult prPt1Res = ed.GetPoint(prPt1Opts);
                if (prPt1Res.Status != PromptStatus.OK)
                    return;
                
                prPt2Opts.BasePoint = prPt1Res.Value;
                prPt2Opts.UseBasePoint = true;
                PromptPointResult prPt2Res = ed.GetPoint(prPt2Opts);
                if (prPt2Res.Status != PromptStatus.OK)
                    return;

                using (Autodesk.AutoCAD.DatabaseServices.Transaction tr = tm.StartTransaction()) {
                    AecDb.Geo ent = (AecDb.Geo)tr.GetObject(prEntRes.ObjectId, OpenMode.ForWrite);
                    
                    Vector3d vDir = MgdDbg.Utils.Db.UcsToWcs(prPt2Res.Value - prPt1Res.Value);
                    if (funcId == 0) {
                        ent.Direction = vDir.GetNormal();
                    }
                    else if (funcId == 1) {
                        ent.Normal = vDir.GetNormal();
                    }
                    else if (funcId == 2) {
                        ent.TranslateBy(vDir);
                    }
                    else if (funcId == 3) {
                        ent.MoveInXY(vDir);
                    }
                    
                    tr.Commit();
                }
            }
        }
        
        public void
        SetRotation()
        {
            m_db      = MgdDbg.Utils.Db.GetCurDwg();

            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;

            PromptEntityOptions prEntOpts = new PromptEntityOptions("\nSelect entity");
            prEntOpts.SetRejectMessage("\nSelected entity must be of type Geo.");
            prEntOpts.AddAllowedClass(typeof(AecDb.Geo), false);

            PromptAngleOptions prRotOpts = new PromptAngleOptions("\nRotation angle");
            
            Autodesk.AutoCAD.DatabaseServices.TransactionManager tm = m_db.TransactionManager;
            
            while (true) {
                PromptEntityResult prEntRes = ed.GetEntity(prEntOpts);
                if (prEntRes.Status != PromptStatus.OK)
                    return;
                    
                PromptDoubleResult prRotRes = ed.GetAngle(prRotOpts);
                if (prRotRes.Status != PromptStatus.OK)
                    return;
                
                using (Autodesk.AutoCAD.DatabaseServices.Transaction tr = tm.StartTransaction()) {
                    AecDb.Geo ent = (AecDb.Geo)tr.GetObject(prEntRes.ObjectId, OpenMode.ForWrite);
                    
                    ent.Rotation = prRotRes.Value;
                    
                    tr.Commit();
                }
            }
        }
        
        public void
        SetEcs()
        {
            m_db      = MgdDbg.Utils.Db.GetCurDwg();

            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;
            Autodesk.AutoCAD.DatabaseServices.TransactionManager tm = m_db.TransactionManager;

            PromptEntityOptions prEnt1Opts = new PromptEntityOptions("\nSelect entity");
            prEnt1Opts.SetRejectMessage("\nSelected entity must be of type Geo.");
            prEnt1Opts.AddAllowedClass(typeof(AecDb.Geo), false);
 
            PromptEntityOptions prEnt2Opts = new PromptEntityOptions("\nSelect entity to match ECS");
            
            while (true) {
                PromptEntityResult prEnt1Res = ed.GetEntity(prEnt1Opts);
                if (prEnt1Res.Status != PromptStatus.OK)
                    return;
                    
                PromptEntityResult prEnt2Res = ed.GetEntity(prEnt2Opts);
                if (prEnt2Res.Status != PromptStatus.OK)
                    return;
                
                using (Autodesk.AutoCAD.DatabaseServices.Transaction tr = tm.StartTransaction()) {
                    AecDb.Geo ent = (AecDb.Geo)tr.GetObject(prEnt1Res.ObjectId, OpenMode.ForWrite);
                    AcDb.Entity ecsEnt = (AcDb.Entity)tr.GetObject(prEnt2Res.ObjectId, OpenMode.ForRead);
                    Matrix3d mat = ecsEnt.Ecs;

                    ent.GeoEcs = mat;
                    
                    tr.Commit();
                }
            }
        }
        
        public void
        MoveInZ()
        {
            m_db      = MgdDbg.Utils.Db.GetCurDwg();

            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;
            Autodesk.AutoCAD.DatabaseServices.TransactionManager tm = m_db.TransactionManager;

            PromptEntityOptions prEntOpts = new PromptEntityOptions("\nSelect entity");
            prEntOpts.SetRejectMessage("\nSelected entity must be of type Geo.");
            prEntOpts.AddAllowedClass(typeof(AecDb.Geo), false);
             
            PromptDistanceOptions prDistOpts = new PromptDistanceOptions("\nDistance in Z dir");
            
            while (true) {
                PromptEntityResult prEntRes = ed.GetEntity(prEntOpts);
                if (prEntRes.Status != PromptStatus.OK)
                    return;
                    
                PromptDoubleResult prDistRes = ed.GetDistance(prDistOpts);
                if (prDistRes.Status != PromptStatus.OK)
                    return;
                                
                using (Autodesk.AutoCAD.DatabaseServices.Transaction tr = tm.StartTransaction()) {
                    AecDb.Geo ent = (AecDb.Geo)tr.GetObject(prEntRes.ObjectId, OpenMode.ForWrite);
                    
                    ent.MoveInZ(prDistRes.Value);
                    
                    tr.Commit();
                }
            }
        }
        
        public void
        ReleaseAnchor()
        {
            m_db      = MgdDbg.Utils.Db.GetCurDwg();

            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;
            Autodesk.AutoCAD.DatabaseServices.TransactionManager tm = m_db.TransactionManager;

            PromptEntityOptions prEntOpts = new PromptEntityOptions("\nSelect entity");
            prEntOpts.SetRejectMessage("\nSelected entity must be of type Geo.");
            prEntOpts.AddAllowedClass(typeof(AecDb.Geo), false);
            
            while (true) {
                PromptEntityResult prEntRes = ed.GetEntity(prEntOpts);
                if (prEntRes.Status != PromptStatus.OK)
                    return;
                    
                using (Autodesk.AutoCAD.DatabaseServices.Transaction tr = tm.StartTransaction()) {
                    AecDb.Geo ent = (AecDb.Geo)tr.GetObject(prEntRes.ObjectId, OpenMode.ForWrite);
                    
                    if (ent.AnchorId.IsNull)
                        MgdDbg.Utils.AcadUi.PrintToCmdLine("\nThis object is not currently anchored.");
                    else {
                        ent.ReleaseAnchor();
                        MgdDbg.Utils.AcadUi.PrintToCmdLine("\nAnchor has been released.");
                    }
                    
                    tr.Commit();
                }
            }
        }

#endregion Geo Tests

#region MassGroup Tests

        public void
        MassGroupAttach()
        {
            m_db      = MgdDbg.Utils.Db.GetCurDwg();

            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;

            PromptEntityOptions prMassGroupOpts = new PromptEntityOptions("\nSelect a Mass Group");
            prMassGroupOpts.SetRejectMessage("\nSelected entity must be of type MassGroup.");
            prMassGroupOpts.AddAllowedClass(typeof(AecDb.MassGroup), false);
            
            PromptEntityOptions prAttachOpts = new PromptEntityOptions("\nSelect an element to attach");
            prAttachOpts.SetRejectMessage("\nSelected entity must be of type MassGroup or MassElement.");
            prAttachOpts.AddAllowedClass(typeof(AecDb.MassGroup), false);
            prAttachOpts.AddAllowedClass(typeof(AecDb.MassElement), false);
            
            PromptEntityResult prMassGroupRes = ed.GetEntity(prMassGroupOpts);
            if (prMassGroupRes.Status != PromptStatus.OK)
                return;
                
            PromptEntityResult prAttachRes = ed.GetEntity(prAttachOpts);
            if (prAttachRes.Status != PromptStatus.OK)
                return;
                            
            using (TransactionHelper trHlp = new TransactionHelper(m_db)) {
                trHlp.Start();
                
                AecDb.MassGroup massGroup = (AecDb.MassGroup)trHlp.Transaction.GetObject(prMassGroupRes.ObjectId, OpenMode.ForWrite);
                AcDb.Entity attachEnt = (AcDb.Entity)trHlp.Transaction.GetObject(prAttachRes.ObjectId, OpenMode.ForRead);
                
                    // TBD: seems to work, but asserts in AecImpGeo
                try {
                    massGroup.AddNode(attachEnt, Autodesk.Aec.DatabaseServices.Operation.Additive);
                }
                catch (Autodesk.AutoCAD.Runtime.Exception e) {
                    if (e.ErrorStatus == ErrorStatus.SelfReference)
                        MgdDbg.Utils.AcadUi.PrintToCmdLine("\nCannot attach an entity to itself!");
                    else if (e.ErrorStatus == ErrorStatus.RepeatEntity)
                        MgdDbg.Utils.AcadUi.PrintToCmdLine("\nThis entity is already attached!");
                    else
                        throw e;
                }
                
                trHlp.Commit();
            }
        }
            
        public void
        MassGroupRemoveAll()
        {
            m_db      = MgdDbg.Utils.Db.GetCurDwg();

            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;

            PromptEntityOptions prMassGroupOpts = new PromptEntityOptions("\nSelect a Mass Group");
            prMassGroupOpts.SetRejectMessage("\nSelected entity must be of type MassGroup.");
            prMassGroupOpts.AddAllowedClass(typeof(AecDb.MassGroup), false);
            
            PromptEntityResult prMassGroupRes = ed.GetEntity(prMassGroupOpts);
            if (prMassGroupRes.Status != PromptStatus.OK)
                return;
                                            
            using (TransactionHelper trHlp = new TransactionHelper(m_db)) {
                trHlp.Start();
                
                AecDb.MassGroup massGroup = (AecDb.MassGroup)trHlp.Transaction.GetObject(prMassGroupRes.ObjectId, OpenMode.ForWrite);
                massGroup.RemoveAllNodes();
                
                trHlp.Commit();
            }
        }
        
        public void
        MassGroupRemoveOdd()
        {
            m_db      = MgdDbg.Utils.Db.GetCurDwg();

            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;

            PromptEntityOptions prMassGroupOpts = new PromptEntityOptions("\nSelect a Mass Group");
            prMassGroupOpts.SetRejectMessage("\nSelected entity must be of type MassGroup.");
            prMassGroupOpts.AddAllowedClass(typeof(AecDb.MassGroup), false);
            
            PromptEntityResult prMassGroupRes = ed.GetEntity(prMassGroupOpts);
            if (prMassGroupRes.Status != PromptStatus.OK)
                return;
                                            
            using (TransactionHelper trHlp = new TransactionHelper(m_db)) {
                trHlp.Start();
                
                //AecDb.MassGroup massGroup = (AecDb.MassGroup)trHlp.Transaction.GetObject(prMassGroupRes.ObjectId, OpenMode.ForWrite);
                //ObjectIdCollection nodeIds = new ObjectIdCollection();;
                ////for (int i=1; i<massGroup.NodeCount; i+=2) {
                //    nodeIds.Add(massGroup.GetNodeAt(i));
                //}
                
                    // TBD: is a bit bizzarre that all the other signatures except AddNode and RemoveNode
                    // deal with ObjectIds and not an Open entity.  Might need to change?
                //if (nodeIds.Count == 0)
                //    MgdDbg.Utils.AcadUi.PrintToCmdLine("\nNo odd numbered nodes to remove.");
                //else {
                //    foreach (ObjectId nodeId in nodeIds) {
                //        AcDb.Entity ent = (AcDb.Entity)trHlp.Transaction.GetObject(nodeId, OpenMode.ForWrite);
                //        massGroup.RemoveNode(ent);
                //    }
               // }
                
                trHlp.Commit();
            }
        }
        
#endregion  MassGroup tests

#region Layout Curve tests
        public void
        MoveLayoutNode()
        {
            m_db      = MgdDbg.Utils.Db.GetCurDwg();
            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;

            PromptEntityOptions prLayoutCurveOpts = new PromptEntityOptions("\nSelect a Layout Curve");
            prLayoutCurveOpts.SetRejectMessage("\nSelected entity must be of type LayoutCurve.");
            prLayoutCurveOpts.AddAllowedClass(typeof(AecDb.LayoutCurve), false);
            
            PromptEntityResult prLayoutCurveRes = ed.GetEntity(prLayoutCurveOpts);
            if (prLayoutCurveRes.Status != PromptStatus.OK)
                return;
                                            
            using (TransactionHelper trHlp = new TransactionHelper(m_db)) 
            {
                trHlp.Start();
                
                AecDb.LayoutCurve LayoutCurve = (AecDb.LayoutCurve)trHlp.Transaction.GetObject(prLayoutCurveRes.ObjectId, OpenMode.ForWrite);
                AecDb.LayoutCurveNode tempNode = null;
                if (LayoutCurve.NodeCount == 0)
                    MgdDbg.Utils.AcadUi.PrintToCmdLine("\nNo nodes to move.");
                else {
                    double dOldPos;
                    double dNewPos;
                    for (int i=0; i<LayoutCurve.NodeCount; i++) 
                    {
                        tempNode = LayoutCurve.GetLayoutCurveNodeAt(i);
                        dOldPos = tempNode.UPosition;
                        dNewPos = dOldPos + 10.0;
                        LayoutCurve.MoveLayoutNode(dOldPos, dNewPos);
                    }
                }
                
                trHlp.Commit();
            }
        }


        public void
        RemoveLayoutNode()
        {
            m_db      = MgdDbg.Utils.Db.GetCurDwg();
            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;

            PromptEntityOptions prLayoutCurveOpts = new PromptEntityOptions("\nSelect a Layout Curve to have nodes removed");
            prLayoutCurveOpts.SetRejectMessage("\nSelected entity must be of type LayoutCurve.");
            prLayoutCurveOpts.AddAllowedClass(typeof(AecDb.LayoutCurve), false);
            
            PromptEntityResult prLayoutCurveRes = ed.GetEntity(prLayoutCurveOpts);
            if (prLayoutCurveRes.Status != PromptStatus.OK)
                return;
                                            
            using (TransactionHelper trHlp = new TransactionHelper(m_db)) 
            {
                trHlp.Start();
                
                AecDb.LayoutCurve LayoutCurve = (AecDb.LayoutCurve)trHlp.Transaction.GetObject(prLayoutCurveRes.ObjectId, OpenMode.ForWrite);
                AecDb.LayoutCurveNode tempNode = null;
                if (LayoutCurve.NodeCount == 0) 
                {
                    MgdDbg.Utils.AcadUi.PrintToCmdLine("\nNo nodes to remove.");
                    trHlp.Abort();
                    return;
                }
                else 
                {
                    while ( LayoutCurve.NodeCount > 0) 
                    {
                        MgdDbg.Utils.AcadUi.PrintToCmdLine("\nNode Count: "+ LayoutCurve.NodeCount.ToString() + "\n" );
                        // Once you remove a node, the indexes/ids are rearranged internally.
                        // So if you get the node at the 0 index and remove it repeatedly, you
                        // can remove all of them!
                        tempNode = LayoutCurve.GetLayoutCurveNodeAt(0);
                        MgdDbg.Utils.AcadUi.PrintToCmdLine("Node id: "+ tempNode.NodeId.ToString() + "\n" );
                        LayoutCurve.RemoveLayoutNode(tempNode.NodeId);
                    }
                    LayoutCurve.UpdateParameters();
                }
                
                trHlp.Commit();
            }
        }


        public void
        RemoveLayoutNodeAt()
        {
            m_db      = MgdDbg.Utils.Db.GetCurDwg();
            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;

            PromptEntityOptions prLayoutCurveOpts = new PromptEntityOptions("\nSelect a Layout Curve to have nodes removed");
            prLayoutCurveOpts.SetRejectMessage("\nSelected entity must be of type LayoutCurve.");
            prLayoutCurveOpts.AddAllowedClass(typeof(AecDb.LayoutCurve), false);
            
            PromptEntityResult prLayoutCurveRes = ed.GetEntity(prLayoutCurveOpts);
            if (prLayoutCurveRes.Status != PromptStatus.OK)
                return;
                                            
            using (TransactionHelper trHlp = new TransactionHelper(m_db)) 
            {
                trHlp.Start();
                
                AecDb.LayoutCurve LayoutCurve = (AecDb.LayoutCurve)trHlp.Transaction.GetObject(prLayoutCurveRes.ObjectId, OpenMode.ForWrite);
                if (LayoutCurve.NodeCount == 0) 
                {
                    MgdDbg.Utils.AcadUi.PrintToCmdLine("\nNo nodes to remove.");
                    trHlp.Abort();
                    return;
                }
                else 
                {
                    // check if layout curve has at least 2 nodes, since we will remove
                    // the first and last nodes
                    int nNodes = LayoutCurve.NodeCount;
                    if (nNodes > 1) 
                    {
                        MgdDbg.Utils.AcadUi.PrintToCmdLine("\nRemoving first and last nodes.");
                        LayoutCurve.RemoveLayoutNodeAt(nNodes - 1);
                        LayoutCurve.RemoveLayoutNodeAt(0);
                    }
                    else {
                        MgdDbg.Utils.AcadUi.PrintToCmdLine("\nNot enough nodes to remove (need at least 2 for this test).");
                    }

                    LayoutCurve.UpdateParameters();
                }
                
                trHlp.Commit();
            }
        }

#endregion

#region LayoutGrid2d tests

        public void
        SetBoundary()
        {
            m_db      = MgdDbg.Utils.Db.GetCurDwg();
            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;

            // select grid
            PromptEntityOptions prLayoutGridOpts = new PromptEntityOptions("\nSelect a LayoutGrid2d");
            prLayoutGridOpts.SetRejectMessage("\nSelected entity must be of type LayoutGrid2d.");
            prLayoutGridOpts.AddAllowedClass(typeof(AecDb.LayoutGrid2d), false);
            
            PromptEntityResult prLayoutGridRes = ed.GetEntity(prLayoutGridOpts);
            if (prLayoutGridRes.Status != PromptStatus.OK)
                return;
                 
            // select boundary               
            PromptEntityOptions prPolyOpts = new PromptEntityOptions("\nSelect a polyline");
            prPolyOpts.SetRejectMessage("\nSelected entity must be of type polyline.");
            prPolyOpts.AddAllowedClass(typeof(AcDb.Polyline), false);
            
            PromptEntityResult prPolyRes = ed.GetEntity(prPolyOpts);
            if (prPolyRes.Status != PromptStatus.OK)
                return;

            using (TransactionHelper trHlp = new TransactionHelper(m_db)) 
            {
                trHlp.Start();
                
                AecDb.LayoutGrid2d layoutGrid = (AecDb.LayoutGrid2d)trHlp.Transaction.GetObject(prLayoutGridRes.ObjectId, OpenMode.ForWrite);
                AcDb.Polyline pline = (AcDb.Polyline)trHlp.Transaction.GetObject(prPolyRes.ObjectId, OpenMode.ForRead);

                layoutGrid.BoundaryId = pline.ObjectId;

                trHlp.Commit();
            }
        }

        public void
        AddHoles()
        {
            m_db      = MgdDbg.Utils.Db.GetCurDwg();

            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;

            // select grid
            PromptEntityOptions prLayoutGridOpts = new PromptEntityOptions("\nSelect a LayoutGrid2d");
            prLayoutGridOpts.SetRejectMessage("\nSelected entity must be of type LayoutGrid2d.");
            prLayoutGridOpts.AddAllowedClass(typeof(AecDb.LayoutGrid2d), false);
            
            PromptEntityResult prLayoutGridRes = ed.GetEntity(prLayoutGridOpts);
            if (prLayoutGridRes.Status != PromptStatus.OK)
                return;
                 
            // select holes               
            PromptSelectionOptions prPolyOpts = new PromptSelectionOptions();
            prPolyOpts.AllowDuplicates = false;
            
            PromptSelectionResult prPolyRes = ed.GetSelection(prPolyOpts);
            if (prPolyRes.Status != PromptStatus.OK)
                return;

            using (TransactionHelper trHlp = new TransactionHelper(m_db)) 
            {
                trHlp.Start();
                
                AecDb.LayoutGrid2d layoutGrid = (AecDb.LayoutGrid2d)trHlp.Transaction.GetObject(prLayoutGridRes.ObjectId, OpenMode.ForWrite);
                SelectionSet ss = prPolyRes.Value;
                foreach(SelectedObject obj in ss ) {
                    // TBD: check the object's type to make sure its something that could be used as a hole!
                    layoutGrid.AppendBoundaryHole(obj.ObjectId);
                }

                trHlp.Commit();
            }
        }

        public void
        RemoveHoles()
        {
            m_db      = MgdDbg.Utils.Db.GetCurDwg();
            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;

            // select grid
            PromptEntityOptions prLayoutGridOpts = new PromptEntityOptions("\nSelect a LayoutGrid2d");
            prLayoutGridOpts.SetRejectMessage("\nSelected entity must be of type LayoutGrid2d.");
            prLayoutGridOpts.AddAllowedClass(typeof(AecDb.LayoutGrid2d), false);
            
            PromptEntityResult prLayoutGridRes = ed.GetEntity(prLayoutGridOpts);
            if (prLayoutGridRes.Status != PromptStatus.OK)
                return;
                 
            using (TransactionHelper trHlp = new TransactionHelper(m_db)) 
            {
                trHlp.Start();
                
                AecDb.LayoutGrid2d layoutGrid = (AecDb.LayoutGrid2d)trHlp.Transaction.GetObject(prLayoutGridRes.ObjectId, OpenMode.ForWrite);
                ObjectIdCollection ids = layoutGrid.BoundaryHoles;
                foreach(ObjectId id in ids)
                {
                    layoutGrid.RemoveBoundaryHole(id);
                }
                trHlp.Commit();
            }
        }


        public void
        MoveColumnOrRow()
        {
            m_db      = MgdDbg.Utils.Db.GetCurDwg();
            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;

            // select grid
            PromptEntityOptions prLayoutGridOpts = new PromptEntityOptions("\nSelect a LayoutGrid2d");
            prLayoutGridOpts.SetRejectMessage("\nSelected entity must be of type LayoutGrid2d.");
            prLayoutGridOpts.AddAllowedClass(typeof(AecDb.LayoutGrid2d), false);
            
            PromptEntityResult prLayoutGridRes = ed.GetEntity(prLayoutGridOpts);
            if (prLayoutGridRes.Status != PromptStatus.OK)
                return;
                 
            using (TransactionHelper trHlp = new TransactionHelper(m_db)) 
            {
                trHlp.Start();
                
                AecDb.LayoutGrid2d layoutGrid = (AecDb.LayoutGrid2d)trHlp.Transaction.GetObject(prLayoutGridRes.ObjectId, OpenMode.ForWrite);
                AecDb.GridUv grid = layoutGrid.BaseGrid;

                if (grid == null) 
                {
                    MgdDbg.Utils.AcadUi.PrintToCmdLine("\nNo base grid available.");
                    trHlp.Abort();
                    return;
                }

                PromptKeywordOptions prWhichComponent = new PromptKeywordOptions("\nWhich component do you want to move");
                prWhichComponent.Keywords.Add("Column");
                prWhichComponent.Keywords.Add("Row");
            
                PromptResult prWhichComponentRes = ed.GetKeywords(prWhichComponent);
                if (prWhichComponentRes.Status != PromptStatus.OK)
                    return;

                if (prWhichComponentRes.StringResult == "Column")
                {
                    DoubleCollection uParams = grid.UParameters;
                    if (uParams.Count < 3) {  // needs to have more than one column (2 grid lines!)
                        MgdDbg.Utils.AcadUi.PrintToCmdLine("\nCan't move any more columns.");
                        trHlp.Abort();
                        return;
                    }
                    // move last column
                    double oldParam, lastParam, newParam;
                    oldParam = uParams[uParams.Count - 2];
                    lastParam = uParams[uParams.Count - 1];
                    newParam = ((lastParam - oldParam) / 2.0) + oldParam;

                    grid.MoveGridColumn(oldParam, newParam);
                }
                else {
                    DoubleCollection vParams = grid.VParameters;
                    if (vParams.Count < 3) 
                    {  // needs to have more than one row (2 grid lines!)
                        MgdDbg.Utils.AcadUi.PrintToCmdLine("\nCan't move any more rows.");
                        trHlp.Abort();
                        return;
                    }
                    // move last row
                    double oldParam, lastParam, newParam;
                    oldParam = vParams[vParams.Count - 2];
                    lastParam = vParams[vParams.Count - 1];
                    newParam = ((lastParam - oldParam) / 2.0) + oldParam;

                    grid.MoveGridRow(oldParam, newParam);
                }


                trHlp.Commit();
            }
        }


        public void
        RemoveLastGridRow()
        {
            m_db      = MgdDbg.Utils.Db.GetCurDwg();
            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;

            // select grid
            PromptEntityOptions prLayoutGridOpts = new PromptEntityOptions("\nSelect a LayoutGrid2d");
            prLayoutGridOpts.SetRejectMessage("\nSelected entity must be of type LayoutGrid2d.");
            prLayoutGridOpts.AddAllowedClass(typeof(AecDb.LayoutGrid2d), false);
            
            PromptEntityResult prLayoutGridRes = ed.GetEntity(prLayoutGridOpts);
            if (prLayoutGridRes.Status != PromptStatus.OK)
                return;
                 
            using (TransactionHelper trHlp = new TransactionHelper(m_db)) 
            {
                trHlp.Start();
                
                AecDb.LayoutGrid2d layoutGrid = (AecDb.LayoutGrid2d)trHlp.Transaction.GetObject(prLayoutGridRes.ObjectId, OpenMode.ForWrite);
                AecDb.GridUv grid = layoutGrid.BaseGrid;

                if (grid == null) 
                {
                    MgdDbg.Utils.AcadUi.PrintToCmdLine("\nNo base grid available.");
                    trHlp.Abort();
                    return;
                }

                DoubleCollection vParams = grid.VParameters;
                if (vParams.Count < 3) 
                {  // needs to have more than one row (2 grid lines!)
                    MgdDbg.Utils.AcadUi.PrintToCmdLine("\nCan't remove any more rows.");
                    trHlp.Abort();
                    return;
                }

                // remove last column
                grid.RemoveGridRow(vParams[vParams.Count - 1]);

                trHlp.Commit();
            }
        }


      
        public void
        RemoveLastGridColumn()
        {
            m_db      = MgdDbg.Utils.Db.GetCurDwg();
            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;

            // select grid
            PromptEntityOptions prLayoutGridOpts = new PromptEntityOptions("\nSelect a LayoutGrid2d");
            prLayoutGridOpts.SetRejectMessage("\nSelected entity must be of type LayoutGrid2d.");
            prLayoutGridOpts.AddAllowedClass(typeof(AecDb.LayoutGrid2d), false);
            
            PromptEntityResult prLayoutGridRes = ed.GetEntity(prLayoutGridOpts);
            if (prLayoutGridRes.Status != PromptStatus.OK)
                return;
                 
            using (TransactionHelper trHlp = new TransactionHelper(m_db)) 
            {
                trHlp.Start();
                
                AecDb.LayoutGrid2d layoutGrid = (AecDb.LayoutGrid2d)trHlp.Transaction.GetObject(prLayoutGridRes.ObjectId, OpenMode.ForWrite);
                AecDb.GridUv grid = layoutGrid.BaseGrid;

                if (grid == null) {
                    MgdDbg.Utils.AcadUi.PrintToCmdLine("\nNo base grid available.");
                    trHlp.Abort();
                    return;
                }

                DoubleCollection vParams = grid.VParameters;
                if (vParams.Count < 3) 
                {  // needs to have more than one row (2 grid lines!)
                    MgdDbg.Utils.AcadUi.PrintToCmdLine("\nCan't remove any more rows.");
                    trHlp.Abort();
                    return;
                }

                // remove last column
                grid.RemoveGridRow(vParams[vParams.Count - 1]);

                trHlp.Commit();
            }
        }


        public void
        AddNode()
        {
            m_db      = MgdDbg.Utils.Db.GetCurDwg();
            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;

            // select grid
            PromptEntityOptions prLayoutGridOpts = new PromptEntityOptions("\nSelect a LayoutGrid2d");
            prLayoutGridOpts.SetRejectMessage("\nSelected entity must be of type LayoutGrid2d.");
            prLayoutGridOpts.AddAllowedClass(typeof(AecDb.LayoutGrid2d), false);
            
            PromptEntityResult prLayoutGridRes = ed.GetEntity(prLayoutGridOpts);
            if (prLayoutGridRes.Status != PromptStatus.OK)
                return;
                 
            using (TransactionHelper trHlp = new TransactionHelper(m_db)) 
            {
                trHlp.Start();
                
                AecDb.LayoutGrid2d layoutGrid = (AecDb.LayoutGrid2d)trHlp.Transaction.GetObject(prLayoutGridRes.ObjectId, OpenMode.ForWrite);
                AecDb.GridUv grid = layoutGrid.BaseGrid;

                if (grid == null) 
                {
                    MgdDbg.Utils.AcadUi.PrintToCmdLine("\nNo base grid available.");
                    trHlp.Abort();
                    return;
                }

                DoubleCollection uParams = grid.UParameters;
                if (uParams.Count < 3) 
                {  // needs to have at least one column (2 grid lines!)
                    MgdDbg.Utils.AcadUi.PrintToCmdLine("\nCan't add a column.");
                    trHlp.Abort();
                    return;
                }
                // add between the last node
                double oldUParam, lastUParam, newUParam;
                oldUParam = uParams[uParams.Count - 2];
                lastUParam = uParams[uParams.Count - 1];
                newUParam = ((lastUParam - oldUParam) / 2.0) + oldUParam;


                DoubleCollection vParams = grid.VParameters;
                if (vParams.Count < 3) 
                {  // needs to have at least one row (2 grid lines!)
                    MgdDbg.Utils.AcadUi.PrintToCmdLine("\nCan't add a row.");
                    trHlp.Abort();
                    return;
                }
                // add node
                double oldVParam, lastVParam, newVParam;
                oldVParam = vParams[vParams.Count - 2];
                lastVParam = vParams[vParams.Count - 1];
                newVParam = ((lastVParam - oldVParam) / 2.0) + oldVParam;

                grid.AddNode(newUParam, newVParam);

                trHlp.Commit();
            }
        }


#endregion

#region LayoutGrid3d tests

        public void
        AddNode3d()
        {
            m_db      = MgdDbg.Utils.Db.GetCurDwg();
            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;

            // select grid
            PromptEntityOptions prLayoutGridOpts = new PromptEntityOptions("\nSelect a LayoutGrid3d");
            prLayoutGridOpts.SetRejectMessage("\nSelected entity must be of type LayoutGrid3d.");
            prLayoutGridOpts.AddAllowedClass(typeof(AecDb.LayoutGrid3d), false);
            
            PromptEntityResult prLayoutGridRes = ed.GetEntity(prLayoutGridOpts);
            if (prLayoutGridRes.Status != PromptStatus.OK)
                return;
                 
            using (TransactionHelper trHlp = new TransactionHelper(m_db)) 
            {
                trHlp.Start();
                
                AecDb.LayoutGrid3d layoutGrid = (AecDb.LayoutGrid3d)trHlp.Transaction.GetObject(prLayoutGridRes.ObjectId, OpenMode.ForWrite);
                AecDb.GridUvw grid = layoutGrid.BaseGrid;

                if (grid == null) 
                {
                    MgdDbg.Utils.AcadUi.PrintToCmdLine("\nNo base grid available.");
                    trHlp.Abort();
                    return;
                }

                DoubleCollection uParams = grid.UParameters;
                if (uParams.Count < 3) 
                {  // needs to have at least one column (2 grid lines!)
                    MgdDbg.Utils.AcadUi.PrintToCmdLine("\nCan't add a column.");
                    trHlp.Abort();
                    return;
                }
                // add between the last node
                double oldUParam, lastUParam, newUParam;
                oldUParam = uParams[uParams.Count - 2];
                lastUParam = uParams[uParams.Count - 1];
                newUParam = ((lastUParam - oldUParam) / 2.0) + oldUParam;


                DoubleCollection vParams = grid.VParameters;
                if (vParams.Count < 3) 
                {  // needs to have at least one row (2 grid lines!)
                    MgdDbg.Utils.AcadUi.PrintToCmdLine("\nCan't add a row.");
                    trHlp.Abort();
                    return;
                }
                double oldVParam, lastVParam, newVParam;
                oldVParam = vParams[vParams.Count - 2];
                lastVParam = vParams[vParams.Count - 1];
                newVParam = ((lastVParam - oldVParam) / 2.0) + oldVParam;

                DoubleCollection wParams = grid.WParameters;
                if (wParams.Count < 3) 
                {  // needs to have at least one level (2 grid lines!)
                    MgdDbg.Utils.AcadUi.PrintToCmdLine("\nCan't add a level.");
                    trHlp.Abort();
                    return;
                }
                double oldWParam, lastWParam, newWParam;
                oldWParam = wParams[wParams.Count - 2];
                lastWParam = wParams[wParams.Count - 1];
                newWParam = ((lastWParam - oldWParam) / 2.0) + oldWParam;

                // add node
                grid.AddNode(newUParam, newVParam, newWParam);

                trHlp.Commit();
            }
        }

        public void
        AppendGridLevel()
        {
            m_db      = MgdDbg.Utils.Db.GetCurDwg();
            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;

            // select grid
            PromptEntityOptions prLayoutGridOpts = new PromptEntityOptions("\nSelect a LayoutGrid3d");
            prLayoutGridOpts.SetRejectMessage("\nSelected entity must be of type LayoutGrid3d.");
            prLayoutGridOpts.AddAllowedClass(typeof(AecDb.LayoutGrid3d), false);
            
            PromptEntityResult prLayoutGridRes = ed.GetEntity(prLayoutGridOpts);
            if (prLayoutGridRes.Status != PromptStatus.OK)
                return;
                 
            using (TransactionHelper trHlp = new TransactionHelper(m_db)) 
            {
                trHlp.Start();
                
                AecDb.LayoutGrid3d layoutGrid = (AecDb.LayoutGrid3d)trHlp.Transaction.GetObject(prLayoutGridRes.ObjectId, OpenMode.ForWrite);
                AecDb.GridUvw grid = layoutGrid.BaseGrid;

                if (grid == null) 
                {
                    MgdDbg.Utils.AcadUi.PrintToCmdLine("\nNo base grid available.");
                    trHlp.Abort();
                    return;
                }

                DoubleCollection wParams = grid.WParameters;
                if (wParams.Count < 2) 
                {  // needs to have at least one level (2 grid lines!)
                    MgdDbg.Utils.AcadUi.PrintToCmdLine("\nCan't add a level.");
                    trHlp.Abort();
                    return;
                }
                double oldWParam, lastWParam, newWParam;
                oldWParam = wParams[wParams.Count - 2];
                lastWParam = wParams[wParams.Count - 1];
                // new level will have same height of the last level.
                newWParam = (lastWParam - oldWParam) + lastWParam;

                // add node
                grid.AppendGridLevel(newWParam);

                trHlp.Commit();
            }
        }


        public void
        MoveGridLevel()
        {
            m_db      = MgdDbg.Utils.Db.GetCurDwg();
            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;

            // select grid
            PromptEntityOptions prLayoutGridOpts = new PromptEntityOptions("\nSelect a LayoutGrid3d");
            prLayoutGridOpts.SetRejectMessage("\nSelected entity must be of type LayoutGrid3d.");
            prLayoutGridOpts.AddAllowedClass(typeof(AecDb.LayoutGrid3d), false);
            
            PromptEntityResult prLayoutGridRes = ed.GetEntity(prLayoutGridOpts);
            if (prLayoutGridRes.Status != PromptStatus.OK)
                return;
                 
            using (TransactionHelper trHlp = new TransactionHelper(m_db)) 
            {
                trHlp.Start();
                
                AecDb.LayoutGrid3d layoutGrid = (AecDb.LayoutGrid3d)trHlp.Transaction.GetObject(prLayoutGridRes.ObjectId, OpenMode.ForWrite);
                AecDb.GridUvw grid = layoutGrid.BaseGrid;

                if (grid == null) 
                {
                    MgdDbg.Utils.AcadUi.PrintToCmdLine("\nNo base grid available.");
                    trHlp.Abort();
                    return;
                }

                DoubleCollection wParams = grid.WParameters;
                if (wParams.Count < 3) 
                {  // needs to have at least one level (2 grid lines!)
                    MgdDbg.Utils.AcadUi.PrintToCmdLine("\nCan't add a level.");
                    trHlp.Abort();
                    return;
                }
                double oldWParam, lastWParam, newWParam;
                oldWParam = wParams[wParams.Count - 2];
                lastWParam = wParams[wParams.Count - 1];
                // the level will be move upwards by 1/2 the height of the last level.
                newWParam = ((lastWParam - oldWParam) / 2.0) + oldWParam;

                // move level
                grid.MoveGridLevel(oldWParam, newWParam);

                trHlp.Commit();
            }
        }

        public void
        RemoveLastGridLevel()
        {
            m_db      = MgdDbg.Utils.Db.GetCurDwg();
            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;

            // select grid
            PromptEntityOptions prLayoutGridOpts = new PromptEntityOptions("\nSelect a LayoutGrid3d");
            prLayoutGridOpts.SetRejectMessage("\nSelected entity must be of type LayoutGrid3d.");
            prLayoutGridOpts.AddAllowedClass(typeof(AecDb.LayoutGrid3d), false);
            
            PromptEntityResult prLayoutGridRes = ed.GetEntity(prLayoutGridOpts);
            if (prLayoutGridRes.Status != PromptStatus.OK)
                return;
                 
            using (TransactionHelper trHlp = new TransactionHelper(m_db)) 
            {
                trHlp.Start();
                
                AecDb.LayoutGrid3d layoutGrid = (AecDb.LayoutGrid3d)trHlp.Transaction.GetObject(prLayoutGridRes.ObjectId, OpenMode.ForWrite);
                AecDb.GridUvw grid = layoutGrid.BaseGrid;

                if (grid == null) 
                {
                    MgdDbg.Utils.AcadUi.PrintToCmdLine("\nNo base grid available.");
                    trHlp.Abort();
                    return;
                }

                DoubleCollection wParams = grid.WParameters;
                if (wParams.Count < 3) 
                {  // needs to have at least one level (2 grid lines!)
                    MgdDbg.Utils.AcadUi.PrintToCmdLine("\nCan't remove any more levels.");
                    trHlp.Abort();
                    return;
                }
                double lastWParam;
                lastWParam = wParams[wParams.Count - 1];
                grid.RemoveGridLevel(lastWParam);

                trHlp.Commit();
            }
        }


#endregion

#region GridAssembly tests

        public void
        AddInterferences()
        {
            m_db      = MgdDbg.Utils.Db.GetCurDwg();
            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;

            // select grid assembly
            PromptEntityOptions prGridAssemblyOpts = new PromptEntityOptions("\nSelect a GridAssembly");
            prGridAssemblyOpts.SetRejectMessage("\nSelected entity must be of type GridAssembly.");
            prGridAssemblyOpts.AddAllowedClass(typeof(AecDb.GridAssembly), false);
            
            PromptEntityResult prGridAssemblyRes = ed.GetEntity(prGridAssemblyOpts);
            if (prGridAssemblyRes.Status != PromptStatus.OK)
                return;
                 
            // select entities to be added as interferences               
            PromptSelectionOptions prPolyOpts = new PromptSelectionOptions();
            prPolyOpts.AllowDuplicates = false;
            
            PromptSelectionResult prPolyRes = ed.GetSelection(prPolyOpts);
            if (prPolyRes.Status != PromptStatus.OK)
                return;

            using (TransactionHelper trHlp = new TransactionHelper(m_db)) 
            {
                trHlp.Start();
                
                AecDb.GridAssembly gridAssembly = (AecDb.GridAssembly)trHlp.Transaction.GetObject(prGridAssemblyRes.ObjectId, OpenMode.ForWrite);
                SelectionSet ss = prPolyRes.Value;
                foreach(SelectedObject obj in ss ) 
                {
                    // TBD: check the object's type to make sure its something that could be used as an interefence!
                    AecDb.GridAssemblyInterference gaInterference = new AecDb.GridAssemblyInterference();
                    gaInterference.EntityId = obj.ObjectId;
                    gaInterference.AppliesToFlags = AecDb.ApplyToFlags.ApplyToBoundaryEdges | AecDb.ApplyToFlags.ApplyToInfill | AecDb.ApplyToFlags.ApplyToInteriorEdges;
                    gridAssembly.Interferences.Add(gaInterference);
                }

                gridAssembly.UpdateGridContents();
                trHlp.Commit();
            }
        }


        public void
        RemoveAllInterferences()
        {
            m_db      = MgdDbg.Utils.Db.GetCurDwg();
            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;

            // select grid assembly
            PromptEntityOptions prGridAssemblyOpts = new PromptEntityOptions("\nSelect a GridAssembly");
            prGridAssemblyOpts.SetRejectMessage("\nSelected entity must be of type GridAssembly.");
            prGridAssemblyOpts.AddAllowedClass(typeof(AecDb.GridAssembly), false);
            
            PromptEntityResult prGridAssemblyRes = ed.GetEntity(prGridAssemblyOpts);
            if (prGridAssemblyRes.Status != PromptStatus.OK)
                return;
                 
            using (TransactionHelper trHlp = new TransactionHelper(m_db)) 
            {
                trHlp.Start();
                
                AecDb.GridAssembly gridAssembly = (AecDb.GridAssembly)trHlp.Transaction.GetObject(prGridAssemblyRes.ObjectId, OpenMode.ForWrite);
                while(gridAssembly.Interferences.Count > 0) 
                {
                    // removing the first entity rearranges the collection of interfering entities, 
                    // so keep removing first entity until all have been removed
                    gridAssembly.Interferences.RemoveAt(0); 
                }

                gridAssembly.UpdateGridContents();
                trHlp.Commit();
            }
        }

        public void
        RemoveInterferences()
        {
            m_db      = MgdDbg.Utils.Db.GetCurDwg();
            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;

            // select grid assembly
            PromptEntityOptions prGridAssemblyOpts = new PromptEntityOptions("\nSelect a GridAssembly");
            prGridAssemblyOpts.SetRejectMessage("\nSelected entity must be of type GridAssembly.");
            prGridAssemblyOpts.AddAllowedClass(typeof(AecDb.GridAssembly), false);
            
            PromptEntityResult prGridAssemblyRes = ed.GetEntity(prGridAssemblyOpts);
            if (prGridAssemblyRes.Status != PromptStatus.OK)
                return;
                 
            // select entities to be removed as interferences               
            PromptSelectionOptions prPolyOpts = new PromptSelectionOptions();
            prPolyOpts.AllowDuplicates = false;
            
            PromptSelectionResult prPolyRes = ed.GetSelection(prPolyOpts);
            if (prPolyRes.Status != PromptStatus.OK)
                return;

            using (TransactionHelper trHlp = new TransactionHelper(m_db)) 
            {
                trHlp.Start();
                
                AecDb.GridAssembly gridAssembly = (AecDb.GridAssembly)trHlp.Transaction.GetObject(prGridAssemblyRes.ObjectId, OpenMode.ForWrite);
                SelectionSet ss = prPolyRes.Value;
                foreach(SelectedObject obj in ss ) 
                {
                    AcDb.ObjectId id = obj.ObjectId;
                    gridAssembly.Interferences.RemoveId(id);
                }

                gridAssembly.UpdateGridContents();
                trHlp.Commit();
            }
        }


        public void 
        RectangleClippingBoundary()
        {
            m_db      = MgdDbg.Utils.Db.GetCurDwg();
            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;

            // select grid assembly
            PromptEntityOptions prGridAssemblyOpts = new PromptEntityOptions("\nSelect a GridAssembly");
            prGridAssemblyOpts.SetRejectMessage("\nSelected entity must be of type GridAssembly.");
            prGridAssemblyOpts.AddAllowedClass(typeof(AecDb.GridAssembly), false);
            
            PromptEntityResult prGridAssemblyRes = ed.GetEntity(prGridAssemblyOpts);
            if (prGridAssemblyRes.Status != PromptStatus.OK)
                return;

            PromptPointOptions prPtOpt = new PromptPointOptions("\nPick the start point of the clipping rectangle");
            PromptPointResult prPtRes = ed.GetPoint(prPtOpt);
            if (prPtRes.Status != PromptStatus.OK)
                return;
                
            Point3d startPt = prPtRes.Value;

            PromptPointOptions prPtOpt2 = new PromptPointOptions("\nPick the end point of the clipping rectangle");
            PromptPointResult prPtRes2 = ed.GetPoint(prPtOpt2);
            if (prPtRes2.Status != PromptStatus.OK)
                return;
                
            Point3d endPt = prPtRes2.Value;

            using (TransactionHelper trHlp = new TransactionHelper(m_db)) 
            {
                trHlp.Start();
                
                AecDb.GridAssembly gridAssembly = (AecDb.GridAssembly)trHlp.Transaction.GetObject(prGridAssemblyRes.ObjectId, OpenMode.ForWrite);

                // MH: modified for 2010 
                // gridAssembly.SetRectangleClippingBoundary(startPt.X, startPt.Y, endPt.X, endPt.Y);
                AecDb.ClippingRectangleInfo clipRect = new AecDb.ClippingRectangleInfo(startPt.X, startPt.Y, endPt.X, endPt.Y);
                gridAssembly.ClippingRectangle = clipRect;

                gridAssembly.UpdateGridContents();
                trHlp.Commit();
            }
        }

        public void 
        RemoveClippingBoundary()
        {
            m_db      = MgdDbg.Utils.Db.GetCurDwg();
            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;

            // select grid assembly
            PromptEntityOptions prGridAssemblyOpts = new PromptEntityOptions("\nSelect a GridAssembly");
            prGridAssemblyOpts.SetRejectMessage("\nSelected entity must be of type GridAssembly.");
            prGridAssemblyOpts.AddAllowedClass(typeof(AecDb.GridAssembly), false);
            
            PromptEntityResult prGridAssemblyRes = ed.GetEntity(prGridAssemblyOpts);
            if (prGridAssemblyRes.Status != PromptStatus.OK)
                return;

            using (TransactionHelper trHlp = new TransactionHelper(m_db)) 
            {
                trHlp.Start();
                
                AecDb.GridAssembly gridAssembly = (AecDb.GridAssembly)trHlp.Transaction.GetObject(prGridAssemblyRes.ObjectId, OpenMode.ForWrite);
                gridAssembly.RemoveClippingBoundary();

                gridAssembly.UpdateGridContents();
                trHlp.Commit();
            }

        }

        public void 
        SetMiterAngles()
        {
            m_db      = MgdDbg.Utils.Db.GetCurDwg();
            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;

            // select grid assembly
            PromptEntityOptions prGridAssemblyOpts = new PromptEntityOptions("\nSelect a GridAssembly");
            prGridAssemblyOpts.SetRejectMessage("\nSelected entity must be of type GridAssembly.");
            prGridAssemblyOpts.AddAllowedClass(typeof(AecDb.GridAssembly), false);
            
            PromptEntityResult prGridAssemblyRes = ed.GetEntity(prGridAssemblyOpts);
            if (prGridAssemblyRes.Status != PromptStatus.OK)
                return;

            using (TransactionHelper trHlp = new TransactionHelper(m_db)) 
            {
                trHlp.Start();
                
                AecDb.GridAssembly gridAssembly = (AecDb.GridAssembly)trHlp.Transaction.GetObject(prGridAssemblyRes.ObjectId, OpenMode.ForWrite);
                gridAssembly.SetHasStartMiterAngle(true, 0.5235987756);
                gridAssembly.SetHasEndMiterAngle(true, 0.5235987756);

                gridAssembly.UpdateGridContents();
                trHlp.Commit();
            }

        }

        public void 
        SetEditDepth()
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();
            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;

            // select grid assembly
            PromptEntityOptions prGridAssemblyOpts = new PromptEntityOptions("\nSelect a GridAssembly");
            prGridAssemblyOpts.SetRejectMessage("\nSelected entity must be of type GridAssembly.");
            prGridAssemblyOpts.AddAllowedClass(typeof(AecDb.GridAssembly), false);
            
            PromptEntityResult prGridAssemblyRes = ed.GetEntity(prGridAssemblyOpts);
            if (prGridAssemblyRes.Status != PromptStatus.OK)
                return;

            using (TransactionHelper trHlp = new TransactionHelper(m_db)) 
            {
                trHlp.Start();
                
                AecDb.GridAssembly gridAssembly = (AecDb.GridAssembly)trHlp.Transaction.GetObject(prGridAssemblyRes.ObjectId, OpenMode.ForWrite);

                int value = 0;
                if (MgdDbg.Utils.AcadUi.PromptInteger("\nEnter edit depth (0 - 255)", true, false, true, false, true, value, out value) != PromptStatus.OK)
                    return;

                if(value < 0 || value > 255)
                    return;

                gridAssembly.EditDepth = (short) value;

//                gridAssembly.UpdateGridContents();
                trHlp.Commit();
            }

        }


        public void
        RemoveFirstComponent()
        {
            m_db      = MgdDbg.Utils.Db.GetCurDwg();
            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;

            // select grid assembly
            PromptEntityOptions prGridAssemblyOpts = new PromptEntityOptions("\nSelect a GridAssembly");
            prGridAssemblyOpts.SetRejectMessage("\nSelected entity must be of type GridAssembly.");
            prGridAssemblyOpts.AddAllowedClass(typeof(AecDb.GridAssembly), false);
            
            PromptEntityResult prGridAssemblyRes = ed.GetEntity(prGridAssemblyOpts);
            if (prGridAssemblyRes.Status != PromptStatus.OK)
                return;

            using (TransactionHelper trHlp = new TransactionHelper(m_db)) 
            {
                trHlp.Start();
                
                AecDb.GridAssembly gridAssembly = (AecDb.GridAssembly)trHlp.Transaction.GetObject(prGridAssemblyRes.ObjectId, OpenMode.ForWrite);

                AecDb.GridAssemblyComponentSet gridAssemblyCompSet = gridAssembly.ComponentSet;

                if( gridAssemblyCompSet == null ) 
                {
                    MgdDbg.Utils.AcadUi.PrintToCmdLine("\nGridAssemblyComponentSet was null");
                    return;
                }

                PromptKeywordOptions prWhichComponent = new PromptKeywordOptions("\nWhich component do you want to remove the first instance of");
                prWhichComponent.Keywords.Add("Boundary edge");
                prWhichComponent.Keywords.Add("Division");
                prWhichComponent.Keywords.Add("Infill");
                prWhichComponent.Keywords.Add("Internal edge");
            
                PromptResult prWhichComponentRes = ed.GetKeywords(prWhichComponent);
                if (prWhichComponentRes.Status != PromptStatus.OK)
                    return;

                bool useId = false;
                if (MgdDbg.Utils.AcadUi.PromptYesNo("\nRemove using id", useId, out useId) != PromptStatus.OK)
                    return;

                short id;
                if (prWhichComponentRes.StringResult == "Boundary edge")
                {
                    int becCount = gridAssemblyCompSet.BoundaryEdgeComponents.Count;
                    if(becCount > 0) 
                    {
                        if(useId)
                        {
                            AecDb.NestedGridEdgeDefinition ngeDef = null;
                            ngeDef = gridAssemblyCompSet.BoundaryEdgeComponents[0];
                            id = ngeDef.Id;
                            gridAssemblyCompSet.BoundaryEdgeComponents.RemoveId(id);
                        }
                        else
                        {
                            gridAssemblyCompSet.BoundaryEdgeComponents.RemoveAt(0);
                        }
                    }
                    else
                    {
                        MgdDbg.Utils.AcadUi.PrintToCmdLine("\nThere were no Boundary Edge Components to remove");
                    }
                }
                else if (prWhichComponentRes.StringResult == "Division")
                {
                    int divCount = gridAssemblyCompSet.DivisionComponents.Count;
                    if(divCount > 0) 
                    {
                        if(useId)
                        {
                            AecDb.NestedGridDivision ngDiv = null;
                            ngDiv = gridAssemblyCompSet.DivisionComponents[0];
                            id = ngDiv.Id;
                            gridAssemblyCompSet.DivisionComponents.RemoveId(id);
                        }
                        else
                        {
                            gridAssemblyCompSet.DivisionComponents.RemoveAt(0);
                        }
                    }
                    else
                    {
                        MgdDbg.Utils.AcadUi.PrintToCmdLine("\nThere were no Division Components to remove");
                    }
                }
                else if (prWhichComponentRes.StringResult == "Infill")
                {
                    int inCount = gridAssemblyCompSet.InfillComponents.Count;
                    if(inCount > 0) 
                    {
                        if(useId)
                        {
                            AecDb.NestedGridInfillDefinition ngInComp = null;
                            ngInComp = gridAssemblyCompSet.InfillComponents[0];
                            id = ngInComp.Id;
                            gridAssemblyCompSet.InfillComponents.RemoveId(id);
                        }
                        else
                        {
                            gridAssemblyCompSet.InfillComponents.RemoveAt(0);
                        }
                    }
                    else
                    {
                        MgdDbg.Utils.AcadUi.PrintToCmdLine("\nThere were no Infill Components to remove");
                    }
                }
                else if (prWhichComponentRes.StringResult == "Internal edge")
                {
                    int ieCount = gridAssemblyCompSet.InteriorEdgeComponents.Count;
                    if(ieCount > 0) 
                    {
                        if(useId)
                        {
                            AecDb.NestedGridEdgeDefinition ngeDef = null;
                            ngeDef = gridAssemblyCompSet.InteriorEdgeComponents[0];
                            id = ngeDef.Id;
                            gridAssemblyCompSet.InteriorEdgeComponents.RemoveId(id);
                        }
                        else
                        {
                            gridAssemblyCompSet.InteriorEdgeComponents.RemoveAt(0);
                        }
                    }
                    else
                    {
                        MgdDbg.Utils.AcadUi.PrintToCmdLine("\nThere were no Internal Edge Components to remove");
                    }
                }
                else
                    MgdDbg.Utils.AcadUi.PrintToCmdLine("\nUnknown option. Please select correct option.");

                trHlp.Commit();
            }

        }

        public void
        ReplaceFirstDivisionComponentTest()
        {
            m_db      = MgdDbg.Utils.Db.GetCurDwg();
            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;

            // select grid assembly
            PromptEntityOptions prGridAssemblyOpts = new PromptEntityOptions("\nSelect a GridAssembly");
            prGridAssemblyOpts.SetRejectMessage("\nSelected entity must be of type GridAssembly.");
            prGridAssemblyOpts.AddAllowedClass(typeof(AecDb.GridAssembly), false);
            
            PromptEntityResult prGridAssemblyRes = ed.GetEntity(prGridAssemblyOpts);
            if (prGridAssemblyRes.Status != PromptStatus.OK)
                return;

            using (TransactionHelper trHlp = new TransactionHelper(m_db)) 
            {
                trHlp.Start();
                
                AecDb.GridAssembly gridAssembly = (AecDb.GridAssembly)trHlp.Transaction.GetObject(prGridAssemblyRes.ObjectId, OpenMode.ForWrite);

                AecDb.GridAssemblyComponentSet gridAssemblyCompSet = gridAssembly.ComponentSet;

                if( gridAssemblyCompSet == null ) 
                {
                    MgdDbg.Utils.AcadUi.PrintToCmdLine("\nGridAssemblyComponentSet was null");
                    return;
                }

                int divCount = gridAssemblyCompSet.DivisionComponents.Count;
                if(divCount > 0) 
                {
                    AecDb.NestedGridDivision oldNgDiv = null;
                    oldNgDiv = gridAssemblyCompSet.DivisionComponents[0];

                    // create a grid division based on the values of the old one (but double everything!)
                    AecDb.NestedGridDivision newNgDiv = new AecDb.NestedGridDivision();
                    newNgDiv.SetToStandard(m_db);
                    newNgDiv.Description = "created by MgdDbgAec";
                    newNgDiv.DivisionDirection = oldNgDiv.DivisionDirection;
                    newNgDiv.StartOffset = (oldNgDiv.StartOffset == 0 ? 2.0 : oldNgDiv.StartOffset*2.0);
                    newNgDiv.EndOffset = (oldNgDiv.EndOffset == 0 ? 2.0 : oldNgDiv.EndOffset*2.0);
                    newNgDiv.Id = oldNgDiv.Id;
                    newNgDiv.Name = "New Grid Division Doubled";

                    // now replace them!
                    gridAssemblyCompSet.DivisionComponents.ReplaceAt(0, newNgDiv);
                }
                else
                {
                    MgdDbg.Utils.AcadUi.PrintToCmdLine("\nThere were no Division Components to replace");
                }

                trHlp.Commit();
            }

        }
#endregion

#region GridAssemblyDefinition tests

        public void
        RemoveSpecificTest()
        {
            m_db      = MgdDbg.Utils.Db.GetCurDwg();
            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;

            // select grid assembly
            PromptEntityOptions prGridAssemblyOpts = new PromptEntityOptions("\nSelect a GridAssembly");
            prGridAssemblyOpts.SetRejectMessage("\nSelected entity must be of type GridAssembly.");
            prGridAssemblyOpts.AddAllowedClass(typeof(AecDb.GridAssembly), false);
            
            PromptEntityResult prGridAssemblyRes = ed.GetEntity(prGridAssemblyOpts);
            if (prGridAssemblyRes.Status != PromptStatus.OK)
                return;

            using (TransactionHelper trHlp = new TransactionHelper(m_db)) 
            {
                trHlp.Start();
                
                AecDb.GridAssembly gridAssembly = (AecDb.GridAssembly)trHlp.Transaction.GetObject(prGridAssemblyRes.ObjectId, OpenMode.ForWrite);

                AecDb.GridAssemblyDefinition gridDef = gridAssembly.GridAssemblyDefinition;

                if( gridDef == null ) 
                {
                    MgdDbg.Utils.AcadUi.PrintToCmdLine("\nGridAssemblyDefinition was null");
                    trHlp.Abort();
                    return;
                }

                PromptKeywordOptions prWhichComponent = new PromptKeywordOptions("\nWhich specific component assignment do you want to remove");
                prWhichComponent.Keywords.Add("BoundaryEdge");
                prWhichComponent.Keywords.Add("Cell");
                prWhichComponent.Keywords.Add("InternalEdge");
            
                PromptResult prWhichComponentRes = ed.GetKeywords(prWhichComponent);
                if (prWhichComponentRes.Status != PromptStatus.OK)
                    return;

                if (prWhichComponentRes.StringResult == "BoundaryEdge")
                {
                    int becCount = gridDef.SpecificBoundaryEdgeAssignments.Count;
                    if(becCount > 0)
                    {
                        gridDef.SpecificBoundaryEdgeAssignments.RemoveAt(0);
                        gridAssembly.UpdateGridContents();
                    }
                    else
                    {
                        MgdDbg.Utils.AcadUi.PrintToCmdLine("\nThere were no Specific Boundary Edge Assignments to remove.");
                    }
                }
                else if (prWhichComponentRes.StringResult == "Cell")
                {
                    int cellCount = gridDef.SpecificCellAssignments.Count; ;
                    if(cellCount > 0) 
                    {
                        gridDef.SpecificCellAssignments.RemoveAt(0);
                        gridAssembly.UpdateGridContents();
                    }
                    else
                    {
                        MgdDbg.Utils.AcadUi.PrintToCmdLine("\nThere were no Specific Cell Assignments to remove");
                    }
                }
                else if (prWhichComponentRes.StringResult == "InternalEdge")
                {
                    int intEdgeCount = gridDef.SpecificInteriorEdgeAssignments.Count;
                    if(intEdgeCount > 0) 
                    {
                        gridDef.SpecificInteriorEdgeAssignments.RemoveAt(0);
                        gridAssembly.UpdateGridContents();
                    }
                    else
                    {
                        MgdDbg.Utils.AcadUi.PrintToCmdLine("\nThere were no Specific Internal Edge Assignments to remove");
                    }
                }
                else
                    MgdDbg.Utils.AcadUi.PrintToCmdLine("\nUnknown option. Please select correct option.");

                trHlp.Commit();
            }

        }

#endregion

#region GridCustom tests

        public void
        InsertSegmentAt()
        {
            m_db      = MgdDbg.Utils.Db.GetCurDwg();
            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;

            // select grid assembly
            PromptEntityOptions prGridCustomOpts = new PromptEntityOptions("\nSelect a Custom LayoutGrid2d");
            prGridCustomOpts.SetRejectMessage("\nSelected entity must be of type LayoutGrid2d.");
            prGridCustomOpts.AddAllowedClass(typeof(AecDb.LayoutGrid2d), false);
            
            PromptEntityResult prGridCustomRes = ed.GetEntity(prGridCustomOpts);
            if (prGridCustomRes.Status != PromptStatus.OK)
                return;

            using (TransactionHelper trHlp = new TransactionHelper(m_db)) 
            {
                trHlp.Start();
                
                AecDb.LayoutGrid2d layoutGrid = (AecDb.LayoutGrid2d)trHlp.Transaction.GetObject(prGridCustomRes.ObjectId, OpenMode.ForWrite);

                if(layoutGrid == null)
                    return;

                AecDb.GridCustom customGrid = layoutGrid.CustomGrid;
                if(customGrid == null)
                {
                    MgdDbg.Utils.AcadUi.PrintToCmdLine("\nThis layout grid is not a custom grid.");
                    trHlp.Abort();
                    return;
                }

#if !ACA2009
                int count = customGrid.MainSegmentCount;
#endif

#if ACA2009
                int count = customGrid.Segments.Count;
#endif

                if (count == 0)
                {
                    MgdDbg.Utils.AcadUi.PrintToCmdLine("\nNo main segments...");
                    trHlp.Abort();
                    return;
                }

                // select line to be added             
                PromptEntityOptions prLineOpts = new PromptEntityOptions("\nSelect a line");
                prLineOpts .SetRejectMessage("\nSelected entity must be of type Line.");
                prLineOpts .AddAllowedClass(typeof(AcDb.Line), false);
            
                PromptEntityResult prLineRes = ed.GetEntity(prLineOpts );
                if (prLineRes .Status != PromptStatus.OK)
                    return;

                AcDb.Line line= (AcDb.Line)trHlp.Transaction.GetObject(prLineRes.ObjectId, OpenMode.ForWrite);

                if(line== null) {
                    MgdDbg.Utils.AcadUi.PrintToCmdLine("\nUnable to get the LineSegment2d ...");
                    trHlp.Abort();
                    return;
                }

                LineSegment2d lseg = new LineSegment2d(new Point2d(line.StartPoint.X, line.StartPoint.Y), new Point2d(line.EndPoint.X, line.EndPoint.Y) );
                AecDb.GridCustomSegment customSeg = new AecDb.GridCustomSegment(lseg);
                // TBD: need to transform the line to fall on the right spot
                //customSeg.TransformBy(???);

#if !ACA2009
                customGrid.InsertSegmentAt(customGrid.MainSegmentCount, customSeg); // inserts as last segment
#endif

#if ACA2009
                customGrid.Segments.Insert(customGrid.Segments.Count, customSeg); // inserts as last segment
#endif

                customGrid.RequiresUpdate = true;

                trHlp.Commit();
            }
        }



        public void
        RemoveLastSegment()
        {
            m_db      = MgdDbg.Utils.Db.GetCurDwg();
            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;

            // select grid assembly
            PromptEntityOptions prGridCustomOpts = new PromptEntityOptions("\nSelect a Custom LayoutGrid2d");
            prGridCustomOpts.SetRejectMessage("\nSelected entity must be of type LayoutGrid2d.");
            prGridCustomOpts.AddAllowedClass(typeof(AecDb.LayoutGrid2d), false);
            
            PromptEntityResult prGridCustomRes = ed.GetEntity(prGridCustomOpts);
            if (prGridCustomRes.Status != PromptStatus.OK)
                return;

            Point3d pt = prGridCustomRes.PickedPoint;

            using (TransactionHelper trHlp = new TransactionHelper(m_db)) 
            {
                trHlp.Start();
                
                AecDb.LayoutGrid2d layoutGrid = (AecDb.LayoutGrid2d)trHlp.Transaction.GetObject(prGridCustomRes.ObjectId, OpenMode.ForWrite);

                if(layoutGrid == null)
                    return;

                AecDb.GridCustom customGrid = layoutGrid.CustomGrid;
                if(customGrid == null)
                {
                    MgdDbg.Utils.AcadUi.PrintToCmdLine("\nThis layout grid is not a custom grid.");
                    trHlp.Abort();
                    return;
                }

#if !ACA2009
                int count = customGrid.MainSegmentCount;
#endif

#if ACA2009
                int count = customGrid.Segments.Count;
#endif

                if (count == 0)
                {
                    MgdDbg.Utils.AcadUi.PrintToCmdLine("\nNo main segments...");
                    trHlp.Abort();
                    return;
                }

#if !ACA2009
                customGrid.RemoveSegmentAt(customGrid.MainSegmentCount - 1); // removes last segment
#endif

#if ACA2009
                customGrid.Segments.RemoveAt(customGrid.Segments.Count - 1); // removes last segment
#endif

                customGrid.RequiresUpdate = true;

                trHlp.Commit();
            }
        }



#endregion



        }
}
