
// $Header: //Desktop/Source/TestTools/MgdDbgAec/ObjTests/PropData.cs#1 $
// $Author: fergusja $ $DateTime: 2007/03/16 11:08:08 $ $Change: 68854 $
//
// (C) Copyright 2006 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//


using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;
using System.Windows.Forms;

using Autodesk.AutoCAD.Runtime;
using Autodesk.AutoCAD.EditorInput;
using Autodesk.AutoCAD.DatabaseServices;
using Autodesk.Aec.PropertyData.DatabaseServices;

using MgdDbg;
using MgdDbg.Test;

namespace MgdDbgAec.ObjTests {

    class PropData : MgdDbgTestFuncs {

		public
		PropData()
		{		    
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Add Hardwired PropertySet (setting values based on object type", "Hardwire a PropSetDef and then add to all Doors, Windows, and MassElements", "PropertyDataServices", new MgdDbgTestFuncInfo.TestFunc(AddHardwiredPropertySet), MgdDbgTestFuncInfo.TestType.Modify));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Add PropertySet", "AddPropertySet()", "PropertyDataServices", new MgdDbgTestFuncInfo.TestFunc(AddPropertySet), MgdDbgTestFuncInfo.TestType.Modify));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Get PropertySetDefs Used", "GetPropertySetDefinitionsUsed()", "PropertyDataServices", new MgdDbgTestFuncInfo.TestFunc(GetPropSetDefsUsed), MgdDbgTestFuncInfo.TestType.Query));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Get PropertySets Attached", "GetPropertySets()", "PropertyDataServices", new MgdDbgTestFuncInfo.TestFunc(GetPropSets), MgdDbgTestFuncInfo.TestType.Query));

            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Hardwire PropertySetDef", "Hardwire a Property Set Definition", typeof(PropertySetDefinition), new MgdDbgTestFuncInfo.TestFunc(HardwirePropertySetDef), MgdDbgTestFuncInfo.TestType.Create));
        }

        /// <summary>
        /// Test adding a PropertySet to an object.  User selects an object, then a PropertySetDef.
        /// </summary>
        
        public void
        AddPropertySet()
        {
            Database db = MgdDbg.Utils.Db.GetCurDwg();
            ObjectId objId;

            while (SelectObject(out objId)) {   // select an object to add the PropertySet to
                    // have user select a PropertySetDef
                ObjectId propSetDefId = MgdDbgAec.Utils.Ui.SelectStyle.GetPropertySetDefId(db);
                if (propSetDefId.IsNull)
                    return;     // user cancelled

                using (TransactionHelper trHelp = new TransactionHelper(db)) {
                    trHelp.Start();

                    if (propSetDefId.IsNull == false) {
                        PropertySetDefinition psDef = (PropertySetDefinition)trHelp.Transaction.GetObject(propSetDefId, OpenMode.ForRead);
                        DBObject tmpObj = trHelp.Transaction.GetObject(objId, OpenMode.ForWrite);

                            // if the PropSetDef doesn't apply to this type of object, allow user to 
                            // bail out.
                        bool doAttach = true;
                        if (!PropSetDefAppliesToObject(psDef, tmpObj)) {
                            DialogResult result = MessageBox.Show("The selected PropertySetDef does not apply to this object type.  Attach anyway?", "AppliesTo Filter", MessageBoxButtons.YesNo);
                            if (result != DialogResult.Yes)
                                doAttach = false;
                        }

                        if (doAttach) {
                            PropertyDataServices.AddPropertySet(tmpObj, propSetDefId);
                            MgdDbg.Utils.AcadUi.PrintToCmdLine(string.Format("\nAttached PropertySet \"{0}\".", psDef.Name));
                        }

                        trHelp.Commit();
                    }
                }
            }
        }
        
        /// <summary>
        /// Select an object, see what PropertySetDefs are used on it (this object only... not extended objects like Styles)
        /// </summary>
        
        public void
        GetPropSetDefsUsed()
        {
            Database db = MgdDbg.Utils.Db.GetCurDwg();
            ObjectId objId;

            while (SelectObject(out objId)) {
                using (TransactionHelper trHelp = new TransactionHelper(db)) {
                    trHelp.Start();

                    DBObject tmpObj = trHelp.Transaction.GetObject(objId, OpenMode.ForRead);
                    ObjectIdCollection propSetDefsUsed = PropertyDataServices.GetPropertySetDefinitionsUsed(tmpObj);
                    if (propSetDefsUsed.Count == 0) {
                        Autodesk.AutoCAD.ApplicationServices.Application.ShowAlertDialog("No PropertySets are attached to this object.");
                    }
                    else {
                        MgdDbg.Snoop.Forms.DBObjects dbox = new MgdDbg.Snoop.Forms.DBObjects(propSetDefsUsed, trHelp);
                        dbox.Text = "Property Set Definitions Used";
                        Autodesk.AutoCAD.ApplicationServices.Application.ShowModalDialog(dbox);
                    }

                    trHelp.Commit();
                }
            }
        }

        /// <summary>
        /// Select an object, see what PropertySets are attached to it (just this object, not extended objects like Styles)
        /// </summary>
        
        public void
        GetPropSets()
        {
            Database db = MgdDbg.Utils.Db.GetCurDwg();
            ObjectId objId;

            while (SelectObject(out objId)) {
                using (TransactionHelper trHelp = new TransactionHelper(db)) {
                    trHelp.Start();

                    DBObject tmpObj = trHelp.Transaction.GetObject(objId, OpenMode.ForRead);
                    ObjectIdCollection propSets = PropertyDataServices.GetPropertySets(tmpObj);
                    if (propSets.Count == 0) {
                        Autodesk.AutoCAD.ApplicationServices.Application.ShowAlertDialog("No PropertySets are attached to this object.");
                    }
                    else {
                        MgdDbg.Snoop.Forms.DBObjects dbox = new MgdDbg.Snoop.Forms.DBObjects(propSets, trHelp);
                        dbox.Text = "Property Sets Attached";
                        Autodesk.AutoCAD.ApplicationServices.Application.ShowModalDialog(dbox);
                    }

                    trHelp.Commit();
                }
            }
        }

        /// <summary>
        /// hardwire a PropSetDef and then apply it to a set of selected objects.
        /// </summary>
        
        public void
        AddHardwiredPropertySet()
        {
            Database db = MgdDbg.Utils.Db.GetCurDwg();

            HardwirePropertySetDef();   // make sure our def is hardwired first
            DictionaryPropertySetDefinitions dict = new DictionaryPropertySetDefinitions(db);
            ObjectId propSetDefId = dict.GetAt("MgdDbgAec_PropSetDef");

            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;
		
                // prompt for objects of type Door, Window, or MassElement
            PromptSelectionOptions selOpts = new PromptSelectionOptions();
            selOpts.MessageForAdding = "\nSelect Doors, Windows, and MassElements";

            RXClass classDoor     = (RXClass)SystemObjects.ClassDictionary["AecDbDoor"];
            RXClass classWindow   = (RXClass)SystemObjects.ClassDictionary["AecDbWindow"];
            RXClass classMassElem = (RXClass)SystemObjects.ClassDictionary["AecDbMassElem"];

            TypedValue[] filterPairs = {
                new TypedValue((int)DxfCode.Operator, "<or"),
                new TypedValue((int)DxfCode.Start, classDoor.DxfName),
                new TypedValue((int)DxfCode.Start, classWindow.DxfName),
                new TypedValue((int)DxfCode.Start, classMassElem.DxfName),
                new TypedValue((int)DxfCode.Operator, "or>"),
            };

            PromptSelectionResult result = ed.GetSelection(selOpts, new SelectionFilter(filterPairs));
            if (result.Status == PromptStatus.OK) {
                using (TransactionHelper tr = new TransactionHelper(db)) {
                    tr.Start();

                    ObjectId[] objIds = result.Value.GetObjectIds();
                    foreach (ObjectId tmpObjId in objIds) {
                        Entity tmpEnt = (Entity)tr.Transaction.GetObject(tmpObjId, OpenMode.ForWrite);

                            // first add it to the entity
                        PropertyDataServices.AddPropertySet(tmpEnt, propSetDefId);

                            // now open it back up and edit the values
                        ObjectId propSetId = PropertyDataServices.GetPropertySet(tmpEnt, propSetDefId);
                        PropertySet propSet = (PropertySet)tr.Transaction.GetObject(propSetId, OpenMode.ForWrite);
                        
                            // look up the PropertyIds we need in the 'set' operation.  NOTE: if doing
                            // this alot, it would be more efficient to look them up in the PropSetDef
                            // instead of doing it each time for each PropertySet.
                        int propMfgId = propSet.PropertyNameToId("Manufacturer");
                        int propSkuId = propSet.PropertyNameToId("SKU");
                        int propCostId = propSet.PropertyNameToId("Cost");
                        int propRecycledId = propSet.PropertyNameToId("Made from recycled materials");
                        
                            // based on the type of object, set some data accordingly
                        if (tmpEnt is Autodesk.Aec.Arch.DatabaseServices.Door) {
                            //propSet.SetAt(propMfgId, "Alpha Corp.");      // Just to show two ways to do this
                            PropertySetData psData = propSet.GetPropertySetDataAt(propMfgId);
                            psData.SetData("Alpha Corp.");
                            propSet.SetAt(propSkuId, "SKU-Door-001299834");
                            propSet.SetAt(propCostId, 450.39);
                            propSet.SetAt(propRecycledId, false);
                        }
                        else if (tmpEnt is Autodesk.Aec.Arch.DatabaseServices.Window) {
                            propSet.SetAt(propMfgId, "Beta Corp.");
                            propSet.SetAt(propSkuId, "SKU-Window-98666612");
                            propSet.SetAt(propCostId, 380.98);
                            propSet.SetAt(propRecycledId, true);
                        }
                        else if (tmpEnt is Autodesk.Aec.DatabaseServices.MassElement) {
                            propSet.SetAt(propMfgId, "Gamma Corp.");
                            propSet.SetAt(propSkuId, "SKU-MassElement-11011100");
                            propSet.SetAt(propCostId, 3.45);
                            propSet.SetAt(propRecycledId, true);
                        }
                        else {
                            propSet.SetAt(propMfgId, "Who made this thing?");
                            propSet.SetAt(propSkuId, "???");
                            propSet.SetAt(propCostId, 0.0);
                            propSet.SetAt(propRecycledId, false);
                        }
                    }

                    tr.Commit();
                }
            }
        }

        /// <summary>
        /// Hardwire a PropSetDef.
        /// </summary>
        
        public void
        HardwirePropertySetDef()
        {
            Database db = MgdDbg.Utils.Db.GetCurDwg();
            string symName = "MgdDbgAec_PropSetDef";    // don't change without changing in function AddHardwiredPropertySet() above
            
            using (TransactionHelper tr = new TransactionHelper(db)) {
                tr.Start();
                
                using (DictionaryPropertySetDefinitions dict = new DictionaryPropertySetDefinitions(db)) {
                    if (dict.Has(symName, tr.Transaction)) {
                        MgdDbg.Utils.AcadUi.PrintToCmdLine(string.Format("\nProperty Set Definition \"{0}\" already exists.", symName));
                        return;
                    }
                    
                    PropertySetDefinition psDef = new PropertySetDefinition();
                    psDef.Description = "Test properties (applies to Doors, Windows, and MassElements)";

                        // set up a filter (just for fun, we'll use Doors, Windows, and MassElements)
                    System.Collections.Specialized.StringCollection filterList = new System.Collections.Specialized.StringCollection();
                    filterList.Add("AecDbDoor");
                    filterList.Add("AecDbWindow");
                    filterList.Add("AecDbMassElem");
                    psDef.SetAppliesToFilter(filterList, false);
                    
                        // now add some basic properties
                    PropertyDefinitionCollection propDefs = psDef.Definitions;

                    PropertyDefinition propDef1 = new PropertyDefinition();
                    propDef1.Name = "Manufacturer";
                    propDef1.Description = "Sample property created by MgdDbgAec";
                    propDef1.DataType = Autodesk.Aec.PropertyData.DataType.Text;
                    propDef1.DefaultData = "-- default --";
                    propDefs.Add(propDef1);

                    PropertyDefinition propDef2 = new PropertyDefinition();
                    propDef2.Name = "SKU";
                    propDef2.Description = "Sample property created by MgdDbgAec";
                    propDef2.DataType = Autodesk.Aec.PropertyData.DataType.Text;
                    propDef2.DefaultData = "-- default --";
                    propDefs.Add(propDef2);

                    PropertyDefinition propDef3 = new PropertyDefinition();
                    propDef3.Name = "Cost";
                    propDef3.Description = "Sample property created by MgdDbgAec";
                    propDef3.DataType = Autodesk.Aec.PropertyData.DataType.Real;
                    propDef3.DefaultData = 0.0;
                    propDefs.Add(propDef3);

                    PropertyDefinition propDef4 = new PropertyDefinition();
                    propDef4.Name = "Made from recycled materials";
                    propDef4.Description = "Sample property created by MgdDbgAec";
                    propDef4.DataType = Autodesk.Aec.PropertyData.DataType.TrueFalse;
                    propDef4.DefaultData = false;
                    propDefs.Add(propDef4);
                    
                    dict.AddNewRecord(symName, psDef);
                    tr.Transaction.AddNewlyCreatedDBObject(psDef, true);
                }
                
                tr.Commit();
                MgdDbg.Utils.AcadUi.PrintToCmdLine(string.Format("\nCreated Property Set Definition \"{0}\".", symName));
            }
        }

		private bool
		SelectObject(out ObjectId objId)
		{
		    Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;
		
                // prompt for the Object
            PromptEntityOptions prOpts = new PromptEntityOptions("\nSelect an entity");
            
            PromptEntityResult prRes = ed.GetEntity(prOpts);
            if (prRes.Status != PromptStatus.OK) {
                objId = ObjectId.Null;
                return false;
            }
            else {
                objId = prRes.ObjectId;
                return true;
            }
        }

        private bool
        PropSetDefAppliesToObject(PropertySetDefinition psDef, DBObject objToAttachTo)
        {
            if (psDef.AppliesToAll)
                return true;

            if (psDef.AppliesToFilter.Contains(objToAttachTo.GetRXClass().Name))
                return true;

            return false;       // check Classification?
        }
    }
}

