
// $Header: //Desktop/Source/TestTools/MgdDbgAec/ObjTests/QueryArch.cs#2 $
// $Author: shanbhp $ $DateTime: 2008/02/06 15:03:54 $ $Change: 93065 $
//
// (C) Copyright 2004 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//

using System;
using System.Diagnostics;
using System.Collections;
using System.Windows.Forms;
using Autodesk.AutoCAD.DatabaseServices;
using Autodesk.AutoCAD.Geometry;
using Autodesk.AutoCAD.Colors;
using Autodesk.AutoCAD.ApplicationServices;
using Autodesk.AutoCAD.EditorInput;
using Autodesk.AutoCAD.Runtime;
using MgdDbg;
using MgdDbg.Test;

using AcDb        = Autodesk.AutoCAD.DatabaseServices;
using AecDb       = Autodesk.Aec.DatabaseServices;
using AecGe       = Autodesk.Aec.Geometry;
using AecAppSrvcs = Autodesk.Aec.ApplicationServices;
using ArchDb      = Autodesk.Aec.Arch.DatabaseServices;
using AcadApp     = Autodesk.AutoCAD.ApplicationServices.Application;
using AecModeler  = Autodesk.Aec.Modeler;

namespace MgdDbgAec.ObjTests
{
	/// <summary>
	/// Summary description for QueryEntTests.
	/// </summary>
	public class QueryArch : MgdDbgTestFuncs
	{
        private Database    m_db = null;

		public
		QueryArch()
		{		    
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Cleanup Group Definition Dictionary",  "Basic dictionary query", typeof(AecDb.Dictionary), new MgdDbgTestFuncInfo.TestFunc(CleanupGroupDefinitionDictionary), MgdDbgTestFuncInfo.TestType.Query));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Curtain Wall Layout Style Dictionary", "Basic dictionary query", typeof(AecDb.Dictionary), new MgdDbgTestFuncInfo.TestFunc(CurtainWallLayoutStyleDictionary), MgdDbgTestFuncInfo.TestType.Query));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Curtain Wall Unit Style Dictionary", "Basic dictionary query", typeof(AecDb.Dictionary), new MgdDbgTestFuncInfo.TestFunc(CurtainWallUnitStyleDictionary), MgdDbgTestFuncInfo.TestType.Query));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Door Style Dictionary", "Basic dictionary query", typeof(AecDb.Dictionary), new MgdDbgTestFuncInfo.TestFunc(DoorStyleDictionary), MgdDbgTestFuncInfo.TestType.Query));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Endcap Style Dictionary", "Basic dictionary query", typeof(AecDb.Dictionary), new MgdDbgTestFuncInfo.TestFunc(EndcapStyleDictionary), MgdDbgTestFuncInfo.TestType.Query));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Opening Endcap Style Dictionary", "Basic dictionary query", typeof(AecDb.Dictionary), new MgdDbgTestFuncInfo.TestFunc(OpeningEndcapStyleDictionary), MgdDbgTestFuncInfo.TestType.Query));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Roof Slab Edge Style Dictionary", "Basic dictionary query", typeof(AecDb.Dictionary), new MgdDbgTestFuncInfo.TestFunc(RoofSlabEdgeStyleDictionary), MgdDbgTestFuncInfo.TestType.Query));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Roof Slab Style Dictionary", "Basic dictionary query", typeof(AecDb.Dictionary), new MgdDbgTestFuncInfo.TestFunc(RoofSlabStyleDictionary), MgdDbgTestFuncInfo.TestType.Query));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Slab Edge Style Dictionary", "Basic dictionary query", typeof(AecDb.Dictionary), new MgdDbgTestFuncInfo.TestFunc(SlabEdgeStyleDictionary), MgdDbgTestFuncInfo.TestType.Query));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Slab Style Dictionary", "Basic dictionary query", typeof(AecDb.Dictionary), new MgdDbgTestFuncInfo.TestFunc(SlabStyleDictionary), MgdDbgTestFuncInfo.TestType.Query));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Space Style Dictionary", "Basic dictionary query", typeof(AecDb.Dictionary), new MgdDbgTestFuncInfo.TestFunc(SpaceStyleDictionary), MgdDbgTestFuncInfo.TestType.Query));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Wall Modifier Style Dictionary", "Basic dictionary query", typeof(AecDb.Dictionary), new MgdDbgTestFuncInfo.TestFunc(WallModifierStyleDictionary), MgdDbgTestFuncInfo.TestType.Query));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Wall Style Dictionary", "Basic dictionary query", typeof(AecDb.Dictionary), new MgdDbgTestFuncInfo.TestFunc(WallStyleDictionary), MgdDbgTestFuncInfo.TestType.Query));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Window Assembly Style Dictionary", "Basic dictionary query", typeof(AecDb.Dictionary), new MgdDbgTestFuncInfo.TestFunc(WindowAssemblyStyleDictionary), MgdDbgTestFuncInfo.TestType.Query));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Window Style Dictionary", "Basic dictionary query", typeof(AecDb.Dictionary), new MgdDbgTestFuncInfo.TestFunc(WindowStyleDictionary), MgdDbgTestFuncInfo.TestType.Query));

            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Width at elevation", "Get the width of the wall at a certain elevation", typeof(ArchDb.Wall), new MgdDbgTestFuncInfo.TestFunc(WallWidthAtElevation), MgdDbgTestFuncInfo.TestType.Query));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Area", "Get the surface area of one side of the wall", typeof(ArchDb.Wall), new MgdDbgTestFuncInfo.TestFunc(WallArea), MgdDbgTestFuncInfo.TestType.Query));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Shrinkwrap outline", "Get the shrinkwrap outline of a wall", typeof(ArchDb.Wall), new MgdDbgTestFuncInfo.TestFunc(WallShrinkwrap), MgdDbgTestFuncInfo.TestType.Query));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Shrinkwrap body", "Get the wall shrink wrap body", typeof(ArchDb.Wall), new MgdDbgTestFuncInfo.TestFunc(WallShrinkwrapBody), MgdDbgTestFuncInfo.TestType.Query));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Get Component Pline", "Get the polyline representing the left or right side of the wall", typeof(ArchDb.Wall), new MgdDbgTestFuncInfo.TestFunc(WallComponentPline), MgdDbgTestFuncInfo.TestType.Query));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Min and Max Heights", "Get the minimum and maximum heights of the wall", typeof(ArchDb.Wall), new MgdDbgTestFuncInfo.TestFunc(WallMinAndMaxHeights), MgdDbgTestFuncInfo.TestType.Query));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Get Actual Height Variations", "Get the actual height variations in the wall", typeof(ArchDb.Wall), new MgdDbgTestFuncInfo.TestFunc(WallActualHeightVariations), MgdDbgTestFuncInfo.TestType.Query));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Get anchored objects", "Get the list of anchored objects in a wall", typeof(ArchDb.Wall), new MgdDbgTestFuncInfo.TestFunc(WallAnchoredOpenings), MgdDbgTestFuncInfo.TestType.Query));
  

/*
    // For future implementation:
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Railing Style Dictionary",             "Basic dictionary query", typeof(AecDb.Dictionary), new MgdDbg.MgdDbgTestFuncInfo.TestFunc(RailingStyleDictionary)));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Stair Style Dictionary",               "Basic dictionary query", typeof(AecDb.Dictionary), new MgdDbg.MgdDbgTestFuncInfo.TestFunc(StairStyleDictionary)));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Stair Winder Style Dictionary",        "Basic dictionary query", typeof(AecDb.Dictionary), new MgdDbg.MgdDbgTestFuncInfo.TestFunc(StairWinderStyleDictionary)));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Stair Winder Type Dictionary",         "Basic dictionary query", typeof(AecDb.Dictionary), new MgdDbg.MgdDbgTestFuncInfo.TestFunc(StairWinderTypeDictionary)));
*/
        }
        

        public void
        DoorStyleDictionary()
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            using (ArchDb.DictionaryDoorStyle dDict = new ArchDb.DictionaryDoorStyle(m_db)) 
            {
                MgdDbg.Snoop.Forms.Objects snoopForm = new MgdDbg.Snoop.Forms.Objects(dDict);
                snoopForm.ShowDialog();
            }
        }

        public void
        CleanupGroupDefinitionDictionary()
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            using (ArchDb.DictionaryWallCleanupGroupDefinition dDict = new ArchDb.DictionaryWallCleanupGroupDefinition(m_db)) 
            {
                MgdDbg.Snoop.Forms.Objects snoopForm = new MgdDbg.Snoop.Forms.Objects(dDict);
                snoopForm.ShowDialog();
            }
        }
        
        public void
        CurtainWallLayoutStyleDictionary()
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            using (ArchDb.DictionaryCurtainWallLayoutStyle dDict = new ArchDb.DictionaryCurtainWallLayoutStyle(m_db)) 
            {
                MgdDbg.Snoop.Forms.Objects snoopForm = new MgdDbg.Snoop.Forms.Objects(dDict);
                snoopForm.ShowDialog();
            }
        }
        
        public void
        CurtainWallUnitStyleDictionary()
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            using (ArchDb.DictionaryCurtainWallUnitStyle dDict = new ArchDb.DictionaryCurtainWallUnitStyle(m_db)) 
            {
                MgdDbg.Snoop.Forms.Objects snoopForm = new MgdDbg.Snoop.Forms.Objects(dDict);
                snoopForm.ShowDialog();
            }
        }
        
        public void
        EndcapStyleDictionary()
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            using (ArchDb.DictionaryWallEndcapStyle dDict = new ArchDb.DictionaryWallEndcapStyle(m_db)) 
            {
                MgdDbg.Snoop.Forms.Objects snoopForm = new MgdDbg.Snoop.Forms.Objects(dDict);
                snoopForm.ShowDialog();
            }
        }
        
        public void
        OpeningEndcapStyleDictionary()
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            using (ArchDb.DictionaryOpeningEndcapStyle dDict = new ArchDb.DictionaryOpeningEndcapStyle(m_db)) 
            {
                MgdDbg.Snoop.Forms.Objects snoopForm = new MgdDbg.Snoop.Forms.Objects(dDict);
                snoopForm.ShowDialog();
            }
        }
       
/*
 // For future implementation:
  
        public void
        RailingStyleDictionary()
        {
            using (ArchDb.DictionaryRailingStyle dDict = new ArchDb.DictionaryRailingStyle(m_db)) 
            {
                MgdDbg.Snoop.Forms.Objects snoopForm = new MgdDbg.Snoop.Forms.Objects(dDict);
                snoopForm.ShowDialog();
            }
        }
*/
        
        public void
        RoofSlabEdgeStyleDictionary()
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            using (ArchDb.DictionaryRoofSlabEdgeStyle dDict = new ArchDb.DictionaryRoofSlabEdgeStyle(m_db)) 
            {
                MgdDbg.Snoop.Forms.Objects snoopForm = new MgdDbg.Snoop.Forms.Objects(dDict);
                snoopForm.ShowDialog();
            }
        }
        
        public void
        RoofSlabStyleDictionary()
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            using (ArchDb.DictionaryRoofSlabStyle dDict = new ArchDb.DictionaryRoofSlabStyle(m_db)) 
            {
                MgdDbg.Snoop.Forms.Objects snoopForm = new MgdDbg.Snoop.Forms.Objects(dDict);
                snoopForm.ShowDialog();
            }
        }
        
        public void
        SlabEdgeStyleDictionary()
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            using (ArchDb.DictionarySlabEdgeStyle dDict = new ArchDb.DictionarySlabEdgeStyle(m_db)) 
            {
                MgdDbg.Snoop.Forms.Objects snoopForm = new MgdDbg.Snoop.Forms.Objects(dDict);
                snoopForm.ShowDialog();
            }
        }
        
        public void
        SlabStyleDictionary()
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            using (ArchDb.DictionarySlabStyle dDict = new ArchDb.DictionarySlabStyle(m_db)) 
            {
                MgdDbg.Snoop.Forms.Objects snoopForm = new MgdDbg.Snoop.Forms.Objects(dDict);
                snoopForm.ShowDialog();
            }
        }
        
        public void
        SpaceStyleDictionary()
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            using (ArchDb.DictionarySpaceStyle dDict = new ArchDb.DictionarySpaceStyle(m_db)) 
            {
                MgdDbg.Snoop.Forms.Objects snoopForm = new MgdDbg.Snoop.Forms.Objects(dDict);
                snoopForm.ShowDialog();
            }
        }
        
/*
    // For future implementation:
  
        public void
        StairStyleDictionary()
        {
            using (ArchDb.DictionaryStairStyle dDict = new ArchDb.DictionaryStairStyle(m_db)) 
            {
                MgdDbg.Snoop.Forms.Objects snoopForm = new MgdDbg.Snoop.Forms.Objects(dDict);
                snoopForm.ShowDialog();
            }
        }
        
        public void
        StairWinderStyleDictionary()
        {
            using (ArchDb.DictionaryStairWinderStyle dDict = new ArchDb.DictionaryStairWinderStyle(m_db)) 
            {
                MgdDbg.Snoop.Forms.Objects snoopForm = new MgdDbg.Snoop.Forms.Objects(dDict);
                snoopForm.ShowDialog();
            }
        }
        
        public void
        StairWinderTypeDictionary()
        {
            using (ArchDb.DictionaryStairWinderType dDict = new ArchDb.DictionaryStairWinderType(m_db)) 
            {
                MgdDbg.Snoop.Forms.Objects snoopForm = new MgdDbg.Snoop.Forms.Objects(dDict);
                snoopForm.ShowDialog();
            }
        }
*/
        
        public void
        WallModifierStyleDictionary()
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            using (ArchDb.DictionaryWallModifierStyle dDict = new ArchDb.DictionaryWallModifierStyle(m_db)) 
            {
                MgdDbg.Snoop.Forms.Objects snoopForm = new MgdDbg.Snoop.Forms.Objects(dDict);
                snoopForm.ShowDialog();
            }
        }
        
        public void
        WallStyleDictionary()
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            using (ArchDb.DictionaryWallStyle dDict = new ArchDb.DictionaryWallStyle(m_db)) 
            {
                MgdDbg.Snoop.Forms.Objects snoopForm = new MgdDbg.Snoop.Forms.Objects(dDict);
                snoopForm.ShowDialog();
            }
        }
        
        public void
        WindowAssemblyStyleDictionary()
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            using (ArchDb.DictionaryWindowAssemblyStyle dDict = new ArchDb.DictionaryWindowAssemblyStyle(m_db)) 
            {
                MgdDbg.Snoop.Forms.Objects snoopForm = new MgdDbg.Snoop.Forms.Objects(dDict);
                snoopForm.ShowDialog();
            }
        }
        
        public void
        WindowStyleDictionary()
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            using (ArchDb.DictionaryWindowStyle dDict = new ArchDb.DictionaryWindowStyle(m_db)) 
            {
                MgdDbg.Snoop.Forms.Objects snoopForm = new MgdDbg.Snoop.Forms.Objects(dDict);
                snoopForm.ShowDialog();
            }
        }
                
		public void
		WallWidthAtElevation()
		{
            m_db = MgdDbg.Utils.Db.GetCurDwg();

		    Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;
            Autodesk.AutoCAD.DatabaseServices.TransactionManager tm = m_db.TransactionManager;

                // prompt for the Wall Object
            PromptEntityOptions prWallOpts = new PromptEntityOptions("\nSelect a wall");
            prWallOpts.SetRejectMessage("\nSelected entity must be of type Wall.");
            prWallOpts.AddAllowedClass(typeof(ArchDb.Wall), false);
            
            while (true) {
                PromptEntityResult prWallRes = ed.GetEntity(prWallOpts);
                if (prWallRes.Status != PromptStatus.OK)
                    return;
                    
                    // prompt for the Elevation
                PromptDoubleResult prElevRes = ed.GetDistance("\nElevation");
                if (prElevRes.Status != PromptStatus.OK)
                    return;

			    using (TransactionHelper trHlp = new TransactionHelper(m_db)) {
				    trHlp.Start();
    				
                    ArchDb.Wall wall = (ArchDb.Wall)trHlp.Transaction.GetObject(prWallRes.ObjectId, OpenMode.ForRead);
                    double width = wall.WidthAtElevation(prElevRes.Value);
                    MgdDbg.Utils.AcadUi.PrintToCmdLine(string.Format("\nWidth at elevation {0}: {1}", Converter.DistanceToString(prElevRes.Value), Converter.DistanceToString(width)));				
                    
				    trHlp.Commit();
			    }
			}
		}

		public void
		WallArea()
		{
            m_db = MgdDbg.Utils.Db.GetCurDwg();

		    Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;
            Autodesk.AutoCAD.DatabaseServices.TransactionManager tm = m_db.TransactionManager;

                // prompt for the Wall Object
            PromptEntityOptions prWallOpts = new PromptEntityOptions("\nSelect a wall");
            prWallOpts.SetRejectMessage("\nSelected entity must be of type Wall.");
            prWallOpts.AddAllowedClass(typeof(ArchDb.Wall), false);
            
            while (true) {
                PromptEntityResult prWallRes = ed.GetEntity(prWallOpts);
                if (prWallRes.Status != PromptStatus.OK)
                    return;
                
			    using (TransactionHelper trHlp = new TransactionHelper(m_db)) {
				    trHlp.Start();
    				
                    ArchDb.Wall wall = (ArchDb.Wall)trHlp.Transaction.GetObject(prWallRes.ObjectId, OpenMode.ForRead);
                    double netAreaLeft = wall.Area(ArchDb.WallSide.Left, true);
                    double grossAreaLeft = wall.Area(ArchDb.WallSide.Left, false);
                    double netAreaRight = wall.Area(ArchDb.WallSide.Right, true);
                    double grossAreaRight = wall.Area(ArchDb.WallSide.Right, false);
                 
                        // we have the raw number for area, but we want to print it out in 
                        // human readable form.  Use the DwgSetup object to tell us how the
                        // user wants these values displayed.
				    ObjectId dwgSetVarsId = AecAppSrvcs.DrawingSetupVariables.GetInstance(m_db, true);
                    AecAppSrvcs.DrawingSetupVariables dwgVars = (AecAppSrvcs.DrawingSetupVariables)trHlp.Transaction.GetObject(dwgSetVarsId, OpenMode.ForRead);
                    
                    netAreaLeft    = dwgVars.ConvertToCurrentAreaDisplay(netAreaLeft);
                    grossAreaLeft  = dwgVars.ConvertToCurrentAreaDisplay(grossAreaLeft);
                    netAreaRight   = dwgVars.ConvertToCurrentAreaDisplay(netAreaRight);
                    grossAreaRight = dwgVars.ConvertToCurrentAreaDisplay(grossAreaRight);

                    ed.WriteMessage(string.Format("\nGross area left side:  {0}{1}", Converter.DistanceToString(grossAreaLeft, DistanceUnitFormat.Decimal, dwgVars.AreaPrecision), dwgVars.AreaSuffix));				
                    ed.WriteMessage(string.Format("\nNet area left side:    {0}{1}", Converter.DistanceToString(netAreaLeft, DistanceUnitFormat.Decimal, dwgVars.AreaPrecision), dwgVars.AreaSuffix));				
                    ed.WriteMessage(string.Format("\nGross area right side: {0}{1}", Converter.DistanceToString(grossAreaRight, DistanceUnitFormat.Decimal, dwgVars.AreaPrecision), dwgVars.AreaSuffix));				
                    ed.WriteMessage(string.Format("\nNet area right side:   {0}{1}", Converter.DistanceToString(netAreaRight, DistanceUnitFormat.Decimal, dwgVars.AreaPrecision), dwgVars.AreaSuffix));				
                    
				    trHlp.Commit();
			    }
			}
		}
		
	    public void
		WallShrinkwrap()
		{
            m_db = MgdDbg.Utils.Db.GetCurDwg();

		    Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;
            Autodesk.AutoCAD.DatabaseServices.TransactionManager tm = m_db.TransactionManager;
            
                // set up the default param options
            bool cutInterference = true;
			bool doEndcaps = true;
			bool cutDoorFrames = true;
			bool cutWindowFrames = true;
			bool alwaysCutBelowOpenings = true;
			bool alwaysCutBelowOpeningsAbove = true;
			bool cacheIt = false;
			bool doTrueCut = true;
			bool combineRings = true;
			
			double cutPlaneHeight = 42.0;
			bool cutOpenings = true;

                // prompt for the Wall Object
            PromptEntityOptions prWallOpts = new PromptEntityOptions("\nSelect a wall");
            prWallOpts.SetRejectMessage("\nSelected entity must be of type Wall.");
            prWallOpts.AddAllowedClass(typeof(ArchDb.Wall), false);
            
                // ask user for cut plane height and whether to include openings
            PromptDistanceOptions prCutPlaneOpt = new PromptDistanceOptions("\nCut plane height");
            prCutPlaneOpt.DefaultValue = cutPlaneHeight;
            prCutPlaneOpt.UseDefaultValue = true;
            
            while (true) {
                PromptEntityResult prWallRes = ed.GetEntity(prWallOpts);
                if (prWallRes.Status != PromptStatus.OK)
                    return;
                    
                PromptDoubleResult prCutPlaneRes = ed.GetDistance(prCutPlaneOpt);
                if (prCutPlaneRes.Status != PromptStatus.OK)
                    return;
                else
                    cutPlaneHeight = prCutPlaneRes.Value;
                    
                if (MgdDbg.Utils.AcadUi.PromptYesNo("\nCut openings", cutOpenings, out cutOpenings) != PromptStatus.OK)
                    return;
                
			    using (TransactionHelper trHlp = new TransactionHelper(m_db)) {
				    trHlp.Start();
				        				
                    ArchDb.Wall wall = (ArchDb.Wall)trHlp.Transaction.GetObject(prWallRes.ObjectId, OpenMode.ForRead);
                    AecGe.Profile prof = wall.GetWallShrinkWrap(cutPlaneHeight, cutOpenings, cutInterference, doEndcaps, cutDoorFrames, cutWindowFrames,
                                alwaysCutBelowOpenings, alwaysCutBelowOpeningsAbove, cacheIt, doTrueCut, combineRings);
                                
                        // add a polyline for each Ring of the resulting profile
                    if (prof != null) {
                        prof.TransformBy(wall.Ecs);
                        int numRings = prof.RingCount;
                        for (int i=0; i<numRings; i++) {
                            Polyline pline = Utils.Db.CompCurve2dToPolyline(prof.Ring(i));
                            if (pline != null) {
                                pline.ColorIndex = 1;
                                MgdDbg.Utils.SymTbl.AddToCurrentSpace(pline, m_db, trHlp.Transaction);
                            }
                        }
                    }

				    trHlp.Commit();
			    }
			}
		}
		


        public void
        WallShrinkwrapBody()
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;
            Autodesk.AutoCAD.DatabaseServices.TransactionManager tm = m_db.TransactionManager;
            
            // set up the default param options
            bool cacheGraphics      = false;
            bool doMergers          = false;
            bool applyBodyModifiers = false;
            bool cutOpenings        = true;
            bool doInterferences    = true;

            // prompt for the Wall Object
            PromptEntityOptions prWallOpts = new PromptEntityOptions("\nSelect a wall");
            prWallOpts.SetRejectMessage("\nSelected entity must be of type Wall.");
            prWallOpts.AddAllowedClass(typeof(ArchDb.Wall), false);
            
            // ask user whether to include openings and interferences
            
            while (true) 
            {
                PromptEntityResult prWallRes = ed.GetEntity(prWallOpts);
                if (prWallRes.Status != PromptStatus.OK)
                    return;
                    
                if (MgdDbg.Utils.AcadUi.PromptYesNo("\nCut openings", cutOpenings, out cutOpenings) != PromptStatus.OK)
                    return;
                
                if (MgdDbg.Utils.AcadUi.PromptYesNo("\nDo interferences", doInterferences, out doInterferences) != PromptStatus.OK)
                    return;
                
                using (TransactionHelper trHlp = new TransactionHelper(m_db)) 
                {
                    trHlp.Start();
				        				
                    ArchDb.Wall wall = (ArchDb.Wall)trHlp.Transaction.GetObject(prWallRes.ObjectId, OpenMode.ForRead);

                    AecModeler.Body shrinkWrapBody = null;

                    wall.GetShrinkWrapBody(out shrinkWrapBody, cacheGraphics, doMergers, 
                                           cutOpenings, doInterferences, applyBodyModifiers );

                    AecDb.MassElement mass = new AecDb.MassElement();

                    // entity properties
                    mass.Description = "Created in .NET";

                    // geo properties
                    mass.Location = wall.Location;
                    mass.Normal = wall.Normal;
                    mass.Rotation = wall.Rotation;

                    // body
                    mass.ChangeTo(AecDb.ShapeType.BoundaryRepresentation);
                    mass.SetBody(shrinkWrapBody, false);

                    MgdDbg.Utils.SymTbl.AddToCurrentSpace(mass, m_db, trHlp.Transaction);

                    trHlp.Commit();
                } //end using
            }
        }

		/// <summary>
		/// TBD: there seems to be a bug in the Wall.GetComponentPline() function that returns
		/// the last point way off the end of the wall. (jma - 11/09/04)
		/// </summary>
	    public void
		WallComponentPline()
		{
            m_db = MgdDbg.Utils.Db.GetCurDwg();

		    Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;
            Autodesk.AutoCAD.DatabaseServices.TransactionManager tm = m_db.TransactionManager;
            
                // set up some defaults
            bool leftSide = true;
            double height = 0.0;
            double startParam = 0.0;
            double endParam = 0.0;
            
                // prompt for the Wall Object
            PromptEntityOptions prWallOpts = new PromptEntityOptions("\nSelect a wall");
            prWallOpts.SetRejectMessage("\nSelected entity must be of type Wall.");
            prWallOpts.AddAllowedClass(typeof(ArchDb.Wall), false);
            
                // ask user for cut plane height and whether to include openings
            PromptDistanceOptions prHeightOpt = new PromptDistanceOptions("\nHeight");
            prHeightOpt.DefaultValue = height;
            prHeightOpt.UseDefaultValue = true;
                        
            while (true) {
                PromptEntityResult prWallRes = ed.GetEntity(prWallOpts);
                if (prWallRes.Status != PromptStatus.OK)
                    return;
                    
                PromptDoubleResult prHeightRes = ed.GetDistance(prHeightOpt);
                if (prHeightRes.Status != PromptStatus.OK)
                    return;
                height = prHeightRes.Value;
                prHeightOpt.DefaultValue = height;
                    
                if (MgdDbg.Utils.AcadUi.PromptYesNo("\nLeft side", leftSide, out leftSide) != PromptStatus.OK)
                    return;
                
			    using (TransactionHelper trHlp = new TransactionHelper(m_db)) {
				    trHlp.Start();
				        				
                    ArchDb.Wall wall = (ArchDb.Wall)trHlp.Transaction.GetObject(prWallRes.ObjectId, OpenMode.ForRead);
                    ArchDb.WallStyle wallStyle = (ArchDb.WallStyle)trHlp.Transaction.GetObject(wall.StyleId, OpenMode.ForRead);
                    
                    startParam = wall.StartParam;
                    endParam = wall.EndParam;
                    
                    ArchDb.WallStyleComponentCollection wsCompColl = wallStyle.Components;
                    for(int i = 0; i < wsCompColl.Count; i++) {
                        AecGe.Polyline2d pline2d = wall.GetComponentPline(i, leftSide, wallStyle, startParam, endParam, height);
                        if (pline2d != null) {
                            pline2d.TransformBy(wall.Ecs);
                            Polyline pline = Utils.Db.CompCurve2dToPolyline(pline2d);
                            if (pline != null) {
                                pline.ColorIndex = i+1;
                                MgdDbg.Utils.SymTbl.AddToCurrentSpace(pline, m_db, trHlp.Transaction);
                            }
                        }
                    }

				    trHlp.Commit();
			    }
			}
		}
		
		public void
		WallMinAndMaxHeights()
		{
            m_db = MgdDbg.Utils.Db.GetCurDwg();

		    Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;
		    ObjectId wallId;
		    
		    while (SelectWall(out wallId)) {
                using (TransactionHelper trHlp = new TransactionHelper(m_db)) {
				    trHlp.Start();
                    
                    double minHt, maxHt;
                    ArchDb.Wall wall = (ArchDb.Wall)trHlp.Transaction.GetObject(wallId, OpenMode.ForRead);
                    wall.GetMinAndMaxHeight(out minHt, out maxHt);
                    
                    ed.WriteMessage(string.Format("\nMin height: {0}", Converter.DistanceToString(minHt)));
                    ed.WriteMessage(string.Format("\nMax height: {0}", Converter.DistanceToString(maxHt)));
                    
                    trHlp.Commit();
                }
            }
		}
		
	    public void
		WallActualHeightVariations()
		{
            m_db = MgdDbg.Utils.Db.GetCurDwg();

		    Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;
		    ObjectId wallId;
		    
		    while (SelectWall(out wallId)) {
                using (TransactionHelper trHlp = new TransactionHelper(m_db)) {
				    trHlp.Start();
                    
                    DoubleCollection heights = new DoubleCollection();
                    ArchDb.Wall wall = (ArchDb.Wall)trHlp.Transaction.GetObject(wallId, OpenMode.ForRead);
                    wall.GetActualHeightVariations(ref heights);
                    
                    for (int i=0; i < heights.Count; i++) {
                        ed.WriteMessage(string.Format("\nHeight {0:d}: {1}", i, Converter.DistanceToString(heights[i])));
                    }
                    
                    trHlp.Commit();
                }
            }
		}

        public void
        WallAnchoredOpenings()
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;
            Autodesk.AutoCAD.DatabaseServices.TransactionManager tm = m_db.TransactionManager;

            // prompt for the Wall Object
            PromptEntityOptions prWallOpts = new PromptEntityOptions("\nSelect a wall");
            prWallOpts.SetRejectMessage("\nSelected entity must be of type Wall.");
            prWallOpts.AddAllowedClass(typeof(ArchDb.Wall), false);

            while (true)
            {
                PromptEntityResult prWallRes = ed.GetEntity(prWallOpts);
                if (prWallRes.Status != PromptStatus.OK)
                    return;

                using (TransactionHelper trHlp = new TransactionHelper(m_db))
                {
                    trHlp.Start();

                    ArchDb.Wall wall = (ArchDb.Wall)trHlp.Transaction.GetObject(prWallRes.ObjectId, OpenMode.ForRead);
                    AcDb.ObjectIdCollection openingIds = wall.GetOpeningsFor();

                    if (openingIds.Count == 0) {
                        MgdDbg.Utils.AcadUi.PrintToCmdLine(string.Format("\nNo openings are anchored to this wall"));
                    }
                    else {
                        int iDoorCount = 0;
                        int iWindowCount = 0;
                        int iOpeningCount = 0;
                        int iDoorWinAssemblyCount = 0;
                        foreach (ObjectId id in openingIds)
                        {
                            if (id.ObjectClass.IsDerivedFrom(RXObject.GetClass(typeof(ArchDb.Door)))) {
                                iDoorCount++;
                            }
                            else if (id.ObjectClass.IsDerivedFrom(RXObject.GetClass(typeof(ArchDb.Window))))
                            {
                                iWindowCount++;
                            }
                            else if (id.ObjectClass.IsDerivedFrom(RXObject.GetClass(typeof(ArchDb.Opening))))
                            {
                                iOpeningCount++;
                            }
                            else if (id.ObjectClass.IsDerivedFrom(RXObject.GetClass(typeof(ArchDb.WindowAssembly))))
                            {
                                iDoorWinAssemblyCount++;
                            }


                        }

                        MgdDbg.Utils.AcadUi.PrintToCmdLine(string.Format("\nThis wall has - {0} Doors, {1} Windows , {2} Openings, {3} Door window assemblies anchored to it", iDoorCount, iWindowCount, iOpeningCount, iDoorWinAssemblyCount));				

                    }

                    trHlp.Commit();
                }
            }
        }

		
		private bool
		SelectWall(out ObjectId wallId)
		{
            m_db      = MgdDbg.Utils.Db.GetCurDwg();
		    Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;
		
                // prompt for the Wall Object
            PromptEntityOptions prWallOpts = new PromptEntityOptions("\nSelect a wall");
            prWallOpts.SetRejectMessage("\nSelected entity must be of type Wall.");
            prWallOpts.AddAllowedClass(typeof(ArchDb.Wall), false);
            
            PromptEntityResult prWallRes = ed.GetEntity(prWallOpts);
            if (prWallRes.Status != PromptStatus.OK) {
                wallId = ObjectId.Null;
                return false;
            }
            else {
                wallId = prWallRes.ObjectId;
                return true;
            }
		}
	}	
}
