
// $Header: //Desktop/Source/TestTools/MgdDbgAec/ObjTests/QueryBase.cs#5 $
// $Author: mehtam $ $DateTime: 2008/03/03 11:13:18 $ $Change: 94460 $
//
// (C) Copyright 2004 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//



//#define VS2003

using System;
using System.Diagnostics;
using System.Collections;
using System.Windows.Forms;
using Autodesk.AutoCAD.DatabaseServices;
using Autodesk.AutoCAD.Geometry;
using Autodesk.AutoCAD.Colors;
using Autodesk.AutoCAD.ApplicationServices;
using Autodesk.AutoCAD.EditorInput;
using Autodesk.AutoCAD.Runtime;
using MgdDbg;
using MgdDbg.Test;

using AcDb        = Autodesk.AutoCAD.DatabaseServices;
using AcGe        = Autodesk.AutoCAD.Geometry;
using AecDb       = Autodesk.Aec.DatabaseServices;
using AecGe       = Autodesk.Aec.Geometry;
using AecAppSrvcs = Autodesk.Aec.ApplicationServices;
using ArchDb      = Autodesk.Aec.Arch.DatabaseServices;
using AcadApp     = Autodesk.AutoCAD.ApplicationServices.Application;
using AecDbArch   = Autodesk.Aec.Arch.DatabaseServices;

namespace MgdDbgAec.ObjTests
{
	/// <summary>
	/// Summary description for QueryEntTests.
	/// </summary>
	public class QueryBase : MgdDbgTestFuncs
	{
        private Database    m_db = null;

		public
		QueryBase()
		{		    

            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Get Base Curve", "Add a base curve to the drawing", typeof(AecDb.Entity), new MgdDbgTestFuncInfo.TestFunc(BaseCurve), MgdDbgTestFuncInfo.TestType.Query));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Get Base Profile", "Add curves to the drawing representing base profile", typeof(AecDb.Entity), new MgdDbgTestFuncInfo.TestFunc(BaseProfile), MgdDbgTestFuncInfo.TestType.Query));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Get Cut Profile", "Add curves to the drawing representing a cut plane", typeof(AecDb.Entity), new MgdDbgTestFuncInfo.TestFunc(CutProfile), MgdDbgTestFuncInfo.TestType.Query));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Get Usage Area", "Add curves to the drawing representing usage area", typeof(AecDb.Entity), new MgdDbgTestFuncInfo.TestFunc(UsageArea), MgdDbgTestFuncInfo.TestType.Query));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Get Local Model Cached Extents", "Add a box to show cached extents", typeof(AecDb.Entity), new MgdDbgTestFuncInfo.TestFunc(LocalModelCachedExtents), MgdDbgTestFuncInfo.TestType.Query));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Get World Extents", "Add a box to show world extents", typeof(AecDb.Entity), new MgdDbgTestFuncInfo.TestFunc(WorldExtents), MgdDbgTestFuncInfo.TestType.Query));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Get Local Extents", "Add a box to show local extents", typeof(AecDb.Entity), new MgdDbgTestFuncInfo.TestFunc(LocalExtents), MgdDbgTestFuncInfo.TestType.Query));
            
			m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Mass Element Style Dictionary", "Basic dictionary query", typeof(AecDb.Dictionary), new MgdDbgTestFuncInfo.TestFunc(MassElementStyleDictionary), MgdDbgTestFuncInfo.TestType.Query));

            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Get Drawing Setup Variables", "Queries the values of the Drawing Setup Variables", typeof(AecAppSrvcs.DrawingSetupVariables), new MgdDbgTestFuncInfo.TestFunc(DwgSetupVariables), MgdDbgTestFuncInfo.TestType.Query));

            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Get Closest Layout Cell",   "Gets the closest layout cell for a given point, plus the centroid of the cell and its profile", typeof(AecDb.CellLayoutTool), new MgdDbgTestFuncInfo.TestFunc(GetClosestLayoutCell), MgdDbgTestFuncInfo.TestType.Query));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Get Closest Layout node",   "Gets the closest layout node for a given point, plus the coordinate system and location of that node.", typeof(AecDb.LayoutTool), new MgdDbgTestFuncInfo.TestFunc(GetClosestLayoutNode), MgdDbgTestFuncInfo.TestType.Query));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Get Closest Layout volume", "Gets the closest layout volume for a given point, plus the volume body, centroid, and extents", typeof(AecDb.VolumeLayoutTool), new MgdDbgTestFuncInfo.TestFunc(GetClosestLayoutVolume), MgdDbgTestFuncInfo.TestType.Query));

            // LayoutGrid2d tests
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Get Boundary Profile", "Add curves to the drawing representing the boundary profile", typeof(AecDb.LayoutGrid2d), new MgdDbgTestFuncInfo.TestFunc(BoundaryProfile), MgdDbgTestFuncInfo.TestType.Query));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("List opposite layout nodes", "Lists the opposite layout node for each node in the LayoutGrid2d", typeof(AecDb.LayoutGrid2d), new MgdDbgTestFuncInfo.TestFunc(ListOpposite), MgdDbgTestFuncInfo.TestType.Query));

            // LayoutGrid3d tests
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Get Layout Volume Extent", "Gets the layout volume extent of all volumes of the LayoutGrid3d", typeof(AecDb.LayoutGrid3d), new MgdDbgTestFuncInfo.TestFunc(GetLayoutVolumeExtent), MgdDbgTestFuncInfo.TestType.Query));

            // GridAssembly tests
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Find All Boundary Edge Assignments", "Displays all the Boundary Edge Assignments of a GridDefinition of a GridAssembly", typeof(AecDb.GridAssembly), new MgdDbgTestFuncInfo.TestFunc(FindBoundaryEdgeAssignment), MgdDbgTestFuncInfo.TestType.Query));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Find All Internal Edge Assignments", "Displays all the Internal Edge Assignments of a GridDefinition of a GridAssembly", typeof(AecDb.GridAssembly), new MgdDbgTestFuncInfo.TestFunc(FindInternalEdgeAssignment), MgdDbgTestFuncInfo.TestType.Query));
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Find All Cell Assignments", "Displays all the Cell Assignments of a GridDefinition of a GridAssembly", typeof(AecDb.GridAssembly), new MgdDbgTestFuncInfo.TestFunc(FindCellAssignments), MgdDbgTestFuncInfo.TestType.Query));

            // GridCustom tests
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Find All intersection points on a GridCustom", "Adds points on all intersections of a GridCustom", typeof(AecDb.GridCustom), new MgdDbgTestFuncInfo.TestFunc(FindIntersections), MgdDbgTestFuncInfo.TestType.Query));

            // Number of Xrefs test
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("Find number of Xrefs", "Find number of Xrefs in the database for the entity selected", typeof(AecDb.Entity), new MgdDbgTestFuncInfo.TestFunc(FindNumberOfXrefs), MgdDbgTestFuncInfo.TestType.Query));

            // Entity Components test
            m_testFrameworkFuncs.Add(new MgdDbgTestFuncInfo("List material components", "Lists material components for the selected entity and their material names from entity style", typeof(AecDb.Entity), new MgdDbgTestFuncInfo.TestFunc(ListMaterialComponents), MgdDbgTestFuncInfo.TestType.Query));        
        }
        
        public void
        BaseCurve()
        {
            m_db      = MgdDbg.Utils.Db.GetCurDwg();

            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;
            Autodesk.AutoCAD.DatabaseServices.TransactionManager tm = m_db.TransactionManager;

            PromptEntityOptions prEntOpts = new PromptEntityOptions("\nSelect entity to create base curve");
            prEntOpts.SetRejectMessage("\nSelected entity must be of type AecEntity.");
            prEntOpts.AddAllowedClass(typeof(AecDb.Entity), false);

            while (true) {
                PromptEntityResult prEntRes = ed.GetEntity(prEntOpts);
                if (prEntRes.Status != PromptStatus.OK)
                    return;
                    
                using (Autodesk.AutoCAD.DatabaseServices.Transaction tr = tm.StartTransaction()) {
                    AecDb.Entity ent = (AecDb.Entity)tr.GetObject(prEntRes.ObjectId, OpenMode.ForRead);
                    
                    AcDb.Curve baseCurve = ent.BaseCurve;
                    if (baseCurve != null) {
                        baseCurve.ColorIndex = 1;
                        MgdDbg.Utils.SymTbl.AddToCurrentSpace(baseCurve, m_db, tr);
                    }
                    tr.Commit();
                }
            }
        }
        
        public void
        BaseProfile()
        {
            m_db      = MgdDbg.Utils.Db.GetCurDwg();

            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;
            Autodesk.AutoCAD.DatabaseServices.TransactionManager tm = m_db.TransactionManager;

            PromptEntityOptions prEntOpts = new PromptEntityOptions("\nSelect entity to create base profile");
            prEntOpts.SetRejectMessage("\nSelected entity must be of type AecEntity.");
            prEntOpts.AddAllowedClass(typeof(AecDb.Entity), false);

            while (true) {
                PromptEntityResult prEntRes = ed.GetEntity(prEntOpts);
                if (prEntRes.Status != PromptStatus.OK)
                    return;
                    
                using (Autodesk.AutoCAD.DatabaseServices.Transaction tr = tm.StartTransaction()) {
                    AecDb.Entity ent = (AecDb.Entity)tr.GetObject(prEntRes.ObjectId, OpenMode.ForRead);
                    
                    if (ent.SupportsProfileCommands) {
                        Matrix3d ecsToWcs;
                        AecGe.Profile prof = ent.GetBaseProfile(out ecsToWcs);
                        if (prof != null) {
                            int numRings = prof.RingCount;
                            for (int i=0; i<numRings; i++) {
                                Polyline pline = Utils.Db.CompCurve2dToPolyline(prof.Ring(i));
                                if (pline != null) {
                                    pline.TransformBy(ecsToWcs);
                                    pline.ColorIndex = 2;
                                    MgdDbg.Utils.SymTbl.AddToCurrentSpace(pline, m_db, tr);
                                }
                            }
                        }
                    }
                    else {
                        MgdDbg.Utils.AcadUi.PrintToCmdLine("\nThis entity does not support Profile commands.");
                    }
                    
                    tr.Commit();
                }
            }
        }
        
        public void
        CutProfile()
        {
            DoProfilePlaneGuts(0, 2);
        }
        
        public void
        UsageArea()
        {
            DoProfilePlaneGuts(1, 3);
        }
        
        /// <summary>
        /// Do the common work for several functions
        /// </summary>
        /// <param name="funcId"></param>
        /// <param name="colorIndex"></param>
        private void
        DoProfilePlaneGuts(int funcId, int colorIndex)
        {
            m_db      = MgdDbg.Utils.Db.GetCurDwg();

            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;
            Autodesk.AutoCAD.DatabaseServices.TransactionManager tm = m_db.TransactionManager;

            PromptEntityOptions prEntOpts = new PromptEntityOptions("\nSelect entity");
            prEntOpts.SetRejectMessage("\nSelected entity must be of type AecEntity.");
            prEntOpts.AddAllowedClass(typeof(AecDb.Entity), false);
            
            PromptEntityOptions prEntPlaneOpts = new PromptEntityOptions("\nSelect entity to define plane");
            
            while (true) {
                PromptEntityResult prEntRes = ed.GetEntity(prEntOpts);
                if (prEntRes.Status != PromptStatus.OK)
                    return;
                    
                PromptEntityResult prEntPlaneRes = ed.GetEntity(prEntPlaneOpts);
                if (prEntPlaneRes.Status != PromptStatus.OK)
                    return;

                using (Autodesk.AutoCAD.DatabaseServices.Transaction tr = tm.StartTransaction()) {
                    AecDb.Entity ent = (AecDb.Entity)tr.GetObject(prEntRes.ObjectId, OpenMode.ForRead);
                    AcDb.Entity entCut = (AcDb.Entity)tr.GetObject(prEntPlaneRes.ObjectId, OpenMode.ForRead);
                                        
                    if (entCut.IsPlanar) {
                        Plane plane = entCut.GetPlane();
                        AecGe.Profile prof = null;
                        if (funcId == 0)
                            prof = ent.GetProfile(plane);
                        else if (funcId == 1)
                            prof = ent.GetUsageArea(plane);
                        
                        // The returned profile is relative to the plane we used to cut it.  If we want
                        // it to line up, we need to transform it back.
                        // TBD: GetCoordSystem() is crahsing, and I'm not sure how to do this anyway! (jma - 11/02/04)
                        /*CoordinateSystem2d planeEcs = plane.GetCoordinateSystem();
                        Point3d originPt = new Point3d(planeEcs.Origin.X, planeEcs.Origin.Y, 0.0);
                        Vector3d uDir = new Vector3d(planeEcs.Xaxis.X, planeEcs.Xaxis.Y, 0.0);
                        Vector3d vDir = new Vector3d(planeEcs.Yaxis.X, planeEcs.Yaxis.Y, 0.0);
                        Matrix3d m = Matrix3d.AlignCoordinateSystem(MgdDbg.Utils.Ge.kOrigin,
                                                    MgdDbg.Utils.Ge.kXAxis, MgdDbg.Utils.Ge.kYAxis, MgdDbg.Utils.Ge.kZAxis,
                                                    originPt, uDir, vDir, plane.Normal);*/
                                                    
                        if (prof != null) {
                            //prof.TransformBy(m);
                            int numRings = prof.RingCount;
                            for (int i=0; i<numRings; i++) {
                                Polyline pline = Utils.Db.CompCurve2dToPolyline(prof.Ring(i));
                                if (pline != null) {
                                    pline.ColorIndex = colorIndex;
                                    MgdDbg.Utils.SymTbl.AddToCurrentSpace(pline, m_db, tr);
                                }
                            }
                        }
                    }
                    else {
                        MgdDbg.Utils.AcadUi.PrintToCmdLine("\nEntity is not planar.");
                    }
                    
                    tr.Commit();
                }
            }
        }
                        
        public void
        LocalExtents()
        {
            DoExtentsGuts(0, 5);
        }
        
        public void
        WorldExtents()
        {
            DoExtentsGuts(1, 6);
        }

        public void
        LocalModelCachedExtents()
        {
            DoExtentsGuts(2, 3);
        }
        
        /// <summary>
        /// We're doing the same thing for the various extents functions, so just abstract out
        /// some things here.
        /// </summary>
        /// <param name="funcId">Hard coded Id of which function to use</param>
        /// <param name="colorIndex">color index to use for resulting Polylines</param>
        
        private void
        DoExtentsGuts(int funcId, int colorIndex)
        {
            m_db      = MgdDbg.Utils.Db.GetCurDwg();
            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;
            Autodesk.AutoCAD.DatabaseServices.TransactionManager tm = m_db.TransactionManager;

            PromptEntityOptions prEntOpts = new PromptEntityOptions("\nSelect entity to create extents box");
            prEntOpts.SetRejectMessage("\nSelected entity must be of type AecEntity.");
            prEntOpts.AddAllowedClass(typeof(AecDb.Entity), false);
                        
            while (true) {
                PromptEntityResult prEntRes = ed.GetEntity(prEntOpts);
                if (prEntRes.Status != PromptStatus.OK)
                    return;

                using (Autodesk.AutoCAD.DatabaseServices.Transaction tr = tm.StartTransaction()) {
                    AecDb.Entity ent = (AecDb.Entity)tr.GetObject(prEntRes.ObjectId, OpenMode.ForRead);
                    
                    BoundBlock3d bndBlk = null;
                    if (funcId == 0)
                        ent.GetLocalExtents(out bndBlk);
                    else if (funcId == 1) {
                        try {
                            ent.GetWorldExtents(out bndBlk);
                        }
                        catch (Autodesk.AutoCAD.Runtime.Exception e) {
                            if (e.ErrorStatus == Autodesk.AutoCAD.Runtime.ErrorStatus.NotApplicable)
                                ed.WriteMessage("\nWorld Extents: Not Applicable");
                            else
                                throw e;
                        }
                    }
                    else if (funcId == 2)
                        ent.GetLocalModelCachedExtents(out bndBlk);
                    else {
                        Debug.Assert(false);
                    }

                    Point3d centerPt = MgdDbg.Utils.Ge.Midpoint(bndBlk.GetMinimumPoint(), bndBlk.GetMaximumPoint());
                    
                    MgdDbg.Utils.AcadUi.PrintToCmdLine(string.Format("\nEXTMIN:    {0}", MgdDbg.Utils.AcadUi.PtToStr(bndBlk.GetMinimumPoint())));
                    MgdDbg.Utils.AcadUi.PrintToCmdLine(string.Format("\nEXTMAX:    {0}", MgdDbg.Utils.AcadUi.PtToStr(bndBlk.GetMaximumPoint())));
                    MgdDbg.Utils.AcadUi.PrintToCmdLine(string.Format("\nCENTER PT: {0}", MgdDbg.Utils.AcadUi.PtToStr(centerPt)));
                    
                    tr.Commit();
                    
                    Utils.Db.MakeExtentsBlock(bndBlk, colorIndex, m_db);
                }
            }
        }

        public void
        MassElementStyleDictionary()
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            using (AecDb.DictionaryMassElementStyle meDict = new AecDb.DictionaryMassElementStyle(m_db)) {
                MgdDbg.Snoop.Forms.Objects snoopForm = new MgdDbg.Snoop.Forms.Objects(meDict);
                snoopForm.ShowDialog();
            }
        }
		
		public void
		DwgSetupVariables()
		{
            m_db = MgdDbg.Utils.Db.GetCurDwg();

			ObjectId objId = AecAppSrvcs.DrawingSetupVariables.GetInstance(m_db, false);
			if (objId.IsNull == false) {
				using (TransactionHelper trHlp = new TransactionHelper(m_db)) {
				    trHlp.Start();
				    
					ObjectIdCollection coll = new ObjectIdCollection();
					coll.Add(objId);
					MgdDbg.Snoop.Forms.DBObjects snoopForm = new MgdDbg.Snoop.Forms.DBObjects(coll, trHlp);
					snoopForm.ShowDialog();
					
					trHlp.Commit();
				}
			}
			else {
				MgdDbg.Utils.AcadUi.PrintToCmdLine("\nDrawing Setup Variables not found.");
			}
		}


        public void
        GetClosestLayoutCell()
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;

            PromptEntityOptions prLayoutOpts = new PromptEntityOptions("\nSelect an object derived from CellLayoutTool");
            prLayoutOpts.SetRejectMessage("\nSelected object must be derived from CellLayoutTool.");
            prLayoutOpts.AddAllowedClass(typeof(AecDb.CellLayoutTool), false);
            prLayoutOpts.AllowObjectOnLockedLayer = false;
            
            PromptEntityResult prCellLayout = ed.GetEntity(prLayoutOpts);
            if (prCellLayout.Status != PromptStatus.OK)
                return;

            PromptPointOptions prPtOpt = new PromptPointOptions("\nPick a point to find the closest layout cell");
            PromptPointResult prPtRes = ed.GetPoint(prPtOpt);
            if (prPtRes.Status != PromptStatus.OK)
                return;
                
            Point3d pt = prPtRes.Value;

            using (TransactionHelper trHlp = new TransactionHelper(m_db)) 
            {
                trHlp.Start();
    			
                AecDb.CellLayoutTool cellLayoutTool = (AecDb.CellLayoutTool)trHlp.Transaction.GetObject(prCellLayout.ObjectId, OpenMode.ForRead);
                int cellId = cellLayoutTool.GetClosestLayoutCell(pt);
                MgdDbg.Utils.AcadUi.PrintToCmdLine("\nThe Closest cell id is: " + cellId.ToString() );
                Matrix3d m = cellLayoutTool.GetLayoutCellCentroid(cellId);
                MgdDbg.Utils.AcadUi.PrintToCmdLine("\nThe centroid for it is: " + m.ToString() );

                Matrix3d profCoords;
                AecGe.Profile prof = cellLayoutTool.GetLayoutCellProfile(cellId, out profCoords);
                if (prof != null) 
                {
                    int numRings = prof.RingCount;
                    for (int i=0; i<numRings; i++) 
                    {
                        Polyline pline = Utils.Db.CompCurve2dToPolyline(prof.Ring(i));
                        if (pline != null) 
                        {
                            pline.TransformBy(profCoords);
                            pline.ColorIndex = 2;
                            MgdDbg.Utils.SymTbl.AddToCurrentSpace(pline, m_db, trHlp.Transaction);
                        }
                    }
                }

                trHlp.Commit();  
            }

        }


        public void
        GetClosestLayoutNode()
        {
            m_db      = MgdDbg.Utils.Db.GetCurDwg();

            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;

            PromptEntityOptions prLayoutOpts = new PromptEntityOptions("\nSelect an object derived from LayoutTool");
            prLayoutOpts.SetRejectMessage("\nSelected object must be derived from LayoutTool.");
            prLayoutOpts.AddAllowedClass(typeof(AecDb.LayoutTool), false);
            prLayoutOpts.AllowObjectOnLockedLayer = false;
            
            PromptEntityResult prLayout = ed.GetEntity(prLayoutOpts);
            if (prLayout.Status != PromptStatus.OK)
                return;

            PromptPointOptions prPtOpt = new PromptPointOptions("\nPick a point to find the closest layout node");
            PromptPointResult prPtRes = ed.GetPoint(prPtOpt);
            if (prPtRes.Status != PromptStatus.OK)
                return;
                
            Point3d pt = prPtRes.Value;

            using (TransactionHelper trHlp = new TransactionHelper(m_db)) 
            {
                trHlp.Start();
    			
                AecDb.LayoutTool layoutTool = (AecDb.LayoutTool)trHlp.Transaction.GetObject(prLayout.ObjectId, OpenMode.ForRead);
                int nodeId = layoutTool.GetClosestLayoutNode(pt);
                MgdDbg.Utils.AcadUi.PrintToCmdLine("\nThe Closest node id is: " + nodeId.ToString() );
                Matrix3d m = layoutTool.GetLayoutNodeCoordinateSystem(nodeId);
                MgdDbg.Utils.AcadUi.PrintToCmdLine("\nThe coordinate system for it is: " + m.ToString() );
                Point3d ptNode = layoutTool.GetLayoutNodeLocation(nodeId);
                MgdDbg.Utils.AcadUi.PrintToCmdLine("\nThe location for it is: " + ptNode.ToString() );
                Utils.Db.MakePointEnt(ptNode, 1, trHlp.Transaction, m_db);

                trHlp.Commit();  
            }

        }


        public void
        GetClosestLayoutVolume()
        {
            m_db      = MgdDbg.Utils.Db.GetCurDwg();
            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;

            PromptEntityOptions prVolumeLayoutOpts = new PromptEntityOptions("\nSelect an object derived from VolumeLayoutTool");
            prVolumeLayoutOpts.SetRejectMessage("\nSelected object must be derived from VolumeLayoutTool.");
            prVolumeLayoutOpts.AddAllowedClass(typeof(AecDb.VolumeLayoutTool), false);
            prVolumeLayoutOpts.AllowObjectOnLockedLayer = false;
            
            PromptEntityResult prVolumeLayout = ed.GetEntity(prVolumeLayoutOpts);
            if (prVolumeLayout.Status != PromptStatus.OK)
                return;

            while(true)
            {
                PromptPointOptions prPtOpt = new PromptPointOptions("\nPick a point to find the closest VolumeLayout node");
                PromptPointResult prPtRes = ed.GetPoint(prPtOpt);
                if (prPtRes.Status != PromptStatus.OK)
                    return;
                    
                Point3d pt = prPtRes.Value;

                using (TransactionHelper trHlp = new TransactionHelper(m_db)) 
                {
                    trHlp.Start();
        			
                    AecDb.VolumeLayoutTool volumeLayoutTool = (AecDb.VolumeLayoutTool)trHlp.Transaction.GetObject(prVolumeLayout.ObjectId, OpenMode.ForRead);
                    int volumeId = volumeLayoutTool.GetClosestLayoutVolume(pt);
                    MgdDbg.Utils.AcadUi.PrintToCmdLine("\nThe Closest volume id is: " + volumeId.ToString() );

                    Matrix3d m = volumeLayoutTool.GetLayoutVolumeCentroid(volumeId);
                    MgdDbg.Utils.AcadUi.PrintToCmdLine("\nThe volume centroid for it is: " + m.ToString() );

                    Matrix3d toWcs;
                    double length, width, height;

                    volumeLayoutTool.GetLayoutVolumeExtent(volumeId, out length, out width, out height, out toWcs);
                    using (MgdDbg.CompBldrCurSpace compBldr = new MgdDbg.CompBldrCurSpace(m_db)) 
                    {
                        compBldr.Start();
                        compBldr.PushXform(MgdDbg.Utils.Db.GetUcsMatrix(m_db));

                        AecDb.MassElement massElem = AecDb.MassElement.Create(AecDb.ShapeType.Box);
                        massElem.ColorIndex = 3;
                        //massElem.Location = new Point3d(0.0, 0.0, 0.0);
                        // toWcs is the centroid in this case, so we have to 
                        // adjust the Z coordinate to position our volume on the 
                        // correct location!
                        Point3d ptOrigin = toWcs.CoordinateSystem3d.Origin;
                        massElem.Location = new Point3d(ptOrigin.X, ptOrigin.Y , ptOrigin.Z - (height / 2.0));
                        massElem.Width = width;
                        massElem.Depth = length;
                        massElem.Height = height;

                        compBldr.AddToDb(massElem);

                        compBldr.Commit();
                    }

                    trHlp.Commit();  
                }
            }
        }



#region LayoutGrid2d tests

        public void
        BoundaryProfile()
        {
            m_db      = MgdDbg.Utils.Db.GetCurDwg();
            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;
            Autodesk.AutoCAD.DatabaseServices.TransactionManager tm = m_db.TransactionManager;

            PromptEntityOptions prEntOpts = new PromptEntityOptions("\nSelect a LayoutGrid2d to create boundary profile");
            prEntOpts.SetRejectMessage("\nSelected entity must be of type LayoutGrid2d.");
            prEntOpts.AddAllowedClass(typeof(AecDb.LayoutGrid2d), false);

            while (true) 
            {
                PromptEntityResult prEntRes = ed.GetEntity(prEntOpts);
                if (prEntRes.Status != PromptStatus.OK)
                    return;
                    
                using (Autodesk.AutoCAD.DatabaseServices.Transaction tr = tm.StartTransaction()) 
                {
                    AecDb.LayoutGrid2d grid = (AecDb.LayoutGrid2d)tr.GetObject(prEntRes.ObjectId, OpenMode.ForRead);
                    AecGe.Profile prof = grid.BoundaryProfile;

                    if (prof != null) 
                    {
                        int numRings = prof.RingCount;
                        for (int i=0; i<numRings; i++) 
                        {
                            Polyline pline = Utils.Db.CompCurve2dToPolyline(prof.Ring(i));
                            if (pline != null) 
                            {
                                pline.ColorIndex = 2;
                                // set the location to the same as the layoutgrid's
                                Matrix3d ecsToWcs = grid.Ecs;
                                pline.TransformBy(ecsToWcs);

                                MgdDbg.Utils.SymTbl.AddToCurrentSpace(pline, m_db, tr);
                            }
                        }
                    }
                    else 
                    {
                        MgdDbg.Utils.AcadUi.PrintToCmdLine("\nThis LayoutGrid2d does not have a boundary profile.");
                    }
                    
                    tr.Commit();
                }
            }
        }


        public void
        ListOpposite()
        {
            m_db      = MgdDbg.Utils.Db.GetCurDwg();
            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;
            Autodesk.AutoCAD.DatabaseServices.TransactionManager tm = m_db.TransactionManager;

            PromptEntityOptions prEntOpts = new PromptEntityOptions("\nSelect a LayoutGrid2d");
            prEntOpts.SetRejectMessage("\nSelected entity must be of type LayoutGrid2d.");
            prEntOpts.AddAllowedClass(typeof(AecDb.LayoutGrid2d), false);

            while (true) 
            {
                PromptEntityResult prEntRes = ed.GetEntity(prEntOpts);
                if (prEntRes.Status != PromptStatus.OK)
                    return;
                    
                using (Autodesk.AutoCAD.DatabaseServices.Transaction tr = tm.StartTransaction()) 
                {
                    AecDb.LayoutGrid2d grid = (AecDb.LayoutGrid2d)tr.GetObject(prEntRes.ObjectId, OpenMode.ForRead);

#if(VS2003)
                    double angleOffset = 0.0;
                    double oppositeAngleOffset;
                    int oppositeId;
                    bool found;
                    
                    IntegerCollection ids = grid.LayoutNodeIds;
                    foreach(int id in ids)
                    {
                        found = grid.OppositeLayoutNode(id, angleOffset, out oppositeId, out oppositeAngleOffset);
                        if( found )
                            MgdDbg.Utils.AcadUi.PrintToCmdLine("\nNode id: " + id.ToString() + " with angle offset = " + angleOffset.ToString() + " has opposite node id: " + oppositeId.ToString() + " with opposite angle offset = " + oppositeAngleOffset.ToString() );
                        else
                            MgdDbg.Utils.AcadUi.PrintToCmdLine("\nDid not find an opposite node for Node id: " + id.ToString() );
                    }
#endif

                    tr.Commit();
                }
            }
        }
#endregion


#region LayoutGrid3d tests


        public void
        GetLayoutVolumeExtent()
        {
            m_db      = MgdDbg.Utils.Db.GetCurDwg();
            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;

            // select grid
            PromptEntityOptions prLayoutGridOpts = new PromptEntityOptions("\nSelect a LayoutGrid3d");
            prLayoutGridOpts.SetRejectMessage("\nSelected entity must be of type LayoutGrid3d.");
            prLayoutGridOpts.AddAllowedClass(typeof(AecDb.LayoutGrid3d), false);
            
            PromptEntityResult prLayoutGridRes = ed.GetEntity(prLayoutGridOpts);
            if (prLayoutGridRes.Status != PromptStatus.OK)
                return;
                 
            using (TransactionHelper trHlp = new TransactionHelper(m_db)) 
            {
                trHlp.Start();
                
                AecDb.LayoutGrid3d layoutGrid = (AecDb.LayoutGrid3d)trHlp.Transaction.GetObject(prLayoutGridRes.ObjectId, OpenMode.ForRead);
                AecDb.GridUvw grid = layoutGrid.BaseGrid;

                if (grid == null) 
                {
                    MgdDbg.Utils.AcadUi.PrintToCmdLine("\nNo base grid available.");
                    trHlp.Abort();
                    return;
                }

                IntegerCollection volIds = grid.LayoutVolumeIds;
                String tempStr;
                double length;
                double width;
                double height;
                Matrix3d mat;
                MgdDbg.Utils.AcadUi.PrintToCmdLine("\nNumber of volumes: " + volIds.Count.ToString() );

                // due to a compiler bug in VS2002 I can't use a foreach here. If you are using VS2003 or above, then you can use foreach on IntegerCollections.
                for (int n = 0; n < volIds.Count; n++) {
                    grid.GetLayoutVolumeExtent(volIds[n], out length, out width, out height, out mat);
                    tempStr = "\n\nVolume Id: " + volIds[n].ToString() + " length: " + length.ToString() + " width: " + width.ToString() + " height: " + height.ToString() + "\n toWcs: " + mat.ToString();
                    MgdDbg.Utils.AcadUi.PrintToCmdLine(tempStr);
                }

                trHlp.Commit();
            }
        }


#endregion


#region LayoutGrid3d tests

        public void
        FindBoundaryEdgeAssignment()
        {
            FindEdgeAssignments(false);
        }


        public void
        FindInternalEdgeAssignment()
        {
            FindEdgeAssignments(true);
        }

        public void
        FindEdgeAssignments(bool lookForInternalEdge)
        {
            m_db      = MgdDbg.Utils.Db.GetCurDwg();
            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;

            // select grid assembly
            PromptEntityOptions prGridAssemblyOpts = new PromptEntityOptions("\nSelect a GridAssembly");
            prGridAssemblyOpts.SetRejectMessage("\nSelected entity must be of type GridAssembly.");
            prGridAssemblyOpts.AddAllowedClass(typeof(AecDb.GridAssembly), false);
            
            PromptEntityResult prGridAssemblyRes = ed.GetEntity(prGridAssemblyOpts);
            if (prGridAssemblyRes.Status != PromptStatus.OK)
                return;

            using (TransactionHelper trHlp = new TransactionHelper(m_db)) {
                trHlp.Start();
                
                AecDb.GridAssembly gridAssembly = (AecDb.GridAssembly)trHlp.Transaction.GetObject(prGridAssemblyRes.ObjectId, OpenMode.ForRead);

                if (gridAssembly.GridDefinitionLocation == AecDb.GridDefinitionLocationType.StyleBased) {
                    MgdDbg.Utils.AcadUi.PrintToCmdLine("\nGridAssembly is style based.");
                    return;
                }
                AecDb.GridAssemblyDefinition gridAssemblyDef = gridAssembly.GridAssemblyDefinition;

                if (gridAssemblyDef == null) {
                    MgdDbg.Utils.AcadUi.PrintToCmdLine("\nGridAssemblyDefinition was null.");
                    return;
                }
                int count = gridAssembly.EdgeCount;
                if (count == 0) {
                    MgdDbg.Utils.AcadUi.PrintToCmdLine("\nNo edges...");
                    return;
                }
                MgdDbg.Utils.AcadUi.PrintToCmdLine("\nNumber of edges to search for: " + count.ToString() + " edges.");
                AecDb.GridAssemblyEdge edge = null;
                AecDb.NestedGridEdgeAssignment edgeAsn = null;
                ArrayList array = new ArrayList();
                for (int n=0; n < count; n++) {
                    edge =  gridAssembly.GetEdgeAt(n);
                    MgdDbg.Utils.AcadUi.PrintToCmdLine("\nSearching edge with id = " + edge.EdgeId );

                    if (edge == null)
                        continue;

                    if (lookForInternalEdge)  {
                        if (!edge.IsInternalEdge)
                            continue;
                        edgeAsn = gridAssemblyDef.SpecificInteriorEdgeAssignments.GetId(edge.EdgeAssignmentId);
                    }
                    else {
                        if (edge.IsInternalEdge)
                            continue;
                        edgeAsn = gridAssemblyDef.SpecificBoundaryEdgeAssignments.GetId(edge.EdgeAssignmentId);
                    }

                    if (edgeAsn != null) {
                        array.Add(edgeAsn);
                        MgdDbg.Utils.AcadUi.PrintToCmdLine("\nEdge with id = " + edge.EdgeId.ToString() + " has edge assignment id = " + edge.EdgeAssignmentId.ToString() );
                    }
                }
                if (array.Count == 0) {
                    MgdDbg.Utils.AcadUi.PrintToCmdLine("\nDidn't find any edge assignments.");
                    return;
                }
                else {
                    String tempStr = (lookForInternalEdge ? " internal" : " boundary" );
                    MgdDbg.Utils.AcadUi.PrintToCmdLine("\nFound " + array.Count.ToString() + tempStr + " edge assignments.");
                }
                // display form with edge assignments
                MgdDbg.Snoop.Forms.Objects snoopForm = new MgdDbg.Snoop.Forms.Objects(array);
                snoopForm.ShowDialog();
            }
        }


        public void
        FindCellAssignments()
        {
            m_db      = MgdDbg.Utils.Db.GetCurDwg();
            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;

            // select grid assembly
            PromptEntityOptions prGridAssemblyOpts = new PromptEntityOptions("\nSelect a GridAssembly");
            prGridAssemblyOpts.SetRejectMessage("\nSelected entity must be of type GridAssembly.");
            prGridAssemblyOpts.AddAllowedClass(typeof(AecDb.GridAssembly), false);
            
            PromptEntityResult prGridAssemblyRes = ed.GetEntity(prGridAssemblyOpts);
            if (prGridAssemblyRes.Status != PromptStatus.OK)
                return;

            using (TransactionHelper trHlp = new TransactionHelper(m_db)) 
            {
                trHlp.Start();
                
                AecDb.GridAssembly gridAssembly = (AecDb.GridAssembly)trHlp.Transaction.GetObject(prGridAssemblyRes.ObjectId, OpenMode.ForRead);

                if(gridAssembly.GridDefinitionLocation == AecDb.GridDefinitionLocationType.StyleBased)
                {
                    MgdDbg.Utils.AcadUi.PrintToCmdLine("\nGridAssembly is style based.");
                    return;
                }
                AecDb.GridAssemblyDefinition gridAssemblyDef = gridAssembly.GridAssemblyDefinition;

                if( gridAssemblyDef == null ) 
                {
                    MgdDbg.Utils.AcadUi.PrintToCmdLine("\nGridAssemblyDefinition was null.");
                    return;
                }
                int count = gridAssembly.CellCount;
                if (count == 0)
                {
                    MgdDbg.Utils.AcadUi.PrintToCmdLine("\nNo cells...");
                    return;
                }
                MgdDbg.Utils.AcadUi.PrintToCmdLine("\nNumber of cells to search for: " + count.ToString());
                AecDb.GridAssemblyCell cell = null;
                AecDb.NestedGridCellAssignment cellAsn = null;
                ArrayList array = new ArrayList();
                for (int n=0; n < count; n++)
                {
                    cell =  gridAssembly.GetCellAt(n);
                    MgdDbg.Utils.AcadUi.PrintToCmdLine("\nSearching cell with id = " + cell.CellId );

                    if (cell == null)
                        continue;

                    cellAsn = gridAssemblyDef.SpecificCellAssignments.GetId(cell.CellAssignmentId);

                    if (cellAsn != null) 
                    {
                        array.Add(cellAsn);
                        MgdDbg.Utils.AcadUi.PrintToCmdLine("\nCell with id = " + cell.CellId.ToString() + " has cell assignment id = " + cell.CellAssignmentId.ToString() );
                    }
                }
                if (array.Count == 0) 
                {
                    MgdDbg.Utils.AcadUi.PrintToCmdLine("\nDidn't find any cell assignments.");
                    return;
                }
                else 
                    MgdDbg.Utils.AcadUi.PrintToCmdLine("\nFound " + array.Count.ToString() + " cell assignments.");

                // display form with cell assignments
                MgdDbg.Snoop.Forms.Objects snoopForm = new MgdDbg.Snoop.Forms.Objects(array);
                snoopForm.ShowDialog();
            }
        }


#endregion

#region GridCustom tests

        public void
        FindIntersections()
        {
            m_db      = MgdDbg.Utils.Db.GetCurDwg();
            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;

            // select grid assembly
            PromptEntityOptions prGridCustomOpts = new PromptEntityOptions("\nSelect a Custom LayoutGrid2d");
            prGridCustomOpts.SetRejectMessage("\nSelected entity must be of type LayoutGrid2d.");
            prGridCustomOpts.AddAllowedClass(typeof(AecDb.LayoutGrid2d), false);
            
            PromptEntityResult prGridCustomRes = ed.GetEntity(prGridCustomOpts);
            if (prGridCustomRes.Status != PromptStatus.OK)
                return;

            using (TransactionHelper trHlp = new TransactionHelper(m_db)) 
            {
                trHlp.Start();
                
                AecDb.LayoutGrid2d layoutGrid = (AecDb.LayoutGrid2d)trHlp.Transaction.GetObject(prGridCustomRes.ObjectId, OpenMode.ForRead);

                if(layoutGrid == null)
                    return;

                AecDb.GridCustom customGrid = layoutGrid.CustomGrid;
                if(customGrid == null)
                {
                    MgdDbg.Utils.AcadUi.PrintToCmdLine("\nThis layout grid is not a custom grid.");
                    trHlp.Abort();
                    return;
                }
                int count = customGrid.NodeCount;
                if (count == 0)
                {
                    MgdDbg.Utils.AcadUi.PrintToCmdLine("\nNo nodes...");
                    trHlp.Abort();
                    return;
                }
                MgdDbg.Utils.AcadUi.PrintToCmdLine("\nNumber of nodes to search for: " + count.ToString());

                AecDb.GridCustomNode node = null;
                Matrix3d mat = layoutGrid.Ecs;

                for (int n=0; n < count; n++)
                {
                    node =  customGrid.CustomGridNodeAt(n);
                    MgdDbg.Utils.AcadUi.PrintToCmdLine("\nSearching node with id = " + node.NodeId );

                    if (node == null)
                        continue;
                    
                    IntegerCollection segIds = null;
                    DoubleCollection segParams = null; 
                    node.GetIntersectionIds(out segIds, out segParams);

                    // need 2 segments to have an intersection
                    if(segIds.Count > 1)
                    {
                        MgdDbg.Utils.AcadUi.PrintToCmdLine("\nFound " + segIds.Count.ToString() + " segment ids and " + segParams.Count.ToString() + " segmParameters." );
                        AecDb.GridCustomSegment seg = null;
                        AcGe.PointOnCurve2d ptOnCurve = null;
                        Vector2d segDir;

                        for(int i = 0; i < segIds.Count; i++)
                        {
#if !ACA2009
                            seg = customGrid.MainSegment(segIds[i]);
#endif

#if ACA2009
                            seg = customGrid.Segments.FindById(segIds[i]);
#endif

                            ptOnCurve = seg.GetPointOnCurve2d(segParams[i]);
                            segDir = ptOnCurve.GetDerivative(1);
                            
                            AcGe.Point3d pt3d = new AcGe.Point3d(ptOnCurve.Point.X, ptOnCurve.Point.Y, 0.0);
                            
                            pt3d = pt3d.TransformBy(mat);
                            Utils.Db.MakePointEnt(pt3d, 2, trHlp.Transaction, m_db);
                        }
                    }
                }

                trHlp.Commit();
            }
        }



#endregion


#region Number of Xrefs

        public bool
        IsBlockReferenceALayout(ObjectId blockRefId, TransactionHelper trHlp)
        {
            BlockReference blockRef = trHlp.Transaction.GetObject(blockRefId, OpenMode.ForRead) as BlockReference;
            AcDb.ObjectId blockId = blockRef.BlockId;
            if (blockId.IsErased)
            {
                return false;
            }

            BlockTableRecord xrefBlockTableRec = trHlp.Transaction.GetObject(blockId, OpenMode.ForRead) as BlockTableRecord;
            if (xrefBlockTableRec.IsLayout)
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        public void
        FindNumberOfXrefs()
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();
            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;
            while (true)
            {
                PromptNestedEntityOptions prOptNent = new PromptNestedEntityOptions("\nSelect entity: ");
                prOptNent.AppendKeywordsToMessage = true;
                prOptNent.Keywords.Add("Done");

                PromptNestedEntityResult result = ed.GetNestedEntity(prOptNent);
                if (result.Status == PromptStatus.OK)
                {
                    using (TransactionHelper trHlp = new TransactionHelper(m_db))
                    {
                        trHlp.Start();

                        // Getting database of entity to check if the entity lies in host database or Xref
                        AcDb.BlockTable blockTable = trHlp.Transaction.GetObject(m_db.BlockTableId, AcDb.OpenMode.ForRead) as AcDb.BlockTable;
                        AcDb.ObjectId modelSpaceId = blockTable[AcDb.BlockTableRecord.ModelSpace];

                        AcDb.ObjectId selectedObjectId = result.ObjectId;
                        AecDb.Entity entity = (AecDb.Entity)trHlp.Transaction.GetObject(selectedObjectId, OpenMode.ForRead);

                        AcDb.Database entityDatabase = entity.Database;
                        if (entityDatabase == m_db)
                        {
                            // Entity is from host database
                            MgdDbg.Utils.AcadUi.PrintToCmdLine(string.Format("\nThe enity is not from Xref"));
                        }
                        else
                        {
                            // Entity is from Xref

                            // Getting entity's block table record
                            AcDb.ObjectId xrefBlockId = entityDatabase.XrefBlockId;
                            BlockTableRecord xrefBlockTableRec = (BlockTableRecord)trHlp.Transaction.GetObject(xrefBlockId, OpenMode.ForRead);

                            // Getting total number of copies of the Xref
                            ObjectIdCollection blockRefSet = new ObjectIdCollection();
                            ObjectIdCollection layoutBlockRefSet = new ObjectIdCollection();
                            blockRefSet = xrefBlockTableRec.GetBlockReferenceIds(true, false);
                            int iLen = blockRefSet.Count;

                            // Removing erased/non-layout Xrefs from the set
                            foreach (ObjectId blockRefId in blockRefSet)
                            {
                                if (IsBlockReferenceALayout(blockRefId, trHlp))
                                {
                                    layoutBlockRefSet.Add(blockRefId);
                                }
                            }
                            iLen = layoutBlockRefSet.Count;

                            // Printing number of Xrefs
                            MgdDbg.Utils.AcadUi.PrintToCmdLine(string.Format("\nThe selected entity is from Xref and there are {0} copy of the Xref in the drawing\n", iLen));

                        }

                        trHlp.Commit();


                    }


                }

                else if (result.Status == PromptStatus.Keyword)
                    break;
                else
                    return;
            }
        }

        public void
        ListMaterialComponents()
        {
            m_db = MgdDbg.Utils.Db.GetCurDwg();

            Editor ed = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor;
            Autodesk.AutoCAD.DatabaseServices.TransactionManager tm = m_db.TransactionManager;

            PromptEntityOptions prEntOpts = new PromptEntityOptions("\nSelect entity:");
            prEntOpts.SetRejectMessage("\nSelected entity must be of type AecEntity.");
            prEntOpts.AddAllowedClass(typeof(AecDb.Entity), false);

            while (true)
            {
                PromptEntityResult prEntRes = ed.GetEntity(prEntOpts);
                if (prEntRes.Status != PromptStatus.OK)
                    return;

                using (Autodesk.AutoCAD.DatabaseServices.Transaction tr = tm.StartTransaction())
                {
                    AecDb.Entity ent = (AecDb.Entity)tr.GetObject(prEntRes.ObjectId, OpenMode.ForRead);
                    AecDbArch.OpenFiller opnFlr = ent as AecDbArch.OpenFiller;

                    AecDb.DBObject styleId = (AecDb.DBObject)tr.GetObject(opnFlr.StyleId, OpenMode.ForRead);

                    if (styleId != null)
                    {
                        MgdDbg.Utils.AcadUi.PrintToCmdLine(string.Format("\nEntity name: {0}", ent.DisplayName));                        

                        IntegerCollection compIds = null;
                        System.Collections.Specialized.StringCollection compNames = null;
                        AcDb.ObjectIdCollection materialIds = null;
                        styleId.GetMaterialComponents(out compIds, out compNames, out materialIds);
                        Debug.Assert((compIds.Count == compNames.Count) && (compIds.Count == materialIds.Count));
                        
                        int len = compIds.Count;
                        for (int i = 0; i < len; i++)
                        {
                            using (Autodesk.AutoCAD.DatabaseServices.Transaction tr2 = tm.StartTransaction())
                            {
                                AecDb.DictionaryRecord dictRec = (AecDb.DictionaryRecord)tr2.GetObject(materialIds[i], OpenMode.ForRead);
                                if (dictRec != null)
                                {
                                    MgdDbg.Utils.AcadUi.PrintToCmdLine(string.Format("\nComponent ID: {0}, Component Name: {1}, Material Name: {2}", compIds[i], compNames[i], dictRec.Name));
                                }
                                else
                                {
                                    MgdDbg.Utils.AcadUi.PrintToCmdLine(string.Format("\nComponent ID: {0}, Component Name: {1}, Material ID: {2}", compIds[i], compNames[i], materialIds[i]));
                                }
                                tr2.Commit();
                            }
                        }

                        tr.Commit();
                    }
                }
            }
        }

#endregion

    }	
}
