
// $Header: //Desktop/Source/TestTools/MgdDbgAec/Snoop/CollectorExts/DictArch.cs#2 $
// $Author: ayyara $ $DateTime: 2007/03/28 11:46:10 $ $Change: 69505 $
//
// (C) Copyright 2004 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//


using System;
using System.Diagnostics;
using System.Collections;
using System.Windows.Forms;

using MgdDbg.Snoop;
using Autodesk.Aec.Geometry;
using Autodesk.AutoCAD.Geometry;
using Autodesk.Aec.DatabaseServices;
using Autodesk.Aec.Arch.DatabaseServices;


namespace MgdDbgAec.Snoop.CollectorExts
{
	/// <summary>
	/// This is a Snoop Collector Extension object to collect data from AEC ImpObjs.
	/// </summary>
	public class DictArch : MgdDbg.Snoop.CollectorExts.CollectorExt
	{
		public DictArch()
		{
		}
		
        protected override void
        CollectEvent(object sender, MgdDbg.Snoop.Collectors.CollectorEventArgs e)
        {
                // bail early if this isn't an object we are responsible for
            Dictionary dict = e.ObjToSnoop as Dictionary;
            if (dict == null)
                return;

                // cast the sender object to the SnoopCollector we are expecting
            MgdDbg.Snoop.Collectors.Collector snoopCollector = sender as MgdDbg.Snoop.Collectors.Collector;
            if (snoopCollector == null) {
                Debug.Assert(false);    // why did someone else send us the message?
                return;
            }
            
            DictionaryWallCleanupGroupDefinition cgDict = dict as DictionaryWallCleanupGroupDefinition;
            if (cgDict != null) {
                Stream(snoopCollector.Data(), cgDict);
                return;
            }
                
            DictionaryCurtainWallLayoutStyle cwlDict = dict as DictionaryCurtainWallLayoutStyle;
            if (cwlDict != null) {
                Stream(snoopCollector.Data(), cwlDict);
                return;
            }

            DictionaryCurtainWallUnitStyle cwuDict = dict as DictionaryCurtainWallUnitStyle;
            if (cwuDict != null) {
                Stream(snoopCollector.Data(), cwuDict);
                return;
            }
                
            DictionaryDoorStyle dDict = dict as DictionaryDoorStyle;
            if (dDict != null) {
                Stream(snoopCollector.Data(), dDict);
                return;
            }
                
            DictionaryWallEndcapStyle ecDict = dict as DictionaryWallEndcapStyle;
            if (ecDict != null) {
                Stream(snoopCollector.Data(), ecDict);
                return;
            }
                
            DictionaryOpeningEndcapStyle oecDict = dict as DictionaryOpeningEndcapStyle;
            if (oecDict != null) {
                Stream(snoopCollector.Data(), oecDict);
                return;
            }           
            
            DictionaryRailingStyle rDict = dict as DictionaryRailingStyle;
            if (rDict != null) {
                Stream(snoopCollector.Data(), rDict);
                return;
            }
                
            DictionaryRoofSlabEdgeStyle rseDict = dict as DictionaryRoofSlabEdgeStyle;
            if (rseDict != null) {
                Stream(snoopCollector.Data(), rseDict);
                return;
            }
                
            DictionaryRoofSlabStyle rsDict = dict as DictionaryRoofSlabStyle;
            if (rsDict != null) {
                Stream(snoopCollector.Data(), rsDict);
                return;
            }
                
            DictionarySlabEdgeStyle seDict = dict as DictionarySlabEdgeStyle;
            if (seDict != null) {
                Stream(snoopCollector.Data(), seDict);
                return;
            }
                
            DictionarySlabStyle ssDict = dict as DictionarySlabStyle;
            if (ssDict != null) {
                Stream(snoopCollector.Data(), ssDict);
                return;
            }
                
            DictionarySpaceStyle spDict = dict as DictionarySpaceStyle;
            if (spDict != null) {
                Stream(snoopCollector.Data(), spDict);
                return;
            }              
             
            DictionaryStairStyle stDict = dict as DictionaryStairStyle;
            if (stDict != null) {
                Stream(snoopCollector.Data(), stDict);
                return;
            }
                
            DictionaryStairWinderStyle swDict = dict as DictionaryStairWinderStyle;
            if (swDict != null) {
                Stream(snoopCollector.Data(), swDict);
                return;
            }
                
            DictionaryStairWinderType swtDict = dict as DictionaryStairWinderType;
            if (swtDict != null) {
                Stream(snoopCollector.Data(), swtDict);
                return;
            }
                
            DictionaryWallModifierStyle wmDict = dict as DictionaryWallModifierStyle;
            if (wmDict != null) {
                Stream(snoopCollector.Data(), wmDict);
                return;
            }
                
            DictionaryWallStyle  wallDict = dict as DictionaryWallStyle;
            if (wallDict != null) {
                Stream(snoopCollector.Data(), wallDict);
                return;
            }
                
            DictionaryWindowAssemblyStyle wassDict = dict as DictionaryWindowAssemblyStyle;
            if (wassDict != null) {
                Stream(snoopCollector.Data(), wassDict);
                return;
            }

            DictionaryWindowStyle winDict = dict as DictionaryWindowStyle;
            if (winDict != null) {
                Stream(snoopCollector.Data(), winDict);
                return;
            }

            DictionaryZoneDefinition zoneDefDict = dict as DictionaryZoneDefinition;
            if (zoneDefDict != null) {
                Stream(snoopCollector.Data(), zoneDefDict);
                return;
            }

            DictionaryZoneStyle zsDict = dict as DictionaryZoneStyle;
            if (zsDict != null) {
                Stream(snoopCollector.Data(), zsDict);
                return;
            }           
        }

        private void
        Stream(ArrayList data, DictionaryWallCleanupGroupDefinition cgDict)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(DictionaryWallCleanupGroupDefinition)));

            data.Add(new MgdDbg.Snoop.Data.ObjectId("Standard definition", DictionaryWallCleanupGroupDefinition.GetStandardDefinition(cgDict.Database)));
        }

        private void
        Stream(ArrayList data, DictionaryCurtainWallLayoutStyle cwlDict)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(DictionaryCurtainWallLayoutStyle)));

            data.Add(new MgdDbg.Snoop.Data.ObjectId("Standard style", DictionaryCurtainWallLayoutStyle.GetStandardStyle(cwlDict.Database)));
        }

        private void
        Stream(ArrayList data, DictionaryCurtainWallUnitStyle cwuDict)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(DictionaryCurtainWallUnitStyle)));

            data.Add(new MgdDbg.Snoop.Data.ObjectId("Standard style", DictionaryCurtainWallUnitStyle.GetStandardStyle(cwuDict.Database)));
        }

        private void
        Stream(ArrayList data, DictionaryDoorStyle dDict)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(DictionaryDoorStyle)));

            data.Add(new MgdDbg.Snoop.Data.ObjectId("Standard style", DictionaryDoorStyle.GetStandardStyle(dDict.Database)));
        }

        private void
        Stream(ArrayList data, DictionaryWallEndcapStyle ecDict)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(DictionaryWallEndcapStyle)));

            data.Add(new MgdDbg.Snoop.Data.ObjectId("Standard style", DictionaryWallEndcapStyle.GetStandardStyle(ecDict.Database)));
        }

        private void
        Stream(ArrayList data, DictionaryOpeningEndcapStyle oecDict)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(DictionaryOpeningEndcapStyle)));

            data.Add(new MgdDbg.Snoop.Data.ObjectId("Standard style", DictionaryOpeningEndcapStyle.GetStandardStyle(oecDict.Database)));
        }       

        private void
        Stream(ArrayList data, DictionaryRailingStyle rDict)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(DictionaryRailingStyle)));

            data.Add(new MgdDbg.Snoop.Data.ObjectId("Standard style", DictionaryRailingStyle.GetStandardStyle(rDict.Database)));
        }
       
        private void
        Stream(ArrayList data, DictionaryRoofSlabEdgeStyle rseDict)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(DictionaryRoofSlabEdgeStyle)));

            data.Add(new MgdDbg.Snoop.Data.ObjectId("Standard style", DictionaryRoofSlabEdgeStyle.GetStandardStyle(rseDict.Database)));
        }

        private void
        Stream(ArrayList data, DictionaryRoofSlabStyle rsDict)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(DictionaryRoofSlabStyle)));

            data.Add(new MgdDbg.Snoop.Data.ObjectId("Standard style", DictionaryRoofSlabStyle.GetStandardStyle(rsDict.Database)));
        }

        private void
        Stream(ArrayList data, DictionarySlabEdgeStyle seDict)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(DictionarySlabEdgeStyle)));

            data.Add(new MgdDbg.Snoop.Data.ObjectId("Standard style", DictionarySlabEdgeStyle.GetStandardStyle(seDict.Database)));
        }

        private void
        Stream(ArrayList data, DictionarySlabStyle ssDict)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(DictionarySlabStyle)));

            data.Add(new MgdDbg.Snoop.Data.ObjectId("Standard style", DictionarySlabStyle.GetStandardStyle(ssDict.Database)));
        }

        private void
        Stream(ArrayList data, DictionarySpaceStyle spDict)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(DictionarySpaceStyle)));

            data.Add(new MgdDbg.Snoop.Data.ObjectId("Standard style", DictionarySpaceStyle.GetStandardStyle(spDict.Database)));
        }


        private void
        Stream(ArrayList data, DictionaryStairStyle stDict)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(DictionaryStairStyle)));

            data.Add(new MgdDbg.Snoop.Data.ObjectId("Standard style", DictionaryStairStyle.GetStandardStyle(stDict.Database)));
        }
        
        private void
        Stream(ArrayList data, DictionaryStairWinderStyle swDict)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(DictionaryStairWinderStyle)));

            data.Add(new MgdDbg.Snoop.Data.ObjectId("Standard style", DictionaryStairWinderStyle.GetStandardStyle(swDict.Database)));
        }

        private void
        Stream(ArrayList data, DictionaryStairWinderType swtDict)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(DictionaryStairWinderType)));

            data.Add(new MgdDbg.Snoop.Data.ObjectId("Standard type", DictionaryStairWinderType.GetStandardType(swtDict.Database)));
        }

        private void
        Stream(ArrayList data, DictionaryWallModifierStyle wmDict)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(DictionaryWallModifierStyle)));

            data.Add(new MgdDbg.Snoop.Data.ObjectId("Standard style", DictionaryWallModifierStyle.GetStandardStyle(wmDict.Database)));
        }

        private void
        Stream(ArrayList data, DictionaryWallStyle wallDict)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(DictionaryWallStyle)));

            data.Add(new MgdDbg.Snoop.Data.ObjectId("Standard style", DictionaryWallStyle.GetStandardStyle(wallDict.Database)));
        }

        private void
        Stream(ArrayList data, DictionaryWindowAssemblyStyle wassDict)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(DictionaryWindowAssemblyStyle)));

            data.Add(new MgdDbg.Snoop.Data.ObjectId("Standard style", DictionaryWindowAssemblyStyle.GetStandardStyle(wassDict.Database)));
        }

        private void
        Stream(ArrayList data, DictionaryWindowStyle winDict)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(DictionaryWindowStyle)));

            data.Add(new MgdDbg.Snoop.Data.ObjectId("Standard style", DictionaryWindowStyle.GetStandardStyle(winDict.Database)));
        }

        private void
        Stream(ArrayList data, DictionaryZoneDefinition zoneDefDict)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(DictionaryZoneDefinition)));

            data.Add(new MgdDbg.Snoop.Data.ObjectId("Standard definition", DictionaryZoneDefinition.GetStandardDefinition(zoneDefDict.Database)));
        }

        private void
        Stream(ArrayList data, DictionaryZoneStyle zsDict)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(DictionaryZoneStyle)));

            data.Add(new MgdDbg.Snoop.Data.ObjectId("Standard style", DictionaryZoneStyle.GetStandardStyle(zsDict.Database)));
        }
    }
}
