
// $Header: //Desktop/Source/TestTools/MgdDbgAec/Snoop/CollectorExts/DictBase.cs#2 $
// $Author: ayyara $ $DateTime: 2007/03/28 11:46:10 $ $Change: 69505 $
//
// (C) Copyright 2004 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//


using System;
using System.Diagnostics;
using System.Collections;
using System.Windows.Forms;

using MgdDbg.Snoop;
using Autodesk.Aec.Geometry;
using Autodesk.AutoCAD.Geometry;
using Autodesk.Aec.DatabaseServices;
using Autodesk.Aec.Arch.DatabaseServices;


namespace MgdDbgAec.Snoop.CollectorExts
{
	/// <summary>
	/// This is a Snoop Collector Extension object to collect data from AEC ImpObjs.
	/// </summary>
	public class DictBase : MgdDbg.Snoop.CollectorExts.CollectorExt
	{
		public DictBase()
		{
		}
		
        protected override void
        CollectEvent(object sender, MgdDbg.Snoop.Collectors.CollectorEventArgs e)
        {
                // bail early if this isn't an object we are responsible for
            Dictionary dict = e.ObjToSnoop as Dictionary;
            if (dict == null)
                return;

                // cast the sender object to the SnoopCollector we are expecting
            MgdDbg.Snoop.Collectors.Collector snoopCollector = sender as MgdDbg.Snoop.Collectors.Collector;
            if (snoopCollector == null) {
                Debug.Assert(false);    // why did someone else send us the message?
                return;
            }
            
            Stream(snoopCollector.Data(), dict);
        }
        

        private void
        Stream(ArrayList data, Dictionary dict)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(Dictionary)));
            
            data.Add(new MgdDbg.Snoop.Data.Bool("Has standard entries", dict.HasStandardEntries));
            data.Add(new MgdDbg.Snoop.Data.String("Display name", dict.DisplayName));
            data.Add(new MgdDbg.Snoop.Data.String("Internal name", dict.InternalName));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Dictionary ID", dict.DictionaryId));
            data.Add(new MgdDbg.Snoop.Data.Database("Database", dict.Database));
            data.Add(new MgdDbg.Snoop.Data.String("Record type", dict.RecordType.Name));
            
                // this can be NULL
            Autodesk.AutoCAD.Runtime.RXClass rxClass = dict.ObjectType;
            string objTypeName = "(null)";
            if (rxClass != null)
                objTypeName = rxClass.Name;
            data.Add(new MgdDbg.Snoop.Data.String("Object type", objTypeName));
            
            data.Add(new MgdDbg.Snoop.Data.ObjectIdCollection("Records", dict.Records));
            data.Add(new MgdDbg.Snoop.Data.Object("Names in use", dict.NamesInUse));

                // branch to all known major sub-classes
            DictionaryMultiViewBlockDefinition mvBlockDict = dict as DictionaryMultiViewBlockDefinition;
            if (mvBlockDict != null) {
                Stream(data, mvBlockDict);
                return;
            }        

            DictionaryMassElementStyle meDict = dict as DictionaryMassElementStyle;
            if (meDict != null) {
                Stream(data, meDict);
                return;
            }

            DictionarySection2dStyle sec2dDict = dict as DictionarySection2dStyle;
            if (sec2dDict != null) {
                Stream(data, sec2dDict);
                return;
            }

            DictionaryListDefinition listDefDict = dict as DictionaryListDefinition;
            if (listDefDict != null) {
                Stream(data, listDefDict);
                return;
            }

            DictionaryPolygonStyle polyDict = dict as DictionaryPolygonStyle;
            if (polyDict != null) {
                Stream(data, polyDict);
                return;
            }

            DictionaryLayerKeyStyle lKeyDict = dict as DictionaryLayerKeyStyle;
            if (lKeyDict != null) {
                Stream(data, lKeyDict);
                return;
            }

            DictionaryMaskBlockDefinition mBlockDict = dict as DictionaryMaskBlockDefinition;
            if (mBlockDict != null) {
                Stream(data, mBlockDict);
                return;
            }

            DictionaryMaterialDefinition matDefDict = dict as DictionaryMaterialDefinition;
            if (matDefDict != null) {
                Stream(data, matDefDict);
                return;
            }

            DictionaryProfileDefinition profDefDict = dict as DictionaryProfileDefinition;
            if (profDefDict != null) {
                Stream(data, profDefDict);
                return;
            }
        }

        private void
        Stream(ArrayList data, DictionaryMultiViewBlockDefinition mvBlockDict)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(DictionaryMultiViewBlockDefinition)));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Standard definition", DictionaryMultiViewBlockDefinition.GetStandardDefinition(mvBlockDict.Database)));
        }

        private void
        Stream(ArrayList data, DictionaryMassElementStyle meDict)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(DictionaryMassElementStyle)));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Standard style", DictionaryMassElementStyle.GetStandardStyle(meDict.Database)));
        }

        private void
        Stream(ArrayList data, DictionarySection2dStyle sec2dDict)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(DictionarySection2dStyle)));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Standard style", DictionarySection2dStyle.GetStandardStyle(sec2dDict.Database)));
        }

        private void
        Stream(ArrayList data, DictionaryListDefinition listDefDict)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(DictionaryListDefinition)));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Standard record", DictionaryListDefinition.GetStandardDefinition(listDefDict.Database)));
        }

        private void
        Stream(ArrayList data, DictionaryLayerKeyStyle lKeyDict)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(DictionaryLayerKeyStyle)));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Standard style", DictionaryLayerKeyStyle.GetStandardStyle(lKeyDict.Database)));
        }

        private void
        Stream(ArrayList data, DictionaryMaskBlockDefinition mBlockDict)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(DictionaryMaskBlockDefinition)));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Standard definition", DictionaryMaskBlockDefinition.GetStandardDefinition(mBlockDict.Database)));
        }

        private void
        Stream(ArrayList data, DictionaryMaterialDefinition matDefDict)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(DictionaryMaterialDefinition)));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Standard definition", DictionaryMaterialDefinition.GetStandardDefinition(matDefDict.Database)));
        }

        private void
        Stream(ArrayList data, DictionaryProfileDefinition profDefDict)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(DictionaryProfileDefinition)));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Standard definition", DictionaryProfileDefinition.GetStandardDefinition(profDefDict.Database)));
        }
    }
}
