
// $Header: //Desktop/Source/TestTools/MgdDbgAec/Snoop/CollectorExts/DictRecBase.cs#4 $
// $Author: ayyara $ $DateTime: 2008/03/07 17:52:55 $ $Change: 94950 $
//
// (C) Copyright 2004 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//

//#define VS2003

using System;
using System.Diagnostics;
using System.Collections;
using System.Windows.Forms;
using System.Collections.Specialized;

using MgdDbg.Snoop;
using Autodesk.AutoCAD.Geometry;
using Autodesk.AutoCAD.DatabaseServices;

using AecDb       = Autodesk.Aec.DatabaseServices;
using AecAppSrvcs = Autodesk.Aec.ApplicationServices;
using AcDb        = Autodesk.AutoCAD.DatabaseServices;
using AecDbArch   = Autodesk.Aec.Arch.DatabaseServices;


namespace MgdDbgAec.Snoop.CollectorExts
{
	/// <summary>
	/// This is a Snoop Collector Extension object to collect data from AEC objects.
	/// </summary>
	public class DictRecBase : MgdDbg.Snoop.CollectorExts.CollectorExt
	{
		public DictRecBase()
		{
		}
		
        protected override void
        CollectEvent(object sender, MgdDbg.Snoop.Collectors.CollectorEventArgs e)
        {
                // bail early if this isn't an object we are responsible for
            AecDb.DictionaryRecord dictRec = e.ObjToSnoop as AecDb.DictionaryRecord;
            if (dictRec == null)
                return;

                // cast the sender object to the SnoopCollector we are expecting
            MgdDbg.Snoop.Collectors.Collector snoopCollector = sender as MgdDbg.Snoop.Collectors.Collector;
            if (snoopCollector == null) {
                Debug.Assert(false);    // why did someone else send us the message?
                return;
            }

            Stream(snoopCollector.Data(), dictRec);
        }
        
        
        private void
        Stream(ArrayList data, AecDb.DictionaryRecord dictRec)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDb.DictionaryRecord)));

            data.Add(new MgdDbg.Snoop.Data.String("Name", dictRec.Name));
            data.Add(new MgdDbg.Snoop.Data.String("Alternate name", dictRec.AlternateName));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is locked", dictRec.IsLocked));

#if !ADT2007
            data.Add(new MgdDbg.Snoop.Data.String("Automatically bound spaces", dictRec.AutomaticallyBoundSpaces.ToString()));
#endif

                // the following functions only work in some cases
            try {
                data.Add(new MgdDbg.Snoop.Data.String("Keynote", dictRec.Keynote));
            }
            catch (Autodesk.AutoCAD.Runtime.Exception e) {
                data.Add(new MgdDbg.Snoop.Data.String("Keynote", e.Message));
            }
            try {
                data.Add(new MgdDbg.Snoop.Data.String("Keynote value", dictRec.KeynoteValue));
            }
            catch (Autodesk.AutoCAD.Runtime.Exception e) {
                data.Add(new MgdDbg.Snoop.Data.String("Keynote value", e.Message));
            }

            data.Add(new MgdDbg.Snoop.Data.ObjectCollection("Classifications", dictRec.Classifications));   // TBD: can return null (should it return empty collection?)
            data.Add(new MgdDbg.Snoop.Data.Object("Translator", dictRec.Translator));
            data.Add(new MgdDbg.Snoop.Data.String("Localized name", dictRec.LocalizedName));

            AecDb.GridAssemblyStyle gaStyle = dictRec as AecDb.GridAssemblyStyle;
            if (gaStyle != null) {
                Stream(data, gaStyle);
                return;
            }

            AecDb.MassElementStyle meStyle = dictRec as AecDb.MassElementStyle;
            if (meStyle != null) {
                Stream(data, meStyle);
                return;
            }
            
            AecDb.PolygonStyle polyStyle = dictRec as AecDb.PolygonStyle;
            if (polyStyle != null) {
                Stream(data, polyStyle);
                return;
            }

            AecDb.LayerKeyStyle lkStyle = dictRec as AecDb.LayerKeyStyle;
            if (lkStyle != null) {
                Stream(data, lkStyle);
                return;
            }

            AecDb.Section2dStyle sec2dStyle = dictRec as AecDb.Section2dStyle;
            if (sec2dStyle != null) {
                Stream(data, sec2dStyle);
                return;
            }

            AecAppSrvcs.DBVariables dbVars = dictRec as AecAppSrvcs.DBVariables;
            if (dbVars != null) {
                Stream(data, dbVars);
				return;
			}
			
		    AecDb.MultiViewBlockDefinition mvBlkDef = dictRec as AecDb.MultiViewBlockDefinition;
			if (mvBlkDef != null) {
				Stream(data, mvBlkDef);
				return;
			}

            AecDb.ClassificationDefinition classDef = dictRec as AecDb.ClassificationDefinition;
            if (classDef != null) {
                Stream(data, classDef);
                return;
            }

            AecDb.Classification classific = dictRec as AecDb.Classification;
            if (classific != null) {
                Stream(data, classific);
                return;
            }

            AecDb.MaskBlockDefinition mbDef = dictRec as AecDb.MaskBlockDefinition;
            if (mbDef != null) {
                Stream(data, mbDef);
                return;
            }

            AecDb.ProfileDefinition profileDef = dictRec as AecDb.ProfileDefinition;
            if (profileDef != null) {
                Stream(data, profileDef);
                return;
            }

            AecDb.MaterialDefinition matDef = dictRec as AecDb.MaterialDefinition;
            if (matDef != null) {
                Stream(data, matDef);
                return;
            }

#if !ADT2007
            AecDb.DisplayConfiguration dispConfig = dictRec as AecDb.DisplayConfiguration;
            if (dispConfig != null) {
                Stream(data, dispConfig);
                return;
            }

            AecDb.DisplayRepresentation dispRep = dictRec as AecDb.DisplayRepresentation;
            if (dispRep != null) {
                Stream(data, dispRep);
                return;
            }

            AecDb.DisplayProperties dispProps = dictRec as AecDb.DisplayProperties;
            if (dispProps != null) {
                Stream(data, dispProps);
                return;
            }
#endif
        }
        
        private void
        Stream(ArrayList data, AecDb.MassElementStyle meStyle)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDb.MassElementStyle)));
            
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Material ID", meStyle.MaterialId));
        }
        
        private void
        Stream(ArrayList data, AecDb.GridAssemblyStyle gaStyle)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDb.GridAssemblyStyle)));
            
            data.Add(new MgdDbg.Snoop.Data.Object("Component set", gaStyle.ComponentSet));
            data.Add(new MgdDbg.Snoop.Data.Object("Grid assembly definition", gaStyle.GridAssemblyDefinition));
        }

        private void
        Stream(ArrayList data, AecDb.PolygonStyle polyStyle)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDb.PolygonStyle)));
            
            data.Add(new MgdDbg.Snoop.Data.Distance("Edge width", polyStyle.EdgeWidth));
            data.Add(new MgdDbg.Snoop.Data.String("Edge width justif type", polyStyle.EdgeWidthJustifyType.ToString()));
        }

        private void
        Stream(ArrayList data, AecDb.LayerKeyStyle lkStyle)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDb.LayerKeyStyle)));

#if ADT2007
            data.Add(new MgdDbg.Snoop.Data.Int("Layer key definition count", lkStyle.LayerKeyDefinitionCount));
            ArrayList defs = new ArrayList();
            for (int i=0; i<lkStyle.LayerKeyDefinitionCount; i++) {
                defs.Add(lkStyle.GetLayerKeyDefinitionAt(i));
            }
            data.Add(new MgdDbg.Snoop.Data.ObjectCollection("Layer key definitions", defs));
#else
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Layer key definitions", lkStyle.KeyDefinitions));
#endif

            data.Add(new MgdDbg.Snoop.Data.Int("Override count", lkStyle.OverrideCount));
            StringCollection strs = new StringCollection();
            for (int i=0; i<lkStyle.OverrideCount; i++) {
                string str1 = "";
                string str2 = "";
                lkStyle.GetOverrideAt(i, str1, str2);
                strs.Add(string.Format("key = {0}, value = {1}", str1, str2));  // TBD: these need to be out or ref params
            }
            data.Add(new MgdDbg.Snoop.Data.Object("Overrides", strs));

            data.Add(new MgdDbg.Snoop.Data.Bool("Overrides enabled", lkStyle.OverridesEnabled));
            data.Add(new MgdDbg.Snoop.Data.String("Standard", lkStyle.Standard));
        }

        private void
        Stream(ArrayList data, AecDb.Section2dStyle sec2dStyle)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDb.Section2dStyle)));

            data.Add(new MgdDbg.Snoop.Data.Bool("Keep all hidden line work", sec2dStyle.KeepAllHiddenLineWork));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Line work components", sec2dStyle.LineWorkComponents));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Line work rules", sec2dStyle.LineWorkRules));

            try {
                data.Add(new MgdDbg.Snoop.Data.Bool("Use object display property", sec2dStyle.UseObjectDisplayProperty));
            }
            catch (Autodesk.AutoCAD.Runtime.Exception ex) {
                data.Add(new MgdDbg.Snoop.Data.Exception("Use object display property", ex));
            }
        }

        private void
        Stream(ArrayList data, AecDb.MultiViewBlockDefinition mvBlkDef)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDb.MultiViewBlockDefinition)));
            
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Display representation definitions", mvBlkDef.DisplayRepresentationDefinitions));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Interference block ID", mvBlkDef.InterferenceBlockId));
            data.Add(new MgdDbg.Snoop.Data.ObjectIdCollection("All blocks referenced", mvBlkDef.GetAllBlocksReferenced()));
        }

        private void
        Stream(ArrayList data, AecDb.ClassificationDefinition classDef)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDb.ClassificationDefinition)));

            data.Add(new MgdDbg.Snoop.Data.Bool("Applies to all", classDef.AppliesToAll));
            data.Add(new MgdDbg.Snoop.Data.Object("Applies to filter", classDef.AppliesToFilter));
            data.Add(new MgdDbg.Snoop.Data.Object("Classification tree", classDef.ClassificationTree));

            AecDb.ListDefinition listDef = classDef as AecDb.ListDefinition;
            if (listDef != null) {
                Stream(data, listDef);
                return;
            }
        }

        private void
        Stream(ArrayList data, AecDb.ListDefinition listDef)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDb.ListDefinition)));

            data.Add(new MgdDbg.Snoop.Data.Bool("Applies to all", listDef.AllowToVary));
            data.Add(new MgdDbg.Snoop.Data.ObjectIdCollection("List items", listDef.GetListItems()));
        }

        private void
        Stream(ArrayList data, AecDb.Classification classific)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDb.Classification)));

            data.Add(new MgdDbg.Snoop.Data.String("Name and description", classific.NameAndDescription));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Owning system ID", classific.OwningSystemId));
            data.Add(new MgdDbg.Snoop.Data.String("Owning system name", classific.OwningSystemName));

            AecDb.ListItem listItem = classific as AecDb.ListItem;
            if (listItem != null) {
                Stream(data, listItem);
                return;
            }
        }

        private void
        Stream(ArrayList data, AecDb.ListItem mbDef)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDb.ListItem)));

                // No data at this level
        }

        private void
        Stream(ArrayList data, AecDb.MaskBlockDefinition mbDef)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDb.MaskBlockDefinition)));

            data.Add(new MgdDbg.Snoop.Data.ObjectId("Extra graphics block ID", mbDef.ExtraGraphicsBlockId));
            data.Add(new MgdDbg.Snoop.Data.Object("Cut profile", mbDef.CutProfile));
        }

        private void
        Stream(ArrayList data, AecDb.ProfileDefinition profileDef)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDb.ProfileDefinition)));

            data.Add(new MgdDbg.Snoop.Data.Point2d("Insertion point", profileDef.InsertionPoint));
            data.Add(new MgdDbg.Snoop.Data.String("Extrusion direction", profileDef.ExtrusionDirection.ToString()));
            data.Add(new MgdDbg.Snoop.Data.Object("Profile", profileDef.Profile));
        }

        private void
        Stream(ArrayList data, AecDb.MaterialDefinition matDef)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDb.MaterialDefinition)));

           // no data at this level
        }

        private void
        Stream(ArrayList data, AecAppSrvcs.DBVariables dbVars)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecAppSrvcs.DBVariables)));

            AecAppSrvcs.DrawingSetupVariables dwgSetupVars = dbVars as AecAppSrvcs.DrawingSetupVariables;
            if (dwgSetupVars != null) {
                Stream(data, dwgSetupVars);
                return;
            }
        }

		private void
		Stream(ArrayList data, AecAppSrvcs.DrawingSetupVariables dwgSetupVars)
		{
			data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecAppSrvcs.DrawingSetupVariables)));
	            
			data.Add(new MgdDbg.Snoop.Data.Point3d("AecBasePtNE",                     dwgSetupVars.AecBasePtNE));
			data.Add(new MgdDbg.Snoop.Data.Point3d("AecDwgBasePt",                    dwgSetupVars.AecDwgBasePt));
			data.Add(new MgdDbg.Snoop.Data.Angle  ("AecNorthRotation",                dwgSetupVars.AecNorthRotation));
			data.Add(new MgdDbg.Snoop.Data.Bool   ("AlwaysUpdateLayerKeyStyle",       dwgSetupVars.AlwaysUpdateLayerKeyStyle));
			data.Add(new MgdDbg.Snoop.Data.Int    ("AngularAzimuth",                  dwgSetupVars.AngularAzimuth));
			data.Add(new MgdDbg.Snoop.Data.Int    ("AngularDisplayFormat",            dwgSetupVars.AngularDisplayFormat));
			data.Add(new MgdDbg.Snoop.Data.Int    ("AngularPrecision",                dwgSetupVars.AngularPrecision));
			data.Add(new MgdDbg.Snoop.Data.String ("AngularDisplayFormat",            dwgSetupVars.AngularDisplayFormat.ToString()));
			data.Add(new MgdDbg.Snoop.Data.Int    ("AreaPrecision",                   dwgSetupVars.AreaPrecision));
			data.Add(new MgdDbg.Snoop.Data.String ("AreaSuffix",                      dwgSetupVars.AreaSuffix));
			data.Add(new MgdDbg.Snoop.Data.Bool   ("BlockBasedLayerOffBehavior",      dwgSetupVars.BlockBasedLayerOffBehavior));
			data.Add(new MgdDbg.Snoop.Data.Int    ("CivilDimaltd",                    dwgSetupVars.CivilDimaltd));
			data.Add(new MgdDbg.Snoop.Data.Double ("CivilDimaltf",                    dwgSetupVars.CivilDimaltf));
			data.Add(new MgdDbg.Snoop.Data.String ("CivilDimapost",                   dwgSetupVars.CivilDimapost));
			data.Add(new MgdDbg.Snoop.Data.Double ("CivilDimexo",                     dwgSetupVars.CivilDimexo));
			data.Add(new MgdDbg.Snoop.Data.String ("CivilDimpost",                    dwgSetupVars.CivilDimpost));
			data.Add(new MgdDbg.Snoop.Data.Double ("CivilDimscale",                   dwgSetupVars.CivilDimscale));
			data.Add(new MgdDbg.Snoop.Data.Int    ("CivilStateFlag1",                 dwgSetupVars.CivilStateFlag1));
			data.Add(new MgdDbg.Snoop.Data.Int    ("CivilStateFlag2",                 dwgSetupVars.CivilStateFlag2));
			data.Add(new MgdDbg.Snoop.Data.Int    ("CivilStateFlag3",                 dwgSetupVars.CivilStateFlag3));
			data.Add(new MgdDbg.Snoop.Data.String ("CivilTextStyle",                  dwgSetupVars.CivilTextStyle));
			data.Add(new MgdDbg.Snoop.Data.Int    ("CoordinatePrecision",             dwgSetupVars.CoordinatePrecision));
			data.Add(new MgdDbg.Snoop.Data.Bool   ("CreateDimOverride",               dwgSetupVars.CreateDimOverride));
			data.Add(new MgdDbg.Snoop.Data.String ("CurrentDimUnit",                  dwgSetupVars.CurrentDimUnit.ToString()));
			data.Add(new MgdDbg.Snoop.Data.Double ("DwgScale",                        dwgSetupVars.DwgScale));
			data.Add(new MgdDbg.Snoop.Data.Bool   ("ElevationLabelPlus",              dwgSetupVars.ElevationLabelPlus));
			data.Add(new MgdDbg.Snoop.Data.Bool   ("ElevationLabelPlusMinus",         dwgSetupVars.ElevationLabelPlusMinus));
			data.Add(new MgdDbg.Snoop.Data.Int    ("ElevationPrecision",              dwgSetupVars.ElevationPrecision));
			data.Add(new MgdDbg.Snoop.Data.Double ("FacetDeviation",                  dwgSetupVars.FacetDeviation));
			data.Add(new MgdDbg.Snoop.Data.Bool   ("IsMetric",                        dwgSetupVars.IsMetric));
			data.Add(new MgdDbg.Snoop.Data.Double ("LastLayerStyleUpdate",            dwgSetupVars.LastLayerStyleUpdate));
			data.Add(new MgdDbg.Snoop.Data.String ("LayerFile",                       dwgSetupVars.LayerFile));
			data.Add(new MgdDbg.Snoop.Data.String ("LayerStandard",                   dwgSetupVars.LayerStandard));
			data.Add(new MgdDbg.Snoop.Data.String ("LayerStandardLegacy",             dwgSetupVars.LayerStandardLegacy));
			data.Add(new MgdDbg.Snoop.Data.Int    ("LinearDisplayFormat",             dwgSetupVars.LinearDisplayFormat));
			data.Add(new MgdDbg.Snoop.Data.Int    ("LinearPrecision",                 dwgSetupVars.LinearPrecision));
			data.Add(new MgdDbg.Snoop.Data.String ("LinearUnit",                      dwgSetupVars.LinearUnit.ToString()));
			data.Add(new MgdDbg.Snoop.Data.Int    ("MaxFacetsOnCircle",               dwgSetupVars.MaxFacetsOnCircle));
			data.Add(new MgdDbg.Snoop.Data.String ("ProjectName",                     dwgSetupVars.ProjectName));
			data.Add(new MgdDbg.Snoop.Data.Bool   ("ScaleOnInsert",                   dwgSetupVars.ScaleOnInsert));
			data.Add(new MgdDbg.Snoop.Data.Bool   ("SuperScript",                     dwgSetupVars.SuperScript));
			data.Add(new MgdDbg.Snoop.Data.Bool   ("SuperScriptZeroSupression",       dwgSetupVars.SuperScriptZeroSupression));
			data.Add(new MgdDbg.Snoop.Data.Double ("TextHeight",                      dwgSetupVars.TextHeight));
			data.Add(new MgdDbg.Snoop.Data.String ("UnitType",                        dwgSetupVars.UnitType.ToString()));
			data.Add(new MgdDbg.Snoop.Data.Double ("VerticalScale",                   dwgSetupVars.VerticalScale));
			data.Add(new MgdDbg.Snoop.Data.String ("VolumeDisplayUnit",               dwgSetupVars.VolumeDisplayUnit.ToString()));
			data.Add(new MgdDbg.Snoop.Data.Int    ("VolumePrecision",                 dwgSetupVars.VolumePrecision));
			data.Add(new MgdDbg.Snoop.Data.String ("VolumeSuffix",                    dwgSetupVars.VolumeSuffix));
			data.Add(new MgdDbg.Snoop.Data.Bool   ("XrefOverlaysUseOwnDisplayConfig", dwgSetupVars.XrefOverlaysUseOwnDisplayConfig));
		}

#if !ADT2007

        private void
        Stream(ArrayList data, AecDb.DisplayConfiguration displayConfig)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDb.DisplayConfiguration)));

            data.Add(new MgdDbg.Snoop.Data.Distance("Cut plane above range", displayConfig.CutPlaneAboveRange));
            data.Add(new MgdDbg.Snoop.Data.Distance("Cut plane below range", displayConfig.CutPlaneBelowRange));
            data.Add(new MgdDbg.Snoop.Data.Distance("Cut plane offset from WCS zero", displayConfig.CutPlaneOffsetFromWcsZero));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Display theme", displayConfig.DisplayTheme));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is view-dependent", displayConfig.IsViewDependent));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Default view-dependent view set", displayConfig.DefaultViewDependentViewSet));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("View-dependent combinations", displayConfig.ViewDependentCombinations));
            data.Add(new MgdDbg.Snoop.Data.Vector3d("Hard view direction", displayConfig.HardViewDirection));
            data.Add(new MgdDbg.Snoop.Data.String("Hard view direction type", displayConfig.HardViewDirectionType.ToString()));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Hard view set", displayConfig.HardViewSet));
            data.Add(new MgdDbg.Snoop.Data.Bool("Use viewport view direction", displayConfig.UseViewportViewDirection));
            data.Add(new MgdDbg.Snoop.Data.Icon("State icon", displayConfig.StateIcon));
            data.Add(new MgdDbg.Snoop.Data.String("State image index", displayConfig.StateImageIndex.ToString()));
        }

        private void
        Stream(ArrayList data, AecDb.DisplayRepresentation dispRep)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDb.DisplayRepresentation)));
            
            data.Add(new MgdDbg.Snoop.Data.Int("Base Gs marker value", dispRep.BaseGsMarkerValue));
            data.Add(new MgdDbg.Snoop.Data.Bool("Can be deleted", dispRep.CanBeDeleted));
            data.Add(new MgdDbg.Snoop.Data.Object("Display props class", dispRep.DisplayPropertiesClass));
            data.Add(new MgdDbg.Snoop.Data.Object("Works with", dispRep.WorksWith));
            data.Add(new MgdDbg.Snoop.Data.String("Symbol name", dispRep.SymbolName));
            data.Add(new MgdDbg.Snoop.Data.String("View type display name", dispRep.ViewTypeDisplayName));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Default display props", dispRep.DefaultDisplayPropertiesId));
            data.Add(new MgdDbg.Snoop.Data.String("Default rep name", dispRep.DisplayRepresentationName));
            data.Add(new MgdDbg.Snoop.Data.String("Default record name", dispRep.DefaultRecordName));
            data.Add(new MgdDbg.Snoop.Data.Bool("Default visibility", dispRep.DefaultVisibility));            
            data.Add(new MgdDbg.Snoop.Data.Bool("Has reliable GS markers", dispRep.HasReliableGsMarkers));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is new to this drawing", dispRep.IsNewToThisDrawing));
            data.Add(new MgdDbg.Snoop.Data.String("Is similar to", dispRep.IsSimilarTo));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is user display rep", dispRep.IsUserDisplayRepresentation));            
            data.Add(new MgdDbg.Snoop.Data.Object("Current material component display info", dispRep.CurrentMaterialComponentDisplayInformation));
            data.Add(new MgdDbg.Snoop.Data.Icon("State icon", dispRep.StateIcon));
            data.Add(new MgdDbg.Snoop.Data.String("State image index", dispRep.StateImageIndex.ToString()));
            data.Add(new MgdDbg.Snoop.Data.Bool("Use object snap cache", dispRep.UseObjectSnapCache));            
        }

        private void
        Stream(ArrayList data, AecDb.DisplayProperties dispProps)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDb.DisplayProperties)));

            data.Add(new MgdDbg.Snoop.Data.Bool("Is drawing default", dispProps.IsDrawingDefault));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is style override", dispProps.IsStyleOverride));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Owner object ID", dispProps.OwnerObjectId));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Owning display rep", dispProps.OwningDisplayRep));
            data.Add(new MgdDbg.Snoop.Data.Object("Owning display rep class", dispProps.OwningDisplayRepClass));           
        }
#endif        
	}
}
