// $Header: //Desktop/Source/TestTools/MgdDbgAec/Snoop/CollectorExts/DictRecPropData.cs#3 $
// $Author: ayyara $ $DateTime: 2007/04/13 17:09:53 $ $Change: 70520 $
//
// (C) Copyright 2006 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//

using System;
using System.Diagnostics;
using System.Collections;
using System.Windows.Forms;
using System.Collections.Specialized;

using MgdDbg.Snoop;
using Autodesk.Aec.PropertyData.DatabaseServices;

using AecDb       = Autodesk.Aec.DatabaseServices;


namespace MgdDbgAec.Snoop.CollectorExts
{

	public class DictRecPropData : MgdDbg.Snoop.CollectorExts.CollectorExt
	{
		public DictRecPropData()
		{
		}
		
        protected override void
        CollectEvent(object sender, MgdDbg.Snoop.Collectors.CollectorEventArgs e)
        {
                // bail early if this isn't an object we are responsible for
            AecDb.DictionaryRecord dictRec = e.ObjToSnoop as AecDb.DictionaryRecord;
            if (dictRec == null)
                return;

                // cast the sender object to the SnoopCollector we are expecting
            MgdDbg.Snoop.Collectors.Collector snoopCollector = sender as MgdDbg.Snoop.Collectors.Collector;
            if (snoopCollector == null) {
                Debug.Assert(false);    // why did someone else send us the message?
                return;
            }

            PropertySet propSet = e.ObjToSnoop as PropertySet;
            if (propSet != null) {
                Stream(snoopCollector.Data(), propSet);
                return;
            }

            PropertySetDefinition propSetDef = e.ObjToSnoop as PropertySetDefinition;
            if (propSetDef != null) {
                Stream(snoopCollector.Data(), propSetDef);
                return;
            }

#if !ADT2007
            PropertyDataFormat propDataFormat = e.ObjToSnoop as PropertyDataFormat;
            if (propDataFormat != null)
            {
                Stream(snoopCollector.Data(), propDataFormat);
                return;
            }
#endif

        }
                
        private void
        Stream(ArrayList data, PropertySet propSet)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(PropertySet)));
            
            data.Add(new MgdDbg.Snoop.Data.String("Property set definition name", propSet.PropertySetDefinitionName));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Property set definition", propSet.PropertySetDefinition));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Property set data", propSet.PropertySetData));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Object attached to", propSet.ObjectAttachedTo));
        }

        private void
        Stream(ArrayList data, PropertySetDefinition propSetDef)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(PropertySetDefinition)));
            
            data.Add(new MgdDbg.Snoop.Data.Bool("Applies to all", propSetDef.AppliesToAll));
            data.Add(new MgdDbg.Snoop.Data.Object("Applies to filter", propSetDef.AppliesToFilter));

            try {
                data.Add(new MgdDbg.Snoop.Data.Enumerable("Classification filter", propSetDef.ClassificationFilter));
            }
            catch (System.Runtime.InteropServices.SEHException e) {
                data.Add(new MgdDbg.Snoop.Data.Exception("Classification filter", e));
            }

            data.Add(new MgdDbg.Snoop.Data.Enumerable("Definitions", propSetDef.Definitions));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is style based", propSetDef.IsStyleBased));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is visible", propSetDef.IsVisible));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is writeable", propSetDef.IsWriteable));

        }

#if !ADT2007
        private void
        Stream(ArrayList data, PropertyDataFormat propDataFormat)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(PropertyDataFormat)));


            try {
                data.Add(new MgdDbg.Snoop.Data.Object("Built in type", propDataFormat.BuiltinType));
            }
            catch (Autodesk.AutoCAD.Runtime.Exception e) {
                data.Add(new MgdDbg.Snoop.Data.Exception("Built in type", e));
            }

            try {
                data.Add(new MgdDbg.Snoop.Data.Object("Fraction", propDataFormat.Fraction));
            }
            catch (Autodesk.AutoCAD.Runtime.Exception e) {
                data.Add(new MgdDbg.Snoop.Data.Exception("Fraction", e));
            }

            data.Add(new MgdDbg.Snoop.Data.Bool("Leading zeros", propDataFormat.LeadingZeros));
            data.Add(new MgdDbg.Snoop.Data.Int("Precision", propDataFormat.Precision));

            try {
                data.Add(new MgdDbg.Snoop.Data.Object("Rounding mode", propDataFormat.RoundingMode));
            }
            catch (Autodesk.AutoCAD.Runtime.Exception e) {
                data.Add(new MgdDbg.Snoop.Data.Exception("Rounding mode", e));
            }

            data.Add(new MgdDbg.Snoop.Data.Double("Round off", propDataFormat.RoundOff));
            data.Add(new MgdDbg.Snoop.Data.Double("Scale", propDataFormat.Scale));

            try {
                data.Add(new MgdDbg.Snoop.Data.Object("Separator", propDataFormat.Separator));
            }
            catch (Autodesk.AutoCAD.Runtime.Exception e) {
                data.Add(new MgdDbg.Snoop.Data.Exception("Separator", e));
            }

            data.Add(new MgdDbg.Snoop.Data.Bool("Trailing zeros", propDataFormat.TrailingZeros));
            data.Add(new MgdDbg.Snoop.Data.String("True text", propDataFormat.TrueText));
            data.Add(new MgdDbg.Snoop.Data.String("False text", propDataFormat.FalseText));
            data.Add(new MgdDbg.Snoop.Data.String("Prefix", propDataFormat.Prefix));
            data.Add(new MgdDbg.Snoop.Data.String("Suffix", propDataFormat.Suffix));
            data.Add(new MgdDbg.Snoop.Data.String("Not applicable", propDataFormat.NotApplicable));
            data.Add(new MgdDbg.Snoop.Data.String("Undefined", propDataFormat.Undefined));
            data.Add(new MgdDbg.Snoop.Data.Bool("Zero feet", propDataFormat.ZeroFeet));
            data.Add(new MgdDbg.Snoop.Data.Bool("Zero inches", propDataFormat.ZeroInches));
            data.Add(new MgdDbg.Snoop.Data.Int("Zero padding", propDataFormat.ZeroPadding));            

            try {
                data.Add(new MgdDbg.Snoop.Data.Object("Unit type", propDataFormat.UnitType));
            }
            catch (Autodesk.AutoCAD.Runtime.Exception e) {
                data.Add(new MgdDbg.Snoop.Data.Exception("Unit type", e));
            }               
        }
#endif        
	}
}
