// $Header: //Desktop/Source/TestTools/MgdDbgAec/Snoop/CollectorExts/DictStructure.cs#1 $
// $Author: fergusja $ $DateTime: 2008/03/04 16:12:35 $ $Change: 94604 $
//
// (C) Copyright 2008 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//


using System;
using System.Diagnostics;
using System.Collections;
using System.Windows.Forms;

using MgdDbg.Snoop;
using Autodesk.Aec.Geometry;
using Autodesk.AutoCAD.Geometry;
using Autodesk.Aec.DatabaseServices;
using Autodesk.Aec.Structural.DatabaseServices;


namespace MgdDbgAec.Snoop.CollectorExts
{
	/// <summary>
	/// This is a Snoop Collector Extension object to collect data from AEC ImpObjs.
	/// </summary>
	public class DictStructure : MgdDbg.Snoop.CollectorExts.CollectorExt
	{
		public DictStructure()
		{
		}
		
        protected override void
        CollectEvent(object sender, MgdDbg.Snoop.Collectors.CollectorEventArgs e)
        {
                // bail early if this isn't an object we are responsible for
            Dictionary dict = e.ObjToSnoop as Dictionary;
            if (dict == null)
                return;

                // cast the sender object to the SnoopCollector we are expecting
            MgdDbg.Snoop.Collectors.Collector snoopCollector = sender as MgdDbg.Snoop.Collectors.Collector;
            if (snoopCollector == null) {
                Debug.Assert(false);    // why did someone else send us the message?
                return;
            }

            DictionaryMemberStyle msDict = dict as DictionaryMemberStyle;
            if (msDict != null) {
                Stream(snoopCollector.Data(), msDict);
                return;
            }

            DictionaryMemberNodeShape mnsDict = dict as DictionaryMemberNodeShape;
            if (mnsDict != null) {
                Stream(snoopCollector.Data(), mnsDict);
                return;
            }
        }

        private void
        Stream(ArrayList data, DictionaryMemberNodeShape mnsDict)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(DictionaryMemberNodeShape)));

            data.Add(new MgdDbg.Snoop.Data.ObjectId("Standard style", DictionaryMemberNodeShape.GetStandardDefinition(mnsDict.Database)));
        }

        private void
        Stream(ArrayList data, DictionaryMemberStyle msDict)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(DictionaryMemberStyle)));

            data.Add(new MgdDbg.Snoop.Data.ObjectId("Standard style", DictionaryMemberStyle.GetStandardStyle(msDict.Database)));
        }

    }
}
