
// $Header: //Desktop/Source/TestTools/MgdDbgAec/Snoop/CollectorExts/EntityArch.cs#8 $
// $Author: ayyara $ $DateTime: 2008/03/07 17:52:55 $ $Change: 94950 $
//
// (C) Copyright 2004 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//


using System;
using System.Diagnostics;
using System.Collections;
using System.Windows.Forms;
using System.Collections.Specialized;

using MgdDbg.Snoop;
using Autodesk.Aec.Modeler;
using Autodesk.Aec.Geometry;
using Autodesk.AutoCAD.Geometry;

using AecGeoArch = Autodesk.Aec.Arch.Geometry;
using AecDb      = Autodesk.Aec.DatabaseServices;
using AcDb       = Autodesk.AutoCAD.DatabaseServices;
using AecDbArch  = Autodesk.Aec.Arch.DatabaseServices;

namespace MgdDbgAec.Snoop.CollectorExts
{
	/// <summary>
	/// This is a Snoop Collector Extension object to collect data from AEC entities.
	/// </summary>
	public class EntityArch : MgdDbg.Snoop.CollectorExts.CollectorExt
	{
		public EntityArch()
		{
		}
		
        protected override void
        CollectEvent(object sender, MgdDbg.Snoop.Collectors.CollectorEventArgs e)
        {
                // bail early if this isn't an object we are responsible for
            AecDb.Entity aecEnt = e.ObjToSnoop as AecDb.Entity;
            if (aecEnt == null)
                return;

                // cast the sender object to the SnoopCollector we are expecting
            MgdDbg.Snoop.Collectors.Collector snoopCollector = sender as MgdDbg.Snoop.Collectors.Collector;
            if (snoopCollector == null) {
                Debug.Assert(false);    // why did someone else send us the message?
                return;
            }
            
            AecDbArch.OpeningBase opnBase = aecEnt as AecDbArch.OpeningBase;
            if (opnBase != null) {
                Stream(snoopCollector.Data(), opnBase);
                return;
            }
            
            AecDbArch.Wall wall = aecEnt as AecDbArch.Wall;
            if (wall != null) {
                Stream(snoopCollector.Data(), wall);
                return;
            }

            AecDbArch.Stair stair = aecEnt as AecDbArch.Stair;
            if (stair != null) {
                Stream(snoopCollector.Data(), stair);
                return;
            }

            AecDbArch.Zone zone = aecEnt as AecDbArch.Zone;
            if (zone != null) {
                Stream(snoopCollector.Data(), zone);
                return;
            }
            
            AecDbArch.Space space = aecEnt as AecDbArch.Space;
            if (space != null) {
                Stream(snoopCollector.Data(), space);
                return;
            }

            AecDbArch.Roof roof = aecEnt as AecDbArch.Roof;
            if (roof != null) {
                Stream(snoopCollector.Data(), roof);
                return;
            }

            AecDbArch.Railing rail = aecEnt as AecDbArch.Railing;
            if (rail != null) {
                Stream(snoopCollector.Data(), rail);
                return;
            }

            AecDbArch.SlabBase sBase = aecEnt as AecDbArch.SlabBase;
            if (sBase != null) {
                Stream(snoopCollector.Data(), sBase);
                return;
            }

            AecDbArch.CurtainWallLayout curtainWallLayout = aecEnt as AecDbArch.CurtainWallLayout;
            if (curtainWallLayout != null) {
                Stream(snoopCollector.Data(), curtainWallLayout);
                return;
            }

            AecDbArch.CurtainWallUnit curtainWallUnit = aecEnt as AecDbArch.CurtainWallUnit;
            if (curtainWallUnit != null) {
                Stream(snoopCollector.Data(), curtainWallUnit);
                return;
            }

            AecDbArch.WindowAssembly windowAssembly = aecEnt as AecDbArch.WindowAssembly;
            if (windowAssembly != null) {
                Stream(snoopCollector.Data(), windowAssembly);
                return;
            }

            AecDbArch.CeilingGrid ceilingGrid = aecEnt as AecDbArch.CeilingGrid;
            if (ceilingGrid != null) {
                Stream(snoopCollector.Data(), ceilingGrid);
                return;
            }

            AecDbArch.BuildingElevationLine buildingElev = aecEnt as AecDbArch.BuildingElevationLine;
            if (buildingElev != null) {
                Stream(snoopCollector.Data(), buildingElev);
                return;
            }

            AecDbArch.BuildingSectionLine buildingSectLine = aecEnt as AecDbArch.BuildingSectionLine;
            if (buildingSectLine != null) {
                Stream(snoopCollector.Data(), buildingSectLine);
                return;
            }

            AecDbArch.BuildingSection buildingSect = aecEnt as AecDbArch.BuildingSection;
            if (buildingSect != null) {
                Stream(snoopCollector.Data(), buildingSect);
                return;
            }
        }
                
        private void
        Stream(ArrayList data, AecDbArch.OpeningBase opnBase)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDbArch.OpeningBase)));
            
            data.Add(new MgdDbg.Snoop.Data.Object("Open shape", opnBase.OpenShape));
            data.Add(new MgdDbg.Snoop.Data.Distance("Width", opnBase.Width));
            data.Add(new MgdDbg.Snoop.Data.Distance("Height", opnBase.Height));
            data.Add(new MgdDbg.Snoop.Data.Distance("Inner width", opnBase.InnerWidth));
            data.Add(new MgdDbg.Snoop.Data.Distance("Inner height", opnBase.InnerHeight));
            data.Add(new MgdDbg.Snoop.Data.Distance("Rough width", opnBase.RoughWidth));
            data.Add(new MgdDbg.Snoop.Data.Distance("Rough height", opnBase.RoughHeight));
            data.Add(new MgdDbg.Snoop.Data.Distance("Rise", opnBase.Rise));
                        
            AecDbArch.OpenFiller opnFlr = opnBase as AecDbArch.OpenFiller;
            if (opnFlr != null) {
                Stream(data, opnFlr);
            }
        }
        
        private void
        Stream(ArrayList data, AecDbArch.OpenFiller opnFlr)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDbArch.OpenFiller)));
            
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Style ID", opnFlr.StyleId));
            data.Add(new MgdDbg.Snoop.Data.Distance("Frame width", opnFlr.FrameWidth));
            data.Add(new MgdDbg.Snoop.Data.Distance("Frame depth", opnFlr.FrameDepth));
            data.Add(new MgdDbg.Snoop.Data.Distance("Leaf width", opnFlr.LeafWidth));
            data.Add(new MgdDbg.Snoop.Data.Double("Open percent", opnFlr.OpenPercent));
            data.Add(new MgdDbg.Snoop.Data.Bool("Storing rough dimensions", opnFlr.StoringRoughDimensions));
            
            AecDbArch.Door door = opnFlr as AecDbArch.Door;
            if (door != null) {
                Stream(data, door);
            }
            
            AecDbArch.Window win = opnFlr as AecDbArch.Window;
            if (win != null) {
                Stream(data, win);
            }
        }
        
        private void
        Stream(ArrayList data, AecDbArch.Door door)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDbArch.Door)));

            // TBD: GetMaterialId()
        }
        
        private void
        Stream(ArrayList data, AecDbArch.Window win)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDbArch.Window)));

            // TBD: GetMaterialId()
        }
        
        private void
        Stream(ArrayList data, AecDbArch.Opening opn)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDbArch.Opening)));
            
            data.Add(new MgdDbg.Snoop.Data.String("Shape type", opn.ShapeType.ToString()));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Custom shape ID", opn.CustomShapeId));
        }
        
        private void
        Stream(ArrayList data, AecDbArch.Wall wall)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDbArch.Wall)));
            
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Style ID", wall.StyleId));
            data.Add(new MgdDbg.Snoop.Data.Point3d("Start point", wall.StartPoint));
            data.Add(new MgdDbg.Snoop.Data.Point3d("End point", wall.EndPoint));
            data.Add(new MgdDbg.Snoop.Data.Point3d("Mid point", wall.MidPoint));
            data.Add(new MgdDbg.Snoop.Data.Distance("Length", wall.Length));
            data.Add(new MgdDbg.Snoop.Data.Distance("Base height", wall.BaseHeight));
            data.Add(new MgdDbg.Snoop.Data.Distance("Instance width", wall.InstanceWidth));
            data.Add(new MgdDbg.Snoop.Data.Distance("Width", wall.Width));
            data.Add(new MgdDbg.Snoop.Data.String("Justification type", wall.JustificationType.ToString()));
            data.Add(new MgdDbg.Snoop.Data.Object("Roof line", wall.RoofLine));
            data.Add(new MgdDbg.Snoop.Data.Object("Floor line", wall.FloorLine));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Cleanup group definition ID", wall.CleanupGroupDefinitionId));
            data.Add(new MgdDbg.Snoop.Data.Bool("Don't cleanup", wall.DontCleanup));
            data.Add(new MgdDbg.Snoop.Data.Distance("Cleanup radius", wall.CleanupRadius));
            data.Add(new MgdDbg.Snoop.Data.Distance("Start cleanup radius", wall.StartCleanupRadius));
            data.Add(new MgdDbg.Snoop.Data.Distance("End cleanup radius", wall.EndCleanupRadius));
            data.Add(new MgdDbg.Snoop.Data.Bool("Use justification line for cleanups", wall.UseJustificationLineForCleanups));
            data.Add(new MgdDbg.Snoop.Data.ObjectIdCollection("Openings", wall.GetOpeningsFor()));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Starting endcap style ID", wall.StartingEndcapStyleId));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Ending endcap style ID", wall.EndingEndcapStyleId));
            data.Add(new MgdDbg.Snoop.Data.String("Wall geometry type", wall.WallGeometryType.ToString()));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Wall interferences", wall.WallInterferences));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Wall merges", wall.WallMerges));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("WallStyle overrides", wall.WallStyleOverrides));
        }
        
        private void
        Stream(ArrayList data, AecDbArch.Stair stair)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDbArch.Stair)));

            data.Add(new MgdDbg.Snoop.Data.ObjectId("Style ID", stair.StyleId));
            data.Add(new MgdDbg.Snoop.Data.Distance("Additional landing width", stair.AdditionalLandingWidth));
            data.Add(new MgdDbg.Snoop.Data.Object("Calculator data", stair.CalculatorData));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Components", stair.Components));

            data.Add(new MgdDbg.Snoop.Data.Int("Clearance body count", stair.ClearanceBodyCount));
            ArrayList clearanceBodies = new ArrayList();
            int len = stair.ClearanceBodyCount;
            for (int i=0; i<len; i++) {
                clearanceBodies.Add(stair.ClearanceBodyAt(i));
            }
            data.Add(new MgdDbg.Snoop.Data.ObjectCollection("Clearance bodies", clearanceBodies));

            data.Add(new MgdDbg.Snoop.Data.Int("Concrete body count", stair.ConcreteBodyCount));
            ArrayList concreteBodies = new ArrayList();
            len = stair.ConcreteBodyCount;
            for (int i=0; i<len; i++) {
                concreteBodies.Add(stair.ConcreteBodyAt(i));
            }
            data.Add(new MgdDbg.Snoop.Data.ObjectCollection("Concrete bodies", concreteBodies));

            data.Add(new MgdDbg.Snoop.Data.Int("Creation time stamp", stair.CreationTimestamp));
            data.Add(new MgdDbg.Snoop.Data.Object("Creator", stair.Creator));
            data.Add(new MgdDbg.Snoop.Data.Object("Defined left polyline", stair.DefinedLeftPolyline));
            data.Add(new MgdDbg.Snoop.Data.Object("Defined right polyline", stair.DefinedRightPolyline));
            data.Add(new MgdDbg.Snoop.Data.Object("Defined walkline", stair.DefinedWalkline));
            data.Add(new MgdDbg.Snoop.Data.Distance("Distance from narrow edge", stair.DistanceFromNarrowEdge));

            data.Add(new MgdDbg.Snoop.Data.Int("Edge polyline count", stair.EdgePolylineCount));
            ArrayList edgePlines = new ArrayList();
            len = stair.EdgePolylineCount;
            for (int i=0; i<len; i++) {
                edgePlines.Add(stair.EdgePolylineAt(i));
            }
            data.Add(new MgdDbg.Snoop.Data.ObjectCollection("Edge polylines", edgePlines));

            data.Add(new MgdDbg.Snoop.Data.String("Flags", stair.Flags.ToString()));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is overlapping", stair.IsOverlapping));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is valid", stair.IsValid));
            data.Add(new MgdDbg.Snoop.Data.String("Justification", stair.Justification.ToString()));

            data.Add(new MgdDbg.Snoop.Data.Int("Landing body count", stair.LandingBodyCount));
            ArrayList landingBodies = new ArrayList();
            len = stair.LandingBodyCount;
            for (int i=0; i<len; i++) {
                landingBodies.Add(stair.LandingBodyAt(i));
            }
            data.Add(new MgdDbg.Snoop.Data.ObjectCollection("Landing bodies", landingBodies));

            data.Add(new MgdDbg.Snoop.Data.Object("Left polyline", stair.LeftPolyline));
            data.Add(new MgdDbg.Snoop.Data.Object("Right polyline", stair.RightPolyline));

            data.Add(new MgdDbg.Snoop.Data.String("Maximum flight limit type", stair.MaximumFlightLimitType.ToString()));
            data.Add(new MgdDbg.Snoop.Data.Distance("Maximum height per flight", stair.MaximumHeightPerFlight));
            data.Add(new MgdDbg.Snoop.Data.Int("Maximum treads per flight", stair.MaximumTreadsPerFlight));
            data.Add(new MgdDbg.Snoop.Data.String("Minimum flight limit type", stair.MinimumFlightLimitType.ToString()));
            data.Add(new MgdDbg.Snoop.Data.Distance("Minimum height per flight", stair.MinimumHeightPerFlight));
            data.Add(new MgdDbg.Snoop.Data.Bool("Minimum landing add tread to start extension", stair.MinimumLandingAddTreadToStartExtension));
            data.Add(new MgdDbg.Snoop.Data.Bool("Minimum landing add tread to end extension", stair.MinimumLandingAddTreadToEndExtension));
            data.Add(new MgdDbg.Snoop.Data.Distance("Minimum landing start extension", stair.MinimumLandingStartExtension));
            data.Add(new MgdDbg.Snoop.Data.Distance("Minimum landing start extension with tread", stair.MinimumLandingStartExtensionWithTread));
            data.Add(new MgdDbg.Snoop.Data.Distance("Minimum landing end extension", stair.MinimumLandingEndExtension));
            data.Add(new MgdDbg.Snoop.Data.Distance("Minimum landing end extension with tread", stair.MinimumLandingEndExtensionWithTread));
            data.Add(new MgdDbg.Snoop.Data.Int("Minimum treads per flight", stair.MinimumTreadsPerFlight));
            data.Add(new MgdDbg.Snoop.Data.Distance("Nosing", stair.Nosing));
            data.Add(new MgdDbg.Snoop.Data.Bool("Open riser", stair.OpenRiser));

            try {
                data.Add(new MgdDbg.Snoop.Data.Object("Outline", stair.Outline));
            }
            catch (Autodesk.AutoCAD.Runtime.Exception ex) {
                data.Add(new MgdDbg.Snoop.Data.Exception("Outline", ex));                                             
            }

            data.Add(new MgdDbg.Snoop.Data.Distance("Required path length", stair.RequiredPathLength));

            data.Add(new MgdDbg.Snoop.Data.Int("Riser body count", stair.RiserBodyCount));
            ArrayList riserBodies = new ArrayList();
            len = stair.RiserBodyCount;
            for (int i=0; i<len; i++) {
                riserBodies.Add(stair.RiserBodyAt(i));
            }
            data.Add(new MgdDbg.Snoop.Data.ObjectCollection("Riser bodies", riserBodies));
            data.Add(new MgdDbg.Snoop.Data.String("Riser type", stair.RiserType.ToString()));

            data.Add(new MgdDbg.Snoop.Data.Int("Stringer body count", stair.StringerBodyCount));
            ArrayList stringerBodies = new ArrayList();
            len = stair.StringerBodyCount;
            for (int i=0; i<len; i++) {
                stringerBodies.Add(stair.StringerBodyAt(i));
            }
            data.Add(new MgdDbg.Snoop.Data.ObjectCollection("Stringer bodies", stringerBodies));

            data.Add(new MgdDbg.Snoop.Data.Int("Stringer facet polyline count", stair.StringerFacetPolylineCount));
            ArrayList strFacetPlines = new ArrayList();
            len = stair.StringerFacetPolylineCount;
            for (int i=0; i<len; i++) {
                strFacetPlines.Add(stair.StringerFacetPolylineAt(i));
            }
            data.Add(new MgdDbg.Snoop.Data.ObjectCollection("Stringer facet polylines", strFacetPlines));

            data.Add(new MgdDbg.Snoop.Data.Int("Stringer polyline count", stair.StringerPolylineCount));
            ArrayList strPlines = new ArrayList();
            len = stair.StringerPolylineCount;
            for (int i=0; i<len; i++) {
                strPlines.Add(stair.StringerPolylineAt(i));
            }
            data.Add(new MgdDbg.Snoop.Data.ObjectCollection("Stringer polylines", strPlines));

            data.Add(new MgdDbg.Snoop.Data.String("Terminate with", stair.TerminateWith.ToString()));

            data.Add(new MgdDbg.Snoop.Data.Int("Tread body count", stair.TreadBodyCount));
            ArrayList treadBodies = new ArrayList();
            len = stair.TreadBodyCount;
            for (int i=0; i<len; i++) {
                treadBodies.Add(stair.TreadBodyAt(i));
            }
            data.Add(new MgdDbg.Snoop.Data.ObjectCollection("Tread bodies", treadBodies));

            data.Add(new MgdDbg.Snoop.Data.Int("Tread count", stair.TreadCount));
            ArrayList treads = new ArrayList();
            len = stair.TreadCount;
            for (int i=0; i<len; i++) {
                treads.Add(stair.TreadAt(i));
            }
            data.Add(new MgdDbg.Snoop.Data.ObjectCollection("Treads", treads));

            data.Add(new MgdDbg.Snoop.Data.Int("Tread front polyline count", stair.TreadFrontPolylineCount));
            ArrayList tfPlines = new ArrayList();
            len = stair.TreadFrontPolylineCount;
            for (int i=0; i<len; i++) {
                tfPlines.Add(stair.TreadFrontPolylineAt(i));
            }
            data.Add(new MgdDbg.Snoop.Data.ObjectCollection("Tread front polylines", tfPlines));

            data.Add(new MgdDbg.Snoop.Data.Int("Tread back polyline count", stair.TreadBackPolylineCount));
            ArrayList tbPlines = new ArrayList();
            len = stair.TreadBackPolylineCount;
            for (int i=0; i<len; i++) {
                tbPlines.Add(stair.TreadBackPolylineAt(i));
            }
            data.Add(new MgdDbg.Snoop.Data.ObjectCollection("Tread back polylines", tbPlines));

            data.Add(new MgdDbg.Snoop.Data.Distance("User path offset", stair.UserPathOffset));
            data.Add(new MgdDbg.Snoop.Data.Object("Walkline", stair.Walkline));
            data.Add(new MgdDbg.Snoop.Data.Distance("Width", stair.Width));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Winder style ID", stair.WinderStyleId));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Width constraints", stair.WidthConstraints));           
        }

        private void
        Stream(ArrayList data, AecDbArch.Zone zone)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDbArch.Zone)));
            
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Style ID", zone.StyleId));
            data.Add(new MgdDbg.Snoop.Data.String("Name", zone.Name));
            data.Add(new MgdDbg.Snoop.Data.Distance("Boundary offset", zone.BoundaryOffset));
            data.Add(new MgdDbg.Snoop.Data.Bool("Can contain spaces", zone.CanContainSpaces));
            data.Add(new MgdDbg.Snoop.Data.Bool("Can contain zones", zone.CanContainZones));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is exclusive space style", zone.IsExclusiveSpaceStyle));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is exclusive zone style", zone.IsExclusiveZoneStyle));
            data.Add(new MgdDbg.Snoop.Data.Object("Space errors", zone.SpaceErrors));

                // TBD: this is actually a AecObjectIdCollection... if important, we need a special Snoop.Data
            data.Add(new MgdDbg.Snoop.Data.Int("Space count", zone.SpaceCount));
            AcDb.ObjectIdCollection spaceIds = new AcDb.ObjectIdCollection();
            foreach (AecDb.ObjectId objId in zone.Spaces)
                spaceIds.Add(objId.Id);
            data.Add(new MgdDbg.Snoop.Data.ObjectIdCollection("Spaces", spaceIds));

            data.Add(new MgdDbg.Snoop.Data.Bool("Use incremental names", zone.UseIncrementalNames));
            data.Add(new MgdDbg.Snoop.Data.Object("Zone errors", zone.ZoneErrors));

                // TBD: this is actually a AecObjectIdCollection... if important, we need a special Snoop.Data
            data.Add(new MgdDbg.Snoop.Data.Int("Zone count", zone.ZoneCount));
            AcDb.ObjectIdCollection zoneIds = new AcDb.ObjectIdCollection();
            foreach (AecDb.ObjectId objId in zone.Zones)
                zoneIds.Add(objId.Id);
            data.Add(new MgdDbg.Snoop.Data.ObjectIdCollection("Zones", zoneIds));
        }


        private void
        Stream(ArrayList data, AecDbArch.Space space)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDbArch.Space)));

            data.Add(new MgdDbg.Snoop.Data.ObjectId("Style ID", space.StyleId));
            data.Add(new MgdDbg.Snoop.Data.Double("Area", space.Area));
            data.Add(new MgdDbg.Snoop.Data.Double("Base volume", space.BaseVolume));
            data.Add(new MgdDbg.Snoop.Data.Double("Overall volume", space.OverallVolume));
            data.Add(new MgdDbg.Snoop.Data.Distance("Perimeter", space.Perimeter));
            data.Add(new MgdDbg.Snoop.Data.Object("Base profile", space.BaseProfile));           
            data.Add(new MgdDbg.Snoop.Data.String("Boundary offset type", space.BoundaryOffsetType.ToString()));
            data.Add(new MgdDbg.Snoop.Data.Distance("Calculation plane 1 height", space.CalculationPlane1Height));
            data.Add(new MgdDbg.Snoop.Data.Distance("Calculation plane 2 height", space.CalculationPlane2Height));
            data.Add(new MgdDbg.Snoop.Data.Distance("Ceiling height", space.CeilingHeight));
            data.Add(new MgdDbg.Snoop.Data.Object("Ceiling surface", space.CeilingSurface));
            data.Add(new MgdDbg.Snoop.Data.Distance("Ceiling thickness", space.CeilingThickness));
            data.Add(new MgdDbg.Snoop.Data.Object("Floor surface", space.FloorSurface));
            data.Add(new MgdDbg.Snoop.Data.Distance("Floor thickness", space.FloorThickness));
            data.Add(new MgdDbg.Snoop.Data.String("Geometry type", space.GeometryType.ToString()));
            data.Add(new MgdDbg.Snoop.Data.Distance("Height at bottom of space below floor", space.HeightAtBottomOfSpaceBelowFloor));
            data.Add(new MgdDbg.Snoop.Data.Distance("Height at bottom of surfaces", space.HeightAtBottomOfSurfaces));
            data.Add(new MgdDbg.Snoop.Data.Distance("Height at top of floor", space.HeightAtTopOfFloor));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Interferences", space.Interferences));
            data.Add(new MgdDbg.Snoop.Data.String("Justification", space.Justification.ToString()));
            data.Add(new MgdDbg.Snoop.Data.String("Name", space.Name));
            //data.Add(new MgdDbg.Snoop.Data.Vector3d("Offset base point", space.OffsetBasePoint));       // TBD: Vector or Point?
            data.Add(new MgdDbg.Snoop.Data.Distance("Space above ceiling", space.SpaceAboveCeiling));
            data.Add(new MgdDbg.Snoop.Data.Distance("Space below floor", space.SpaceBelowFloor));       
            data.Add(new MgdDbg.Snoop.Data.Double("Volume above ceiling", space.VolumeAboveCeiling));
            data.Add(new MgdDbg.Snoop.Data.Double("Volume below floor", space.VolumeBelowFloor));
            data.Add(new MgdDbg.Snoop.Data.Distance("Surface height default", space.SurfaceHeightDefault));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Surfaces", space.Surfaces));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Surfaces freeform", space.SurfacesFreeForm));
            data.Add(new MgdDbg.Snoop.Data.Bool("Use incremental names", space.UseIncrementalNames));
            data.Add(new MgdDbg.Snoop.Data.Bool("Draw hatch", space.DrawHatch));

            data.Add(new MgdDbg.Snoop.Data.Point2d("Centroid ECS (closest=true)", space.CentroidECS(true)));
            data.Add(new MgdDbg.Snoop.Data.Point2d("Centroid ECS (closest=false)", space.CentroidECS(false)));
            data.Add(new MgdDbg.Snoop.Data.Point3d("Centroid WCS (closest=true)", space.CentroidWCS(true)));
            data.Add(new MgdDbg.Snoop.Data.Point3d("Centroid WCS (closest=false)", space.CentroidWCS(false)));

            data.Add(new MgdDbg.Snoop.Data.Object("Interference bodies", space.GetInterferenceBodies()));
            data.Add(new MgdDbg.Snoop.Data.Bool("Associative", space.Associative));
            if (space.Associative)
                data.Add(new MgdDbg.Snoop.Data.Object("Rule based offset base body", space.GetRuleBasedOffsetBaseBody()));
            else
                data.Add(new MgdDbg.Snoop.Data.String("Rule based offset base body", "N/A"));

                // TBD: Wall Areas
            //data.Add(new MgdDbg.Snoop.Data.Double("Wall area (bottom of floor, Top of ceiling, net)", space.WallArea(AecDbArch.SpaceZType.ZBottomOfFloor, AecDbArch.SpaceZType.ZTopOfCeiling, true)));
            //data.Add(new MgdDbg.Snoop.Data.Double("Wall area (bottom of floor, Top of ceiling, gross)", space.WallArea(AecDbArch.SpaceZType.ZBottomOfFloor, AecDbArch.SpaceZType.ZTopOfCeiling, false)));

            foreach (AecDbArch.SpaceProfileType spType in System.Enum.GetValues(typeof(AecDbArch.SpaceProfileType))) {
                data.Add(new MgdDbg.Snoop.Data.CategorySeparator(string.Format("SpaceProfileType = {0}", spType.ToString())));
                data.Add(new MgdDbg.Snoop.Data.Object("Body", space.GetBody(spType)));
                data.Add(new MgdDbg.Snoop.Data.Object("Profile", space.GetProfile(spType)));
                data.Add(new MgdDbg.Snoop.Data.Object("Cached profile", space.GetCachedProfile(spType)));
                data.Add(new MgdDbg.Snoop.Data.Double("Calculated area", space.GetCalculatedArea(spType)));
                data.Add(new MgdDbg.Snoop.Data.Double("Calculated permiter", space.GetCalculatedPerimeter(spType)));
            }

            AcDb.ObjectIdCollection boundaryIds = new AcDb.ObjectIdCollection();
            foreach (AecDb.ObjectId objId in space.GetBoundary())
                boundaryIds.Add(objId.Id);
            data.Add(new MgdDbg.Snoop.Data.ObjectIdCollection("Boundary", boundaryIds));

            AcDb.ObjectIdCollection openingIds = new AcDb.ObjectIdCollection();
            foreach (AecDb.ObjectId objId in space.FindBoundingOpenings())
                openingIds.Add(objId.Id);
            data.Add(new MgdDbg.Snoop.Data.ObjectIdCollection("Bounding openings", openingIds));

            // TBD: GetMaterialId()
        }


        private void
        Stream(ArrayList data, AecDbArch.Roof roof)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDbArch.Roof)));
            
            data.Add(new MgdDbg.Snoop.Data.Bool("Is valid", roof.IsValid));

            try {
                data.Add(new MgdDbg.Snoop.Data.Distance("Overhang", roof.Overhang));
            }            
            catch (Autodesk.AutoCAD.Runtime.Exception ex) {
                data.Add(new MgdDbg.Snoop.Data.Exception("Overhang", ex));
            }

            try {
                data.Add(new MgdDbg.Snoop.Data.Distance("Plate height", roof.PlateHeight));
            }
            catch (Autodesk.AutoCAD.Runtime.Exception ex) {
                data.Add(new MgdDbg.Snoop.Data.Exception("Plate height", ex));
            }

            try {
                data.Add(new MgdDbg.Snoop.Data.Distance("Rise", roof.Rise));
            }
            catch (Autodesk.AutoCAD.Runtime.Exception ex) {
                data.Add(new MgdDbg.Snoop.Data.Exception("Rise", ex));
            }

            data.Add(new MgdDbg.Snoop.Data.Object("Roof profile", roof.RoofProfile));
            data.Add(new MgdDbg.Snoop.Data.Distance("Run", roof.Run));

            try {
                data.Add(new MgdDbg.Snoop.Data.Distance("Slope", roof.Slope));
            }
            catch (Autodesk.AutoCAD.Runtime.Exception ex) {
                data.Add(new MgdDbg.Snoop.Data.Exception("Slope", ex));
            }

            data.Add(new MgdDbg.Snoop.Data.Distance("Thickness", roof.Thickness));
            data.Add(new MgdDbg.Snoop.Data.String("Thickness direction", roof.ThicknessDirection.ToString()));

            try {
                data.Add(new MgdDbg.Snoop.Data.Distance("Upper plate height", roof.UpperPlateHeight));
            }
            catch (Autodesk.AutoCAD.Runtime.Exception ex) {
                data.Add(new MgdDbg.Snoop.Data.Exception("Upper plate height", ex));
            }

            try {
                data.Add(new MgdDbg.Snoop.Data.Distance("Upper rise", roof.UpperRise));
            }
            catch (Autodesk.AutoCAD.Runtime.Exception ex) {
                data.Add(new MgdDbg.Snoop.Data.Exception("Upper rise", ex));
            }

            try {
                data.Add(new MgdDbg.Snoop.Data.Distance("Upper run", roof.UpperRun));
            }
            catch (Autodesk.AutoCAD.Runtime.Exception ex) {
                data.Add(new MgdDbg.Snoop.Data.Exception("Upper run", ex));
            }

            try {
                data.Add(new MgdDbg.Snoop.Data.Distance("Upper slope", roof.UpperSlope));
            }
            catch (Autodesk.AutoCAD.Runtime.Exception ex) {
                data.Add(new MgdDbg.Snoop.Data.Exception("Upper slope", ex));
            }

            int len = roof.CountRoofBodies;
            data.Add(new MgdDbg.Snoop.Data.Int("Count bodies", len));
            ArrayList bodyArray = new ArrayList();
            for (int i=0; i<len; i++){
                AecGeoArch.RoofBody body = roof.RoofBodyAt(i);
                bodyArray.Add(body);
            }
            data.Add(new MgdDbg.Snoop.Data.ObjectCollection("Roof Bodies", bodyArray));

            len = roof.CountRoofLines;
            data.Add(new MgdDbg.Snoop.Data.Int("Count lines", len));
            ArrayList lineArray = new ArrayList();
            for (int i=0; i<len; i++) {
                AecGeoArch.RoofLine line = roof.RoofLineAt(i);
                lineArray.Add(line);
            }
            data.Add(new MgdDbg.Snoop.Data.ObjectCollection("Roof Lines", lineArray));
        }

        private void
        Stream(ArrayList data, AecDbArch.Railing rail)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDbArch.Railing)));

            data.Add(new MgdDbg.Snoop.Data.ObjectId("Style ID", rail.StyleId));
            data.Add(new MgdDbg.Snoop.Data.Double("Base height limit (Max)", rail.GetBaseHeightLimitsMax()));
            data.Add(new MgdDbg.Snoop.Data.Double("Base height limit (Min)", rail.GetBaseHeightLimitsMin()));

            data.Add(new MgdDbg.Snoop.Data.Int("Baluster count", rail.BalusterCount));
            ArrayList balusters = new ArrayList();
            int len = rail.BalusterCount;
            for (int i=0; i<len; i++) {
                balusters.Add(rail.BalusterAt(i));
            }
            data.Add(new MgdDbg.Snoop.Data.ObjectCollection("Balusters", balusters));

            data.Add(new MgdDbg.Snoop.Data.Double("Baluster spacing", rail.BalusterSpacing));
            data.Add(new MgdDbg.Snoop.Data.Int("Baluster per tread", rail.BalustersPerTread));
            data.Add(new MgdDbg.Snoop.Data.Bool("Baluster tread override", rail.BalusterTreadOverride));

            data.Add(new MgdDbg.Snoop.Data.Int("Bottom rail count", rail.BottomrailCount));
                // TBD: how to get BottomRails?
            //ArrayList bottomRails = new ArrayList();
            //int len = rail.BottomrailCount;
            //for (int i=0; i<len; i++) {
            //    bottomRails.Add(rail.BottomRailAt(i));
            //}
            //data.Add(new MgdDbg.Snoop.Data.ObjectCollection("Bottom rails", bottomRails));

            data.Add(new MgdDbg.Snoop.Data.Double("Bottom rail height", rail.BottomrailHeight));
            data.Add(new MgdDbg.Snoop.Data.Double("Bottom rail offset", rail.BottomrailOffset));
            data.Add(new MgdDbg.Snoop.Data.Double("Bottom rail sloping height", rail.BottomrailSlopingHeight));
            data.Add(new MgdDbg.Snoop.Data.Double("Bottom rail spacing", rail.BottomrailSpacing));
            data.Add(new MgdDbg.Snoop.Data.String("Bottom placement", rail.BottomrailPlacement.ToString()));

            data.Add(new MgdDbg.Snoop.Data.Int("Guardrail count", rail.GuardrailCount));
            ArrayList guardRails = new ArrayList();
            len = rail.GuardrailCount;
            for (int i=0; i<len; i++) {
                guardRails.Add(rail.GuardrailAt(i));
            }
            data.Add(new MgdDbg.Snoop.Data.ObjectCollection("Guardrails", guardRails));

            data.Add(new MgdDbg.Snoop.Data.Double("Guardrail height", rail.GuardrailHeight));
            data.Add(new MgdDbg.Snoop.Data.Double("Guardrail offset", rail.GuardrailOffset));
            data.Add(new MgdDbg.Snoop.Data.Double("Guardrail sloping height", rail.GuardrailSlopingHeight));
            data.Add(new MgdDbg.Snoop.Data.String("Guardrail placement", rail.GuardrailPlacement.ToString()));

            data.Add(new MgdDbg.Snoop.Data.Int("Handrail count", rail.HandrailCount));
            ArrayList handRails = new ArrayList();
            len = rail.HandrailCount;
            for (int i=0; i<len; i++) {
                handRails.Add(rail.HandrailAt(i));
            }
            data.Add(new MgdDbg.Snoop.Data.ObjectCollection("Handrails", handRails));

            data.Add(new MgdDbg.Snoop.Data.Double("Handrail height", rail.HandrailHeight));
            data.Add(new MgdDbg.Snoop.Data.Double("Handrail offset", rail.HandrailOffset));
            data.Add(new MgdDbg.Snoop.Data.Double("Handrail sloping height", rail.HandrailSlopingHeight));
            data.Add(new MgdDbg.Snoop.Data.String("Handrail placement", rail.HandrailPlacement.ToString()));

            data.Add(new MgdDbg.Snoop.Data.Int("Lower rail count", rail.LowerRailCount));
            ArrayList lowerRails = new ArrayList();
            len = rail.LowerRailCount;
            for (int i=0; i<len; i++) {
                lowerRails.Add(rail.LowerRailAt(i));
            }
            data.Add(new MgdDbg.Snoop.Data.ObjectCollection("Lower rails", lowerRails));

            data.Add(new MgdDbg.Snoop.Data.Int("Main post count", rail.MainPostCount));
            ArrayList mainPosts = new ArrayList();
            len = rail.MainPostCount;
            for (int i=0; i<len; i++) {
                mainPosts.Add(rail.MainPostAt(i));
            }
            data.Add(new MgdDbg.Snoop.Data.ObjectCollection("Main posts", mainPosts));

            data.Add(new MgdDbg.Snoop.Data.Double("Max post spacing", rail.MaxPostSpacing));
            data.Add(new MgdDbg.Snoop.Data.Double("Minimum length", rail.MinimumLength));
            data.Add(new MgdDbg.Snoop.Data.Double("Minimum post spacing", rail.MinPostSpacing));

            data.Add(new MgdDbg.Snoop.Data.Int("Post count", rail.PostCount));
            ArrayList posts = new ArrayList();
            len = rail.PostCount;
            for (int i=0; i<len; i++) {
                posts.Add(rail.PostAt(i));
            }
            data.Add(new MgdDbg.Snoop.Data.ObjectCollection("Posts", posts));
            data.Add(new MgdDbg.Snoop.Data.Bool("Posts maintain slope on landings", rail.PostsMaintainSlopeOnLandings));
            data.Add(new MgdDbg.Snoop.Data.Bool("Posts perpendicular", rail.PostsPerpendicular));
            data.Add(new MgdDbg.Snoop.Data.Bool("Posts perpendicular to surface", rail.PostsPerpendicularToSurface));            

            data.Add(new MgdDbg.Snoop.Data.Int("Sub post count", rail.SubPostCount));
            ArrayList subPosts = new ArrayList();
            len = rail.SubPostCount;
            for (int i=0; i<len; i++) {
                subPosts.Add(rail.SubPostAt(i));
            }
            data.Add(new MgdDbg.Snoop.Data.ObjectCollection("Sub posts", subPosts));

            data.Add(new MgdDbg.Snoop.Data.Bool("Use stair landing extension at floor", rail.UseStairLandingExtensionAtFloor));
            data.Add(new MgdDbg.Snoop.Data.Bool("Use stair landing extension at landing", rail.UseStairLandingExtensionAtLanding));

            data.Add(new MgdDbg.Snoop.Data.Object("Path", rail.GetPath()));
            data.Add(new MgdDbg.Snoop.Data.Object("Post path", rail.GetPostPath()));

            data.Add(new MgdDbg.Snoop.Data.CategorySeparator("Explicit Posts"));

            data.Add(new MgdDbg.Snoop.Data.Int("Explicit post count", rail.ExplicitPostCount));
            len = rail.ExplicitPostCount;
            for (int i=0; i<len; i++) {
                data.Add(new MgdDbg.Snoop.Data.String(string.Format("Explicit post flags [{0}]", i), rail.GetExplicitPostFlags(i).ToString()));
                data.Add(new MgdDbg.Snoop.Data.Point3d(string.Format("Explicit post position [{0}]", i), rail.GetExplicitPostPosition(i)));
                data.Add(new MgdDbg.Snoop.Data.Distance(string.Format("Explicit post position from start [{0}]", i), rail.GetExplicitPostPositionFromStart(i)));
            }
        }

        private void
        Stream(ArrayList data, AecDbArch.SlabBase sBase)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDbArch.SlabBase)));

            data.Add(new MgdDbg.Snoop.Data.ObjectId("Style ID", sBase.StyleId));
            data.Add(new MgdDbg.Snoop.Data.Distance("Base thickness", sBase.BaseThickness));
            data.Add(new MgdDbg.Snoop.Data.Point3d("Centroid", sBase.Centroid));

            try {
                data.Add(new MgdDbg.Snoop.Data.Bool("Does slab edges have style", sBase.DoesSlabEdgesHaveStyle));
            }
            catch (Autodesk.AutoCAD.Runtime.Exception ex) {
                data.Add(new MgdDbg.Snoop.Data.Exception("Does slab edges have style", ex));
            }
 
            data.Add(new MgdDbg.Snoop.Data.Distance("Exterior face offset", sBase.ExteriorFaceOffset));
            data.Add(new MgdDbg.Snoop.Data.Object("Exterior plane", sBase.ExteriorPlane));
            data.Add(new MgdDbg.Snoop.Data.Distance("Interior face offset", sBase.InteriorFaceOffset));
            data.Add(new MgdDbg.Snoop.Data.Object("Interior plane", sBase.InteriorPlane));
            data.Add(new MgdDbg.Snoop.Data.Object("Face", sBase.Face));
            data.Add(new MgdDbg.Snoop.Data.Bool("Fascial height is constant", sBase.FasciaHeightIsConstant));
            data.Add(new MgdDbg.Snoop.Data.Double("Gross area", sBase.GrossArea));
            data.Add(new MgdDbg.Snoop.Data.Distance("High point", sBase.HighPoint));
            data.Add(new MgdDbg.Snoop.Data.Distance("Low point", sBase.LowPoint));
            data.Add(new MgdDbg.Snoop.Data.Distance("Horizontal offset", sBase.HorizontalOffset));
            data.Add(new MgdDbg.Snoop.Data.Distance("Vertical offset", sBase.VerticalOffset));
            data.Add(new MgdDbg.Snoop.Data.Bool("Vertical offset is constant", sBase.VerticalOffsetIsConstant));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is valid", sBase.IsValid));
            data.Add(new MgdDbg.Snoop.Data.Double("Net area", sBase.NetArea));
            data.Add(new MgdDbg.Snoop.Data.Object("Offset face", sBase.OffsetFace));
            data.Add(new MgdDbg.Snoop.Data.Object("Offset true curve face", sBase.OffsetTrueCurveFace));
            data.Add(new MgdDbg.Snoop.Data.Bool("Overhang is constant", sBase.OverhangIsConstant));
            data.Add(new MgdDbg.Snoop.Data.Point3d("Pivot point", sBase.PivotPoint));
            data.Add(new MgdDbg.Snoop.Data.Angle("Slope", sBase.Slope));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Interferences", sBase.Interferences));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Custom geometries", sBase.CustomGeometries));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Fascia bodies", sBase.FasciaBodies));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Layer bodies", sBase.LayerBodies));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Soffit bodies", sBase.SoffitBodies));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Interference bodies", sBase.GetInterferenceBodies()));

            AecDbArch.RoofSlab rSlab = sBase as AecDbArch.RoofSlab;
            if (rSlab != null) {
                Stream(data, rSlab);
                return;
            }

            AecDbArch.Slab slab = sBase as AecDbArch.Slab;
            if (slab != null) {
                Stream(data, slab);
                return;
            }
        }

        private void
        Stream(ArrayList data, AecDbArch.RoofSlab rSlab)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDbArch.RoofSlab)));

            // no data at this level
        }

        private void
        Stream(ArrayList data, AecDbArch.Slab slab)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDbArch.Slab)));

            // no data at this level
        }

        private void
        Stream(ArrayList data, AecDbArch.CurtainWallLayout curtainWallLayout)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDbArch.CurtainWallLayout)));

            data.Add(new MgdDbg.Snoop.Data.String("Base curve type", curtainWallLayout.BaseCurveType.ToString()));
            data.Add(new MgdDbg.Snoop.Data.Distance("Base height", curtainWallLayout.BaseHeight));
            data.Add(new MgdDbg.Snoop.Data.Distance("Length", curtainWallLayout.Length));
            data.Add(new MgdDbg.Snoop.Data.Object("CurtainWallLayout base Curve", curtainWallLayout.CurtainWallLayoutBaseCurve));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Referenced base curve", curtainWallLayout.ReferencedBaseCurve));
            data.Add(new MgdDbg.Snoop.Data.Object("Floor line", curtainWallLayout.FloorLine));
            data.Add(new MgdDbg.Snoop.Data.Object("RoofLine", curtainWallLayout.RoofLine));
        }


        private void
        Stream(ArrayList data, AecDbArch.CurtainWallUnit curtainWallUnit)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDbArch.CurtainWallUnit)));

            data.Add(new MgdDbg.Snoop.Data.Distance("Height", curtainWallUnit.Height));
            data.Add(new MgdDbg.Snoop.Data.Distance("Length", curtainWallUnit.Length));
        }

        private void
        Stream(ArrayList data, AecDbArch.WindowAssembly windowAssembly)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDbArch.WindowAssembly)));

            data.Add(new MgdDbg.Snoop.Data.Distance("Height", windowAssembly.Height));
            data.Add(new MgdDbg.Snoop.Data.Distance("Length", windowAssembly.Length));
            data.Add(new MgdDbg.Snoop.Data.Distance("Rise", windowAssembly.Rise));
            data.Add(new MgdDbg.Snoop.Data.Object("Open shape", windowAssembly.OpenShape));
        }

        private void
        Stream(ArrayList data, AecDbArch.CeilingGrid ceilingGrid)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDbArch.CeilingGrid)));

            data.Add(new MgdDbg.Snoop.Data.Object("Boundary profile", ceilingGrid.BoundaryProfile));
        }

        private void
        Stream(ArrayList data, AecDbArch.BuildingElevationLine buildingElevLine)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDbArch.BuildingElevationLine)));

            data.Add(new MgdDbg.Snoop.Data.String("Default elevation line name (static)", AecDbArch.BuildingElevationLine.DefaultElevationLineName));
        }
            
        private void
        Stream(ArrayList data, AecDbArch.BuildingSectionLine buildingSectLine)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDbArch.BuildingSectionLine)));

            data.Add(new MgdDbg.Snoop.Data.String("Default section line name (static)", AecDbArch.BuildingSectionLine.DefaultSectionLineName));
        }

        private void
        Stream(ArrayList data, AecDbArch.BuildingSection buildingSect)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDbArch.BuildingSection)));                       
        }
    }
}
