
// $Header: //Desktop/Source/TestTools/MgdDbgAec/Snoop/CollectorExts/EntityBase.cs#6 $
// $Author: ayyara $ $DateTime: 2008/02/28 10:13:37 $ $Change: 94226 $
//
// (C) Copyright 2004 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//



using System;
using System.Diagnostics;
using System.Collections;
using System.Windows.Forms;
using System.Collections.Specialized;

using MgdDbg.Snoop;
using Autodesk.AutoCAD.Geometry;

using AecDb     = Autodesk.Aec.DatabaseServices;
using AcDb      = Autodesk.AutoCAD.DatabaseServices;
using AecDbArch = Autodesk.Aec.Arch.DatabaseServices;

namespace MgdDbgAec.Snoop.CollectorExts
{
	/// <summary>
	/// This is a Snoop Collector Extension object to collect data from AEC entities.
	/// </summary>
    public class EntityBase : MgdDbg.Snoop.CollectorExts.CollectorExt {
        public EntityBase()
        {
        }

        protected override void
        CollectEvent(object sender, MgdDbg.Snoop.Collectors.CollectorEventArgs e)
        {
            // bail early if this isn't an object we are responsible for
            AecDb.Entity aecEnt = e.ObjToSnoop as AecDb.Entity;
            if (aecEnt == null)
                return;

            // cast the sender object to the SnoopCollector we are expecting
            MgdDbg.Snoop.Collectors.Collector snoopCollector = sender as MgdDbg.Snoop.Collectors.Collector;
            if (snoopCollector == null) {
                Debug.Assert(false);    // why did someone else send us the message?
                return;
            }

            Stream(snoopCollector.Data(), aecEnt);
        }


        private void
        Stream(ArrayList data, AecDb.Entity ent)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDb.Entity)));

            try {
                data.Add(new MgdDbg.Snoop.Data.String("Automatically bound spaces", ent.AutomaticallyBoundSpaces.ToString()));
            }
            catch (Autodesk.AutoCAD.Runtime.Exception ex) {
                data.Add(new MgdDbg.Snoop.Data.Exception("Automatically bound spaces", ex));
            }

            data.Add(new MgdDbg.Snoop.Data.String("Display name", ent.DisplayName));
            data.Add(new MgdDbg.Snoop.Data.Icon("Type icon", ent.TypeIcon));
            data.Add(new MgdDbg.Snoop.Data.String("Description", ent.Description));
            data.Add(new MgdDbg.Snoop.Data.String("Layer key", ent.LayerKey));

#if !ACA2009
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Classifying style ID", ent.ClassifyingStyleId));
#endif

#if ACA2009
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Style ID", ent.StyleId));
#endif

#if(VS2003)
            IntegerCollection compIds = null;
            StringCollection compNames = null;
            AcDb.ObjectIdCollection materialIds = null;
            ent.GetMaterialComponents(out compIds, out compNames, out materialIds);
            Debug.Assert((compIds.Count == compNames.Count) && (compIds.Count == materialIds.Count));
            
            int len = compIds.Count;
            for (int i=0; i<len; i++) {
                data.Add(new MgdDbg.Snoop.Data.ObjectId(string.Format("Component material ID ({0:d}, {1})", compIds[i], compNames[i]), materialIds[i]));
            }

                // get material location info (if any)
            IntegerCollection locationIds = null;
            AcDb.ObjectIdCollection styleIds = null;
            ent.GetMaterialLocations(out locationIds, out styleIds);
            Debug.Assert(locationIds.Count == styleIds.Count);
            
            len = locationIds.Count;
            for (int i=0; i<len; i++) {
                data.Add(new MgdDbg.Snoop.Data.ObjectId(string.Format("Material location ID ({0:d})", locationIds[i]), styleIds[i]));
            }
#endif

            try {
                BoundBlock3d bndBlk1 = null;
                ent.GetLocalModelCachedExtents(out bndBlk1);
                data.Add(new MgdDbg.Snoop.Data.Object("Local model cached extents", bndBlk1));
            }
            catch (Autodesk.AutoCAD.Runtime.Exception e) {
                data.Add(new MgdDbg.Snoop.Data.Exception("Local model cached extents", e));
            }

            try {
                BoundBlock3d bndBlk2 = null;
                ent.GetLocalExtents(out bndBlk2);
                data.Add(new MgdDbg.Snoop.Data.Object("Local extents", bndBlk2));
            }
            catch (Autodesk.AutoCAD.Runtime.Exception e) {
                data.Add(new MgdDbg.Snoop.Data.Exception("Local extents", e));
            }

            try {
                BoundBlock3d bndBlk3 = null;
                ent.GetWorldExtents(out bndBlk3);
                data.Add(new MgdDbg.Snoop.Data.Object("World extents", bndBlk3));
            }
            catch (Autodesk.AutoCAD.Runtime.Exception e) {
                data.Add(new MgdDbg.Snoop.Data.Exception("World extents", e));
            }

            data.Add(new MgdDbg.Snoop.Data.Object("Base curve", ent.BaseCurve));
            data.Add(new MgdDbg.Snoop.Data.Bool("Supports profile commands", ent.SupportsProfileCommands));
            if (ent.SupportsProfileCommands) {
                Matrix3d toWcs;
                data.Add(new MgdDbg.Snoop.Data.Object("Base profile", ent.GetBaseProfile(out toWcs)));
            }
            data.Add(new MgdDbg.Snoop.Data.Bool("Supports base curve commands", ent.SupportsBaseCurveCommands));

            AecDb.Geo geo = ent as AecDb.Geo;
            if (geo != null) {
                Stream(data, geo);
            }
        }

        private void
        Stream(ArrayList data, AecDb.Geo geo)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDb.Geo)));

            data.Add(new MgdDbg.Snoop.Data.Point3d("Location", geo.Location));
            data.Add(new MgdDbg.Snoop.Data.Vector3d("Direction", geo.Direction));
            data.Add(new MgdDbg.Snoop.Data.Angle("Rotation", geo.Rotation));
            data.Add(new MgdDbg.Snoop.Data.Vector3d("Normal", geo.Normal));
            data.Add(new MgdDbg.Snoop.Data.Vector3d("X Dir", geo.XDir));
            data.Add(new MgdDbg.Snoop.Data.Vector3d("Y Dir", geo.YDir));
            data.Add(new MgdDbg.Snoop.Data.Vector3d("Z Dir", geo.ZDir));
            data.Add(new MgdDbg.Snoop.Data.Object("Geo ECS", geo.GeoEcs));
            data.Add(new MgdDbg.Snoop.Data.Bool("ECS is dirty", geo.GeoEcsIsDirty));
            data.Add(new MgdDbg.Snoop.Data.Bool("Can be anchored", geo.CanBeAnchored));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Anchor ID", geo.AnchorId));

            AecDb.MassElement massElem = geo as AecDb.MassElement;
            if (massElem != null) {
                Stream(data, massElem);
                return;
            }

            AecDb.MassGroup massGroup = geo as AecDb.MassGroup;
            if (massGroup != null) {
                Stream(data, massGroup);
                return;
            }

            AecDb.BlockReference blkRef = geo as AecDb.BlockReference;
            if (blkRef != null) {
                Stream(data, blkRef);
                return;
            }

            AecDb.Polygon poly = geo as AecDb.Polygon;
            if (poly != null) {
                Stream(data, poly);
                return;
            }

            AecDb.LayoutTool layoutTool = geo as AecDb.LayoutTool;
            if (layoutTool != null) {
                Stream(data, layoutTool);
                return;
            }

            AecDb.ClipVolumeResult volRes = geo as AecDb.ClipVolumeResult;
            if (volRes != null) {
                Stream(data, volRes);
                return;
            }

            AecDb.Section2d section2d = geo as AecDb.Section2d;
            if (section2d != null) {
                Stream(data, section2d);
            }
        }

        private void
        Stream(ArrayList data, AecDb.MassElement massElem)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDb.MassElement)));

            data.Add(new MgdDbg.Snoop.Data.ObjectId("Style ID", massElem.StyleId));
            data.Add(new MgdDbg.Snoop.Data.String("Shape type", massElem.ShapeType.ToString()));
            data.Add(new MgdDbg.Snoop.Data.String("Shape sub-type", massElem.ShapeSubType.ToString()));

                // The dimension properties will throw an exception if we aren't doing the ones that make sense
                // for a given shape.  Here, we will put each one in a try/catch so we can see what the return value
                // is in all cases.
            try {
                data.Add(new MgdDbg.Snoop.Data.Distance("Width", massElem.Width));
            }
            catch (Autodesk.AutoCAD.Runtime.Exception e) {
                data.Add(new MgdDbg.Snoop.Data.Exception("Width", e));   
            }

            try {
                data.Add(new MgdDbg.Snoop.Data.Distance("Depth", massElem.Depth));
            }
            catch (Autodesk.AutoCAD.Runtime.Exception e) {
                data.Add(new MgdDbg.Snoop.Data.Exception("Depth", e));   
            }

            try {
                data.Add(new MgdDbg.Snoop.Data.Distance("Height", massElem.Height));
            }
            catch (Autodesk.AutoCAD.Runtime.Exception e) {
                data.Add(new MgdDbg.Snoop.Data.Exception("Height", e));   
            }

            try {
                data.Add(new MgdDbg.Snoop.Data.Distance("Radius", massElem.Radius));
            }
            catch (Autodesk.AutoCAD.Runtime.Exception e) {
                data.Add(new MgdDbg.Snoop.Data.Exception("Radius", e));   
            }

            try {
                data.Add(new MgdDbg.Snoop.Data.Distance("Rise", massElem.Rise));
            }
            catch (Autodesk.AutoCAD.Runtime.Exception e) {
                data.Add(new MgdDbg.Snoop.Data.Exception("Rise", e));   
            }

            data.Add(new MgdDbg.Snoop.Data.Double("Deviation", massElem.Deviation));
            data.Add(new MgdDbg.Snoop.Data.Double("Volume", massElem.Volume));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Profile ID", massElem.ProfileId));

            try {
                data.Add(new MgdDbg.Snoop.Data.Object("Profile", massElem.Profile));
            }
            catch (Autodesk.AutoCAD.Runtime.Exception e) {
                data.Add(new MgdDbg.Snoop.Data.Exception("Profile", e));   
            }

            try {
                data.Add(new MgdDbg.Snoop.Data.Object("Body", massElem.Body));
            }
            catch (Autodesk.AutoCAD.Runtime.Exception e) {
                data.Add(new MgdDbg.Snoop.Data.Exception("Body", e));   
            }

            try {
                AcDb.ObjectId parentId;
                AecDb.Operation parentOp;
                massElem.ParentMassing(out parentId, out parentOp);

                data.Add(new MgdDbg.Snoop.Data.ObjectId("Parent massing ID", parentId));
                data.Add(new MgdDbg.Snoop.Data.String("Parent massing operation", parentOp.ToString()));
            }
            catch (Autodesk.AutoCAD.Runtime.Exception e) {
                data.Add(new MgdDbg.Snoop.Data.Exception("Parent massing", e));   
            }
        }

        private void
        Stream(ArrayList data, AecDb.MassGroup massGroup)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDb.MassGroup)));

            data.Add(new MgdDbg.Snoop.Data.String("Sub-type", massGroup.SubType));
            data.Add(new MgdDbg.Snoop.Data.Object("Body", massGroup.Body));
            data.Add(new MgdDbg.Snoop.Data.Double("Volume", massGroup.Volume));

            data.Add(new MgdDbg.Snoop.Data.ObjectIdCollection("Nodes", massGroup.Nodes));
            int i=0;
            foreach (AcDb.ObjectId nodeId in massGroup.Nodes) {
                data.Add(new MgdDbg.Snoop.Data.ObjectIdAndOperation(string.Format("Node [{0}]", i++), nodeId, massGroup.GetOperation(nodeId)));
            }   

            data.Add(new MgdDbg.Snoop.Data.ObjectIdCollection("Entities", massGroup.Entities));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Root node", massGroup.RootNode));

            // TBD: this doesn't seem to be quite right.  If you have one MassGroup
            // nested inside another, they both report a parentMassing.
            AecDb.Operation op2 = new AecDb.Operation();
            AcDb.ObjectId nodeId2 = massGroup.ParentMassing(out op2);
            if (nodeId2.IsNull)
                data.Add(new MgdDbg.Snoop.Data.ObjectId("Parent massing", nodeId2));
            else
                data.Add(new MgdDbg.Snoop.Data.ObjectIdAndOperation("Parent massing", nodeId2, op2));
        }

        private void
        Stream(ArrayList data, AecDb.BlockReference blkRef)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDb.BlockReference)));

            data.Add(new MgdDbg.Snoop.Data.ObjectId("Block def ID", blkRef.BlockDefId));
            data.Add(new MgdDbg.Snoop.Data.Scale3d("Scale factors", blkRef.Scale));

            AecDb.MultiViewBlockReference mvBlkRef = blkRef as AecDb.MultiViewBlockReference;
            if (mvBlkRef != null) {
                Stream(data, mvBlkRef);
                return;
            }

            AecDb.EntityReference entRef = blkRef as AecDb.EntityReference;
            if (entRef != null) {
                Stream(data, entRef);
                return;
            }

            AecDb.MaskBlockReference maskBlkRef = blkRef as AecDb.MaskBlockReference;
            if (maskBlkRef != null) {
                Stream(data, maskBlkRef);
                return;
            }
        }

        private void
        Stream(ArrayList data, AecDb.MultiViewBlockReference mvBlkRef)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDb.MultiViewBlockReference)));

            data.Add(new MgdDbg.Snoop.Data.Bool("Has attributes", mvBlkRef.HasAttributes));

            try {
                data.Add(new MgdDbg.Snoop.Data.Int("Number of annotation scale contexts", mvBlkRef.NumberOfAnnotationScaleContexts));
            }
            catch (Autodesk.AutoCAD.Runtime.Exception ex) {
                data.Add(new MgdDbg.Snoop.Data.Exception("Number of annotation scale contexts", ex));
            }

            try {
                data.Add(new MgdDbg.Snoop.Data.Vector3d("Scale dependent offset", mvBlkRef.ScaleDependentOffset));
            }
            catch (Autodesk.AutoCAD.Runtime.Exception ex) {
                data.Add(new MgdDbg.Snoop.Data.Exception("Number of annotation scale contexts", ex));
            }


            data.Add(new MgdDbg.Snoop.Data.Enumerable("View instances", mvBlkRef.ViewInstances));
        }
       
        private void
        Stream(ArrayList data, AecDb.EntityReference entRef)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDb.EntityReference)));

            data.Add(new MgdDbg.Snoop.Data.Bool("Use offset", entRef.UseOffset));
            data.Add(new MgdDbg.Snoop.Data.Vector3d("Insertion point offset", entRef.InsertionPointOffset));
            data.Add(new MgdDbg.Snoop.Data.Point3d("Insertion point of referred entity", entRef.InsertionPointOfReferredEntity));
            data.Add(new MgdDbg.Snoop.Data.Object("Referred entity ECS", entRef.ReferredEntityEcs));
            data.Add(new MgdDbg.Snoop.Data.Object("Referred entity ECS inverse", entRef.ReferredEntityEcsInverse));
        }

        private void
        Stream(ArrayList data, AecDb.MaskBlockReference entRef)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDb.MaskBlockReference)));

            data.Add(new MgdDbg.Snoop.Data.Object("Cut profile", entRef.CutProfile));
        }

        private void
        Stream(ArrayList data, AecDb.Polygon poly)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDb.Polygon)));

            data.Add(new MgdDbg.Snoop.Data.ObjectId("Style ID", poly.StyleId));
            data.Add(new MgdDbg.Snoop.Data.Distance("Edge perimeter", poly.EdgePerimeter));
            data.Add(new MgdDbg.Snoop.Data.Distance("Infill perimiter", poly.InfillPerimeter));
            data.Add(new MgdDbg.Snoop.Data.Distance("Gross area", poly.GrossArea));
            data.Add(new MgdDbg.Snoop.Data.Distance("Infill area", poly.InfillArea));
        }

        private void
        Stream(ArrayList data, AecDb.LayoutTool layTool)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDb.LayoutTool)));

            IntegerCollection nodeIds = layTool.LayoutNodeIds;
            data.Add(new MgdDbg.Snoop.Data.Object("Layout node IDs", nodeIds));
            for (int i=0; i<nodeIds.Count; i++) {

                try {
                    data.Add(new MgdDbg.Snoop.Data.Point3d(string.Format("Node Id [{0}], location", i), layTool.GetLayoutNodeLocation(i)));
                }
                catch (Autodesk.AutoCAD.Runtime.Exception ex) {
                    data.Add(new MgdDbg.Snoop.Data.Exception(string.Format("Node Id [{0}], location", i), ex));
                }

                try {
                    data.Add(new MgdDbg.Snoop.Data.Object(string.Format("Node Id [{0}], coord system", i), layTool.GetLayoutNodeCoordinateSystem(i)));
                }
                catch (Autodesk.AutoCAD.Runtime.Exception ex) {
                    data.Add(new MgdDbg.Snoop.Data.Exception(string.Format("Node Id [{0}], coord system", i), ex));
                }
                
            }

            AecDb.CellLayoutTool cellLayout = layTool as AecDb.CellLayoutTool;
            if (cellLayout != null) {
                Stream(data, cellLayout);
                return;
            }

            AecDb.LayoutCurve curveLayout = layTool as AecDb.LayoutCurve;
            if (curveLayout != null) {
                Stream(data, curveLayout);
                return;
            }

            AecDb.ClipVolume clipVol = layTool as AecDb.ClipVolume;
            if (clipVol != null) {
                Stream(data, clipVol);
                return;
            }

            AecDb.Slice slice = layTool as AecDb.Slice;
            if (slice != null) {
                Stream(data, slice);
                return;
            }
        }

        private void
        Stream(ArrayList data, AecDb.CellLayoutTool cellLayout)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDb.CellLayoutTool)));

            data.Add(new MgdDbg.Snoop.Data.Object("Layout cell IDs", cellLayout.LayoutCellIds));

            AecDb.LayoutGrid2d grid2d = cellLayout as AecDb.LayoutGrid2d;
            if (grid2d != null) {
                Stream(data, grid2d);
                return;
            }

            AecDb.VolumeLayoutTool volLayout = cellLayout as AecDb.VolumeLayoutTool;
            if (volLayout != null) {
                Stream(data, volLayout);
                return;
            }

            AecDb.GridAssembly gridAssembly = cellLayout as AecDb.GridAssembly;
            if (gridAssembly != null) {
                Stream(data, gridAssembly);
                return;
            }
        }

        private void
        Stream(ArrayList data, AecDb.LayoutGrid2d grid2d)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDb.LayoutGrid2d)));

            data.Add(new MgdDbg.Snoop.Data.String("Grid type", grid2d.GridType.ToString()));
            data.Add(new MgdDbg.Snoop.Data.Object("Base grid", grid2d.BaseGrid));
            data.Add(new MgdDbg.Snoop.Data.Object("Custom grid", grid2d.CustomGrid));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Boundary ID", grid2d.BoundaryId));

                // NOTE: GetBoundaryProfile will return null if BoundaryId is null
            data.Add(new MgdDbg.Snoop.Data.Object("Boundary profile", grid2d.BoundaryProfile));
            data.Add(new MgdDbg.Snoop.Data.ObjectIdCollection("Boundary holes", grid2d.BoundaryHoles));
        }

        private void
        Stream(ArrayList data, AecDb.VolumeLayoutTool volLayout)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDb.VolumeLayoutTool)));

            data.Add(new MgdDbg.Snoop.Data.Object("Volume layout IDs", volLayout.LayoutVolumeIds));

            AecDb.LayoutGrid3d grid3d = volLayout as AecDb.LayoutGrid3d;
            if (grid3d != null) {
                Stream(data, grid3d);
                return;
            }
        }

        private void
        Stream(ArrayList data, AecDb.GridAssembly gridAssembly)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDb.GridAssembly)));

            // methods

            // GetCell
            int lenCell = gridAssembly.CellCount;
            if (lenCell > 0) {
                AecDb.GridAssemblyCell gaCell = gridAssembly.GetCellAt(0);
                int id = gaCell.CellId;
                // test GetCell(id)
                AecDb.GridAssemblyCell gaCellById = gridAssembly.GetCell(id);
                data.Add(new MgdDbg.Snoop.Data.Object("Get first cell using index", gaCell));
                data.Add(new MgdDbg.Snoop.Data.Object("Get first cell using id", gaCellById));
            }

            // GetCellAt
            ArrayList cells = new ArrayList();
            for (int i = 0; i < lenCell; i++)
                cells.Add(gridAssembly.GetCellAt(i));
            data.Add(new MgdDbg.Snoop.Data.ObjectCollection("GridAssemblyCells", cells));

            // GetEdge(IntegerCollection), GetEdge(IntegerCollection, out edgeIndex), GetEdge(id), 
            int lenEdge = gridAssembly.EdgeCount;
            if (lenEdge > 0) {
                //get first edge
                AecDb.GridAssemblyEdge gaEdge = gridAssembly.GetEdgeAt(0);
                data.Add(new MgdDbg.Snoop.Data.Object("Get first edge using index", gaEdge));

                // I want to get a valid nesting path just to test the GetEdge(path) below
                IntegerCollection path = gaEdge.NestingPath;
                AecDb.GridAssemblyEdge edgeByPath = gridAssembly.GetEdge(path);
                data.Add(new MgdDbg.Snoop.Data.Object("Get first edge using path", edgeByPath));

                // Same thing, but now testing GetEdge(IntegerCollection, out edgeIndex - which should be 0 in this case!)
                int edgeIndex;
                AecDb.GridAssemblyEdge edgeByPath2 = gridAssembly.GetEdge(path, out edgeIndex);
                data.Add(new MgdDbg.Snoop.Data.Int("Index of first edge using path", edgeIndex));

                // test GetEdge(edgeId)
                int edgeId = gaEdge.EdgeId;
                AecDb.GridAssemblyEdge edgeById = gridAssembly.GetEdge(edgeId);
                String tempStr = "Get first edge using its edge id = " + edgeId.ToString();
                data.Add(new MgdDbg.Snoop.Data.Object(tempStr, edgeById));

                // Same thing, but now testing GetEdge(edgeId, out edgeIndex - which should be 0 in this case!)
                AecDb.GridAssemblyEdge edgeById2 = gridAssembly.GetEdge(path, out edgeIndex);
                data.Add(new MgdDbg.Snoop.Data.Int("Index of first edge using id", edgeIndex));
            }

            // GetEdgeAt(Index)
            ArrayList edges = new ArrayList();
            for (int i = 0; i < lenEdge; i++)
                edges.Add(gridAssembly.GetEdgeAt(i));
            data.Add(new MgdDbg.Snoop.Data.ObjectCollection("GridAssemblyEdges", edges));


            if (gridAssembly.HasClippingBoundary) {
                if (gridAssembly.HasClippingRectangle) {

                    // MH: modified for 2010 
                    //double x, y, endX, endY;
                    //gridAssembly.GetClippingRectangle(out x, out y, out endX, out endY);
                    //data.Add(new MgdDbg.Snoop.Data.Double("Clipping rectangle X", x));
                    //data.Add(new MgdDbg.Snoop.Data.Double("Clipping rectangle Y", y));
                    //data.Add(new MgdDbg.Snoop.Data.Double("Clipping rectangle end X", endX));
                    //data.Add(new MgdDbg.Snoop.Data.Double("Clipping rectangle end Y", endY));

                    AecDb.ClippingRectangleInfo clipRect = gridAssembly.ClippingRectangle;
                    data.Add(new MgdDbg.Snoop.Data.Double("Clipping rectangle X", clipRect.ClipX));
                    data.Add(new MgdDbg.Snoop.Data.Double("Clipping rectangle Y", clipRect.ClipY));
                    data.Add(new MgdDbg.Snoop.Data.Double("Clipping rectangle end X", clipRect.ClipEndX));
                    data.Add(new MgdDbg.Snoop.Data.Double("Clipping rectangle end Y", clipRect.ClipEndY));

                }
            }
            // GetMaterialIds
            AcDb.ObjectIdCollection frameIds = null;
            gridAssembly.GetMaterialIds(AecDb.GridAssemblyComponentType.Frame, ref frameIds);
            data.Add(new MgdDbg.Snoop.Data.ObjectIdCollection("Material ids (frame)", frameIds));

            AcDb.ObjectIdCollection infillIds = null; 
            gridAssembly.GetMaterialIds(AecDb.GridAssemblyComponentType.Infill, ref infillIds);
            data.Add(new MgdDbg.Snoop.Data.ObjectIdCollection("Material ids (infill)", infillIds));

            AcDb.ObjectIdCollection mullionIds = null;
            gridAssembly.GetMaterialIds(AecDb.GridAssemblyComponentType.Mullion, ref mullionIds);
            data.Add(new MgdDbg.Snoop.Data.ObjectIdCollection("Material ids (mullion)", mullionIds));

            AecDb.GridAssemblyComponentSet compSet = gridAssembly.ComponentSet;
            if (compSet != null) {
                // GetMinAndMaxElevation
                double minElev, maxElev;
                gridAssembly.GetMinAndMaxElevation(out minElev, out maxElev, compSet);
                data.Add(new MgdDbg.Snoop.Data.Double("Minimum elevation", minElev));
                data.Add(new MgdDbg.Snoop.Data.Double("Maximum elevation", maxElev));

                // GetRightAndLeftExtents
                double leftOff, rightOff;
                gridAssembly.GetRightAndLeftExtents(out leftOff, out rightOff, compSet);
                data.Add(new MgdDbg.Snoop.Data.Double("Left Offset ", leftOff));
                data.Add(new MgdDbg.Snoop.Data.Double("Right Offset ", rightOff));
            }

            double angle = 0.0;
            if (gridAssembly.HasStartMiterAngle(out angle))
                data.Add(new MgdDbg.Snoop.Data.Angle("Start miter angle", angle));
            if (gridAssembly.HasEndMiterAngle(out angle))
                data.Add(new MgdDbg.Snoop.Data.Angle("End miter angle", angle));

            // properties
            data.Add(new MgdDbg.Snoop.Data.Double("Base curve radius", gridAssembly.BaseCurveRadius));
            data.Add(new MgdDbg.Snoop.Data.Int("Cell Count", gridAssembly.CellCount));
            if (gridAssembly.HasClippingBoundary) {
                if (gridAssembly.HasClippingProfile)
                    data.Add(new MgdDbg.Snoop.Data.Object("Clipping profile", gridAssembly.ClippingProfile));
            }
            data.Add(new MgdDbg.Snoop.Data.Object("Component set", compSet));
            data.Add(new MgdDbg.Snoop.Data.Object("Custom line work profile", gridAssembly.CustomLineworkProfile));
            data.Add(new MgdDbg.Snoop.Data.Int("Edge count", gridAssembly.EdgeCount));
            data.Add(new MgdDbg.Snoop.Data.Int("Edit depth", gridAssembly.EditDepth));
            data.Add(new MgdDbg.Snoop.Data.Object("Grid assembly definition", gridAssembly.GridAssemblyDefinition));
            data.Add(new MgdDbg.Snoop.Data.String("Grid definition location", gridAssembly.GridDefinitionLocation.ToString()));
            data.Add(new MgdDbg.Snoop.Data.Bool("Has clipping boundary", gridAssembly.HasClippingBoundary));
            if (gridAssembly.HasClippingBoundary) {
                data.Add(new MgdDbg.Snoop.Data.Bool("Has clipping profile", gridAssembly.HasClippingProfile));
                data.Add(new MgdDbg.Snoop.Data.Bool("Has clipping Rectangle", gridAssembly.HasClippingRectangle));
            }
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Interfering entities", gridAssembly.Interferences));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Style id", gridAssembly.StyleId));
            data.Add(new MgdDbg.Snoop.Data.Bool("Supports instance based grid definitions", gridAssembly.SupportsInstanceBasedGridDefinition));
            data.Add(new MgdDbg.Snoop.Data.Bool("Supports interference", gridAssembly.SupportsInterference));
            data.Add(new MgdDbg.Snoop.Data.Bool("Supports supports style based grid definition", gridAssembly.SupportsStyleBasedGridDefinition));
            data.Add(new MgdDbg.Snoop.Data.Double("X axis length", gridAssembly.XAxisLength));
            data.Add(new MgdDbg.Snoop.Data.Double("Y axis length", gridAssembly.YAxisLength));
            data.Add(new MgdDbg.Snoop.Data.Double("Y axis offset", gridAssembly.YAxisOffset));

            // do any derived class from GridAssembly

        }

        private void
        Stream(ArrayList data, AecDb.LayoutGrid3d grid3d)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDb.LayoutGrid3d)));

            data.Add(new MgdDbg.Snoop.Data.Object("Base grid", grid3d.BaseGrid));
            //data.Add(new MgdDbg.Snoop.Data.String("Grid type", grid3d.GridType.ToString()));  // TBD: why no get() accessor?
        }

        private void
        Stream(ArrayList data, AecDb.LayoutCurve layoutCurve)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDb.LayoutCurve)));

            data.Add(new MgdDbg.Snoop.Data.ObjectId("Curve ID", layoutCurve.CurveId));
            data.Add(new MgdDbg.Snoop.Data.String("Layout mode", layoutCurve.LayoutMode.ToString()));
            data.Add(new MgdDbg.Snoop.Data.Int("Node count", layoutCurve.NodeCount));

                // Get Nodes using GetLayoutCurveNodeAt
            ArrayList nodes = new ArrayList();
            int len = layoutCurve.NodeCount;
            for (int i = 0; i < len; i++) {
                nodes.Add(layoutCurve.GetLayoutCurveNodeAt(i));
            }
            data.Add(new MgdDbg.Snoop.Data.ObjectCollection("Layout curve nodes (by index)", nodes));

                // Get nodes using GetLayoutCurveNode
            ArrayList nodesById = new ArrayList();
            IntegerCollection nodeIds = layoutCurve.LayoutNodeIds;
            data.Add(new MgdDbg.Snoop.Data.Object("Layout node IDs", layoutCurve.LayoutNodeIds));
            for (int n = 0; n < nodeIds.Count; n++) {
                nodesById.Add(layoutCurve.GetLayoutCurveNode(nodeIds[n]));
            }
            data.Add(new MgdDbg.Snoop.Data.ObjectCollection("Layout curve nodes (by ID)", nodesById));

            data.Add(new MgdDbg.Snoop.Data.Object("Node positions", layoutCurve.NodePositions));
            data.Add(new MgdDbg.Snoop.Data.Distance("Start offset", layoutCurve.StartOffset));
            data.Add(new MgdDbg.Snoop.Data.Distance("End offset", layoutCurve.EndOffset));
            data.Add(new MgdDbg.Snoop.Data.Distance("Spacing", layoutCurve.Spacing));
        }
                       
        private void
        Stream(ArrayList data, AecDb.ClipVolumeResult volRes)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDb.ClipVolumeResult)));

            data.Add(new MgdDbg.Snoop.Data.ObjectId("Clip volume ID", volRes.ClipVolumeId));
        }

        private void
        Stream (ArrayList data, AecDb.Section2d section2d)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDb.Section2d)));

            data.Add(new MgdDbg.Snoop.Data.ObjectId("Clip volume ID", section2d.ClipVolumeId));
            data.Add(new MgdDbg.Snoop.Data.Bool("Has object display property information", section2d.HasObjectDisplayPropertyInformation));
            data.Add(new MgdDbg.Snoop.Data.Bool("Has user edits", section2d.HasUserEdits));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Hatch regions", section2d.HatchRegions));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is saved edits", section2d.IsSavedEdits));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is single segment", section2d.IsSingleSegment));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Main section ID", section2d.MainSectionId));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Segment groups", section2d.SegmentGroups));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Segments", section2d.Segments));
            data.Add(new MgdDbg.Snoop.Data.Object("Shrink wrap profile", section2d.ShrinkWrapProfile));
            data.Add(new MgdDbg.Snoop.Data.Object("Shrink wrap segment group", section2d.ShrinkWrapSegmentGroup));
            data.Add(new MgdDbg.Snoop.Data.Object("Single segment", section2d.SingleSegment));
            data.Add(new MgdDbg.Snoop.Data.Int("Single segment ID", section2d.SingleSegmentId));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Style ID", section2d.StyleId));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Vertices", section2d.Vertices));
        }

        private void
        Stream(ArrayList data, AecDb.ClipVolume clipVol)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDb.ClipVolume)));

            data.Add(new MgdDbg.Snoop.Data.Double("Length A", clipVol.LengthA));
            data.Add(new MgdDbg.Snoop.Data.Double("Length B", clipVol.LengthB));
            data.Add(new MgdDbg.Snoop.Data.Angle("Angle A", clipVol.AngleA));
            data.Add(new MgdDbg.Snoop.Data.Angle("Angle B", clipVol.AngleB));
            data.Add(new MgdDbg.Snoop.Data.Double("Height", clipVol.Height));
            data.Add(new MgdDbg.Snoop.Data.Double("Lower extension", clipVol.LowerExtension));
            data.Add(new MgdDbg.Snoop.Data.Bool("Use model extents for height", clipVol.UseModelExtentsForHeight));
            data.Add(new MgdDbg.Snoop.Data.ObjectIdCollection("Included entities", clipVol.IncludedEntities));
            data.Add(new MgdDbg.Snoop.Data.Object("ECS vertices", clipVol.EcsVertices));
            data.Add(new MgdDbg.Snoop.Data.Object("Vertices", clipVol.Vertices));
            data.Add(new MgdDbg.Snoop.Data.Object("Sub-divisions", clipVol.Subdivisions));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("View set ID", clipVol.ViewSetId));
        }
                      
        private void
        Stream(ArrayList data, AecDb.Slice slice)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDb.Slice)));

            data.Add(new MgdDbg.Snoop.Data.Double("Area", slice.Area));
            data.Add(new MgdDbg.Snoop.Data.ObjectIdCollection("Entities", slice.GetEntities()));
            data.Add(new MgdDbg.Snoop.Data.Distance("Plane depth", slice.PlaneDepth));
            data.Add(new MgdDbg.Snoop.Data.Distance("Plane width", slice.PlaneWidth));
            data.Add(new MgdDbg.Snoop.Data.Object("Profile", slice.Profile));            
        }
    }
}
