
// $Header: //Desktop/Source/TestTools/MgdDbgAec/Snoop/CollectorExts/EntityStructure.cs#1 $
// $Author: fergusja $ $DateTime: 2008/03/04 16:12:35 $ $Change: 94604 $
//
// (C) Copyright 2008 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//


using System;
using System.Diagnostics;
using System.Collections;
using System.Windows.Forms;
using System.Collections.Specialized;

using MgdDbg.Snoop;
using Autodesk.Aec.Modeler;
using Autodesk.Aec.Geometry;
using Autodesk.AutoCAD.Geometry;

using AecDb      = Autodesk.Aec.DatabaseServices;
using AcDb       = Autodesk.AutoCAD.DatabaseServices;
using StructureDB = Autodesk.Aec.Structural.DatabaseServices;

namespace MgdDbgAec.Snoop.CollectorExts
{
	/// <summary>
	/// This is a Snoop Collector Extension object to collect data from AEC entities.
	/// </summary>
	public class EntityStructure : MgdDbg.Snoop.CollectorExts.CollectorExt
	{
		public EntityStructure()
		{
		}
		
        protected override void
        CollectEvent(object sender, MgdDbg.Snoop.Collectors.CollectorEventArgs e)
        {
                // bail early if this isn't an object we are responsible for
            AecDb.Entity aecEnt = e.ObjToSnoop as AecDb.Entity;
            if (aecEnt == null)
                return;

                // cast the sender object to the SnoopCollector we are expecting
            MgdDbg.Snoop.Collectors.Collector snoopCollector = sender as MgdDbg.Snoop.Collectors.Collector;
            if (snoopCollector == null) {
                Debug.Assert(false);    // why did someone else send us the message?
                return;
            }

            StructureDB.Member member = aecEnt as StructureDB.Member;
            if (member != null) {
                Stream(snoopCollector.Data(), member);
                return;
            }
        }
                
        private void
        Stream(ArrayList data, StructureDB.Member member)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(StructureDB.Member)));

            data.Add(new MgdDbg.Snoop.Data.ObjectId("Style ID", member.StyleId));
            data.Add(new MgdDbg.Snoop.Data.Distance("Length", member.Length));
            data.Add(new MgdDbg.Snoop.Data.Distance("Start offset", member.StartOffset));
            data.Add(new MgdDbg.Snoop.Data.Distance("End offset", member.EndOffset));
            data.Add(new MgdDbg.Snoop.Data.Distance("Roll", member.Roll));
            data.Add(new MgdDbg.Snoop.Data.String("Justification", member.Justification.ToString()));
            data.Add(new MgdDbg.Snoop.Data.String("Member type", member.MemberType.ToString()));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Trim planes", member.TrimPlanes));
        }
    }
}

