
// $Header: //Desktop/Source/TestTools/MgdDbgAec/Snoop/CollectorExts/Geometry.cs#2 $
// $Author: ayyara $ $DateTime: 2007/03/28 11:46:10 $ $Change: 69505 $
//
// (C) Copyright 2004 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//

using System;
using System.Text;
using System.Diagnostics;
using System.Collections;
using System.Collections.Generic;

using Autodesk.Aec.Geometry;
using Autodesk.AutoCAD.Geometry;


using MgdDbg.Snoop;

namespace MgdDbgAec.Snoop.CollectorExts {

    class Geometry : MgdDbg.Snoop.CollectorExts.CollectorExt{

		public Geometry()
		{
		}
		
        protected override void
        CollectEvent(object sender, MgdDbg.Snoop.Collectors.CollectorEventArgs e)
        {
                // cast the sender object to the SnoopCollector we are expecting
            MgdDbg.Snoop.Collectors.Collector snoopCollector = sender as MgdDbg.Snoop.Collectors.Collector;
            if (snoopCollector == null) {
                Debug.Assert(false);    // why did someone else send us the message?
                return;
            }
            
            BoundBox2d bb2d = e.ObjToSnoop as BoundBox2d;
            if (bb2d != null) {
                Stream(snoopCollector.Data(), bb2d);
                return;
            }

            BoundBox3d bb3d = e.ObjToSnoop as BoundBox3d;
            if (bb3d != null) {
                Stream(snoopCollector.Data(), bb3d);
                return;
            }

            CompoundCurve2dVertex vertex2d = e.ObjToSnoop as CompoundCurve2dVertex;
            if (vertex2d != null) {
                Stream(snoopCollector.Data(), vertex2d);
                return;
            }

        }

        private void
        Stream(ArrayList data, BoundBox2d bbox)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(BoundBox2d)));

            data.Add(new MgdDbg.Snoop.Data.Double("Area", bbox.Area));
            data.Add(new MgdDbg.Snoop.Data.Distance("Width", bbox.Width));
            data.Add(new MgdDbg.Snoop.Data.Distance("Depth", bbox.Depth));
            data.Add(new MgdDbg.Snoop.Data.Point2d("Max point", bbox.MaxPoint));
            data.Add(new MgdDbg.Snoop.Data.Point2d("Min point", bbox.MinPoint));
        }

        private void
        Stream(ArrayList data, BoundBox3d bbox)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(BoundBox3d)));

            data.Add(new MgdDbg.Snoop.Data.Double("Volume", bbox.Volume));
            data.Add(new MgdDbg.Snoop.Data.Distance("Width", bbox.Width));
            data.Add(new MgdDbg.Snoop.Data.Distance("Depth", bbox.Depth));
            data.Add(new MgdDbg.Snoop.Data.Distance("Height", bbox.Height));
            data.Add(new MgdDbg.Snoop.Data.Point3d("Max point", bbox.MaxPoint));
            data.Add(new MgdDbg.Snoop.Data.Point3d("Min point", bbox.MinPoint));
        }

        private void
        Stream(ArrayList data, CompoundCurve2dVertex vertex)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(CompoundCurve2dVertex)));

            data.Add(new MgdDbg.Snoop.Data.Point2d("Point", vertex.Point));
            data.Add(new MgdDbg.Snoop.Data.Object("Compound curve", vertex.CompoundCurve));
            data.Add(new MgdDbg.Snoop.Data.Object("Next segment", vertex.NextSegment));
            data.Add(new MgdDbg.Snoop.Data.Object("Previous segment", vertex.PreviousSegment));
        }
    }
}
