// $Header: //Desktop/Source/TestTools/MgdDbgAec/Snoop/CollectorExts/ImpObjPropData.cs#1 $
// $Author: fergusja $ $DateTime: 2007/03/16 11:08:08 $ $Change: 68854 $
//
// (C) Copyright 2006 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//

using System;
using System.Diagnostics;
using System.Collections;
using System.Windows.Forms;

using MgdDbg.Snoop;
using Autodesk.Aec.PropertyData.DatabaseServices;
using AecDb       = Autodesk.Aec.DatabaseServices;

namespace MgdDbgAec.Snoop.CollectorExts
{

	public class ImpObjectPropData : MgdDbg.Snoop.CollectorExts.CollectorExt
	{
		public ImpObjectPropData()
		{
		}
		
        protected override void
        CollectEvent(object sender, MgdDbg.Snoop.Collectors.CollectorEventArgs e)
        {
                // bail early if this isn't an object we are responsible for
            AecDb.ImpObject impObj = e.ObjToSnoop as AecDb.ImpObject;
            if (impObj == null)
                return;

                // cast the sender object to the SnoopCollector we are expecting
            MgdDbg.Snoop.Collectors.Collector snoopCollector = sender as MgdDbg.Snoop.Collectors.Collector;
            if (snoopCollector == null) {
                Debug.Assert(false);    // why did someone else send us the message?
                return;
            }
            
            PropertyDefinition propDef = impObj as PropertyDefinition;
            if (propDef != null) {
                Stream(snoopCollector.Data(), propDef);
                return;
            }

            PropertySetData propData = impObj as PropertySetData;
            if (propData != null) {
                Stream(snoopCollector.Data(), propData);
                return;
            }

#if !ADT2007
            DisplayThemeScheduleDataRule dataRule = impObj as DisplayThemeScheduleDataRule;
            if (dataRule != null) {
                Stream(snoopCollector.Data(), dataRule);
                return;
            }
#endif
        }
        
        private void
        Stream(ArrayList data, PropertyDefinition propDef)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(PropertyDefinition)));

            data.Add(new MgdDbg.Snoop.Data.String("Name", propDef.Name));
            data.Add(new MgdDbg.Snoop.Data.Int("ID", propDef.Id));
            data.Add(new MgdDbg.Snoop.Data.String("Data type", propDef.DataType.ToString()));

            try {
                data.Add(new MgdDbg.Snoop.Data.Object("Unit type", propDef.UnitType));
            }
            catch (Autodesk.AutoCAD.Runtime.Exception e) {
                data.Add(new MgdDbg.Snoop.Data.Exception("Unit type", e));
            }

            data.Add(new MgdDbg.Snoop.Data.Bool("Automatic", propDef.Automatic));
            data.Add(new MgdDbg.Snoop.Data.Bool("Contains fields", propDef.ContainsFields));

            if (propDef.DataType == Autodesk.Aec.PropertyData.DataType.Text)
                data.Add(new MgdDbg.Snoop.Data.String("Default data", (string)propDef.DefaultData));
            else if (propDef.DataType == Autodesk.Aec.PropertyData.DataType.AlphaIncrement)
                data.Add(new MgdDbg.Snoop.Data.String("Default data", (string)propDef.DefaultData));
            else if (propDef.DataType == Autodesk.Aec.PropertyData.DataType.AutoIncrement)
                data.Add(new MgdDbg.Snoop.Data.Int("Default data", (int)propDef.DefaultData));
            else if (propDef.DataType == Autodesk.Aec.PropertyData.DataType.Integer)
                data.Add(new MgdDbg.Snoop.Data.Int("Default data", (int)propDef.DefaultData));
            else if (propDef.DataType == Autodesk.Aec.PropertyData.DataType.Real)
                data.Add(new MgdDbg.Snoop.Data.Double("Default data", (double)propDef.DefaultData));
            else if (propDef.DataType == Autodesk.Aec.PropertyData.DataType.TrueFalse)
                data.Add(new MgdDbg.Snoop.Data.Bool("Default data", (bool)propDef.DefaultData));
            else
                data.Add(new MgdDbg.Snoop.Data.Object("Default data", propDef.DefaultData));

            data.Add(new MgdDbg.Snoop.Data.Bool("Default is unspecified", propDef.DefaultIsUnspecified));
            data.Add(new MgdDbg.Snoop.Data.String("Example string", propDef.ExampleString));
            data.Add(new MgdDbg.Snoop.Data.String("Field bucket ID", propDef.FieldBucketId));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Format ID", propDef.FormatId));
            data.Add(new MgdDbg.Snoop.Data.String("Format string", propDef.FormatString));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is locked", propDef.IsLocked));    // TBD: this isn't get/set (set is "Locked")
            data.Add(new MgdDbg.Snoop.Data.Bool("Is read only", propDef.IsReadOnly));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is visible", propDef.IsVisible));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("List definition ID", propDef.ListDefinitionId));
        }

        private void
        Stream(ArrayList data, PropertySetData propData)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(PropertySetData)));

            data.Add(new MgdDbg.Snoop.Data.Int("ID", propData.Id));
            data.Add(new MgdDbg.Snoop.Data.String("Data type", propData.DataType.ToString()));

            if (propData.DataType == Autodesk.Aec.PropertyData.DataType.Text)
                data.Add(new MgdDbg.Snoop.Data.String("Data", (string)propData.GetData()));
            else if (propData.DataType == Autodesk.Aec.PropertyData.DataType.AlphaIncrement)
                data.Add(new MgdDbg.Snoop.Data.String("Data", (string)propData.GetData()));
            else if (propData.DataType == Autodesk.Aec.PropertyData.DataType.AutoIncrement)
                data.Add(new MgdDbg.Snoop.Data.Int("Data", (int)propData.GetData()));
            else if (propData.DataType == Autodesk.Aec.PropertyData.DataType.Integer)
                data.Add(new MgdDbg.Snoop.Data.Int("Data", (int)propData.GetData()));
            else if (propData.DataType == Autodesk.Aec.PropertyData.DataType.Real)
                data.Add(new MgdDbg.Snoop.Data.Double("Data", (double)propData.GetData()));
            else if (propData.DataType == Autodesk.Aec.PropertyData.DataType.TrueFalse)
                data.Add(new MgdDbg.Snoop.Data.Bool("Data", (bool)propData.GetData()));
            else
                data.Add(new MgdDbg.Snoop.Data.Object("Data", propData.GetData()));

            try {
                data.Add(new MgdDbg.Snoop.Data.Object("Unit type", propData.UnitType));
            }
            catch (Autodesk.AutoCAD.Runtime.Exception e) {
                data.Add(new MgdDbg.Snoop.Data.Exception("Unit type", e));
            }

            data.Add(new MgdDbg.Snoop.Data.Bool("Contains fields", propData.ContainsFields));
            if (propData.ContainsFields) {
                data.Add(new MgdDbg.Snoop.Data.ObjectId("Field ID", propData.GetFieldId()));
                data.Add(new MgdDbg.Snoop.Data.String("Field bucket ID", propData.FieldBucketId));
            }
            data.Add(new MgdDbg.Snoop.Data.Bool("Is unspecified", propData.IsUnspecified));
        }

#if !ADT2007
        private void
        Stream(ArrayList data, DisplayThemeScheduleDataRule dispThemeRule)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(DisplayThemeScheduleDataRule)));

            data.Add(new MgdDbg.Snoop.Data.ObjectId("Property set definition ID", dispThemeRule.PropertySetDefinitionId));
            data.Add(new MgdDbg.Snoop.Data.Int("Property definition ID", dispThemeRule.PropertyDefinitionId));

            // TBD: GetValidPropertySetDefinitions(Autodesk.AutoCAD.DatabaseServices.Database, Autodesk.Aec.DatabaseServices.NameObjectIdPairCollection)
        }
#endif

    }
}

