
// $Header: //Desktop/Source/TestTools/MgdDbgAec/Snoop/CollectorExts/ImpObjectArch.cs#2 $
// $Author: ayyara $ $DateTime: 2007/03/28 11:46:10 $ $Change: 69505 $
//
// (C) Copyright 2004 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//


//#define VS2003

using System;
using System.Diagnostics;
using System.Collections;
using System.Windows.Forms;

using MgdDbg.Snoop;

using Autodesk.Aec.Modeler;
using Autodesk.Aec.Geometry;
using Autodesk.AutoCAD.Geometry;
using Autodesk.Aec.Arch.Geometry;
using Autodesk.Aec.DatabaseServices;
using Autodesk.Aec.Arch.DatabaseServices;


namespace MgdDbgAec.Snoop.CollectorExts
{
	/// <summary>
	/// This is a Snoop Collector Extension object to collect data from AEC ImpObjs.
	/// </summary>
	public class ImpObjectArch : MgdDbg.Snoop.CollectorExts.CollectorExt
	{
		public ImpObjectArch()
		{
		}
		
        protected override void
        CollectEvent(object sender, MgdDbg.Snoop.Collectors.CollectorEventArgs e)
        {
                // bail early if this isn't an object we are responsible for
            ImpObject impObj = e.ObjToSnoop as ImpObject;
            if (impObj == null)
                return;

                // cast the sender object to the SnoopCollector we are expecting
            MgdDbg.Snoop.Collectors.Collector snoopCollector = sender as MgdDbg.Snoop.Collectors.Collector;
            if (snoopCollector == null) {
                Debug.Assert(false);    // why did someone else send us the message?
                return;
            }
            
            WallStyleComponent wallStyleComp = impObj as WallStyleComponent;
            if (wallStyleComp != null) {
                Stream(snoopCollector.Data(), wallStyleComp);
                return;
            }
            
            WallStyleComponentPosition wallStyleCompPos = impObj as WallStyleComponentPosition;
            if (wallStyleCompPos != null) {
                Stream(snoopCollector.Data(), wallStyleCompPos);
                return;
            }
            
            InstanceBasedValue instBasedVal = impObj as InstanceBasedValue;
            if (instBasedVal != null) {
                Stream(snoopCollector.Data(), instBasedVal);
                return;
            }
            
            WallElevation wallElev = impObj as WallElevation;
            if (wallElev != null) {
                Stream(snoopCollector.Data(), wallElev);
                return;
            }

            WallMerge wallMerge = impObj as WallMerge;
            if (wallMerge != null) {
                Stream(snoopCollector.Data(), wallMerge);
                return;
            }
                      
            WallModifier wallMod = impObj as WallModifier;
            if (wallMod != null) {
                Stream(snoopCollector.Data(), wallMod);
                return;
            }

            WallInterference wallInt = impObj as WallInterference;
            if (wallInt != null) {
                Stream(snoopCollector.Data(), wallInt);
                return;
            }

            WallStyleOverride wStyleOverride = impObj as WallStyleOverride;
            if (wStyleOverride != null) {
                Stream(snoopCollector.Data(), wStyleOverride);
                return;
            }

            WallCutLine wCutLine = impObj as WallCutLine;
            if (wCutLine != null) {
                Stream(snoopCollector.Data(), wCutLine);
                return;
            }

            WallCutLineAnchor wcLineAnchor = impObj as WallCutLineAnchor;
            if (wcLineAnchor != null) {
                Stream(snoopCollector.Data(), wcLineAnchor);
                return;
            }

            RoofBody roofBody = impObj as RoofBody;
            if (roofBody != null) {
                Stream(snoopCollector.Data(), roofBody);
                return;
            }

            RoofLine roofLine = impObj as RoofLine;
            if (roofLine != null) {
                Stream(snoopCollector.Data(), roofLine);
                return;
            }

            RoofFace roofFace = impObj as RoofFace;
            if (roofFace != null) {
                Stream(snoopCollector.Data(), roofFace);
                return;
            }

            RoofProfile roofProfile = impObj as RoofProfile;
            if (roofProfile != null) {
                Stream(snoopCollector.Data(), roofProfile);
                return;
            }

            SlabLoop slabLoop = impObj as SlabLoop;
            if (slabLoop != null) {
                Stream(snoopCollector.Data(), slabLoop);
                return;
            }            

            SlabEdge slabEdge = impObj as SlabEdge;
            if (slabEdge != null) {
                Stream(snoopCollector.Data(), slabEdge);
                return;
            }

            SlabStyleComponent ssComp = impObj as SlabStyleComponent;
            if (ssComp != null) {
                Stream(snoopCollector.Data(), ssComp);
                return;            
            }
            
            SlabStyleComponentPosition ssCompPos = impObj as SlabStyleComponentPosition;
            if (ssCompPos != null) {
                Stream(snoopCollector.Data(), ssCompPos);
                return;
            }
            
            SlabFace slabFace = impObj as SlabFace;
            if (slabFace != null) {
                Stream(snoopCollector.Data(), slabFace);
                return;
            }

            SlabInterference slabInt = impObj as SlabInterference;
            if (slabInt != null) {
                Stream(snoopCollector.Data(), slabInt);
                return;
            }

            RoofSegment roofSeg = impObj as RoofSegment;
            if (roofSeg != null) {
                Stream(snoopCollector.Data(), roofSeg);
                return;
            }

            SpaceSurfaceComponent sSurfComp = impObj as SpaceSurfaceComponent;
            if (sSurfComp != null) {
                Stream(snoopCollector.Data(), sSurfComp);
                return;
            }

            SpaceSurface spaceSurf = impObj as SpaceSurface;
            if (spaceSurf != null) {
                Stream(snoopCollector.Data(), spaceSurf);
                return;
            }            

            NestedGridDivisionByBaseHeights ngDivHeights = impObj as NestedGridDivisionByBaseHeights;
            if (ngDivHeights != null) {
                Stream(snoopCollector.Data(), ngDivHeights);
                return;
            }

            NestedGridDivisionByRise ngDivRise = impObj as NestedGridDivisionByRise;
            if (ngDivRise != null) {
                Stream(snoopCollector.Data(), ngDivRise);
                return;
            }

            NestedGridDivisionPolyline ngDivPolyline = impObj as NestedGridDivisionPolyline;
            if (ngDivPolyline != null) {
                Stream(snoopCollector.Data(), ngDivPolyline);
                return;
            }

            StairWidthConstraint stairWidConst = impObj as StairWidthConstraint;
            if (stairWidConst != null) {
                Stream(snoopCollector.Data(), stairWidConst);
            }

            StairComponentDefinition stairCompDef = impObj as StairComponentDefinition;
            if (stairCompDef != null) {
                Stream(snoopCollector.Data(), stairCompDef);
                return;
            }

            StairComponent stairComp = impObj as StairComponent;
            if (stairComp != null) {
                Stream(snoopCollector.Data(), stairComp);
                return;
            }

            StairCustomTreadDefinition scTreadDef = impObj as StairCustomTreadDefinition;
            if (scTreadDef != null) {
                Stream(snoopCollector.Data(), scTreadDef);
                return;
            }

            StairCalculatorData stairCalcData = impObj as StairCalculatorData;
            if (stairCalcData != null) {
                Stream(snoopCollector.Data(), stairCalcData);
                return;
            }

            StairCalculatorLimits stairCalcLim = impObj as StairCalculatorLimits;
            if (stairCalcLim != null) {
                Stream(snoopCollector.Data(), stairCalcLim);
                return;
            }

            StairCreator stairCreator = impObj as StairCreator;
            if (stairCreator != null) {
                Stream(snoopCollector.Data(), stairCreator);
                return;
            }

            RailingComponent railComp = impObj as RailingComponent;
            if (railComp != null) {
                Stream(snoopCollector.Data(), railComp);
                return;
            }

            RailingVerticalComponent railVertComp = impObj as RailingVerticalComponent;
            if (railVertComp != null) {
                Stream(snoopCollector.Data(), railVertComp);
                return;
            }

            StandardSizeOpening stdSizeOpen = impObj as StandardSizeOpening;
            if (stdSizeOpen != null) {
                Stream(snoopCollector.Data(), stdSizeOpen);
                return;
            }

            SlabCustomGeometry slabCustGeo = impObj as SlabCustomGeometry;
            if (slabCustGeo != null) {
                Stream(snoopCollector.Data(), slabCustGeo);
                return;
            }

            AbstractTreeNodeSpace abTreeNodeSpace = impObj as AbstractTreeNodeSpace;
            if (abTreeNodeSpace != null) {
                Stream(snoopCollector.Data(), abTreeNodeSpace);
                return;
            }
        }
        
        private void
        Stream(ArrayList data, WallStyleComponent comp)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(WallStyleComponent)));

            data.Add(new MgdDbg.Snoop.Data.String("Name", comp.Name));
            data.Add(new MgdDbg.Snoop.Data.Int("Component ID", comp.ComponentId));
            data.Add(new MgdDbg.Snoop.Data.Int("Priority", comp.Priority));
            data.Add(new MgdDbg.Snoop.Data.Object("Position", comp.Position));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Material ID", comp.MaterialId)); 
        }
        
        private void
        Stream(ArrayList data, WallStyleComponentPosition pos)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(WallStyleComponentPosition)));

            data.Add(new MgdDbg.Snoop.Data.Bool("Depends on wall instance width", pos.DependsOnWallInstanceWidth));
            data.Add(new MgdDbg.Snoop.Data.Object("Instance edge offset", pos.InstanceEdgeOffset));
            data.Add(new MgdDbg.Snoop.Data.Object("Instance width", pos.InstanceWidth));
            data.Add(new MgdDbg.Snoop.Data.Object("Start elevation", pos.StartElevation));
            data.Add(new MgdDbg.Snoop.Data.Object("End elevation", pos.EndElevation));
        }
        
        private void
        Stream(ArrayList data, InstanceBasedValue val)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(InstanceBasedValue)));

            data.Add(new MgdDbg.Snoop.Data.Bool("Use instance value", val.UseInstanceValue));
            data.Add(new MgdDbg.Snoop.Data.Distance("Base value", val.BaseValue));
            data.Add(new MgdDbg.Snoop.Data.Double("Operand", val.Operand));
            data.Add(new MgdDbg.Snoop.Data.String("Operator type", val.OperatorType.ToString()));
            data.Add(new MgdDbg.Snoop.Data.String("Formula description", val.FormulaDescription));

            SlabStyleComponentInstanceBasedValue ssCompIbv = val as SlabStyleComponentInstanceBasedValue;
            if (ssCompIbv != null) {
                Stream(data, ssCompIbv);
                return;
            }
        }

        private void
        Stream(ArrayList data, SlabStyleComponentInstanceBasedValue val)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(SlabStyleComponentInstanceBasedValue)));

            // no data at this level
        }
        
        private void
        Stream(ArrayList data, WallElevation elev)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(WallElevation)));

            data.Add(new MgdDbg.Snoop.Data.String("Elevation type", elev.ElevationType.ToString()));
            data.Add(new MgdDbg.Snoop.Data.Distance("Elevation offset", elev.ElevationOffset));
            data.Add(new MgdDbg.Snoop.Data.String("Formula description", elev.FormulaDescription));
        }

        private void
        Stream(ArrayList data, WallMerge wallMerge)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(WallMerge)));

            data.Add(new MgdDbg.Snoop.Data.ObjectId("Wall ID", wallMerge.WallId));
        }

        private void
        Stream(ArrayList data, WallModifier wallMod)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(WallModifier)));

            data.Add(new MgdDbg.Snoop.Data.ObjectId("Modifier style ID", wallMod.ModifierStyleId));
            data.Add(new MgdDbg.Snoop.Data.String("Applies to faces", wallMod.AppliesToFaces.ToString()));
            data.Add(new MgdDbg.Snoop.Data.Int("Component ID", wallMod.ComponentId));
            data.Add(new MgdDbg.Snoop.Data.Double("Width", wallMod.Width));
            data.Add(new MgdDbg.Snoop.Data.Distance("Depth", wallMod.Depth));
            data.Add(new MgdDbg.Snoop.Data.Object("Start elevation", wallMod.StartElevation));
            data.Add(new MgdDbg.Snoop.Data.Object("End elevation", wallMod.EndElevation));
            data.Add(new MgdDbg.Snoop.Data.Bool("Mirrored X", wallMod.MirroredX));
            data.Add(new MgdDbg.Snoop.Data.Bool("Mirrored Y", wallMod.MirroredY));
            data.Add(new MgdDbg.Snoop.Data.Bool("To end of wall", wallMod.ToEndOfWall));
            data.Add(new MgdDbg.Snoop.Data.Bool("Use actual size", wallMod.UseActualSize));
            data.Add(new MgdDbg.Snoop.Data.Object("X position", wallMod.XPosition));
        }

        private void
        Stream(ArrayList data, WallInterference wallInt)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(WallInterference)));

            data.Add(new MgdDbg.Snoop.Data.ObjectIdCollection("Entity block ref path", wallInt.GetEntityBlockRefPath()));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Entity ID", wallInt.EntityId));
            data.Add(new MgdDbg.Snoop.Data.String("Shrink wrap effect", wallInt.ShrinkWrapEffect.ToString()));
        }

        private void
        Stream(ArrayList data, WallStyleOverride wStyleOverride)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(WallStyleOverride)));

            WallCleanupRadiusOverride wcrOverride = wStyleOverride as WallCleanupRadiusOverride;
            if (wcrOverride != null) {
                Stream(data, wcrOverride);
                return;
            }

            WallStyleEndcapOverride wseOverride = wStyleOverride as WallStyleEndcapOverride;
            if (wseOverride != null) {
                Stream(data, wseOverride);
                return;
            }

            WallStylePriorityOverride wspOverride = wStyleOverride as WallStylePriorityOverride;
            if (wspOverride != null) {
                Stream(data, wspOverride);
                return;
            }
        }

        private void
        Stream(ArrayList data, WallCleanupRadiusOverride wcrOverride)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(WallCleanupRadiusOverride)));

            data.Add(new MgdDbg.Snoop.Data.Distance("Cleanup radius", wcrOverride.CleanupRadius));
            data.Add(new MgdDbg.Snoop.Data.String("Joint location", wcrOverride.JointLocation.ToString()));           
        }

        private void
        Stream(ArrayList data, WallStyleEndcapOverride wseOverride)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(WallStyleEndcapOverride)));

            data.Add(new MgdDbg.Snoop.Data.String("Endcap location", wseOverride.EndcapLocation.ToString()));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Endcap style ID", wseOverride.EndcapStyleId));
        }

        private void
        Stream(ArrayList data, WallStylePriorityOverride wspOverride)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(WallStylePriorityOverride)));

            data.Add(new MgdDbg.Snoop.Data.Int("Component ID", wspOverride.ComponentId));
            data.Add(new MgdDbg.Snoop.Data.Int("Priority", wspOverride.Priority));
            data.Add(new MgdDbg.Snoop.Data.String("Priority location", wspOverride.PriorityLocation.ToString()));
        }

        private void
        Stream(ArrayList data, WallCutLine wCutLine)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(WallCutLine)));

            data.Add(new MgdDbg.Snoop.Data.Enumerable("Anchors", wCutLine.Anchors));
            data.Add(new MgdDbg.Snoop.Data.String("Cut line type", wCutLine.CutLineType.ToString()));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is self intersecting", wCutLine.IsSelfIntersecting));
            data.Add(new MgdDbg.Snoop.Data.Bool("Priority location", wCutLine.IsStandardCutLine));

            data.Add(new MgdDbg.Snoop.Data.CategorySeparator("Segments"));

            data.Add(new MgdDbg.Snoop.Data.Int("Segment count", wCutLine.SegmentCount));
            for (int i=0; i<wCutLine.SegmentCount; i++) {
                data.Add(new MgdDbg.Snoop.Data.Distance(string.Format("Segment height at [{0}]", i), wCutLine.SegmentHeightAt(i)));
                data.Add(new MgdDbg.Snoop.Data.Distance(string.Format("Segment width at [{0}]", i), wCutLine.SegmentWidthAt(i)));
                data.Add(new MgdDbg.Snoop.Data.Bool(string.Format("Valid anchor position at [{0}]", i), wCutLine.ValidAnchorPositionAt(i)));
            }
        }

        private void
        Stream(ArrayList data, WallCutLineAnchor wcLineAnchor)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(WallCutLineAnchor)));

            data.Add(new MgdDbg.Snoop.Data.Distance("Distance offset", wcLineAnchor.DistanceOffset));
            data.Add(new MgdDbg.Snoop.Data.String("Distance offset type", wcLineAnchor.DistanceOffsetType.ToString()));
            data.Add(new MgdDbg.Snoop.Data.Distance("Height offset", wcLineAnchor.HeightOffset));
            data.Add(new MgdDbg.Snoop.Data.String("Height offset type", wcLineAnchor.HeightOffsetType.ToString()));            
        }

        private void
        Stream(ArrayList data, RoofSegment roofSeg)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(RoofSegment)));

            data.Add(new MgdDbg.Snoop.Data.Distance("Base height", roofSeg.BaseHeight));
            data.Add(new MgdDbg.Snoop.Data.Int("Bend count", roofSeg.BendCount));  
            data.Add(new MgdDbg.Snoop.Data.Int("Facet count", roofSeg.FacetCount)); 
            data.Add(new MgdDbg.Snoop.Data.Double("Eaves", roofSeg.Eaves)); 
            data.Add(new MgdDbg.Snoop.Data.Bool("Facet is valid", roofSeg.FacetIsValid)); 
            data.Add(new MgdDbg.Snoop.Data.Angle("First angle", roofSeg.FirstAngle)); 
            data.Add(new MgdDbg.Snoop.Data.Distance("First height", roofSeg.FirstHeight)); 
            data.Add(new MgdDbg.Snoop.Data.Angle("First slope", roofSeg.FirstSlope)); 
            data.Add(new MgdDbg.Snoop.Data.Bool("Is changeable", roofSeg.IsChangeable)); 
            bool isGable = roofSeg.IsGable(Tolerance.Global.EqualPoint);
            data.Add(new MgdDbg.Snoop.Data.Bool("Is gable (tol = Tolerance.Global.EqualPoint)", isGable)); 
            data.Add(new MgdDbg.Snoop.Data.Bool("Is linear", roofSeg.IsLinear)); 
            bool isValid = roofSeg.IsValid(Tolerance.Global);
            data.Add(new MgdDbg.Snoop.Data.Bool("Is valid (tol = Tolerance.Global)", isValid)); 
            data.Add(new MgdDbg.Snoop.Data.Point3d("Mid segment overhang Point3d", roofSeg.MidSegmentOverhangPoint3d));
            data.Add(new MgdDbg.Snoop.Data.Point2d("Mid segment Point2d", roofSeg.MidSegmentPoint2d)); 
            data.Add(new MgdDbg.Snoop.Data.Point3d("Mid segment Point3d", roofSeg.MidSegmentPoint3d));
            data.Add(new MgdDbg.Snoop.Data.Int("Number of facets", roofSeg.NumberOfFacets)); 
            data.Add(new MgdDbg.Snoop.Data.Object("Offset points", roofSeg.OffsetPoints)); 
            data.Add(new MgdDbg.Snoop.Data.Vector2d("Offset vector", roofSeg.OffsetVector)); 
            data.Add(new MgdDbg.Snoop.Data.Distance("Overhang", roofSeg.Overhang)); 
            data.Add(new MgdDbg.Snoop.Data.Int("Part", roofSeg.Part));

            double radius;
            bool radiusOk = roofSeg.Radius(out radius);
            if (radiusOk) {
                data.Add(new MgdDbg.Snoop.Data.Distance("Radius", radius)); 
            }

            data.Add(new MgdDbg.Snoop.Data.Int("Segment Id", roofSeg.Id)); 
            data.Add(new MgdDbg.Snoop.Data.Distance("Segment height", roofSeg.SegmentHeight)); 
            data.Add(new MgdDbg.Snoop.Data.Point3d("Segment start point", roofSeg.SegmentStartPoint)); 
            data.Add(new MgdDbg.Snoop.Data.Point3d("Segment mid point", roofSeg.SegmentMidPoint)); 
            data.Add(new MgdDbg.Snoop.Data.Point3d("Segment end point", roofSeg.SegmentEndPoint));
            data.Add(new MgdDbg.Snoop.Data.Point3d("Start eave point", roofSeg.StartEavePoint)); 
            data.Add(new MgdDbg.Snoop.Data.Point3d("End eave point", roofSeg.EndEavePoint)); 
            data.Add(new MgdDbg.Snoop.Data.Point2d("Start segment Point2d", roofSeg.StartSegmentPoint2d)); 
            data.Add(new MgdDbg.Snoop.Data.Point2d("End segment Point2d", roofSeg.EndSegmentPoint2d)); 
            data.Add(new MgdDbg.Snoop.Data.Point3d("Start segment Point3d", roofSeg.StartSegmentPoint3d));
            data.Add(new MgdDbg.Snoop.Data.Point3d("End segment Point3d", roofSeg.EndSegmentPoint3d));
        }

        private void
        Stream(ArrayList data, SlabEdge slabEdge)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(SlabEdge)));

            data.Add(new MgdDbg.Snoop.Data.Double("Angle", slabEdge.Angle));
            data.Add(new MgdDbg.Snoop.Data.Vector2d("Edge vector", slabEdge.EdgeVector));
            data.Add(new MgdDbg.Snoop.Data.Int("Gs marker", slabEdge.GsMarker));
            data.Add(new MgdDbg.Snoop.Data.Bool("Has style", slabEdge.HasStyle));
            data.Add(new MgdDbg.Snoop.Data.Int("Id", slabEdge.Id));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is marked new", slabEdge.IsMarkedNew));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is plumb cut", slabEdge.IsPlumbCut));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is square cut", slabEdge.IsSquareCut));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is update needed", slabEdge.IsUpdateNeeded));
            data.Add(new MgdDbg.Snoop.Data.Point2d("Mid point", slabEdge.MidPoint));
            data.Add(new MgdDbg.Snoop.Data.Double("Offset", slabEdge.Offset));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Style id", slabEdge.StyleId));
        }

        private void
        Stream(ArrayList data, SlabStyleComponent ssComp)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(SlabStyleComponent)));

            data.Add(new MgdDbg.Snoop.Data.Int("ID", ssComp.Id));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Material ID", ssComp.MaterialId));
            data.Add(new MgdDbg.Snoop.Data.String("Name", ssComp.Name));
            data.Add(new MgdDbg.Snoop.Data.Object("Position", ssComp.Position));            
        }

        private void
        Stream(ArrayList data, SlabStyleComponentPosition ssCompPos)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(SlabStyleComponentPosition)));

            data.Add(new MgdDbg.Snoop.Data.Bool("Depends on slab instance thickness", ssCompPos.DependsOnSlabInstanceThickness));            
            data.Add(new MgdDbg.Snoop.Data.Object("Offset", ssCompPos.Offset));            
            data.Add(new MgdDbg.Snoop.Data.Object("Thickness", ssCompPos.Thickness));            
        }

        private void
        Stream(ArrayList data, SlabFace slabFace)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(SlabFace)));

            data.Add(new MgdDbg.Snoop.Data.Double("Area", slabFace.Area));
            data.Add(new MgdDbg.Snoop.Data.Point2d("Centroid", slabFace.Centroid));
            data.Add(new MgdDbg.Snoop.Data.Int("Edge count", slabFace.EdgeCount));
            data.Add(new MgdDbg.Snoop.Data.Object("Extent", slabFace.Extent));
            data.Add(new MgdDbg.Snoop.Data.Int("Gs marker", slabFace.GsMarker));
            data.Add(new MgdDbg.Snoop.Data.Int("Id", slabFace.Id));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is update needed", slabFace.IsUpdateNeeded));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is valid", slabFace.IsValid));
            data.Add(new MgdDbg.Snoop.Data.Int("Loop count", slabFace.LoopCount));
            data.Add(new MgdDbg.Snoop.Data.Int("New loop id", slabFace.NewLoopId));
        }

        private void
        Stream(ArrayList data, SlabInterference slabInt)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(SlabInterference)));

            data.Add(new MgdDbg.Snoop.Data.ObjectId("Entity ID", slabInt.EntityId));
            data.Add(new MgdDbg.Snoop.Data.String("Option", slabInt.Option.ToString()));           
        }

        private void
        Stream(ArrayList data, SpaceSurfaceComponent sSurfComp)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(SpaceSurfaceComponent)));

            data.Add(new MgdDbg.Snoop.Data.Distance("Base height", sSurfComp.BaseHeight));
            data.Add(new MgdDbg.Snoop.Data.Distance("Height", sSurfComp.Height));
            data.Add(new MgdDbg.Snoop.Data.String("Name", sSurfComp.Name));
            data.Add(new MgdDbg.Snoop.Data.String("Opening type", sSurfComp.OpeningType.ToString()));
            data.Add(new MgdDbg.Snoop.Data.Bool("Opening type is door", sSurfComp.OpeningTypeIsDoor));
            data.Add(new MgdDbg.Snoop.Data.Bool("Opening type is window", sSurfComp.OpeningTypeIsWindow));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Property set ID", sSurfComp.PropertySet));
            data.Add(new MgdDbg.Snoop.Data.Distance("Start offset", sSurfComp.StartOffset));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Style ID", sSurfComp.StyleId));
            data.Add(new MgdDbg.Snoop.Data.Distance("Width", sSurfComp.Width));
        }


        private void
        Stream(ArrayList data, SpaceSurface spaceSurf)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(SpaceSurface)));

            data.Add(new MgdDbg.Snoop.Data.String("Name", spaceSurf.Name));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Style ID", spaceSurf.StyleId));
            data.Add(new MgdDbg.Snoop.Data.Double("Area", spaceSurf.Area));
            data.Add(new MgdDbg.Snoop.Data.Distance("Height", spaceSurf.Height));
            data.Add(new MgdDbg.Snoop.Data.Bool("Height is overridden", spaceSurf.HeightIsOverridden));
            if (spaceSurf.HeightIsOverridden)   // asserts if you don't check first
                data.Add(new MgdDbg.Snoop.Data.Distance("Height override", spaceSurf.HeightOverride));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Property set ID", spaceSurf.PropertySet));
            data.Add(new MgdDbg.Snoop.Data.String("Surface type", spaceSurf.SurfaceType.ToString()));
            data.Add(new MgdDbg.Snoop.Data.Distance("Thickness", spaceSurf.Thickness));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Components", spaceSurf.Components));
            data.Add(new MgdDbg.Snoop.Data.Object("Coordinates", spaceSurf.GetCoordinates()));
            data.Add(new MgdDbg.Snoop.Data.String("Next default opening name", spaceSurf.GetNextDefaultOpeningName()));
        }       

        private void
        Stream(ArrayList data, RoofBody roofBody)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(RoofBody)));

            data.Add(new MgdDbg.Snoop.Data.Int("Body ID", roofBody.BodyId));
            data.Add(new MgdDbg.Snoop.Data.Int("Body ID Gs Marker", roofBody.BodyIdGsMarker));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is valid", roofBody.IsValid));
            data.Add(new MgdDbg.Snoop.Data.Distance("Thickness", roofBody.Thickness));
            data.Add(new MgdDbg.Snoop.Data.String("Thickness direction", roofBody.ThicknessDirection.ToString()));
            data.Add(new MgdDbg.Snoop.Data.Object("Segment IDs", roofBody.SegmentIdCollection));

            Body bodyModel = roofBody.GetBodyModel(Tolerance.Global);
            data.Add(new MgdDbg.Snoop.Data.Object("Body Model", bodyModel));

            Body bodyPlan = roofBody.GetBodyPlan(Tolerance.Global);
            data.Add(new MgdDbg.Snoop.Data.Object("Body Plan", bodyPlan));

            Body bodyCombineFaces = roofBody.GetCombineFaces(Tolerance.Global);
            data.Add(new MgdDbg.Snoop.Data.Object("Combine Faces", bodyCombineFaces));

            RoofFace faceExt = roofBody.GetRoofFace(RoofFaceType.ExteriorFace);
            data.Add(new MgdDbg.Snoop.Data.Object("Roof face - exterior", faceExt));

            RoofFace faceInt = roofBody.GetRoofFace(RoofFaceType.InteriorFace);
            data.Add(new MgdDbg.Snoop.Data.Object("Roof face - interior", faceInt));
        }
        
        private void
        Stream(ArrayList data, RoofLine roofLine)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(RoofLine)));

            data.Add(new MgdDbg.Snoop.Data.Point3d("Start point", roofLine.StartPoint));
            data.Add(new MgdDbg.Snoop.Data.Point3d("End point", roofLine.EndPoint));
            data.Add(new MgdDbg.Snoop.Data.String("Face type", roofLine.FaceType.ToString()));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is plan", roofLine.IsPlan));
            data.Add(new MgdDbg.Snoop.Data.Object("Line segment", roofLine.LineSegment));
        }
        
        private void
        Stream(ArrayList data, RoofFace roofFace)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(RoofFace)));

            data.Add(new MgdDbg.Snoop.Data.Object("Body", roofFace.Body));
            data.Add(new MgdDbg.Snoop.Data.Bool("Convex", roofFace.Convex));
            data.Add(new MgdDbg.Snoop.Data.Int("Count points", roofFace.CountPoints));
            data.Add(new MgdDbg.Snoop.Data.Object("Face points", roofFace.Face));
            data.Add(new MgdDbg.Snoop.Data.String("Face type", roofFace.FaceType.ToString()));
            data.Add(new MgdDbg.Snoop.Data.Int("First Segment ID", roofFace.FirstSegmentId));
            data.Add(new MgdDbg.Snoop.Data.Object("Geometry Extents ", roofFace.GeometryExtents));
            data.Add(new MgdDbg.Snoop.Data.Int("ID", roofFace.Id));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is exterior", roofFace.IsExterior));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is interior", roofFace.IsInterior));
            data.Add(new MgdDbg.Snoop.Data.Int("Level segment", roofFace.LevelSegment));
            data.Add(new MgdDbg.Snoop.Data.Distance("Min height", roofFace.MinHeight));
            data.Add(new MgdDbg.Snoop.Data.Vector3d("Normal", roofFace.Normal));
            data.Add(new MgdDbg.Snoop.Data.Object("Segment IDs", roofFace.SegmentIdCollection));
            data.Add(new MgdDbg.Snoop.Data.Angle("Slope", roofFace.Slope));
            data.Add(new MgdDbg.Snoop.Data.Object("Transform by ECS", roofFace.TransformByEcs));
        }
        

        private void
        Stream(ArrayList data, RoofProfile roofProf)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(RoofProfile)));

            data.Add(new MgdDbg.Snoop.Data.Bool("Is curved", roofProf.IsCurved(Tolerance.Global)));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is gable", roofProf.IsGable));
            data.Add(new MgdDbg.Snoop.Data.Distance("Max eaves height", roofProf.MaxEavesHeight));
            data.Add(new MgdDbg.Snoop.Data.Distance("Min eaves height", roofProf.MinEavesHeight));
            data.Add(new MgdDbg.Snoop.Data.Distance("Min height", roofProf.MinHeight));
            data.Add(new MgdDbg.Snoop.Data.Point3d("Segment start point", roofProf.SegmentStartPoint));
            data.Add(new MgdDbg.Snoop.Data.Object("Vertices", roofProf.Vertices));
            data.Add(new MgdDbg.Snoop.Data.Int("Count segments", roofProf.CountSegments));
            data.Add(new MgdDbg.Snoop.Data.Int("Segment count ", roofProf.SegmentCount));

            ArrayList roofSegs = new ArrayList();
            for (int i=0; i<roofProf.SegmentCount; i++)
                roofSegs.Add(roofProf.SegmentAt(i));
            data.Add(new MgdDbg.Snoop.Data.ObjectCollection("Roof segments", roofSegs));
        }

        private void
        Stream(ArrayList data, SlabLoop slabLoop)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(SlabLoop)));

            data.Add(new MgdDbg.Snoop.Data.Int("Gs marker", slabLoop.GsMarker));
            data.Add(new MgdDbg.Snoop.Data.Int("Id", slabLoop.Id));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is update needed", slabLoop.IsUpdateNeeded));
            data.Add(new MgdDbg.Snoop.Data.Int("New edge id", slabLoop.NewEdgeId));
        }

        private void
        Stream(ArrayList data, NestedGridDivisionByBaseHeights ngDivHeights)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(NestedGridDivisionByBaseHeights)));

            data.Add(new MgdDbg.Snoop.Data.Distance("Base height offset", ngDivHeights.BaseHeightOffset));
            data.Add(new MgdDbg.Snoop.Data.Distance("Baseline offset", ngDivHeights.BaselineOffset));
            data.Add(new MgdDbg.Snoop.Data.String("Divisions type", ngDivHeights.DivisionsType.ToString()));
        }

        private void
        Stream(ArrayList data, NestedGridDivisionByRise ngDivRise)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(NestedGridDivisionByRise)));

            data.Add(new MgdDbg.Snoop.Data.Distance("Rise offset", ngDivRise.RiseOffset));
        }

        private void
        Stream(ArrayList data, NestedGridDivisionPolyline ngDivPolyline)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(NestedGridDivisionPolyline)));

            // no data at this level
        }

        private void
        Stream(ArrayList data, StairWidthConstraint stairWidConst)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(StairWidthConstraint)));

            data.Add(new MgdDbg.Snoop.Data.String("Alignment", stairWidConst.Alignment.ToString()));
            data.Add(new MgdDbg.Snoop.Data.Distance("Distance", stairWidConst.Distance));
            data.Add(new MgdDbg.Snoop.Data.Distance("Left", stairWidConst.Left));
            data.Add(new MgdDbg.Snoop.Data.Distance("Right", stairWidConst.Right));
            data.Add(new MgdDbg.Snoop.Data.Distance("Width", stairWidConst.Width));
        }

        private void
        Stream(ArrayList data, StairComponentDefinition stairCompDef)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(StairComponentDefinition)));

            data.Add(new MgdDbg.Snoop.Data.Object("Constraint", stairCompDef.Constraint));
            data.Add(new MgdDbg.Snoop.Data.Int("ID", stairCompDef.Id));

            StairStringerComponentDefinition ssCompDef = stairCompDef as StairStringerComponentDefinition;
            if (ssCompDef != null) {
                Stream(data, ssCompDef);
                return;
            }
        }

        private void
        Stream(ArrayList data, StairStringerComponentDefinition ssCompDef)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(StairStringerComponentDefinition)));

            data.Add(new MgdDbg.Snoop.Data.String("Anchor type", ssCompDef.AnchorType.ToString()));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Material ID", ssCompDef.MaterialId));
            data.Add(new MgdDbg.Snoop.Data.String("Name", ssCompDef.Name));
            data.Add(new MgdDbg.Snoop.Data.Distance("Offset", ssCompDef.Offset));
            data.Add(new MgdDbg.Snoop.Data.Distance("Thickness total at flight", ssCompDef.ThicknessTotalAtFlight));
            data.Add(new MgdDbg.Snoop.Data.Distance("Thickness total at landing", ssCompDef.ThicknessTotalAtLanding));
            data.Add(new MgdDbg.Snoop.Data.Bool("Truncate at landing", ssCompDef.TruncateAtLanding));
            data.Add(new MgdDbg.Snoop.Data.String("Type", ssCompDef.Type.ToString()));
            data.Add(new MgdDbg.Snoop.Data.Distance("Waist at flight", ssCompDef.WaistAtFlight));
            data.Add(new MgdDbg.Snoop.Data.Distance("Waist at landing", ssCompDef.WaistAtLanding));
            data.Add(new MgdDbg.Snoop.Data.Distance("Width", ssCompDef.Width));
        }

        private void
        Stream(ArrayList data, StairComponent stairComp)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(StairComponent)));

            data.Add(new MgdDbg.Snoop.Data.Object("Front polyline", stairComp.FrontPolyline));
            data.Add(new MgdDbg.Snoop.Data.Object("Back polyline", stairComp.BackPolyline));
            data.Add(new MgdDbg.Snoop.Data.String("Error code", stairComp.ErrorCode.ToString()));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is front side customized", stairComp.IsFrontSideCustomized));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is back side customized", stairComp.IsBacksideCustomized));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is left side customized", stairComp.IsLeftSideCustomized));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is right side customized", stairComp.IsRightSideCustomized));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is valid", stairComp.IsValid));
            data.Add(new MgdDbg.Snoop.Data.Object("Left polyline", stairComp.LeftPolyline));
            data.Add(new MgdDbg.Snoop.Data.Object("Right polyline", stairComp.RightPolyline));
            data.Add(new MgdDbg.Snoop.Data.Object("Outline", stairComp.Outline));
            data.Add(new MgdDbg.Snoop.Data.Distance("Maximum height", stairComp.MaximumHeight));
            data.Add(new MgdDbg.Snoop.Data.Distance("Minimum height", stairComp.MinimumHeight));
            data.Add(new MgdDbg.Snoop.Data.Distance("Start extension", stairComp.StartExtension));
            data.Add(new MgdDbg.Snoop.Data.Distance("End extension", stairComp.EndExtension));
            data.Add(new MgdDbg.Snoop.Data.Object("Walking line", stairComp.WalkingLine));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Treads", stairComp.Treads));

            foreach (StairWidthType stairWidthType in System.Enum.GetValues(typeof(StairWidthType)))
                data.Add(new MgdDbg.Snoop.Data.Distance(string.Format("Width (type = {0})", stairWidthType.ToString()), stairComp.GetWidth(stairWidthType)));           

            StairComponentCustom sCompCust = stairComp as StairComponentCustom;
            if (sCompCust != null) {
                Stream(data, sCompCust);
                return;
            }

            StairFlight stairFlight = stairComp as StairFlight;
            if (stairFlight != null) {
                Stream(data, stairFlight);
                return;
            }

            StairTurn stairTurn = stairComp as StairTurn;
            if (stairTurn != null) {
                Stream(data, stairTurn);
                return;
            }
        }

        private void
        Stream(ArrayList data, StairComponentCustom sCompCust)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(StairComponentCustom)));

            data.Add(new MgdDbg.Snoop.Data.Distance("First tread height", sCompCust.FirstTreadHeight));
            data.Add(new MgdDbg.Snoop.Data.Distance("Last tread height", sCompCust.LastTreadHeight));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is blended", sCompCust.IsBlended));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is landing", sCompCust.IsLanding));
            data.Add(new MgdDbg.Snoop.Data.Object("Path", sCompCust.Path));            
        }

        private void
        Stream(ArrayList data, StairFlight stairFlight)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(StairFlight)));

            data.Add(new MgdDbg.Snoop.Data.Point2d("Start point", stairFlight.StartPoint));
            data.Add(new MgdDbg.Snoop.Data.Point2d("Mid point", stairFlight.MidPoint));
            data.Add(new MgdDbg.Snoop.Data.Point2d("End point", stairFlight.EndPoint));
            data.Add(new MgdDbg.Snoop.Data.Distance("Start offset", stairFlight.StartOffset));
            data.Add(new MgdDbg.Snoop.Data.Distance("End offset", stairFlight.EndOffset));
            data.Add(new MgdDbg.Snoop.Data.Object("Center curve", stairFlight.CenterCurve));
        }

        private void
        Stream(ArrayList data, StairTurn stairTurn)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(StairTurn)));

            data.Add(new MgdDbg.Snoop.Data.Bool("Is landing", stairTurn.IsLanding));
            data.Add(new MgdDbg.Snoop.Data.Distance("First tread height", stairTurn.FirstTreadHeight));
            data.Add(new MgdDbg.Snoop.Data.Distance("Last tread height", stairTurn.LastTreadHeight));
            data.Add(new MgdDbg.Snoop.Data.Point2d("Turn control point", stairTurn.TurnControlPoint));
            data.Add(new MgdDbg.Snoop.Data.Distance("Turn distance before turn point", stairTurn.TurnDistanceBeforeTurnPoint));
            data.Add(new MgdDbg.Snoop.Data.Distance("Turn distance after turn point", stairTurn.TurnDistanceAfterTurnPoint));
            data.Add(new MgdDbg.Snoop.Data.Object("Turn points", stairTurn.TurnPoints));
        }

        private void
        Stream(ArrayList data, StairCustomTreadDefinition scTreadDef)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(StairCustomTreadDefinition)));

            data.Add(new MgdDbg.Snoop.Data.Object("Back edge", scTreadDef.BackEdge));
            data.Add(new MgdDbg.Snoop.Data.Distance("Distance along path", scTreadDef.DistanceAlongPath));
            data.Add(new MgdDbg.Snoop.Data.String("Error code", scTreadDef.ErrorCode.ToString()));
            data.Add(new MgdDbg.Snoop.Data.Distance("Front distance to left", scTreadDef.FrontDistanceToLeft));
            data.Add(new MgdDbg.Snoop.Data.Distance("Front distance to right", scTreadDef.FrontDistanceToRight));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is landing", scTreadDef.IsLanding));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is valid", scTreadDef.IsValid));
            data.Add(new MgdDbg.Snoop.Data.Object("Left edge", scTreadDef.LeftEdge));
            data.Add(new MgdDbg.Snoop.Data.Object("Right edge", scTreadDef.RightEdge));
            data.Add(new MgdDbg.Snoop.Data.Object("Profile", scTreadDef.Profile));
            data.Add(new MgdDbg.Snoop.Data.Object("Profile ECS", scTreadDef.ProfileEcs));
            data.Add(new MgdDbg.Snoop.Data.Angle("Rotaition angle", scTreadDef.RotationAngle));
            data.Add(new MgdDbg.Snoop.Data.Distance("Tread depth", scTreadDef.TreadDepth));
        }

        private void
        Stream(ArrayList data, StairCalculatorData stairCalcData)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(StairCalculatorData)));

            data.Add(new MgdDbg.Snoop.Data.Distance("Bottom concrete thickness", stairCalcData.BottomConcreteThickness));
            data.Add(new MgdDbg.Snoop.Data.Distance("Bottom floor level", stairCalcData.BottomFloorLevel));
            data.Add(new MgdDbg.Snoop.Data.Distance("Bottom floor offset", stairCalcData.BottomFloorOffset));
            data.Add(new MgdDbg.Snoop.Data.String("Flags", stairCalcData.Flags.ToString()));
            data.Add(new MgdDbg.Snoop.Data.Distance("Headroom height", stairCalcData.HeadroomHeight));
            data.Add(new MgdDbg.Snoop.Data.Distance("Headroom hole length", stairCalcData.HeadroomHoleLength));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is dirty", stairCalcData.IsDirty));
            data.Add(new MgdDbg.Snoop.Data.Distance("Landing thickness", stairCalcData.LandingThickness));
            data.Add(new MgdDbg.Snoop.Data.Distance("Left side clearance", stairCalcData.LeftSideClearance));
            data.Add(new MgdDbg.Snoop.Data.Object("Limits", stairCalcData.Limits));
            data.Add(new MgdDbg.Snoop.Data.Distance("Lower rule limit", stairCalcData.LowerRuleLimit));
            data.Add(new MgdDbg.Snoop.Data.Distance("Minimum landing offset", stairCalcData.MinimumLandingOffset));
            data.Add(new MgdDbg.Snoop.Data.Bool("Override calculator", stairCalcData.OverrideCalculator));
            data.Add(new MgdDbg.Snoop.Data.Distance("Right side clearance", stairCalcData.RightSideClearance));
            data.Add(new MgdDbg.Snoop.Data.Distance("Rise", stairCalcData.Rise));
            data.Add(new MgdDbg.Snoop.Data.Distance("Rise factor", stairCalcData.RiseFactor));
            data.Add(new MgdDbg.Snoop.Data.Distance("Rise length", stairCalcData.RiseLength));
            data.Add(new MgdDbg.Snoop.Data.Angle("Riser angle", stairCalcData.RiserAngle));
            data.Add(new MgdDbg.Snoop.Data.Distance("Rise ratio", stairCalcData.RiseRatio));
            data.Add(new MgdDbg.Snoop.Data.Int("Riser count", stairCalcData.RiserCount));
            data.Add(new MgdDbg.Snoop.Data.Distance("Riser depth", stairCalcData.RiserDepth));
            data.Add(new MgdDbg.Snoop.Data.Distance("Riser thickness", stairCalcData.RiserThickness));
            data.Add(new MgdDbg.Snoop.Data.Distance("Stair height", stairCalcData.StairHeight));
            data.Add(new MgdDbg.Snoop.Data.Distance("Static concrete thickness", stairCalcData.StaticConcreteThickness));
            data.Add(new MgdDbg.Snoop.Data.Distance("Storey height", stairCalcData.StoreyHeight));
            data.Add(new MgdDbg.Snoop.Data.Distance("Straight static concrete height", stairCalcData.StraightStaticConcreteHeight));
            data.Add(new MgdDbg.Snoop.Data.Distance("Top concrete thickness", stairCalcData.TopConcreteThickness));
            data.Add(new MgdDbg.Snoop.Data.Distance("Top floor level", stairCalcData.TopFloorLevel));
            data.Add(new MgdDbg.Snoop.Data.Distance("Top floor offset", stairCalcData.TopFloorOffset));
            data.Add(new MgdDbg.Snoop.Data.Distance("Tread", stairCalcData.Tread));
            data.Add(new MgdDbg.Snoop.Data.Distance("Tread factor", stairCalcData.TreadFactor));
            data.Add(new MgdDbg.Snoop.Data.Distance("Tread overhang", stairCalcData.TreadOverhang));
            data.Add(new MgdDbg.Snoop.Data.Distance("Tread thickness", stairCalcData.TreadThickness));
            data.Add(new MgdDbg.Snoop.Data.Distance("Upper rule limit", stairCalcData.UpperRuleLimit));
            data.Add(new MgdDbg.Snoop.Data.Bool("With riser", stairCalcData.WithRiser));
            data.Add(new MgdDbg.Snoop.Data.Bool("With tread", stairCalcData.WithTread));           
        }

        private void
        Stream(ArrayList data, StairCalculatorLimits stairCalcLim)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(StairCalculatorLimits)));

            foreach (StairCalculatorLimitType scLimitType in System.Enum.GetValues(typeof(StairCalculatorLimitType)))
                data.Add(new MgdDbg.Snoop.Data.Double(string.Format("Limit type = {0}", scLimitType.ToString()), stairCalcLim.GetValue(scLimitType)));           
        }

        private void
        Stream(ArrayList data, StairCreator stairCreator)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(StairCreator)));

            data.Add(new MgdDbg.Snoop.Data.Object("Curve", stairCreator.Curve));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is valid", stairCreator.IsValid));
            data.Add(new MgdDbg.Snoop.Data.Object("Outline", stairCreator.Outline));
            data.Add(new MgdDbg.Snoop.Data.String("Shape type", stairCreator.ShapeType.ToString()));
            data.Add(new MgdDbg.Snoop.Data.Distance("Width", stairCreator.Width));

            StairCreatorCustom scCustom = stairCreator as StairCreatorCustom;
            if (scCustom != null) {
                Stream(data, scCustom);
                return;
            }

            StairCreatorMultiLanding scMultiLand = stairCreator as StairCreatorMultiLanding;
            if (scMultiLand != null) {
                Stream(data, scMultiLand);
                return;
            }

            StairCreatorSpiral scSpiral = stairCreator as StairCreatorSpiral;
            if (scSpiral != null) {
                Stream(data, scSpiral);
                return;
            }

            StairCreatorUShaped scUShaped = stairCreator as StairCreatorUShaped;
            if (scUShaped != null) {
                Stream(data, scUShaped);
                return;
            }
        }

        private void
        Stream(ArrayList data, StairCreatorCustom scCustom)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(StairCreatorCustom)));

            data.Add(new MgdDbg.Snoop.Data.Object("Custom tread distances along path", scCustom.CustomTreadDistancesAlongPath));
            data.Add(new MgdDbg.Snoop.Data.Object("Extended path", scCustom.ExtendedPath));
            data.Add(new MgdDbg.Snoop.Data.Distance("Extension at start", scCustom.ExtensionAtStart));
            data.Add(new MgdDbg.Snoop.Data.Object("Path", scCustom.Path));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Custom tread definitions", scCustom.CustomTreadDefinitions));
        }

        private void
        Stream(ArrayList data, StairCreatorMultiLanding scMultiLand)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(StairCreatorMultiLanding)));

            data.Add(new MgdDbg.Snoop.Data.Object("Points", scMultiLand.Points));
            data.Add(new MgdDbg.Snoop.Data.Distance("Total width", scMultiLand.TotalWidth));
            data.Add(new MgdDbg.Snoop.Data.String("Turn types", scMultiLand.TurnTypes.ToString()));
        }

        private void
        Stream(ArrayList data, StairCreatorSpiral scSpiral)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(StairCreatorSpiral)));

            data.Add(new MgdDbg.Snoop.Data.Distance("Additional terminate with length", scSpiral.AdditionalTerminateWithLength));
            data.Add(new MgdDbg.Snoop.Data.Angle("Angle per step", scSpiral.AnglePerStep));
            data.Add(new MgdDbg.Snoop.Data.Point2d("Center", scSpiral.Center));
            data.Add(new MgdDbg.Snoop.Data.Bool("Counter clockwise", scSpiral.Counterclockwise));
            data.Add(new MgdDbg.Snoop.Data.Distance("Maximum inner width", scSpiral.MaximumInnerWidth));
            data.Add(new MgdDbg.Snoop.Data.Distance("Maximum outer width", scSpiral.MaximumOuterWidth));
            data.Add(new MgdDbg.Snoop.Data.Distance("Path asymmetry", scSpiral.PathAsymmetry));
            data.Add(new MgdDbg.Snoop.Data.Distance("Radius", scSpiral.Radius));
            data.Add(new MgdDbg.Snoop.Data.Point2d("Start", scSpiral.Start));
            data.Add(new MgdDbg.Snoop.Data.Angle("Total angle", scSpiral.TotalAngle));
            data.Add(new MgdDbg.Snoop.Data.Distance("Total automatic landing length", scSpiral.TotalAutomaticLandingLength));
            data.Add(new MgdDbg.Snoop.Data.Distance("Total width", scSpiral.TotalWidth));
            data.Add(new MgdDbg.Snoop.Data.String("Turn constraint", scSpiral.TurnConstraint.ToString()));
        }

        private void
        Stream(ArrayList data, StairCreatorUShaped scUShaped)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(StairCreatorUShaped)));

            data.Add(new MgdDbg.Snoop.Data.String("Alignment", scUShaped.Alignment.ToString()));
            data.Add(new MgdDbg.Snoop.Data.Distance("Alignment offset", scUShaped.AlignmentOffset));
            data.Add(new MgdDbg.Snoop.Data.Point2d("Center", scUShaped.Center));
            data.Add(new MgdDbg.Snoop.Data.Bool("Counter clockwise", scUShaped.Counterclockwise));
            data.Add(new MgdDbg.Snoop.Data.Point2d("End", scUShaped.End));
            data.Add(new MgdDbg.Snoop.Data.Distance("Path asymmetry", scUShaped.PathAsymmetry));
            data.Add(new MgdDbg.Snoop.Data.Point2d("Start", scUShaped.Start));
            data.Add(new MgdDbg.Snoop.Data.Distance("Total width", scUShaped.TotalWidth));
            data.Add(new MgdDbg.Snoop.Data.String("Turn types", scUShaped.TurnTypes.ToString()));
            data.Add(new MgdDbg.Snoop.Data.String("Turn types with final landing", scUShaped.TurnTypesWithFinalLanding.ToString()));
        }

        private void
        Stream(ArrayList data, RailingComponent railComp)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(RailingComponent)));

            data.Add(new MgdDbg.Snoop.Data.String("Shape type", railComp.ShapeType.ToString()));
            data.Add(new MgdDbg.Snoop.Data.String("Flags", railComp.Flags.ToString()));
            data.Add(new MgdDbg.Snoop.Data.Double("Width", railComp.Width));
            data.Add(new MgdDbg.Snoop.Data.Double("Height", railComp.Height));
            data.Add(new MgdDbg.Snoop.Data.String("Justification", railComp.Justification.ToString()));
            data.Add(new MgdDbg.Snoop.Data.Vector2d("Justification offset", railComp.JustificationOffset));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Material ID", railComp.MaterialId));
            data.Add(new MgdDbg.Snoop.Data.Object("Outline", railComp.Outline));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Profile ID", railComp.Profile));
            data.Add(new MgdDbg.Snoop.Data.Object("Railing", railComp.Railing));
            data.Add(new MgdDbg.Snoop.Data.Object("Path", railComp.GetPath()));   

            data.Add(new MgdDbg.Snoop.Data.Int("Segments count", railComp.SegmentsCount));
            ArrayList segments = new ArrayList();
            int len = railComp.SegmentsCount;
            for (int i=0; i<len; i++) {
                segments.Add(railComp.BodySegment(i));
            }
            data.Add(new MgdDbg.Snoop.Data.ObjectCollection("Body segments", segments));
        }

        private void
        Stream(ArrayList data, RailingVerticalComponent railVertComp)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(RailingVerticalComponent)));

            data.Add(new MgdDbg.Snoop.Data.Point3d("Base point", railVertComp.BasePoint));
            data.Add(new MgdDbg.Snoop.Data.Object("Body", railVertComp.Body));
            data.Add(new MgdDbg.Snoop.Data.Distance("Bottom extension", railVertComp.BottomExtension));
            data.Add(new MgdDbg.Snoop.Data.Distance("Depth", railVertComp.Depth));
            data.Add(new MgdDbg.Snoop.Data.Object("Ecs", railVertComp.Ecs));
            data.Add(new MgdDbg.Snoop.Data.String("Flags", railVertComp.Flags.ToString()));
            data.Add(new MgdDbg.Snoop.Data.String("Justification", railVertComp.Justification.ToString()));
            data.Add(new MgdDbg.Snoop.Data.Vector2d("Justification offset", railVertComp.JustificationOffset));
            data.Add(new MgdDbg.Snoop.Data.Object("Justification matrix", railVertComp.GetJustificationMatrix()));
            data.Add(new MgdDbg.Snoop.Data.Distance("Length from start", railVertComp.LengthFromStart));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Material ID", railVertComp.MaterialId));
            data.Add(new MgdDbg.Snoop.Data.Object("Outline", railVertComp.Outline));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Profile", railVertComp.Profile));

            try {
                data.Add(new MgdDbg.Snoop.Data.Distance("Radius", railVertComp.Radius));  // throws exception!
            }
            catch (Autodesk.AutoCAD.Runtime.Exception ex) {               
                data.Add(new MgdDbg.Snoop.Data.Exception("Radius", ex));
            }

            data.Add(new MgdDbg.Snoop.Data.Object("Railing", railVertComp.Railing));
            data.Add(new MgdDbg.Snoop.Data.Angle("Rotation", railVertComp.Rotation));
            data.Add(new MgdDbg.Snoop.Data.String("Shape type", railVertComp.ShapeType.ToString()));
            data.Add(new MgdDbg.Snoop.Data.Distance("Top extension", railVertComp.TopExtension));
            data.Add(new MgdDbg.Snoop.Data.Point3d("Top point", railVertComp.TopPoint));
            data.Add(new MgdDbg.Snoop.Data.Distance("Width", railVertComp.Width));

            Baluster baluster = railVertComp as Baluster;
            if (baluster != null) {
                Stream(data, baluster);
                return;
            }

            RailingPost railPost = railVertComp as RailingPost;
            if (railPost != null) {
                Stream(data, railPost);
                return;
            }
        }

        private void
        Stream(ArrayList data, Baluster baluster)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(Baluster)));

            // no data at this level
        }

        private void
        Stream(ArrayList data, RailingPost railPost)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(RailingPost)));

            data.Add(new MgdDbg.Snoop.Data.String("Type", railPost.Type.ToString()));
        }

        private void
        Stream(ArrayList data, StandardSizeOpening stdSizeOpen)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(StandardSizeOpening)));

            data.Add(new MgdDbg.Snoop.Data.Double("Width", stdSizeOpen.Width));            
            data.Add(new MgdDbg.Snoop.Data.Double("Height", stdSizeOpen.Height));
            data.Add(new MgdDbg.Snoop.Data.Double("Leaf", stdSizeOpen.Leaf));
            data.Add(new MgdDbg.Snoop.Data.Double("Rise", stdSizeOpen.Rise));
        }

        private void
        Stream(ArrayList data, SlabCustomGeometry slabCustGeo)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(SlabCustomGeometry)));

            data.Add(new MgdDbg.Snoop.Data.Int("Component ID", slabCustGeo.ComponentId));
            data.Add(new MgdDbg.Snoop.Data.String("Operation type", slabCustGeo.OperationType.ToString()));

            SlabCustomGeometryBody scGeoBody = slabCustGeo as SlabCustomGeometryBody;
            if (scGeoBody != null) {
                Stream(data, scGeoBody);
                return;
            }
        }

        private void
        Stream(ArrayList data, SlabCustomGeometryBody scGeoBody)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(SlabCustomGeometryBody)));

            data.Add(new MgdDbg.Snoop.Data.Object("Body", scGeoBody.Body));
            data.Add(new MgdDbg.Snoop.Data.Object("Body as MassElement", scGeoBody.GetBodyAsMassElement()));
        }

        private void
        Stream(ArrayList data, AbstractTreeNodeSpace abTreeNodeSpace)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AbstractTreeNodeSpace)));

            data.Add(new MgdDbg.Snoop.Data.Bool("Allow twins", abTreeNodeSpace.AllowTwins));
            data.Add(new MgdDbg.Snoop.Data.Int("Number of children", abTreeNodeSpace.NumChildren));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Children", abTreeNodeSpace.Children));
            data.Add(new MgdDbg.Snoop.Data.Object("Parent node", abTreeNodeSpace.ParentNode));

            ZoneDefinitionNode zoneDefNode = abTreeNodeSpace as ZoneDefinitionNode;
            if (zoneDefNode != null) {
                Stream(data, zoneDefNode);
                return;
            }
        }

        private void
        Stream(ArrayList data, ZoneDefinitionNode zoneDefNode)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(ZoneDefinitionNode)));

            data.Add(new MgdDbg.Snoop.Data.ObjectId("Style ID", zoneDefNode.StyleId));
            data.Add(new MgdDbg.Snoop.Data.String("Node name", zoneDefNode.NodeName));
            data.Add(new MgdDbg.Snoop.Data.String("Node description", zoneDefNode.NodeDescription));
            data.Add(new MgdDbg.Snoop.Data.ObjectIdCollection("Modifiers", zoneDefNode.Modifiers));
            data.Add(new MgdDbg.Snoop.Data.Bool("New child allowed", zoneDefNode.NewChildAllowed));
            data.Add(new MgdDbg.Snoop.Data.Bool("Remove allowed", zoneDefNode.RemoveAllowed));
           data.Add(new MgdDbg.Snoop.Data.String("Ownership mode", zoneDefNode.OwnershipMode.ToString()));
        }
    }
}
