
// $Header: //Desktop/Source/TestTools/MgdDbgAec/Snoop/CollectorExts/ImpObjectBase.cs#6 $
// $Author: ayyara $ $DateTime: 2008/03/07 17:52:55 $ $Change: 94950 $
//
// (C) Copyright 2004 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//


using System;
using System.Diagnostics;
using System.Collections;
using System.Windows.Forms;
using System.Collections.Specialized;

using Autodesk.Aec.Modeler;
using Autodesk.Aec.Geometry;
using Autodesk.AutoCAD.Geometry;
using Autodesk.Aec.DatabaseServices;
using Autodesk.Aec.Arch.DatabaseServices;


using MgdDbg.Snoop;


namespace MgdDbgAec.Snoop.CollectorExts
{
	/// <summary>
	/// This is a Snoop Collector Extension object to collect data from AEC ImpObjs.
	/// </summary>
	public class ImpObjectBase : MgdDbg.Snoop.CollectorExts.CollectorExt
	{
		public ImpObjectBase()
		{
		}
		
        protected override void
        CollectEvent(object sender, MgdDbg.Snoop.Collectors.CollectorEventArgs e)
        {
                // bail early if this isn't an object we are responsible for
            ImpObject impObj = e.ObjToSnoop as ImpObject;
            if (impObj == null)
                return;

                // cast the sender object to the SnoopCollector we are expecting
            MgdDbg.Snoop.Collectors.Collector snoopCollector = sender as MgdDbg.Snoop.Collectors.Collector;
            if (snoopCollector == null) {
                Debug.Assert(false);    // why did someone else send us the message?
                return;
            }
            
            Stream(snoopCollector.Data(), impObj);
        }
        

        private void
        Stream(ArrayList data, ImpObject impObj)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(ImpObject)));

            data.Add(new MgdDbg.Snoop.Data.String("Description", impObj.Description));
            data.Add(new MgdDbg.Snoop.Data.Database("Database", impObj.Database));

                // branch to all known major sub-classes
            Profile prof = impObj as Profile;
            if (prof != null) {
                Stream(data, prof);
                return;
            }

            Segment2d seg = impObj as Segment2d;
            if (seg != null) {
                Stream(data, seg);
                return;
            }
            
            MultiViewBlockViewInstance mvViewBlkInst = impObj as MultiViewBlockViewInstance;
            if (mvViewBlkInst != null) {
                Stream(data, mvViewBlkInst);
                return;
            }
            
            MultiViewBlockAttributeReference mvViewAtt = impObj as MultiViewBlockAttributeReference;
            if (mvViewAtt != null) {
                Stream(data, mvViewAtt);
                return;
            }
            
            MultiViewBlockDisplayRepresentationDefinition mvViewDispRep = impObj as MultiViewBlockDisplayRepresentationDefinition;
            if (mvViewDispRep != null) {
                Stream(data, mvViewDispRep);
                return;
            }
            
            MultiViewBlockViewDefinition mvViewBlkDef = impObj as MultiViewBlockViewDefinition;
            if (mvViewBlkDef != null) {
                Stream(data, mvViewBlkDef);
                return;
            }
            
            GridUv gridUv = impObj as GridUv;
            if (gridUv != null) {
                Stream(data, gridUv);
                return;
            }
            
            LayoutCell layoutCell = impObj as LayoutCell;
            if (layoutCell != null) {
                Stream(data, layoutCell);
                return;
            }
            
            LayoutVolume layoutVol = impObj as LayoutVolume;
            if (layoutVol != null) {
                Stream(data, layoutVol);
                return;
            }
            
            LayoutNode layoutNode = impObj as LayoutNode;
            if (layoutNode != null) {
                Stream(data, layoutNode);
                return;
            }
 
            GridAssemblyComponentSet gridAssemblyCompSet = impObj as GridAssemblyComponentSet;
            if (gridAssemblyCompSet != null) {
                Stream(data, gridAssemblyCompSet);
                return;
            }
           
            GridAssemblyDefinition gridAssemblyDef = impObj as GridAssemblyDefinition;
            if (gridAssemblyDef != null) {
                Stream(data, gridAssemblyDef);
                return;
            }

            GridAssemblyEdge gridAssemblyEdge = impObj as GridAssemblyEdge;
            if (gridAssemblyEdge != null) {
                Stream(data, gridAssemblyEdge);
                return;
            }

            GridAssemblyInterference gridAssemblyInt = impObj as GridAssemblyInterference;
            if (gridAssemblyInt != null) {
                Stream(data, gridAssemblyInt);
                return;
            }

            NestedGridDivision nestedGridDiv = impObj as NestedGridDivision;
            if (nestedGridDiv != null) {
                Stream(data, nestedGridDiv);
                return;
            }

            NestedGridEdgeDefinition nestedGridEdge = impObj as NestedGridEdgeDefinition;
            if (nestedGridEdge != null) {
                Stream(data, nestedGridEdge);
                return;
            }

            NestedGridInfillDefinition nestedGridInfill = impObj as NestedGridInfillDefinition;
            if (nestedGridInfill != null) {
                Stream(data, nestedGridInfill);
                return;
            }

            NestedGridCellAssignment ngCellAssignment = impObj as NestedGridCellAssignment;
            if (ngCellAssignment != null) {
                Stream(data, ngCellAssignment);
                return;
            }

            NestedGridEdgeAssignment ngEdgeAssignment = impObj as NestedGridEdgeAssignment;
            if (ngEdgeAssignment != null) {
                Stream(data, ngEdgeAssignment);
                return;
            }

            NestedGridDivisionManualEdgeDefinition ngManEdgeDef = impObj as NestedGridDivisionManualEdgeDefinition;
            if (ngManEdgeDef != null) {
                Stream(data, ngManEdgeDef);
                return;
            }

            IndexSpecifier indexSpec = impObj as IndexSpecifier;
            if ( indexSpec != null) {
                Stream(data, indexSpec);
                return;
            }

            AnchorToCurveX anchorCurveX = impObj as AnchorToCurveX;
            if (anchorCurveX != null) {
                Stream(data, anchorCurveX);
                return;
            }
            
            AnchorToCurveY anchorCurveY = impObj as AnchorToCurveY;
            if (anchorCurveY != null) {
                Stream(data, anchorCurveY);
                return;
            }

            AnchorToCurveZ anchorCurveZ = impObj as AnchorToCurveZ;
            if (anchorCurveZ != null) {
                Stream(data, anchorCurveZ);
                return;
            }

            CompoundCurve2d compCurve2d = impObj as CompoundCurve2d;
            if (compCurve2d != null) {
                Stream(data, compCurve2d);
                return;
            }

            ImpTree impTree = impObj as ImpTree;
            if (impTree != null) {
                Stream(data, impTree);
                return;
            }

            ClassificationCollection classColl = impObj as ClassificationCollection;
            if (classColl != null) {
                Stream(data, classColl);
                return;
            }

            LayerKeyDefinition lKeyDef = impObj as LayerKeyDefinition;
            if (lKeyDef != null) {
                Stream(data, lKeyDef);
                return;
            }

            ConnectionComponent connComp = impObj as ConnectionComponent;
            if (connComp != null) {
                Stream(data, connComp);
                return;
            }

            Section2dHatchRegion sec2dHatchReg = impObj as Section2dHatchRegion;
            if (sec2dHatchReg != null) {
                Stream(data, sec2dHatchReg);
                return;
            }

            Section2dSegmentGroup sec2dSegGroup = impObj as Section2dSegmentGroup;
            if (sec2dSegGroup != null) {
                Stream(data, sec2dSegGroup);
                return;
            }

            SectionSegment secSegment = impObj as SectionSegment;
            if (secSegment != null) {
                Stream(data, secSegment);
                return;
            }

            SectionVertex secVertex = impObj as SectionVertex;
            if (secVertex != null) {
                Stream(data, secVertex);
                return;
            }

            LineWorkComponent lwComp = impObj as LineWorkComponent;
            if (lwComp != null) {
                Stream(data, lwComp);
                return;
            }

            LineWorkRule lwRule = impObj as LineWorkRule;
            if (lwRule != null) {
                Stream(data, lwRule);
                return;
            }

            Override overRide = impObj as Override;
            if (overRide != null) {
                Stream(data, overRide);
                return;
            }

            UnitType unitType = impObj as UnitType;
            if (unitType != null) {
                Stream(data, unitType);
                return;
            }

            UnitVariable unitVar = impObj as UnitVariable;
            if (unitVar != null) {
                Stream(data, unitVar);
                return;
            }

            GraphicsStorage graphStorage = impObj as GraphicsStorage;
            if (graphStorage != null) {
                Stream(data, graphStorage);
                return;
            }

#if !ADT2007
            DisplayComponent dispComp = impObj as DisplayComponent;
            if (dispComp != null) {
                Stream(data, dispComp);
                return;
            }
#endif

        }

        private void
        Stream(ArrayList data, Profile prof)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(Profile)));
            
            data.Add(new MgdDbg.Snoop.Data.Double("Area", prof.Area(Tolerance.Global.EqualPoint)));

            Point2d topLeft, bottomRight;
            prof.Extent(out topLeft, out bottomRight);
            data.Add(new MgdDbg.Snoop.Data.Point2d("Extent (top Left)", topLeft));
            data.Add(new MgdDbg.Snoop.Data.Point2d("Extent (bottom right)", bottomRight));
            data.Add(new MgdDbg.Snoop.Data.Distance("Perimeter", prof.Perimeter(Tolerance.Global.EqualPoint)));

            data.Add(new MgdDbg.Snoop.Data.Point2d("Center of gravity", prof.CenterOfGravity));
            data.Add(new MgdDbg.Snoop.Data.Point2d("Centroid", prof.Centroid));
            data.Add(new MgdDbg.Snoop.Data.Bool("Dirty flag", prof.DirtyFlag));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is self intersecting", prof.IsSelfIntersecting));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is valid profile (additional checks = false)", prof.IsValidProfile(false)));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is valid profile (additional checks = true)", prof.IsValidProfile(true)));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is valid ring nesting (check level = 0)", prof.IsValidRingNesting(0)));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is valid ring nesting (check level = 1)", prof.IsValidRingNesting(1)));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is valid ring nesting (check level = 2)", prof.IsValidRingNesting(2)));

            try
            {
                data.Add(new MgdDbg.Snoop.Data.Enumerable("Rings", prof.Rings));
            }            
            catch (System.Exception e)
            {
                data.Add(new MgdDbg.Snoop.Data.Exception("Rings", e));                
            }

            data.Add(new MgdDbg.Snoop.Data.Enumerable("Vertices", prof.Vertices));
        }

        private void
        Stream(ArrayList data, Segment2d seg)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(Segment2d)));

            data.Add(new MgdDbg.Snoop.Data.Object("Bound box 2D", seg.BoundBox2d));            
            data.Add(new MgdDbg.Snoop.Data.Double("Bulge", seg.Bulge));
            data.Add(new MgdDbg.Snoop.Data.Object("Curve", seg.Curve));            
            data.Add(new MgdDbg.Snoop.Data.Bool("Curve direction positive", seg.CurveDirectionPositive));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is closed", seg.IsClosed));            
            data.Add(new MgdDbg.Snoop.Data.Bool("Is curve circular arc 2D", seg.IsCurveCircularArc2d));            
            data.Add(new MgdDbg.Snoop.Data.Bool("Is curve line segment 2D", seg.IsCurveLineSegment2d));            
            data.Add(new MgdDbg.Snoop.Data.Distance("Length", seg.Length));
            data.Add(new MgdDbg.Snoop.Data.Double("Start parameter", seg.StartParameter));
            data.Add(new MgdDbg.Snoop.Data.Double("End parameter", seg.EndParameter));
            data.Add(new MgdDbg.Snoop.Data.Point2d("Start point", seg.StartPoint));
            data.Add(new MgdDbg.Snoop.Data.Point2d("End point", seg.EndPoint));
            data.Add(new MgdDbg.Snoop.Data.Point2d("Mid point", seg.MidPoint));
            data.Add(new MgdDbg.Snoop.Data.Object("Interval", seg.Interval));            
            data.Add(new MgdDbg.Snoop.Data.String("Segment position", seg.SegmentPosition.ToString()));            
            data.Add(new MgdDbg.Snoop.Data.Bool("Visible", seg.Visible));            

            CompoundSegment2d compSeg2d = seg as CompoundSegment2d;
            if (compSeg2d != null) {
                Stream(data, compSeg2d);
                return;
            }

            GridCustomSegment gcSegment = seg as GridCustomSegment;
            if (gcSegment != null) {
                Stream(data, gcSegment);
                return;
            }
        }

        private void
        Stream(ArrayList data, CompoundSegment2d compSeg2d)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(CompoundSegment2d)));

            data.Add(new MgdDbg.Snoop.Data.Int("Graphic system marker", compSeg2d.GraphicSystemMarker));
        }

        private void
        Stream(ArrayList data, MultiViewBlockViewInstance viewBlk)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(MultiViewBlockViewInstance)));

            data.Add(new MgdDbg.Snoop.Data.ObjectId("Block ID", viewBlk.BlockId));
            data.Add(new MgdDbg.Snoop.Data.Vector3d("Offset", viewBlk.Offset));
            data.Add(new MgdDbg.Snoop.Data.Bool("Needs attribute update", viewBlk.NeedsAttributeUpdate));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Attribute references", viewBlk.AttributeReferences));
        }
        
        private void
        Stream(ArrayList data, MultiViewBlockAttributeReference att)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(MultiViewBlockAttributeReference)));

            data.Add(new MgdDbg.Snoop.Data.String("Prompt", att.Prompt));
            data.Add(new MgdDbg.Snoop.Data.Object("Attribute", att.Attribute));
            data.Add(new MgdDbg.Snoop.Data.Bool("Needs update", att.NeedsUpdate));
            data.Add(new MgdDbg.Snoop.Data.Object("MText", att.MText));
        }
        
        private void
        Stream(ArrayList data, MultiViewBlockDisplayRepresentationDefinition dispRepDef)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(MultiViewBlockDisplayRepresentationDefinition)));

            data.Add(new MgdDbg.Snoop.Data.ObjectId("Display representation ID", dispRepDef.DisplayRepresentationId));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("View definitions", dispRepDef.ViewDefinitions));
        }
        
        private void
        Stream(ArrayList data, MultiViewBlockViewDefinition viewBlk)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(MultiViewBlockViewDefinition)));

            data.Add(new MgdDbg.Snoop.Data.ObjectId("Block ID", viewBlk.BlockId));

            foreach (ViewDirection vDir in System.Enum.GetValues(typeof(ViewDirection))) {
                data.Add(new MgdDbg.Snoop.Data.Bool(string.Format("Is view on [{0}]", vDir.ToString()), viewBlk.IsViewOn(vDir)));
            }

            data.Add(new MgdDbg.Snoop.Data.Bool("Attribute grips disabled", viewBlk.AttributeGripsDisabled));
        }
        
        private void
        Stream(ArrayList data, GridUv gridUv)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(GridUv)));

            data.Add(new MgdDbg.Snoop.Data.String("Grid type", gridUv.GridType.ToString()));

            IntegerCollection cellIds = gridUv.LayoutCellIds;
            data.Add(new MgdDbg.Snoop.Data.Object("Layout cell IDs", cellIds));

            ArrayList cells = new ArrayList();
            ArrayList cellProfs = new ArrayList();
            Profile prof = null;
            Matrix3d mat;
            for (int n=0; n < cellIds.Count; n++) {
                cells.Add(gridUv.GetLayoutGrid2dCell(cellIds[n]));
                prof = gridUv.GetLayoutCellProfile(cellIds[n], out mat);
                cellProfs.Add(prof);
            }
            data.Add(new MgdDbg.Snoop.Data.ObjectCollection("2D grid cells using id", cells));
            data.Add(new MgdDbg.Snoop.Data.Object("Layout cell profiles", cellProfs));

            IntegerCollection nodeIds = gridUv.LayoutNodeIds;
            data.Add(new MgdDbg.Snoop.Data.Object("Layout node IDs", cellIds));

            ArrayList nodes = new ArrayList();
            ArrayList nodeCoordSystems = new ArrayList();
            ArrayList nodeCoords = new ArrayList();
            for (int n=0; n < nodeIds.Count; n++) {
                nodes.Add(gridUv.GetLayoutGrid2dNode(nodeIds[n]));
                nodeCoordSystems.Add(gridUv.GetLayoutNodeCoordinateSystem(nodeIds[n]));
                nodeCoords.Add(gridUv.GetLayoutNodeLocation(nodeIds[n]));
            }
            data.Add(new MgdDbg.Snoop.Data.ObjectCollection("2D grid nodes using id", nodes));
            data.Add(new MgdDbg.Snoop.Data.ObjectCollection("2D grid nodes coordinates", nodeCoords));
            data.Add(new MgdDbg.Snoop.Data.ObjectCollection("2D grid nodes coordinate systems", nodeCoordSystems));

            ArrayList nodes2d = new ArrayList();
            int len = gridUv.NodeCount;
            for (int i=0; i<len; i++)
                nodes2d.Add(gridUv.GetLayoutGrid2dNodeAt(i));
            data.Add(new MgdDbg.Snoop.Data.ObjectCollection("2D grid nodes", nodes2d));
            
            ArrayList cells2d = new ArrayList();
            len = gridUv.CellCount;
            for (int i=0; i<len; i++)
                cells2d.Add(gridUv.GetLayoutGrid2dCellAt(i));
            data.Add(new MgdDbg.Snoop.Data.ObjectCollection("2D grid cells", cells2d));

            data.Add(new MgdDbg.Snoop.Data.Distance("U axis length", gridUv.UAxisLength));
            data.Add(new MgdDbg.Snoop.Data.Distance("U spacing", gridUv.USpacing));
            data.Add(new MgdDbg.Snoop.Data.Distance("U start offset", gridUv.UStartOffset));
            data.Add(new MgdDbg.Snoop.Data.Distance("U end offset", gridUv.UEndOffset));
            data.Add(new MgdDbg.Snoop.Data.String("U layout mode", gridUv.ULayoutMode.ToString()));
            data.Add(new MgdDbg.Snoop.Data.Object("U parameters", gridUv.UParameters));
            data.Add(new MgdDbg.Snoop.Data.Distance("V axis length", gridUv.VAxisLength));
            data.Add(new MgdDbg.Snoop.Data.Distance("V spacing", gridUv.VSpacing));
            data.Add(new MgdDbg.Snoop.Data.Distance("V start offset", gridUv.VStartOffset));
            data.Add(new MgdDbg.Snoop.Data.Distance("V end offset", gridUv.VEndOffset));
            data.Add(new MgdDbg.Snoop.Data.String("V layout mode", gridUv.VLayoutMode.ToString()));
            data.Add(new MgdDbg.Snoop.Data.Object("V parameters", gridUv.VParameters));

            GridUvw gridUvw = gridUv as GridUvw;
            if (gridUvw != null) {
                Stream(data, gridUvw);
                return;
            }

            GridUvRadial gridUvRadial = gridUv as GridUvRadial;
            if (gridUvRadial != null) {
                Stream(data, gridUvRadial);
                return;
            }

            GridUvRectangular gridUvRect = gridUv as GridUvRectangular;
            if (gridUvRect != null) {
                Stream(data, gridUvRect);
                return;
            }
        }
        
        private void
        Stream(ArrayList data, GridUvw gridUvw)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(GridUvw)));

                // cells
            IntegerCollection cellIds = gridUvw.LayoutCellIds;
            data.Add(new MgdDbg.Snoop.Data.Object("Layout cell IDs", cellIds));

            ArrayList cells = new ArrayList();
            for (int n=0; n < cellIds.Count; n++) {
                cells.Add(gridUvw.GetLayoutGrid3dCell(cellIds[n]));
            }
            data.Add(new MgdDbg.Snoop.Data.ObjectCollection("3D grid cells using ID", cells));

            ArrayList cells3d = new ArrayList();
            int len = gridUvw.CellCount;
            for (int i=0; i<len; i++)
                cells3d.Add(gridUvw.GetLayoutGrid3dCellAt(i));
            data.Add(new MgdDbg.Snoop.Data.ObjectCollection("3D grid cells", cells3d));
            
                // nodes
            IntegerCollection nodeIds = gridUvw.LayoutNodeIds;
            ArrayList nodes = new ArrayList();
            for (int n=0; n < nodeIds.Count; n++) {
                nodes.Add(gridUvw.GetLayoutGrid3dNode(nodeIds[n]));
            }
            data.Add(new MgdDbg.Snoop.Data.ObjectCollection("3D grid nodes using ID", nodes));


            len = gridUvw.NodeCount;
            ArrayList nodes3d = new ArrayList();
            for (int i=0; i<len; i++)
                nodes3d.Add(gridUvw.GetLayoutGrid3dNodeAt(i));
            data.Add(new MgdDbg.Snoop.Data.ObjectCollection("3D grid nodes", nodes3d));
            

                // volumes
            ArrayList vols3d = new ArrayList();
            len = gridUvw.VolumeCount;
            for (int i=0; i<len; i++)
                vols3d.Add(gridUvw.GetLayoutGrid3dVolumeAt(i));
            data.Add(new MgdDbg.Snoop.Data.ObjectCollection("3D grid volumes", vols3d));

            IntegerCollection volIds = gridUvw.LayoutVolumeIds;
            data.Add(new MgdDbg.Snoop.Data.Object("Layout volume IDs", volIds));

            ArrayList volumes = new ArrayList();
            ArrayList volBodies = new ArrayList();
            ArrayList centroids = new ArrayList();
            Body body = null;
            Matrix3d mat;
            for (int n=0; n < volIds.Count; n++) {
                volumes.Add(gridUvw.GetLayoutGrid3dVolume(volIds[n]));
                body = gridUvw.GetLayoutVolumeBody(volIds[n], out mat);
                volBodies.Add(body);
                mat = gridUvw.GetLayoutVolumeCentroid(volIds[n]);
                centroids.Add(mat);
            }
            data.Add(new MgdDbg.Snoop.Data.ObjectCollection("3D grid volumes by ID", volumes));
            data.Add(new MgdDbg.Snoop.Data.Object("Layout volume bodies", volBodies));
            data.Add(new MgdDbg.Snoop.Data.Object("Layout volume centroids", centroids));

            data.Add(new MgdDbg.Snoop.Data.Distance("W axis length", gridUvw.WAxisLength));
            data.Add(new MgdDbg.Snoop.Data.Distance("W spacing", gridUvw.WSpacing));
            data.Add(new MgdDbg.Snoop.Data.Distance("W start offset", gridUvw.WStartOffset));
            data.Add(new MgdDbg.Snoop.Data.Distance("W end offset", gridUvw.WEndOffset));
            data.Add(new MgdDbg.Snoop.Data.String("W layout mode", gridUvw.WLayoutMode.ToString()));
            data.Add(new MgdDbg.Snoop.Data.Object("W parameters", gridUvw.WParameters));
        }

        private void
        Stream(ArrayList data, GridUvRadial gridUvRadial)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(GridUvRadial)));

            data.Add(new MgdDbg.Snoop.Data.Double("U inside radius", gridUvRadial.UInsideRadius));
            data.Add(new MgdDbg.Snoop.Data.Double("U angle spacing", gridUvRadial.VAngleSpacing));
            data.Add(new MgdDbg.Snoop.Data.Double("U axis angle", gridUvRadial.VAxisAngle));
            data.Add(new MgdDbg.Snoop.Data.Double("V end angle offset", gridUvRadial.VEndAngleOffset));
            data.Add(new MgdDbg.Snoop.Data.Double("V start angle offset", gridUvRadial.VStartAngleOffset));         
        }

        private void
        Stream(ArrayList data, GridUvRectangular gridUvRect)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(GridUvRectangular)));

            // no data at this level           
        }
        
        private void
        Stream(ArrayList data, LayoutCurveNode node)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(LayoutCurveNode)));

            data.Add(new MgdDbg.Snoop.Data.Double("U position", node.UPosition));
            
            LayoutGrid2dNode gridNode = node as LayoutGrid2dNode;
            if (gridNode != null) {
                Stream(data, gridNode);
                return;
            }
        }
        
        private void
        Stream(ArrayList data, GridCustomNode node)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(GridCustomNode)));

            data.Add(new MgdDbg.Snoop.Data.String("Node location", node.NodeLocation.ToString() ));
            data.Add(new MgdDbg.Snoop.Data.Int("Segment ID", node.SegmentId));
        }
        
        private void
        Stream(ArrayList data, LayoutGrid2dNode node)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(LayoutGrid2dNode)));

            data.Add(new MgdDbg.Snoop.Data.Double("V position", node.VPosition));
            
            LayoutGrid3dNode gridNode3d = node as LayoutGrid3dNode;
            if (gridNode3d != null) {
                Stream(data, gridNode3d);
                return;
            }
        }
        
        private void
        Stream(ArrayList data, LayoutGrid3dNode node)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(LayoutGrid3dNode)));

            data.Add(new MgdDbg.Snoop.Data.Double("W position", node.WPosition));
        }
        
        private void
        Stream(ArrayList data, LayoutCell cell)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(LayoutCell)));

            data.Add(new MgdDbg.Snoop.Data.Int("Cell ID", cell.CellId));
            
            LayoutGrid2dCell gridCell = cell as LayoutGrid2dCell;
            if (gridCell != null) {
                Stream(data, gridCell);
                return;
            }

            GridAssemblyCell gridAssemblyCell = cell as GridAssemblyCell;
            if (gridAssemblyCell != null) 
            {
                Stream(data, gridAssemblyCell);
                return;
            }

            GridCustomCell gridCustomCell = cell as GridCustomCell;
            if (gridCustomCell != null) 
            {
                Stream(data, gridCustomCell);
                return;
            }
        }
        
        private void
        Stream(ArrayList data, LayoutGrid2dCell gridCell)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(LayoutGrid2dCell)));

            data.Add(new MgdDbg.Snoop.Data.Int("U index", gridCell.UIndex));
            data.Add(new MgdDbg.Snoop.Data.Int("V index", gridCell.VIndex));
            
            LayoutGrid3dCell gridCell3d = gridCell as LayoutGrid3dCell;
            if (gridCell3d != null) {
                Stream(data, gridCell3d);
                return;
            }
        }
        
        private void
        Stream(ArrayList data, LayoutGrid3dCell gridCell)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(LayoutGrid3dCell)));

            data.Add(new MgdDbg.Snoop.Data.Int("W index", gridCell.WIndex));
            data.Add(new MgdDbg.Snoop.Data.String("Plane type", gridCell.Plane.ToString()));
        }
        
        private void
        Stream(ArrayList data, GridAssemblyCell gridAssemblyCell)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(GridAssemblyCell)));

            data.Add(new MgdDbg.Snoop.Data.Bool("At entity start", gridAssemblyCell.AtEntityStart ));
            data.Add(new MgdDbg.Snoop.Data.Bool("At entity end", gridAssemblyCell.AtEntityEnd ));
            data.Add(new MgdDbg.Snoop.Data.Int("Cell assignment Id", gridAssemblyCell.CellAssignmentId ));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Contained entity Id", gridAssemblyCell.ContainedEntityId ));
            data.Add(new MgdDbg.Snoop.Data.Double("Gross start X", gridAssemblyCell.GrossStartX ));
            data.Add(new MgdDbg.Snoop.Data.Double("Gross start Y", gridAssemblyCell.GrossStartY ));
            data.Add(new MgdDbg.Snoop.Data.Double("Gross end X", gridAssemblyCell.GrossEndX ));
            data.Add(new MgdDbg.Snoop.Data.Double("Gross eEnd Y", gridAssemblyCell.GrossEndY ));
            data.Add(new MgdDbg.Snoop.Data.Bool("Has net profile", gridAssemblyCell.HasNetProfile  ));
            data.Add(new MgdDbg.Snoop.Data.Int("Infill override Id", gridAssemblyCell.InfillOverrideId ));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is leaf cell", gridAssemblyCell.IsLeafCell ));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is overridden", gridAssemblyCell.IsOverridden ));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is void", gridAssemblyCell.IsVoid ));

            data.Add(new MgdDbg.Snoop.Data.Object("NestingPath", gridAssemblyCell.NestingPath ));

            if (gridAssemblyCell.HasNetProfile)
                data.Add(new MgdDbg.Snoop.Data.Object("Net cell profile", gridAssemblyCell.NetCellProfile ));
            else {
                    // these guys can only be called if there is no net profile
                data.Add(new MgdDbg.Snoop.Data.Double("Net start X", gridAssemblyCell.NetStartX ));
                data.Add(new MgdDbg.Snoop.Data.Double("Net start Y", gridAssemblyCell.NetStartY ));
                data.Add(new MgdDbg.Snoop.Data.Double("Net end X", gridAssemblyCell.NetEndX ));
                data.Add(new MgdDbg.Snoop.Data.Double("Net end Y", gridAssemblyCell.NetEndY ));
            }
            data.Add(new MgdDbg.Snoop.Data.Bool("Was transfered", gridAssemblyCell.WasTransfered ));
        }


        private void
        Stream(ArrayList data, GridAssemblyComponentSet gridAssemblyCompSet)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(GridAssemblyComponentSet)));

            data.Add(new MgdDbg.Snoop.Data.Enumerable("Boundary edge components", gridAssemblyCompSet.BoundaryEdgeComponents));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Division components", gridAssemblyCompSet.DivisionComponents));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Infill components", gridAssemblyCompSet.InfillComponents));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Internal Edge Components", gridAssemblyCompSet.InteriorEdgeComponents));
        }


        private void
        Stream(ArrayList data, GridAssemblyEdge gridAssemblyEdge)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(GridAssemblyEdge)));

            data.Add(new MgdDbg.Snoop.Data.Int("Edge assignment ID", gridAssemblyEdge.EdgeAssignmentId));
            data.Add(new MgdDbg.Snoop.Data.Object("Edge curve", gridAssemblyEdge.EdgeCurve));
            data.Add(new MgdDbg.Snoop.Data.Int("Edge ID", gridAssemblyEdge.EdgeId));
            data.Add(new MgdDbg.Snoop.Data.Int("Edge override ID", gridAssemblyEdge.EdgeOverrideId));
            data.Add(new MgdDbg.Snoop.Data.Bool("Start at entity start", gridAssemblyEdge.StartAtEntityStart));
            data.Add(new MgdDbg.Snoop.Data.Bool("Start at entity end", gridAssemblyEdge.StartAtEntityEnd));
            data.Add(new MgdDbg.Snoop.Data.Bool("End at entity start", gridAssemblyEdge.EndAtEntityStart));
            data.Add(new MgdDbg.Snoop.Data.Bool("End at entity end", gridAssemblyEdge.EndAtEntityEnd));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is internal edge", gridAssemblyEdge.IsInternalEdge));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is overridden", gridAssemblyEdge.IsOverridden));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is void", gridAssemblyEdge.IsVoid));
            data.Add(new MgdDbg.Snoop.Data.Object("Nesting path", gridAssemblyEdge.NestingPath));
            data.Add(new MgdDbg.Snoop.Data.Bool("Has profile override", gridAssemblyEdge.HasProfileOverride));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Profile override ID", gridAssemblyEdge.ProfileOverrideId));
        }
        
        private void
        Stream(ArrayList data, NestedGridEdgeDefinition nestedGridEdge)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(NestedGridEdgeDefinition)));

            data.Add(new MgdDbg.Snoop.Data.String("Name", nestedGridEdge.Name));
            data.Add(new MgdDbg.Snoop.Data.Int("ID", nestedGridEdge.Id));
            data.Add(new MgdDbg.Snoop.Data.Bool("Cleanup internal edges", nestedGridEdge.CleanupInternalEdges));
            data.Add(new MgdDbg.Snoop.Data.Distance("Depth offset", nestedGridEdge.DepthOffset));
            data.Add(new MgdDbg.Snoop.Data.Distance("Edge depth", nestedGridEdge.EdgeDepth));
            data.Add(new MgdDbg.Snoop.Data.Distance("Edge offset", nestedGridEdge.EdgeOffset));
            data.Add(new MgdDbg.Snoop.Data.Distance("Edge width", nestedGridEdge.EdgeWidth));
            data.Add(new MgdDbg.Snoop.Data.Distance("Start offset", nestedGridEdge.StartOffset));
            data.Add(new MgdDbg.Snoop.Data.Distance("End offset", nestedGridEdge.EndOffset));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Material ID", nestedGridEdge.MaterialId));
            data.Add(new MgdDbg.Snoop.Data.Bool("Mirror profile X", nestedGridEdge.MirrorProfileX));
            data.Add(new MgdDbg.Snoop.Data.Bool("Mirror profile Y", nestedGridEdge.MirrorProfileY));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Profile ID", nestedGridEdge.ProfileId));
            data.Add(new MgdDbg.Snoop.Data.Angle("Profile rotation", nestedGridEdge.ProfileRotation));
            data.Add(new MgdDbg.Snoop.Data.Bool("Scale profile X", nestedGridEdge.ScaleProfileX));
            data.Add(new MgdDbg.Snoop.Data.Bool("Scale profile Y", nestedGridEdge.ScaleProfileY));
        }

        private void
        Stream(ArrayList data, NestedGridDivision nestedGridDivision)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(NestedGridDivision)));

            data.Add(new MgdDbg.Snoop.Data.String("Name", nestedGridDivision.Name));
            data.Add(new MgdDbg.Snoop.Data.Int("ID", nestedGridDivision.Id));
            data.Add(new MgdDbg.Snoop.Data.String("Division direction", nestedGridDivision.DivisionDirection.ToString()));
            data.Add(new MgdDbg.Snoop.Data.Distance("Start offset", nestedGridDivision.StartOffset));
            data.Add(new MgdDbg.Snoop.Data.Distance("End offset", nestedGridDivision.EndOffset));

            NestedGridDivisionRepeat ngDivRepeat = nestedGridDivision as NestedGridDivisionRepeat;
            if (ngDivRepeat != null) {
                Stream(data, ngDivRepeat);
                return;
            }

            NestedGridDivisionManual ngDivManual = nestedGridDivision as NestedGridDivisionManual;
            if (ngDivManual != null) {
                Stream(data, ngDivManual);
                return;
            }

            NestedGridDivisionDivideBy ngDivideBy = nestedGridDivision as NestedGridDivisionDivideBy;
            if (ngDivideBy != null) {
                Stream(data, ngDivideBy);
                return;
            }
        }

        private void
        Stream(ArrayList data, NestedGridDivisionRepeat ngDivRepeat)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(NestedGridDivisionRepeat)));

            data.Add(new MgdDbg.Snoop.Data.Distance("Cell size", ngDivRepeat.CellSize));
            data.Add(new MgdDbg.Snoop.Data.Bool("Distribute slack", ngDivRepeat.DistributeSlack));
            data.Add(new MgdDbg.Snoop.Data.String("Distribution flags", ngDivRepeat.DistributionFlags.ToString()));
        }

        private void
        Stream(ArrayList data, NestedGridDivisionManual ngDivManual)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(NestedGridDivisionManual)));

            data.Add(new MgdDbg.Snoop.Data.Int("Manual edge definition count", ngDivManual.ManualEdgeDefinitionCount));
            ArrayList edgeDefs = new ArrayList();
            for (int i=0; i<ngDivManual.ManualEdgeDefinitionCount; i++) {
                edgeDefs.Add(ngDivManual.ManualEdgeDefinitionAt(i));
            }
            data.Add(new MgdDbg.Snoop.Data.ObjectCollection("Manual edge definitions", edgeDefs));
        }

        private void
        Stream(ArrayList data, NestedGridDivisionDivideBy ngDivideBy)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(NestedGridDivisionDivideBy)));

            data.Add(new MgdDbg.Snoop.Data.Int("Divide by count", ngDivideBy.DivideByCount));
        }

        private void
        Stream(ArrayList data, NestedGridInfillDefinition nestedGridInfill)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(NestedGridInfillDefinition)));

            data.Add(new MgdDbg.Snoop.Data.String("Name", nestedGridInfill.Name));
            data.Add(new MgdDbg.Snoop.Data.Int("Id", nestedGridInfill.Id));

            bool flipX, flipY, flipZ;
            nestedGridInfill.GetOrientation(out flipX, out flipY, out flipZ);

            data.Add(new MgdDbg.Snoop.Data.Bool("Flip X", flipX));
            data.Add(new MgdDbg.Snoop.Data.Bool("Flip Y", flipY));
            data.Add(new MgdDbg.Snoop.Data.Bool("Flip Z", flipZ));

            data.Add(new MgdDbg.Snoop.Data.Distance("Left offset", nestedGridInfill.LeftOffset));
            data.Add(new MgdDbg.Snoop.Data.Distance("Right offset", nestedGridInfill.RightOffset));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Material ID", nestedGridInfill.MaterialId));
            data.Add(new MgdDbg.Snoop.Data.Bool("Supports materials", nestedGridInfill.SupportsMaterials));
            data.Add(new MgdDbg.Snoop.Data.Bool("Supports mitre", nestedGridInfill.SupportsMitre ));
            data.Add(new MgdDbg.Snoop.Data.String("Y alignment", nestedGridInfill.YAlignment.ToString()));
            data.Add(new MgdDbg.Snoop.Data.Distance("Y offset", nestedGridInfill.YOffset));

            NestedGridInfillArch ngInfArch = nestedGridInfill as NestedGridInfillArch;
            if (ngInfArch != null) 
            {
                Stream(data, ngInfArch);
                return;
            }
        }


        private void
        Stream(ArrayList data, NestedGridCellAssignment ngCellAssignment)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(NestedGridCellAssignment)));

            data.Add(new MgdDbg.Snoop.Data.Object("Cell specifier", ngCellAssignment.CellSpecifier));
            data.Add(new MgdDbg.Snoop.Data.Int("ID", ngCellAssignment.Id));
            data.Add(new MgdDbg.Snoop.Data.Int("Infill definition ID", ngCellAssignment.InfillDefinitionId));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is void", ngCellAssignment.IsVoid));
            data.Add(new MgdDbg.Snoop.Data.String("Name", ngCellAssignment.Name));
            data.Add(new MgdDbg.Snoop.Data.Object("Nested grid definition", ngCellAssignment.NestedGridDefinition));
            data.Add(new MgdDbg.Snoop.Data.Object("Parent grid", ngCellAssignment.ParentGrid));
        }

        private void
        Stream(ArrayList data, NestedGridEdgeAssignment ngEdgeAssignment)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(NestedGridEdgeAssignment)));

            data.Add(new MgdDbg.Snoop.Data.String("Name", ngEdgeAssignment.Name));
            data.Add(new MgdDbg.Snoop.Data.Int("ID", ngEdgeAssignment.Id));
            data.Add(new MgdDbg.Snoop.Data.Int("Edge definition ID", ngEdgeAssignment.EdgeDefinitionId));
            data.Add(new MgdDbg.Snoop.Data.Object("Edge specifier", ngEdgeAssignment.EdgeSpecifier ));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is void", ngEdgeAssignment.IsVoid));
            data.Add(new MgdDbg.Snoop.Data.Object("Parent grid", ngEdgeAssignment.ParentGrid));
        }

        private void
        Stream(ArrayList data, NestedGridDivisionManualEdgeDefinition ngManEdgeDef)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(NestedGridDivisionManualEdgeDefinition)));

            data.Add(new MgdDbg.Snoop.Data.String("Axis reference", ngManEdgeDef.AxisReference.ToString())); 
            data.Add(new MgdDbg.Snoop.Data.Distance("Offset", ngManEdgeDef.Offset)); 
        }

        private void
        Stream(ArrayList data, NestedGridDefinition ngDef)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(NestedGridDefinition)));

            data.Add(new MgdDbg.Snoop.Data.Int("Grid division ID", ngDef.GridDivisionId ));
        } 
                
        private void
        Stream(ArrayList data, NestedGridInfillArch ngInfArch)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(NestedGridInfillArch)));

            data.Add(new MgdDbg.Snoop.Data.ObjectId("Infill style ID", ngInfArch.InfillStyleId));
            data.Add(new MgdDbg.Snoop.Data.Distance("Y depth", ngInfArch.YDepth));
        }

        
        private void
        Stream(ArrayList data, GridCustomCell gridCustomCell)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(GridCustomCell)));

            data.Add(new MgdDbg.Snoop.Data.Object("Calculate cell profile", gridCustomCell.CalculateCellProfile ));
            data.Add(new MgdDbg.Snoop.Data.Int("Extract node count ", gridCustomCell.ExtractNodeCount ));
            
            ArrayList nodeInfoColl = new ArrayList();
            foreach(NodeInformation nodeInfo in gridCustomCell.NodeInformationCollection)
            {
                nodeInfoColl.Add(nodeInfo);
            }
            
            data.Add(new MgdDbg.Snoop.Data.ObjectCollection("Nodes", nodeInfoColl));
        }

        private void
        Stream(ArrayList data, LayoutVolume layoutVol)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(LayoutVolume)));

            data.Add(new MgdDbg.Snoop.Data.Int("Volume ID", layoutVol.VolumeId));
            
            LayoutGrid3dVolume vol3d = layoutVol as LayoutGrid3dVolume;
            if (vol3d != null) {
                Stream(data, vol3d);
                return;
            }
        }
        
        private void
        Stream(ArrayList data, LayoutGrid3dVolume vol3d)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(LayoutGrid3dVolume)));

            data.Add(new MgdDbg.Snoop.Data.Int("U index", vol3d.UIndex));
            data.Add(new MgdDbg.Snoop.Data.Int("V index", vol3d.VIndex));
            data.Add(new MgdDbg.Snoop.Data.Int("W index", vol3d.WIndex));
        }
        
        private void
        Stream(ArrayList data, LayoutNode layoutNode)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(LayoutNode)));

            data.Add(new MgdDbg.Snoop.Data.Int("Node ID", layoutNode.NodeId));
            
            LayoutCurveNode curveNode = layoutNode as LayoutCurveNode;
            if (curveNode != null) {
                Stream(data, curveNode);
                return;
            }

            GridCustomNode gCustomNode = layoutNode as GridCustomNode;
            if (gCustomNode != null) 
            {
                Stream(data, gCustomNode);
                return;
            }
        }
                
        private void
        Stream(ArrayList data, GridAssemblyDefinition gridAssemblyDef)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(GridAssemblyDefinition)));

            data.Add(new MgdDbg.Snoop.Data.String("Name", gridAssemblyDef.Name));
            data.Add(new MgdDbg.Snoop.Data.Object("Default boundary edge assignment", gridAssemblyDef.SpecificBoundaryEdgeAssignments.Default));
            data.Add(new MgdDbg.Snoop.Data.Object("Default cell assignment", gridAssemblyDef.SpecificCellAssignments.Default));
            data.Add(new MgdDbg.Snoop.Data.Object("Default internal edge assignment", gridAssemblyDef.SpecificInteriorEdgeAssignments.Default));

            data.Add(new MgdDbg.Snoop.Data.Enumerable("Specific boundary edge assignments", gridAssemblyDef.SpecificBoundaryEdgeAssignments));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Specific cell assignments", gridAssemblyDef.SpecificCellAssignments));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Specific internal edge assignments", gridAssemblyDef.SpecificInteriorEdgeAssignments));

            short id = 0;
            data.Add(new MgdDbg.Snoop.Data.Int("Max cell assignment ID", gridAssemblyDef.MaxCellAssignmentId(id) ));
            id = 0;  // reset it since we will look for the max edge now.
            data.Add(new MgdDbg.Snoop.Data.Int("Max edge assignment ID", gridAssemblyDef.MaxEdgeAssignmentId(id) ));

            NestedGridDefinition ngDef = gridAssemblyDef as NestedGridDefinition;
            if (ngDef != null) {
                Stream(data, ngDef);
                return;
            }

            GridCustom  customGrid = gridAssemblyDef as GridCustom;
            if (customGrid != null) {
                Stream(data, customGrid);
                return;
            }

            GridAssemblyCustomDefinition customAssemblyGrid = gridAssemblyDef as GridAssemblyCustomDefinition;
            if (customAssemblyGrid != null) {
                Stream(data, customAssemblyGrid);
                return;
            }
        }
                
        private void
        Stream(ArrayList data, AnchorToCurveX anchorData)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AnchorToCurveX)));

            data.Add(new MgdDbg.Snoop.Data.Distance("Offset distance", anchorData.OffsetDistance));
            data.Add(new MgdDbg.Snoop.Data.String("Offset type", anchorData.OffsetType.ToString()));
            data.Add(new MgdDbg.Snoop.Data.String("Measure to type", anchorData.MeasureToType.ToString()));
        }
        
        private void
        Stream(ArrayList data, AnchorToCurveY anchorData)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AnchorToCurveY)));

            data.Add(new MgdDbg.Snoop.Data.Distance("Offset distance", anchorData.OffsetDistance));
            data.Add(new MgdDbg.Snoop.Data.String("Offset type", anchorData.OffsetType.ToString()));
            data.Add(new MgdDbg.Snoop.Data.String("Measure to type", anchorData.MeasureToType.ToString()));
        }

        private void
        Stream(ArrayList data, AnchorToCurveZ anchorData)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AnchorToCurveZ)));

            data.Add(new MgdDbg.Snoop.Data.Distance("Offset distance", anchorData.OffsetDistance));
            data.Add(new MgdDbg.Snoop.Data.String("Offset from type", anchorData.OffsetFromType.ToString()));
            data.Add(new MgdDbg.Snoop.Data.String("Offset to type", anchorData.OffsetToType.ToString()));
        }

        private void
        Stream(ArrayList data, GridCustomSegment gcSegment)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(GridCustomSegment)));

            data.Add(new MgdDbg.Snoop.Data.Int("ID", gcSegment.Id));
        }

        private void
        Stream(ArrayList data, CompoundCurve2d compCurve2d)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(CompoundCurve2d)));
            
            data.Add(new MgdDbg.Snoop.Data.Bool("Dirty flag", compCurve2d.DirtyFlag));
            data.Add(new MgdDbg.Snoop.Data.Point2d("Start point", compCurve2d.StartPoint));
            data.Add(new MgdDbg.Snoop.Data.Point2d("End point", compCurve2d.EndPoint));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is closed", compCurve2d.IsClosed));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is continuous", compCurve2d.IsContinuous));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is self intersecting", compCurve2d.IsSelfIntersecting));
            data.Add(new MgdDbg.Snoop.Data.Double("Length", compCurve2d.Length));

            data.Add(new MgdDbg.Snoop.Data.Int("Segment count", compCurve2d.SegmentCount));
            ArrayList segs = new ArrayList();
            for (int i=0; i<compCurve2d.SegmentCount; i++) {
                segs.Add(compCurve2d.Segment(i));
            }
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Segments", segs));

            data.Add(new MgdDbg.Snoop.Data.Enumerable("Vertices", compCurve2d.Vertices));

            Polyline2d pline2d = compCurve2d as Polyline2d;
            if (pline2d != null) {
                Stream(data, pline2d);
                return;
            }

            Ring ring = compCurve2d as Ring;
            if (ring != null) {
                Stream(data, ring);
                return;
            }
        }
                
        private void
        Stream(ArrayList data, Polyline2d pline2d)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(Polyline2d)));

            // no data at this level
        }

        private void
        Stream(ArrayList data, Ring ring)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(Ring)));

            data.Add(new MgdDbg.Snoop.Data.Double("Area", ring.Area));
            data.Add(new MgdDbg.Snoop.Data.Point2d("Centroid", ring.Centroid));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is circle", ring.IsCircle));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is valid ring", ring.IsValidRing));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is void", ring.IsVoid));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Segments", ring.Segments));
        }

        private void
        Stream(ArrayList data, ImpTree impTree)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(ImpTree)));

            data.Add(new MgdDbg.Snoop.Data.Bool("Allow children", impTree.AllowChildren));
            data.Add(new MgdDbg.Snoop.Data.Bool("Allow null objects", impTree.AllowNullObjects));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is root", impTree.IsRoot));
            data.Add(new MgdDbg.Snoop.Data.Object("Parent tree", impTree.ParentTree));
            data.Add(new MgdDbg.Snoop.Data.Object("Root", impTree.Root));
#if !ADT2007
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Children", impTree.Children));
#endif
            data.Add(new MgdDbg.Snoop.Data.Bool("Sub objects might have references", impTree.SubObjectsMightHaveReferences));

            ClassificationTree classTree = impTree as ClassificationTree;
            if (classTree != null) {
                Stream(data, classTree);
                return;
            }
        }

        private void
        Stream(ArrayList data, ClassificationTree classTree)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(ClassificationTree)));

            data.Add(new MgdDbg.Snoop.Data.ObjectId("ID", classTree.Id));
            data.Add(new MgdDbg.Snoop.Data.Object("Parent tree", classTree.ParentTree));
        }

        private void
        Stream(ArrayList data, ClassificationCollection classColl)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(ClassificationCollection)));

            data.Add(new MgdDbg.Snoop.Data.Int("Count", classColl.Count));

            try {
                data.Add(new MgdDbg.Snoop.Data.Bool("Allow duplicate definitions", classColl.AllowDuplicateDefinitions));
            }
            catch (Autodesk.AutoCAD.Runtime.Exception ex) {
                data.Add(new MgdDbg.Snoop.Data.Exception("Allow duplicate definitions", ex));
            } 
        }

        private void
        Stream(ArrayList data, LayerKeyDefinition lKeyDef)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(LayerKeyDefinition)));

            data.Add(new MgdDbg.Snoop.Data.Object("Color", lKeyDef.Color));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is plottable", lKeyDef.IsPlottable));
            data.Add(new MgdDbg.Snoop.Data.String("Key", lKeyDef.Key));
            data.Add(new MgdDbg.Snoop.Data.String("Layer name", lKeyDef.LayerName));
            data.Add(new MgdDbg.Snoop.Data.String("Line type", lKeyDef.Linetype));
            data.Add(new MgdDbg.Snoop.Data.String("Line weight", lKeyDef.Lineweight.ToString()));

#if ADT2007
            data.Add(new MgdDbg.Snoop.Data.Int("Override count", lKeyDef.OverrideCount));
            StringCollection strs = new StringCollection();
            for (int i=0; i<lKeyDef.OverrideCount; i++) {
                strs.Add(lKeyDef.GetOverrideAt(i));
            }
            data.Add(new MgdDbg.Snoop.Data.Object("Overrides", strs));
#else
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Overrides", lKeyDef.Overrides));
#endif

            data.Add(new MgdDbg.Snoop.Data.String("Plotstyle", lKeyDef.Plotstyle));
            data.Add(new MgdDbg.Snoop.Data.Bool("Removable", lKeyDef.Removable));                    
        }

        private void
        Stream(ArrayList data, ConnectionComponent connComp)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(ConnectionComponent)));

            data.Add(new MgdDbg.Snoop.Data.Object("Bound box", connComp.BoundBox));
            data.Add(new MgdDbg.Snoop.Data.Int("Max simultaneous connections", connComp.MaxSimultaneousConnections));
        }

        private void
        Stream(ArrayList data, Section2dHatchRegion sec2dHatchReg)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(Section2dHatchRegion)));

            data.Add(new MgdDbg.Snoop.Data.Bool("Applies to material line work", sec2dHatchReg.AppliesToMaterialLineWork));
            data.Add(new MgdDbg.Snoop.Data.Bool("Applies to section hatch", sec2dHatchReg.AppliesToSectionHatch));
            data.Add(new MgdDbg.Snoop.Data.Bool("Applies to shrink wrap hatch", sec2dHatchReg.AppliesToShrinkWrapHatch));
            data.Add(new MgdDbg.Snoop.Data.Bool("Applies to shrink wrap line work", sec2dHatchReg.AppliesToShrinkWrapLineWork));
            data.Add(new MgdDbg.Snoop.Data.Bool("Applies to surface hatch", sec2dHatchReg.AppliesToSurfaceHatch));
            data.Add(new MgdDbg.Snoop.Data.Bool("Has material filter", sec2dHatchReg.HasMaterialFilter));
            data.Add(new MgdDbg.Snoop.Data.Object("Hatch profile", sec2dHatchReg.HatchProfile));
            data.Add(new MgdDbg.Snoop.Data.Int("ID", sec2dHatchReg.Id));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is subtractive", sec2dHatchReg.IsSubtractive));
            data.Add(new MgdDbg.Snoop.Data.ObjectIdCollection("Material filter", sec2dHatchReg.MaterialFilter));
        }

        private void
        Stream(ArrayList data, SectionSegment sectionSeg)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(SectionSegment)));

            data.Add(new MgdDbg.Snoop.Data.Int("Component ID", sectionSeg.ComponentId));
            data.Add(new MgdDbg.Snoop.Data.Point2d("Start point", sectionSeg.StartPoint));
            data.Add(new MgdDbg.Snoop.Data.Point2d("End point", sectionSeg.EndPoint));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is a point", sectionSeg.IsAPoint));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is merged segment", sectionSeg.IsMergedSegment));
            data.Add(new MgdDbg.Snoop.Data.String("Line work type", sectionSeg.LineWorkType.ToString()));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is merged segment", sectionSeg.IsMergedSegment));
            data.Add(new MgdDbg.Snoop.Data.Bool("Marked from edit", sectionSeg.MarkedFromEdit));
        }

        private void
        Stream(ArrayList data, SectionVertex secVertex)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(SectionVertex)));

            data.Add(new MgdDbg.Snoop.Data.Point2d("Location", secVertex.Location));            
        }

        private void
        Stream(ArrayList data, Section2dSegmentGroup sec2dSegGroup)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(Section2dSegmentGroup)));

            data.Add(new MgdDbg.Snoop.Data.Bool("Has user edits", sec2dSegGroup.HasUserEdits));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Material ID", sec2dSegGroup.MaterialId));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Line work segments", sec2dSegGroup.LineWorkSegments));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Section hatching segments", sec2dSegGroup.SectionHatchingSegments));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Surface hatching segments", sec2dSegGroup.SurfaceHatchingSegments));
        }

        private void
        Stream(ArrayList data, IndexSpecifier indSpec)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(IndexSpecifier)));

            IndexSpecifierBoundaryEdge indSpecBoundEdge = indSpec as IndexSpecifierBoundaryEdge;
            if (indSpecBoundEdge != null) {
                Stream(data, indSpecBoundEdge);
                return;
            }

            IndexSpecifierLocation indSpecLocation = indSpec as IndexSpecifierLocation;
            if (indSpecLocation != null) {
                Stream(data, indSpecLocation);
                return;
            }
        }

        private void
        Stream(ArrayList data, IndexSpecifierBoundaryEdge indSpecBoundEdge)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(IndexSpecifierBoundaryEdge)));

            data.Add(new MgdDbg.Snoop.Data.Bool("On bottom", indSpecBoundEdge.OnBottom));
            data.Add(new MgdDbg.Snoop.Data.Bool("On left", indSpecBoundEdge.OnLeft));
            data.Add(new MgdDbg.Snoop.Data.Bool("On right", indSpecBoundEdge.OnRight));
            data.Add(new MgdDbg.Snoop.Data.Bool("On top", indSpecBoundEdge.OnTop));
        }

        private void
        Stream(ArrayList data, IndexSpecifierLocation indSpecLocation)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(IndexSpecifierLocation)));

            data.Add(new MgdDbg.Snoop.Data.Bool("Applies to end index", indSpecLocation.AppliesToEndIndex));
            data.Add(new MgdDbg.Snoop.Data.Bool("Applies to middle index", indSpecLocation.AppliesToMiddleIndex));
            data.Add(new MgdDbg.Snoop.Data.Bool("Applies to start index", indSpecLocation.AppliesToStartIndex));
        }

        private void
        Stream(ArrayList data, GridAssemblyInterference gaInterference)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(GridAssemblyInterference)));

            data.Add(new MgdDbg.Snoop.Data.String("Applies to flags", gaInterference.AppliesToFlags.ToString() ));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Entity ID", gaInterference.EntityId));
        }

        private void
        Stream(ArrayList data, GridCustom customGrid)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(GridCustom)));

            data.Add(new MgdDbg.Snoop.Data.Int("Cell count", customGrid.CellCount ));

            if (customGrid.CellCount > 0) {
                IntegerCollection cellIds = customGrid.LayoutCellIds;
                ArrayList arrIds = new ArrayList();
                ArrayList arrCells = new ArrayList();
                ArrayList arrCellsById = new ArrayList();
                ArrayList arrCellProfs = new ArrayList();
                Matrix3d mat;
                Profile prof = null;
                for (int n=0; n<customGrid.CellCount; n++) {
                    arrIds.Add(cellIds[n]);
                    arrCells.Add(customGrid.CustomGridCellAt(n));
                    arrCellsById.Add(customGrid.CustomGridCell(cellIds[n]));
                    prof = customGrid.GetLayoutCellProfile(cellIds[n], out mat);
                    arrCellProfs.Add(prof);
                }
                data.Add(new MgdDbg.Snoop.Data.Object("Layout cell IDs", arrIds ));
                data.Add(new MgdDbg.Snoop.Data.ObjectCollection("Layout cells", arrCells ));
                data.Add(new MgdDbg.Snoop.Data.ObjectCollection("Layout cell profiles", arrCellProfs ));
                data.Add(new MgdDbg.Snoop.Data.ObjectCollection("Layout cell by IDs", arrCellsById ));
            }

            data.Add(new MgdDbg.Snoop.Data.Int("Node count", customGrid.NodeCount ));
            if (customGrid.NodeCount > 0) {
                IntegerCollection nodeIds = customGrid.LayoutNodeIds;
                ArrayList arrIds = new ArrayList();
                ArrayList arrNodes = new ArrayList();
                ArrayList arrNodesById = new ArrayList();
                ArrayList arrPts = new ArrayList();
                ArrayList arrCoordSyst = new ArrayList();

                for (int n=0; n < customGrid.NodeCount; n++) {
                    arrIds.Add(nodeIds[n]);
                    arrNodes.Add(customGrid.CustomGridNodeAt(n));
                    arrNodesById.Add(customGrid.CustomGridNode(nodeIds[n]));
                    arrPts.Add(customGrid.GetLayoutNodeLocation(nodeIds[n]));
                    arrCoordSyst.Add(customGrid.GetLayoutNodeCoordinateSystem(nodeIds[n]));
                }
                data.Add(new MgdDbg.Snoop.Data.Object("Layout node IDs", arrIds ));
                data.Add(new MgdDbg.Snoop.Data.ObjectCollection("Layout nodes", arrNodes ));
                data.Add(new MgdDbg.Snoop.Data.ObjectCollection("Layout node by IDs", arrNodesById ));
                data.Add(new MgdDbg.Snoop.Data.ObjectCollection("Layout node locations", arrPts ));
                data.Add(new MgdDbg.Snoop.Data.ObjectCollection("Layout node coordinate systems", arrCoordSyst ));
            }

#if !ACA2009
            data.Add(new MgdDbg.Snoop.Data.Int("Main segment count", customGrid.MainSegmentCount ));
            if (customGrid.MainSegmentCount > 0) {
                ArrayList arrSegs = new ArrayList();
                ArrayList arrSegsById = new ArrayList();
                GridCustomSegment seg = null;
                GridCustomSegment segById = null;
                for (int n=0; n < customGrid.MainSegmentCount; n++) {
                    seg = customGrid.MainSegmentAt(n);
                    // I'm cheating here - just want to see if the 2 segs are the same so I get the id of the one I found by index and try to find it by id.
                    segById = customGrid.MainSegment(seg.Id); 
                    arrSegs.Add(seg);
                    arrSegsById.Add(segById);
                }
                data.Add(new MgdDbg.Snoop.Data.ObjectCollection("Main segments by index", arrSegs ));
                data.Add(new MgdDbg.Snoop.Data.ObjectCollection("Main segments by ID", arrSegsById ));
            }
#endif

#if ACA2009
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Segments", customGrid.Segments));
            data.Add(new MgdDbg.Snoop.Data.Int("Main segment count", customGrid.Segments.Count ));
            if (customGrid.Segments.Count > 0) {
                ArrayList arrSegs = new ArrayList();
                ArrayList arrSegsById = new ArrayList();
                GridCustomSegment seg = null;
                GridCustomSegment segById = null;
                for (int n=0; n < customGrid.Segments.Count; n++) {
                    seg = customGrid.Segments[n];
                    // I'm cheating here - just want to see if the 2 segs are the same so I get the id of the one I found by index and try to find it by id.
                    segById = customGrid.Segments.FindById(seg.Id); 
                    arrSegs.Add(seg);
                    arrSegsById.Add(segById);
                }
                data.Add(new MgdDbg.Snoop.Data.ObjectCollection("Main segments by index", arrSegs ));
                data.Add(new MgdDbg.Snoop.Data.ObjectCollection("Main segments by ID", arrSegsById ));
            }
#endif

            data.Add(new MgdDbg.Snoop.Data.Bool("Persistent solution", customGrid.PersistentSolution ));
            data.Add(new MgdDbg.Snoop.Data.Bool("Requires update", customGrid.RequiresUpdate ));
            data.Add(new MgdDbg.Snoop.Data.Object("Transient exterior cell", customGrid.TransientExteriorCell ));
        }

        private void
        Stream(ArrayList data, GridAssemblyCustomDefinition assmDef)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(GridAssemblyCustomDefinition)));

            data.Add(new MgdDbg.Snoop.Data.Object("Custom grid definition", assmDef.CustomGridDefinition));
        }

        private void
        Stream(ArrayList data, LineWorkComponent lwComp)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(LineWorkComponent)));

            data.Add(new MgdDbg.Snoop.Data.Int("Id", lwComp.Id));
            data.Add(new MgdDbg.Snoop.Data.String("Name", lwComp.Name));

            try {
                data.Add(new MgdDbg.Snoop.Data.String("Description", lwComp.Description));
            }
            catch (Autodesk.AutoCAD.Runtime.Exception ex) {
                data.Add(new MgdDbg.Snoop.Data.Exception("Description", ex));
            }
        }

        private void
        Stream(ArrayList data, LineWorkRule lwRule)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(LineWorkRule)));

            data.Add(new MgdDbg.Snoop.Data.Object("Color", lwRule.Color));
            data.Add(new MgdDbg.Snoop.Data.Int("Component ID", lwRule.ComponentId));

            try {
                data.Add(new MgdDbg.Snoop.Data.String("Description", lwRule.Description));
            }
            catch (Autodesk.AutoCAD.Runtime.Exception ex) {
                data.Add(new MgdDbg.Snoop.Data.Exception("Description", ex));
            }

            data.Add(new MgdDbg.Snoop.Data.Int("Filter flags", lwRule.FilterFlags));
        }

        private void
        Stream(ArrayList data, Override overRide)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(Override)));

            data.Add(new MgdDbg.Snoop.Data.String("Extension dictionary name", Override.ExtensionDictionaryName));

            OverrideMask overMask = overRide as OverrideMask;
            if (overMask != null) {
                Stream(data, overMask);
                return;
            }
        }

        private void
        Stream(ArrayList data, OverrideMask overMask)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(OverrideMask)));

            data.Add(new MgdDbg.Snoop.Data.ObjectId("Mask ID", overMask.MaskId));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("View ID", overMask.ViewId));
        }

        private void
        Stream(ArrayList data, UnitType unitType)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(UnitType)));

            data.Add(new MgdDbg.Snoop.Data.String("Internal name", unitType.InternalName));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is imperial", unitType.IsImperial));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is metric", unitType.IsMetric));
            data.Add(new MgdDbg.Snoop.Data.String("Status", unitType.Status.ToString()));
            data.Add(new MgdDbg.Snoop.Data.String("Type", unitType.Type.ToString()));
        }

        private void
        Stream(ArrayList data, UnitVariable unitVar)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(UnitVariable)));

            data.Add(new MgdDbg.Snoop.Data.String("Status", unitVar.Status.ToString()));
            data.Add(new MgdDbg.Snoop.Data.String("Type", unitVar.Type.ToString()));
            data.Add(new MgdDbg.Snoop.Data.Double("Value", unitVar.Value));
        }

        private void
        Stream(ArrayList data, GraphicsStorage graphStorage)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(GraphicsStorage)));
            data.Add(new MgdDbg.Snoop.Data.Object("Color", graphStorage.Color));
            data.Add(new MgdDbg.Snoop.Data.Int("Color index", graphStorage.ColorIndex));
            data.Add(new MgdDbg.Snoop.Data.Int("Gs marker", graphStorage.GsMarker));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Layer ID", graphStorage.LayerId));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Line type ID", graphStorage.LinetypeId));
            data.Add(new MgdDbg.Snoop.Data.Double("Line type scale", graphStorage.LineTypeScale));
            data.Add(new MgdDbg.Snoop.Data.String("Line weight", graphStorage.LineWeight.ToString()));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Material ID", graphStorage.MaterialId));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Plot style name ID", graphStorage.PlotStyleNameId));
            data.Add(new MgdDbg.Snoop.Data.String("Plot style name type", graphStorage.PlotStyleNameType.ToString()));

            GraphicsStorageArc graphStorageArc = graphStorage as GraphicsStorageArc;
            if (graphStorageArc != null) {
                Stream(data, graphStorageArc);
                return;
            }

            GraphicsStorageBody graphStorageBody = graphStorage as GraphicsStorageBody;
            if (graphStorageBody != null) {
                Stream(data, graphStorageBody);
                return;
            }

            GraphicsStorageFace graphStorageFace = graphStorage as GraphicsStorageFace;
            if (graphStorageFace != null) {
                Stream(data, graphStorageFace);
                return;
            }

            GraphicsStorageHatch graphStorageHatch = graphStorage as GraphicsStorageHatch;
            if (graphStorageBody != null) {
                Stream(data, graphStorageBody);
                return;
            }

            GraphicsStoragePolyline graphStoragePolyline = graphStorage as GraphicsStoragePolyline;
            if (graphStoragePolyline != null) {
                Stream(data, graphStoragePolyline);
                return;
            }

            GraphicsStorageShell graphStorageShell = graphStorage as GraphicsStorageShell;
            if (graphStorageShell != null) {
                Stream(data, graphStorageShell);
                return;
            }    
        }

        private void
        Stream(ArrayList data, GraphicsStorageArc graphStorageArc)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(GraphicsStorageArc)));

            data.Add(new MgdDbg.Snoop.Data.Object("Arc", graphStorageArc.Arc));            
        }

        private void
        Stream(ArrayList data, GraphicsStorageBody graphStorageBody)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(GraphicsStorageBody)));

            data.Add(new MgdDbg.Snoop.Data.Object("Body", graphStorageBody.Body));
            data.Add(new MgdDbg.Snoop.Data.Int("Discarded edge face mask", graphStorageBody.DiscardedEdgeFaceMask));
        }

        private void
        Stream(ArrayList data, GraphicsStorageFace graphStorageFace)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(GraphicsStorageFace)));

            data.Add(new MgdDbg.Snoop.Data.Enumerable("Edge visibility", graphStorageFace.EdgeVisibility));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Points", graphStorageFace.Points));
        }

        private void
        Stream(ArrayList data, GraphicsStorageHatch graphStorageHatch)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(GraphicsStorageHatch)));

            data.Add(new MgdDbg.Snoop.Data.Enumerable("Hatch type", graphStorageHatch.HatchType.ToString()));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Vectors", graphStorageHatch.Vectors));
        }

        private void
        Stream(ArrayList data, GraphicsStoragePolyline graphStoragePolyline)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(GraphicsStoragePolyline)));

            data.Add(new MgdDbg.Snoop.Data.Enumerable("Points", graphStoragePolyline.Points));           
        }

        private void
        Stream(ArrayList data, GraphicsStorageShell graphStorageShell)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(GraphicsStorageShell)));

            data.Add(new MgdDbg.Snoop.Data.Object("Edge data", graphStorageShell.EdgeData));
            data.Add(new MgdDbg.Snoop.Data.Object("Face data", graphStorageShell.FaceData));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Face list", graphStorageShell.FaceList));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Vertices", graphStorageShell.Vertices));

        }

#if !ADT2007
        private void
        Stream(ArrayList data, DisplayComponent dispComp)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(DisplayComponent)));

            data.Add(new MgdDbg.Snoop.Data.Bool("Is applicable", dispComp.IsApplicable));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is inherited", dispComp.IsInherited));

            DisplayComponentEntity dispCompEnt = dispComp as DisplayComponentEntity;
            if (dispCompEnt != null) {
                Stream(data, dispCompEnt);
                return;
            }
        }

        private void
        Stream(ArrayList data, DisplayComponentEntity dispComp)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(DisplayComponentEntity)));

            data.Add(new MgdDbg.Snoop.Data.Bool("Allow ByMaterial", dispComp.AllowByMaterial));
            data.Add(new MgdDbg.Snoop.Data.Bool("ByMaterial", dispComp.ByMaterial));
            data.Add(new MgdDbg.Snoop.Data.Object("Color", dispComp.Color));
            data.Add(new MgdDbg.Snoop.Data.Int("Color index", dispComp.ColorIndex));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is visible", dispComp.IsVisible));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Layer ID", dispComp.LayerId));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Linetype ID", dispComp.LinetypeId));
            data.Add(new MgdDbg.Snoop.Data.Double("Linetype scale", dispComp.LinetypeScale));
            data.Add(new MgdDbg.Snoop.Data.String("Line weight", dispComp.LineWeight.ToString()));

            data.Add(new MgdDbg.Snoop.Data.String("Plotstyle name type", dispComp.GetPlotStyleNameType().ToString()));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Plotstyle ID", dispComp.GetPlotStyleNameId()));

            DisplayComponentHatch dispCompHatch = dispComp as DisplayComponentHatch;
            if (dispCompHatch != null) {
                Stream(data, dispCompHatch);
                return;
            }
        }

        private void
        Stream(ArrayList data, DisplayComponentHatch dispComp)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(DisplayComponentHatch)));

            data.Add(new MgdDbg.Snoop.Data.String("Hatch type", dispComp.HatchType.ToString()));
            data.Add(new MgdDbg.Snoop.Data.String("Pattern name", dispComp.PatternName));
            data.Add(new MgdDbg.Snoop.Data.Angle("Angle", dispComp.Angle));
            data.Add(new MgdDbg.Snoop.Data.Double("Scale", dispComp.Scale));
            data.Add(new MgdDbg.Snoop.Data.Double("Spacing", dispComp.Spacing));
            data.Add(new MgdDbg.Snoop.Data.Double("U offset", dispComp.UOffset));
            data.Add(new MgdDbg.Snoop.Data.Double("V offset", dispComp.VOffset));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is double hatch", dispComp.IsDoubleHatch));
            data.Add(new MgdDbg.Snoop.Data.Bool("Use angle of object", dispComp.UseAngleOfObject));
        }
#endif
    }
}
