// $Header: //Desktop/Source/TestTools/MgdDbgAec/Snoop/CollectorExts/ImpObjectStructure.cs#1 $
// $Author: fergusja $ $DateTime: 2008/03/04 16:12:35 $ $Change: 94604 $
//
// (C) Copyright 2008 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//

using System;
using System.Diagnostics;
using System.Collections;
using System.Windows.Forms;

using MgdDbg.Snoop;

using Autodesk.Aec.Modeler;
using Autodesk.Aec.Geometry;
using Autodesk.AutoCAD.Geometry;
using Autodesk.Aec.DatabaseServices;
using Autodesk.Aec.Structural.DatabaseServices;


namespace MgdDbgAec.Snoop.CollectorExts
{
	/// <summary>
	/// This is a Snoop Collector Extension object to collect data from AEC ImpObjs.
	/// </summary>
	public class ImpObjectStructure : MgdDbg.Snoop.CollectorExts.CollectorExt
	{
		public ImpObjectStructure()
		{
		}
		
        protected override void
        CollectEvent(object sender, MgdDbg.Snoop.Collectors.CollectorEventArgs e)
        {
                // bail early if this isn't an object we are responsible for
            ImpObject impObj = e.ObjToSnoop as ImpObject;
            if (impObj == null)
                return;

                // cast the sender object to the SnoopCollector we are expecting
            MgdDbg.Snoop.Collectors.Collector snoopCollector = sender as MgdDbg.Snoop.Collectors.Collector;
            if (snoopCollector == null) {
                Debug.Assert(false);    // why did someone else send us the message?
                return;
            }

            MemberBodyModifier memberBodyMod = impObj as MemberBodyModifier;
            if (memberBodyMod != null) {
                Stream(snoopCollector.Data(), memberBodyMod);
                return;
            }

            MemberComponent memberComp = impObj as MemberComponent;
            if (memberComp != null) {
                Stream(snoopCollector.Data(), memberComp);
                return;
            }

            MemberInterference memberInterf = impObj as MemberInterference;
            if (memberInterf != null) {
                Stream(snoopCollector.Data(), memberInterf);
                return;
            }

            MemberNodeComponent memberNodeComp = impObj as MemberNodeComponent;
            if (memberNodeComp != null) {
                Stream(snoopCollector.Data(), memberNodeComp);
                return;
            }

            MemberNodeId memberNodeId = impObj as MemberNodeId;
            if (memberNodeId != null) {
                Stream(snoopCollector.Data(), memberNodeId);
                return;
            }
        }
        
        private void
        Stream(ArrayList data, MemberBodyModifier bodyMod)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(MemberBodyModifier)));

            data.Add(new MgdDbg.Snoop.Data.Int("Component ID", bodyMod.ComponentId));
            data.Add(new MgdDbg.Snoop.Data.String("Operation type", bodyMod.OperationType.ToString()));
        }
        
        private void
        Stream(ArrayList data, MemberComponent comp)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(MemberComponent)));

            data.Add(new MgdDbg.Snoop.Data.String("Name", comp.Name));
            data.Add(new MgdDbg.Snoop.Data.Int("ID", comp.Id));
            data.Add(new MgdDbg.Snoop.Data.Object("Start node", comp.StartNode));
            data.Add(new MgdDbg.Snoop.Data.Object("End node", comp.EndNode));
            data.Add(new MgdDbg.Snoop.Data.Distance("Start offset", comp.StartOffset));
            data.Add(new MgdDbg.Snoop.Data.Distance("End offset", comp.EndOffset));
            data.Add(new MgdDbg.Snoop.Data.Int("Priority", comp.Priority));
        }
        
        private void
        Stream(ArrayList data, MemberInterference memberInterf)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(MemberInterference)));

            data.Add(new MgdDbg.Snoop.Data.ObjectId("Entity ID", memberInterf.EntityId));
            data.Add(new MgdDbg.Snoop.Data.Bool("Block Reference Broken", memberInterf.BlockReferenceBroken));
        }

        private void
        Stream(ArrayList data, MemberNodeComponent memberNodeComp)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(MemberNodeComponent)));

            data.Add(new MgdDbg.Snoop.Data.Bool("Mirror", memberNodeComp.Mirror));
            data.Add(new MgdDbg.Snoop.Data.Object("Reference Node Id", memberNodeComp.ReferenceNodeId));
            data.Add(new MgdDbg.Snoop.Data.Double("Rotation", memberNodeComp.Rotation));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Shape ID", memberNodeComp.ShapeId));
        }

        private void
        Stream(ArrayList data, MemberNodeId memberNodeId)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(MemberNodeId)));

            data.Add(new MgdDbg.Snoop.Data.Int("Index", memberNodeId.Index));
            data.Add(new MgdDbg.Snoop.Data.String("RelativeEnd", memberNodeId.RelativeEnd.ToString()));
        }
    }
}
