
// $Header: //Desktop/Source/TestTools/MgdDbgAec/Snoop/CollectorExts/MiscArch.cs#3 $
// $Author: ayyara $ $DateTime: 2008/03/07 17:52:55 $ $Change: 94950 $
//
// (C) Copyright 2004 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//

using System;
using System.Text;
using System.Collections;
using System.Diagnostics;
using System.Collections.Generic;

using Autodesk.Aec.Arch.DatabaseServices;
using Autodesk.Aec.Arch.DatabaseServices.WallSystem;

using MgdDbg.Snoop;


namespace MgdDbgAec.Snoop.CollectorExts {
    class MiscArch : MgdDbg.Snoop.CollectorExts.CollectorExt {

		public MiscArch()
		{
		}
		
        protected override void
        CollectEvent(object sender, MgdDbg.Snoop.Collectors.CollectorEventArgs e)
        {
                // cast the sender object to the SnoopCollector we are expecting
            MgdDbg.Snoop.Collectors.Collector snoopCollector = sender as MgdDbg.Snoop.Collectors.Collector;
            if (snoopCollector == null) {
                Debug.Assert(false);    // why did someone else send us the message?
                return;
            }
            
            OpenShape openShape = e.ObjToSnoop as OpenShape;
            if (openShape != null) {
                Stream(snoopCollector.Data(), openShape);
                return;
            }

            StairTread stairTread = e.ObjToSnoop as StairTread;
            if (stairTread != null) {
                Stream(snoopCollector.Data(), stairTread);
                return;
            }

            Connection conn = e.ObjToSnoop as Connection;
            if (conn != null) {
                Stream(snoopCollector.Data(), conn);
                return;
            }

            Graph graph = e.ObjToSnoop as Graph;
            if (graph != null) {
                Stream(snoopCollector.Data(), graph);
                return;
            }

            Joint joint = e.ObjToSnoop as Joint;
            if (joint != null) {
                Stream(snoopCollector.Data(), joint);
                return;
            }

            Section section = e.ObjToSnoop as Section;
            if (section != null) {
                Stream(snoopCollector.Data(), section);
                return;
            }

            SectionComponent secComp = e.ObjToSnoop as SectionComponent;
            if (secComp != null) {
                Stream(snoopCollector.Data(), secComp);
                return;
            }

            SectionComponentSet secCompSet = e.ObjToSnoop as SectionComponentSet;
            if (secCompSet != null){
                Stream(snoopCollector.Data(), secCompSet);
                return;
            }            
        }

        private void
        Stream(ArrayList data, OpenShape openShape)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(OpenShape)));

            data.Add(new MgdDbg.Snoop.Data.Distance("Frame width", openShape.FrameWidth));
            data.Add(new MgdDbg.Snoop.Data.Distance("Frame depth", openShape.FrameDepth));
            data.Add(new MgdDbg.Snoop.Data.Distance("Width", openShape.Width));
            data.Add(new MgdDbg.Snoop.Data.Distance("Height", openShape.Height));
            data.Add(new MgdDbg.Snoop.Data.Bool("Has rise", openShape.HasRise));
            data.Add(new MgdDbg.Snoop.Data.Distance("Rise", openShape.Rise));
            data.Add(new MgdDbg.Snoop.Data.Bool("Shape is valid", openShape.ShapeIsValid));
            data.Add(new MgdDbg.Snoop.Data.Bool("Trim bottom", openShape.TrimBottom));

            // TBD: get Body funcs
        }

        private void
        Stream(ArrayList data, StairTread tread)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(StairTread)));

            data.Add(new MgdDbg.Snoop.Data.Angle("Angle", tread.Angle));
            data.Add(new MgdDbg.Snoop.Data.Vector3d("Direction", tread.Direction));
            data.Add(new MgdDbg.Snoop.Data.Point3d("Location", tread.Location));
            data.Add(new MgdDbg.Snoop.Data.Object("Stair component", tread.StairComponent));
            data.Add(new MgdDbg.Snoop.Data.Int("Tread index", tread.TreadIndex));
            data.Add(new MgdDbg.Snoop.Data.Vector3d("Walkline direction", tread.WalklineDirection));
        }

        private void
        Stream(ArrayList data, Connection conn)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(Connection)));

            data.Add(new MgdDbg.Snoop.Data.Vector2d("Direction from here", conn.DirectionFromHere));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Elevation variations", conn.ElevationVariations));
            data.Add(new MgdDbg.Snoop.Data.Double("End elevation", conn.EndElevation));
            data.Add(new MgdDbg.Snoop.Data.Object("Joint", conn.Joint));
            data.Add(new MgdDbg.Snoop.Data.Object("Section", conn.Section));
            data.Add(new MgdDbg.Snoop.Data.Bool("Section starts here", conn.SectionStartsHere));
            data.Add(new MgdDbg.Snoop.Data.Double("Start elevation", conn.StartElevation));           
        }

        private void
        Stream(ArrayList data, Graph graph)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(Graph)));

            data.Add(new MgdDbg.Snoop.Data.ObjectId("Block ID", graph.BlockId));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Cleanup group definition ID", graph.CleanupGroupDefinitionId));
            data.Add(new MgdDbg.Snoop.Data.Object("Ecs to Wcs matrix", graph.EcsToWcsMatrix));
            data.Add(new MgdDbg.Snoop.Data.Object("Plane", graph.Plane));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Top level insert id", graph.TopLevelInsertId));
            data.Add(new MgdDbg.Snoop.Data.Int("Wall joint count", graph.WallJointCount));
            
            ArrayList wallJoints = new ArrayList();
            int len = graph.WallJointCount;
            for (int i = 0; i < len; i++) {
                wallJoints.Add(graph.GetWallJoint(i));
            }

            data.Add(new MgdDbg.Snoop.Data.ObjectCollection("Wall joints", wallJoints));

            data.Add(new MgdDbg.Snoop.Data.Int("Wall section count", graph.WallSectionCount));

            ArrayList wallSections = new ArrayList();
            len = graph.WallSectionCount;
            for (int i = 0; i < len; i++) {
                wallSections.Add(graph.GetWallSection(i));
            }

            data.Add(new MgdDbg.Snoop.Data.ObjectCollection("Wall sections", wallSections));

            data.Add(new MgdDbg.Snoop.Data.Object("Wcs to Ecs matrix", graph.WcsToEcsMatrix));
        }

        private void
        Stream(ArrayList data, Joint joint)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(Joint)));

            data.Add(new MgdDbg.Snoop.Data.Object("Bounding box", joint.BoundingBox));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Connections", joint.Connections));
            data.Add(new MgdDbg.Snoop.Data.Point2d("Location", joint.Location));
            data.Add(new MgdDbg.Snoop.Data.Distance("Radius", joint.Radius));
        }

        private void
        Stream(ArrayList data, Section section)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(Section)));

            data.Add(new MgdDbg.Snoop.Data.Object("Bounding box", section.BoundingBox));
            data.Add(new MgdDbg.Snoop.Data.Object("Center curve", section.CenterCurve));
            data.Add(new MgdDbg.Snoop.Data.Int("Component set count", section.ComponentSetCount));

            ArrayList compSet = new ArrayList();
            int len = section.ComponentSetCount;
            for (int i = 0; i < len; i++) {
                compSet.Add(section.GetComponentSet(i));
            }

            data.Add(new MgdDbg.Snoop.Data.ObjectCollection("Component set", compSet));

            data.Add(new MgdDbg.Snoop.Data.Object("End connection", section.EndConnection));
            data.Add(new MgdDbg.Snoop.Data.Bool("End of main wall", section.EndOfMainWall));
            data.Add(new MgdDbg.Snoop.Data.Point2d("End point", section.EndPoint));
            data.Add(new MgdDbg.Snoop.Data.Bool("Has priority overrides", section.HasPriorityOverrides));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is insert", section.IsInsert));
            data.Add(new MgdDbg.Snoop.Data.Double("Left face offset", section.LeftFaceOffset));
            data.Add(new MgdDbg.Snoop.Data.Object("Opening path", section.OpeningPath));
            data.Add(new MgdDbg.Snoop.Data.Bool("Parent curve is closed", section.ParentCurveIsClosed));
            data.Add(new MgdDbg.Snoop.Data.Double("Right face offset", section.RightFaceOffset));
            data.Add(new MgdDbg.Snoop.Data.Object("Start connection", section.StartConnection));
            data.Add(new MgdDbg.Snoop.Data.Bool("Start of main wall", section.StartOfMainWall));
            data.Add(new MgdDbg.Snoop.Data.Point2d("Start point", section.StartPoint));
            data.Add(new MgdDbg.Snoop.Data.Double("Wall end cleanup radius", section.WallEndCleanupRadius));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Wall ID", section.WallId));
            data.Add(new MgdDbg.Snoop.Data.Double("Wall start cleanup radius", section.WallStartCleanupRadius));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Wall style ID", section.WallStyleId));
        }

        private void
        Stream(ArrayList data, SectionComponent secComp)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(SectionComponent)));

            //data.Add(new MgdDbg.Snoop.Data.Int("Component index", secComp.ComponentIndex));  // MH: 
            data.Add(new MgdDbg.Snoop.Data.Int("Component index", secComp.WallComponentIndex)); // MH: 
            data.Add(new MgdDbg.Snoop.Data.Int("Ending priority", secComp.EndingPriority));
            data.Add(new MgdDbg.Snoop.Data.Bool("End on right wall half by priority", secComp.EndOnRightWallHalfByPriority));
            data.Add(new MgdDbg.Snoop.Data.Bool("Has modifier", secComp.HasModifier));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is edge component", secComp.IsEdgeComponent));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is right part", secComp.IsRightPart));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is single line", secComp.IsSingleLine));
            data.Add(new MgdDbg.Snoop.Data.Double("Offset value", secComp.OffsetValue));
            data.Add(new MgdDbg.Snoop.Data.Bool("On right wall half by location", secComp.OnRightWallHalfByLocation));
            data.Add(new MgdDbg.Snoop.Data.Object("Other half", secComp.OtherHalf));
            data.Add(new MgdDbg.Snoop.Data.Object("Parent", secComp.Parent));
            data.Add(new MgdDbg.Snoop.Data.Object("Polyline", secComp.Polyline));
            data.Add(new MgdDbg.Snoop.Data.Bool("Start on right wall half by priority", secComp.StartOnRgihtWallHalfByPriority));
            data.Add(new MgdDbg.Snoop.Data.Int("Start priority", secComp.StartPriority));
        }

        private void
        Stream(ArrayList data, SectionComponentSet secCompSet)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(SectionComponentSet)));

            data.Add(new MgdDbg.Snoop.Data.Enumerable("Components", secCompSet.Components));
            data.Add(new MgdDbg.Snoop.Data.Object("Parent", secCompSet.Parent));            
        }
    }
}
