using System;
using System.Text;
using System.Collections;
using System.Diagnostics;
using System.Collections.Generic;

using Autodesk.Aec.DatabaseServices;

using MgdDbg.Snoop;

namespace MgdDbgAec.Snoop.CollectorExts  {
    class MiscDb : MgdDbg.Snoop.CollectorExts.CollectorExt {

        public MiscDb()
		{
		}
		
        protected override void
        CollectEvent(object sender, MgdDbg.Snoop.Collectors.CollectorEventArgs e)
        {
                // cast the sender object to the SnoopCollector we are expecting
            MgdDbg.Snoop.Collectors.Collector snoopCollector = sender as MgdDbg.Snoop.Collectors.Collector;
            if (snoopCollector == null) {
                Debug.Assert(false);    // why did someone else send us the message?
                return;
            }
         
            DisplayComponentPushPopData dispCompPushPop = e.ObjToSnoop as DisplayComponentPushPopData;
            if (dispCompPushPop != null) {
                Stream(snoopCollector.Data(), dispCompPushPop);
                return;
            }

            DisplayMaterialInformation dispMatInfo = e.ObjToSnoop as DisplayMaterialInformation;
            if (dispMatInfo != null) {
                Stream(snoopCollector.Data(), dispMatInfo);
                return;
            }

            DBObjectRelationshipManager objRelManager = e.ObjToSnoop as DBObjectRelationshipManager;
            if (objRelManager != null) {
                Stream(snoopCollector.Data(), objRelManager);
                return;
            }           

            FaceScanResults faceScanRes = e.ObjToSnoop as FaceScanResults;
            if (faceScanRes != null) {
                Stream(snoopCollector.Data(), faceScanRes);
                return;
            }

            GraphicsSystemEntityDrawData graphSysEntDrawData = e.ObjToSnoop as GraphicsSystemEntityDrawData;
            if (graphSysEntDrawData != null) {
                Stream(snoopCollector.Data(), graphSysEntDrawData);
                return;
            }            

            NameTranslator nameTrans = e.ObjToSnoop as NameTranslator;
            if (nameTrans != null) {
                Stream(snoopCollector.Data(), nameTrans);
                return;
            }            

            // ValueTypes we have to treat a bit different
            if (e.ObjToSnoop is DBObjectRelationship) {
                Stream(snoopCollector.Data(), (DBObjectRelationship)e.ObjToSnoop);
                return;
            }            
        }

        private void
        Stream(ArrayList data, DisplayComponentPushPopData dispCompPushPop)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(DisplayComponentPushPopData)));

            data.Add(new MgdDbg.Snoop.Data.Object("Display component entity", dispCompPushPop.DisplayComponentEntity));
            data.Add(new MgdDbg.Snoop.Data.String("Display component name", dispCompPushPop.DisplayComponentName));
            data.Add(new MgdDbg.Snoop.Data.Bool("Has material", dispCompPushPop.HasMaterial));
            data.Add(new MgdDbg.Snoop.Data.Bool("Has name", dispCompPushPop.HasName));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Material ID", dispCompPushPop.MaterialId));
            data.Add(new MgdDbg.Snoop.Data.Object("Material info", dispCompPushPop.MaterialInformation));           
        }

        private void
        Stream(ArrayList data, DisplayMaterialInformation dispMatInfo)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(DisplayMaterialInformation)));

            data.Add(new MgdDbg.Snoop.Data.Int("Display component entity", dispMatInfo.ComponentId));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is from object override", dispMatInfo.IsFromObjectOverride));
            //data.Add(new MgdDbg.Snoop.Data.Int("Location ID", dispMatInfo.LocationId));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Location style ID", dispMatInfo.LocationStyleId));
            data.Add(new MgdDbg.Snoop.Data.Object("Material location", dispMatInfo.MaterialLocation));                       
            
            MaterialInformation matInfo = dispMatInfo as MaterialInformation;
            if (matInfo != null) {
                Stream(data, matInfo);
                return;
            }           
        }

        private void
        Stream(ArrayList data, MaterialInformation matInfo)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(MaterialInformation)));

            data.Add(new MgdDbg.Snoop.Data.Int("Component ID", matInfo.ComponentId));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is from object override", matInfo.IsFromObjectOverride));
            //data.Add(new MgdDbg.Snoop.Data.Int("Location ID", matInfo.LocationId));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Location ID", matInfo.LocationStyleId));
            data.Add(new MgdDbg.Snoop.Data.Object("Material location", matInfo.MaterialLocation));           
        }

        private void
        Stream(ArrayList data, DBObjectRelationshipManager objRelManager)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(DBObjectRelationshipManager)));

            data.Add(new MgdDbg.Snoop.Data.Database("Database", objRelManager.Database));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is notification suppressed", objRelManager.IsNotificationSuppressed));
            data.Add(new MgdDbg.Snoop.Data.ObjectIdCollection("Objects with enhanced references", objRelManager.ObjectsWithEnhancedReferences));
            data.Add(new MgdDbg.Snoop.Data.ObjectIdCollection("Objects with enhanced references including erased", objRelManager.ObjectsWithEnhancedReferencesIncludingErased));           
        }        
        
        private void
        Stream(ArrayList data, FaceScanResults faceScanRes)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(FaceScanResults)));

            data.Add(new MgdDbg.Snoop.Data.Bool("Exact hit", faceScanRes.ExactHit));
            data.Add(new MgdDbg.Snoop.Data.Object("Face coordinate system", faceScanRes.FaceCoordinateSystem));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Material ID", faceScanRes.MaterialId));
            data.Add(new MgdDbg.Snoop.Data.Object("Face", faceScanRes.PickedFace));            
        }
       
        private void
        Stream(ArrayList data, GraphicsSystemEntityDrawData graphSysEntDrawData)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(GraphicsSystemEntityDrawData)));

            data.Add(new MgdDbg.Snoop.Data.Object("DbObject", graphSysEntDrawData.DbObject));
            data.Add(new MgdDbg.Snoop.Data.Object("Gs marker information tree", graphSysEntDrawData.GsMarkerInformationTree));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is busy", graphSysEntDrawData.IsBusy));                     
        }        

        private void
        Stream(ArrayList data, NameTranslator nameTrans)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(NameTranslator)));

            // no data at this level
        }        

        private void
        Stream(ArrayList data, DBObjectRelationship objRelation)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(DBObjectRelationship)));

            data.Add(new MgdDbg.Snoop.Data.ObjectId("ID", objRelation.Id));
            data.Add(new MgdDbg.Snoop.Data.String("Relationship type", objRelation.RelationshipType.ToString()));         
        }
    }
}
