
// $Header: //Desktop/Source/TestTools/MgdDbgAec/Snoop/CollectorExts/Modeler.cs#3 $
// $Author: ayyara $ $DateTime: 2007/06/19 11:13:41 $ $Change: 74307 $
//
// (C) Copyright 2006 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//

using System;
using System.Text;
using System.Diagnostics;
using System.Collections;
using System.Collections.Generic;

using Autodesk.Aec.Modeler;

namespace MgdDbgAec.Snoop.CollectorExts {
    class Modeler : MgdDbg.Snoop.CollectorExts.CollectorExt {

		public Modeler()
		{
		}
		
        protected override void
        CollectEvent(object sender, MgdDbg.Snoop.Collectors.CollectorEventArgs e)
        {
                // bail early if this isn't an object we are responsible for
            Autodesk.Aec.Modeler.Entity ent = e.ObjToSnoop as Autodesk.Aec.Modeler.Entity;
            if (ent == null)
                return;

                // cast the sender object to the SnoopCollector we are expecting
            MgdDbg.Snoop.Collectors.Collector snoopCollector = sender as MgdDbg.Snoop.Collectors.Collector;
            if (snoopCollector == null) {
                Debug.Assert(false);    // why did someone else send us the message?
                return;
            }
            
            if (ent != null) {
                Stream(snoopCollector.Data(), ent);
                return;
            }
        }

        private void
        Stream(ArrayList data, Autodesk.Aec.Modeler.Entity ent)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(Autodesk.Aec.Modeler.Entity)));

            data.Add(new MgdDbg.Snoop.Data.Int("Flags", (int)ent.Flags));

            Body body = ent as Body;
            if (body != null) {
                Stream(data, body);
                return;
            }

#if !ADT2007
            Curve curve = ent as Curve;
            if (curve != null) {
                Stream(data, curve);
                return;
            }

            Surface surf = ent as Surface;
            if (surf != null) {
                Stream(data, surf);
                return;
            }

            Edge edge = ent as Edge;
            if (edge != null) {
                Stream(data, edge);
                return;
            }

            Face face = ent as Face;
            if (face != null) {
                Stream(data, face);
                return;
            }

            Vertex vert = ent as Vertex;
            if (vert != null) {
                Stream(data, vert);
                return;
            }

            VertexSurfaceData vertSurfData = ent as VertexSurfaceData;
            if (vertSurfData != null) {
                Stream(data, vertSurfData);
                return;
            }
#endif
        }


        private void
        Stream(ArrayList data, Body body)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(Body)));

#if ADT2007
            data.Add(new MgdDbg.Snoop.Data.Bool("Is valid", body.IsValid));
            data.Add(new MgdDbg.Snoop.Data.Double("Area", body.Area));
            data.Add(new MgdDbg.Snoop.Data.Double("Volume", body.Volume));
            data.Add(new MgdDbg.Snoop.Data.Point3d("Centroid", body.Centroid));
            data.Add(new MgdDbg.Snoop.Data.Point3d("Moments", body.Moments));
            data.Add(new MgdDbg.Snoop.Data.Point3d("Products", body.Products));
            data.Add(new MgdDbg.Snoop.Data.Int("Edge count", body.EdgeCount));
            data.Add(new MgdDbg.Snoop.Data.Int("Face count", body.FaceCount));
            data.Add(new MgdDbg.Snoop.Data.Int("Surface count", body.SurfaceCount));
            data.Add(new MgdDbg.Snoop.Data.Int("Vertex count", body.VertexCount));
            data.Add(new MgdDbg.Snoop.Data.Object("Interval", body.Interval));
            data.Add(new MgdDbg.Snoop.Data.Object("Next", body.Next));
#else
            data.Add(new MgdDbg.Snoop.Data.Bool("Is valid", body.IsValid));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is negated", body.IsNegated));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is null", body.IsNull));
            data.Add(new MgdDbg.Snoop.Data.Bool("Contains non-manifold edges", body.ContainsNonManifoldEdges));
            data.Add(new MgdDbg.Snoop.Data.Double("Area", body.Area));
            data.Add(new MgdDbg.Snoop.Data.Double("Volume", body.Volume));
            data.Add(new MgdDbg.Snoop.Data.Point3d("Centroid", body.Centroid));
            data.Add(new MgdDbg.Snoop.Data.Point3d("Moments", body.Moments));
            data.Add(new MgdDbg.Snoop.Data.Point3d("Products", body.Products));
            data.Add(new MgdDbg.Snoop.Data.Object("Interval", body.Interval));
            data.Add(new MgdDbg.Snoop.Data.Object("Next", body.Next));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Curves", body.Curves));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Faces", body.Faces));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Surfaces", body.Surfaces));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Vertices", body.Vertices));
            try {
                data.Add(new MgdDbg.Snoop.Data.Enumerable("Physical edges", body.PhysicalEdges));
            }
            catch (System.IndexOutOfRangeException ex) {
                data.Add(new MgdDbg.Snoop.Data.Exception("Physical edges", ex));
            }
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Physical vertices", body.PhysicalVertices));
#endif
        }

#if !ADT2007
        private void
        Stream(ArrayList data, Curve curve)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(Curve)));

            data.Add(new MgdDbg.Snoop.Data.String("Type", curve.Type.ToString()));

            CircleCurve circCurve = curve as CircleCurve;
            if (circCurve != null) {
                Stream(data, circCurve);
                return;
            }

            UnspecifiedCurve unspCurve = curve as UnspecifiedCurve;
            if (unspCurve != null) {
                data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(UnspecifiedCurve)));
                    // no data 
                return;
            }

        }

        private void
        Stream(ArrayList data, CircleCurve curve)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(CircleCurve)));

            data.Add(new MgdDbg.Snoop.Data.Int("Approx", curve.Approx));
            data.Add(new MgdDbg.Snoop.Data.Object("Axis", curve.Axis));
            data.Add(new MgdDbg.Snoop.Data.Object("Circle", curve.Circle));
            data.Add(new MgdDbg.Snoop.Data.Point3d("Origin", curve.Origin));
            data.Add(new MgdDbg.Snoop.Data.Distance("Radius", curve.Radius));
            data.Add(new MgdDbg.Snoop.Data.Object("Transf", curve.Transf));
        }

        private void
        Stream(ArrayList data, Surface surf)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(Surface)));

            data.Add(new MgdDbg.Snoop.Data.String("Type", surf.Type.ToString()));

            RevolutionSurface revSurf = surf as RevolutionSurface;
            if (revSurf != null) {
                Stream(data, revSurf);
                return;
            }

            UnspecifiedSurface unspSurf = surf as UnspecifiedSurface;
            if (unspSurf != null) {
                data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(UnspecifiedSurface)));
                    // no data 
                return;
            }
        }

        private void
        Stream(ArrayList data, RevolutionSurface surf)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(RevolutionSurface)));

            data.Add(new MgdDbg.Snoop.Data.Object("Axis", surf.Axis));
            data.Add(new MgdDbg.Snoop.Data.Vector3d("Start dir", surf.StartDir));

            ConeSurface coneSurf = surf as ConeSurface;
            if (coneSurf != null) {
                Stream(data, coneSurf);
                return;
            }

            CylinderSurface cylSurf = surf as CylinderSurface;
            if (cylSurf != null) {
                Stream(data, cylSurf);
                return;
            }

            SphereSurface sphereSurf = surf as SphereSurface;
            if (sphereSurf != null) {
                Stream(data, sphereSurf);
                return;
            }

            TorusSurface torusSurf = surf as TorusSurface;
            if (torusSurf != null) {
                Stream(data, torusSurf);
                return;
            }

        }

        private void
        Stream(ArrayList data, ConeSurface surf)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(ConeSurface)));

            data.Add(new MgdDbg.Snoop.Data.Point3d("Apex", surf.Apex));
            data.Add(new MgdDbg.Snoop.Data.Int("Approximation", surf.Approximation));
            data.Add(new MgdDbg.Snoop.Data.Distance("Radius", surf.Radius));
            data.Add(new MgdDbg.Snoop.Data.Angle("Tan angle", surf.TanAngle));
        }

        private void
        Stream(ArrayList data, CylinderSurface surf)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(CylinderSurface)));

            data.Add(new MgdDbg.Snoop.Data.Int("Approx", surf.Approx));
            data.Add(new MgdDbg.Snoop.Data.Distance("Radius", surf.Radius));
        }

        private void
        Stream(ArrayList data, SphereSurface surf)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(SphereSurface)));

            data.Add(new MgdDbg.Snoop.Data.Int("Approx", surf.Approx));
            data.Add(new MgdDbg.Snoop.Data.Point3d("Center", surf.Center));
            data.Add(new MgdDbg.Snoop.Data.Distance("Radius", surf.Radius));
        }

        private void
        Stream(ArrayList data, TorusSurface surf)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(TorusSurface)));

            data.Add(new MgdDbg.Snoop.Data.Point3d("Center", surf.Center));
            data.Add(new MgdDbg.Snoop.Data.Int("Major approx", surf.MajorApprox));
            data.Add(new MgdDbg.Snoop.Data.Int("Minor approx", surf.MinorApprox));
            data.Add(new MgdDbg.Snoop.Data.Distance("MajorRadius", surf.MajorRadius));
            data.Add(new MgdDbg.Snoop.Data.Distance("MinorRadius", surf.MinorRadius));
        }


        private void
        Stream(ArrayList data, Edge edge)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(Edge)));

            data.Add(new MgdDbg.Snoop.Data.Angle("Angle between edges", edge.AngleBetweenEdges));
            data.Add(new MgdDbg.Snoop.Data.Angle("Angle between faces", edge.AngleBetweenFaces));
            data.Add(new MgdDbg.Snoop.Data.Bool("Angle between faces is concave", edge.AngleBetweenFacesIsConcave));
            data.Add(new MgdDbg.Snoop.Data.Bool("Angle between faces is convex", edge.AngleBetweenFacesIsConvex));
            data.Add(new MgdDbg.Snoop.Data.Bool("Angle between faces is straight", edge.AngleBetweenFacesIsStraight));
            data.Add(new MgdDbg.Snoop.Data.Object("Circle", edge.Circle));
            data.Add(new MgdDbg.Snoop.Data.Object("Line", edge.Line));
            data.Add(new MgdDbg.Snoop.Data.Object("Curve", edge.Curve));
            data.Add(new MgdDbg.Snoop.Data.Object("Surface", edge.Surface));
            data.Add(new MgdDbg.Snoop.Data.String("Color", edge.Color.ToString()));
            data.Add(new MgdDbg.Snoop.Data.Object("Euler edge", edge.EulerEdge));
            data.Add(new MgdDbg.Snoop.Data.Object("Face", edge.Face));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is approx", edge.IsApprox));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is bridge", edge.IsBridge));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is Euler edge", edge.IsEulerEdge));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is manifold", edge.IsManifold));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is on circle", edge.IsOnCircle));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is on full circle", edge.IsOnFullCircle));
            data.Add(new MgdDbg.Snoop.Data.Object("Prev skip bridge", edge.PrevSkipBridge));
            data.Add(new MgdDbg.Snoop.Data.Object("Next skip bridge", edge.NextSkipBridge));
            data.Add(new MgdDbg.Snoop.Data.Object("Partner", edge.Partner));
            data.Add(new MgdDbg.Snoop.Data.Object("Partner bridge edge", edge.PartnerBridgeEdge));
            data.Add(new MgdDbg.Snoop.Data.Object("Plane", edge.Plane));
            data.Add(new MgdDbg.Snoop.Data.Point3d("Point", edge.Point));
            data.Add(new MgdDbg.Snoop.Data.Vector3d("Normal", edge.Normal));
            data.Add(new MgdDbg.Snoop.Data.Vector3d("Unit vector", edge.UnitVector));
            data.Add(new MgdDbg.Snoop.Data.Vector3d("Vector", edge.Vector));
            data.Add(new MgdDbg.Snoop.Data.Object("Vertex", edge.Vertex));
            data.Add(new MgdDbg.Snoop.Data.Vector3d("Vertex normal", edge.VertexNormal));

            data.Add(new MgdDbg.Snoop.Data.Enumerable("All connected edges sharing vertex", edge.GetAllConnectedEdgesSharingVertex()));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("All edges starting from vertex", edge.GetAllEdgesStartingFromVertex()));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("All edges ending in vertex", edge.GetAllEdgesEndingInVertex()));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("All edges sharing same faces (connected sequence only = true)", edge.GetAllEdgesSharingSameFaces(true)));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("All edges sharing same faces (connected sequence only = false)", edge.GetAllEdgesSharingSameFaces(false)));
            data.Add(new MgdDbg.Snoop.Data.Bool("Can merge with previous (same color only = true)", edge.CanMergeWithPrevious(true)));
            data.Add(new MgdDbg.Snoop.Data.Bool("Can merge with previous (same color only = false)", edge.CanMergeWithPrevious(false)));
        }

        private void
        Stream(ArrayList data, Face face)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(Face)));

            data.Add(new MgdDbg.Snoop.Data.Double("Area", face.Area));
            //data.Add(new MgdDbg.Snoop.Data.???("Attribute", face.Attribute)); // TBD: not sure how this works?
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Edges", face.Edges));
            data.Add(new MgdDbg.Snoop.Data.String("Face color", face.FaceColor.ToString()));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is self-intersecting", face.IsSelfIntersecting));
            data.Add(new MgdDbg.Snoop.Data.Object("Plane", face.Plane));
            data.Add(new MgdDbg.Snoop.Data.Object("Surface", face.Surface));
        }

        private void
        Stream(ArrayList data, Vertex vert)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(Vertex)));

            data.Add(new MgdDbg.Snoop.Data.Point3d("Point", vert.Point));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Vertex surface data", vert.VertexSurfaceDataCollection));
        }

        private void
        Stream(ArrayList data, VertexSurfaceData vertSurfData)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(VertexSurfaceData)));

            data.Add(new MgdDbg.Snoop.Data.Vector3d("Normal", vertSurfData.Normal));
            data.Add(new MgdDbg.Snoop.Data.Object("Surface", vertSurfData.Surface));
        }

#endif

    }
}
