
// $Header: //Desktop/Source/TestTools/MgdDbgAec/Snoop/CollectorExts/ObjectArch.cs#2 $
// $Author: ayyara $ $DateTime: 2007/03/28 11:46:10 $ $Change: 69505 $
//
// (C) Copyright 2004 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//


using System;
using System.Diagnostics;
using System.Collections;
using System.Windows.Forms;
using System.Collections.Specialized;

using MgdDbg.Snoop;
using Autodesk.AutoCAD.Geometry;
using Autodesk.AutoCAD.DatabaseServices;
using AcDb            = Autodesk.AutoCAD.DatabaseServices;
using AecDb           = Autodesk.Aec.DatabaseServices;
using AecDbArch       = Autodesk.Aec.Arch.DatabaseServices;
using AecAppSrvcs     = Autodesk.Aec.ApplicationServices;
using AecArchAppSrvcs = Autodesk.Aec.Arch.ApplicationServices;


namespace MgdDbgAec.Snoop.CollectorExts
{
	/// <summary>
	/// This is a Snoop Collector Extension object to collect data from AEC objects.
	/// </summary>
	public class ObjectArch : MgdDbg.Snoop.CollectorExts.CollectorExt
	{
		public ObjectArch()
		{
		}
		
        protected override void
        CollectEvent(object sender, MgdDbg.Snoop.Collectors.CollectorEventArgs e)
        {
                // bail early if this isn't an object we are responsible for
            AecDb.DBObject aecObj = e.ObjToSnoop as AecDb.DBObject;
            if (aecObj == null)
                return;

                // cast the sender object to the SnoopCollector we are expecting
            MgdDbg.Snoop.Collectors.Collector snoopCollector = sender as MgdDbg.Snoop.Collectors.Collector;
            if (snoopCollector == null) {
                Debug.Assert(false);    // why did someone else send us the message?
                return;
            }
                        
            AecDbArch.DoorStyle doorStyle = aecObj as AecDbArch.DoorStyle;
            if (doorStyle != null) {
                Stream(snoopCollector.Data(), doorStyle);
                return;
            }
            
            AecDbArch.WindowStyle winStyle = aecObj as AecDbArch.WindowStyle;
            if (winStyle != null) {
                Stream(snoopCollector.Data(), winStyle);
                return;
            }
            
            AecDbArch.WallStyle wallStyle = aecObj as AecDbArch.WallStyle;
            if (wallStyle != null) {
                Stream(snoopCollector.Data(), wallStyle);
                return;
            }

            AecDbArch.StairStyle stairStyle = aecObj as AecDbArch.StairStyle;
            if (stairStyle != null) {
                Stream(snoopCollector.Data(), stairStyle);
                return;
            }

            AecDbArch.StairWinderStyle swStyle = aecObj as AecDbArch.StairWinderStyle;
            if (swStyle != null) {
                Stream(snoopCollector.Data(), swStyle);
                return;
            }

            AecDbArch.ZoneStyle zoneStyle = aecObj as AecDbArch.ZoneStyle;
            if (zoneStyle != null) {
                Stream(snoopCollector.Data(), zoneStyle);
                return;
            }

            AecDbArch.ZoneDefinition zoneDef = aecObj as AecDbArch.ZoneDefinition;
            if (zoneDef != null) {
                Stream(snoopCollector.Data(), zoneDef);
                return;
            }

            AecDbArch.StairWinderType swType = aecObj as AecDbArch.StairWinderType;
            if (swType != null) {
                Stream(snoopCollector.Data(), swType);
                return;
            }

            AecDbArch.SpaceStyle spaceStyle = aecObj as AecDbArch.SpaceStyle;
            if (spaceStyle != null) {
                Stream(snoopCollector.Data(), spaceStyle);
                return;
            }

            AecDbArch.RailingStyle railStyle = aecObj as AecDbArch.RailingStyle;
            if (railStyle != null) {
                Stream(snoopCollector.Data(), railStyle);
                return;
            }

            AecDbArch.SlabEdgeStyle seStyle = aecObj as AecDbArch.SlabEdgeStyle;
            if (seStyle != null) {
                Stream(snoopCollector.Data(), seStyle);
                return;
            }

            AecDbArch.SlabStyle sStyle = aecObj as AecDbArch.SlabStyle;
            if (sStyle != null) {
                Stream(snoopCollector.Data(), sStyle);
                return;
            }

            AecDbArch.WallEndcapStyle weStyle = aecObj as AecDbArch.WallEndcapStyle;
            if (weStyle != null) {
                Stream(snoopCollector.Data(), weStyle);
                return;
            }

            AecDbArch.WallModifierStyle wmStyle = aecObj as AecDbArch.WallModifierStyle;
            if (wmStyle != null) {
                Stream(snoopCollector.Data(), wmStyle);
                return;
            }

            AecDbArch.AnchorEntityToStairLanding anchorLanding = aecObj as AecDbArch.AnchorEntityToStairLanding;
            if (anchorLanding != null) {
                Stream(snoopCollector.Data(), anchorLanding);
                return;
            }

            AecDbArch.AnchorRailing anchorRailing = aecObj as AecDbArch.AnchorRailing;
            if (anchorRailing != null) {
                Stream(snoopCollector.Data(), anchorRailing);
                return;
            }

            AecDbArch.AnchorEntityToWall anchorWall = aecObj as AecDbArch.AnchorEntityToWall;
			if (anchorWall != null) {
				Stream(snoopCollector.Data(), anchorWall);
				return;
			}


            AecDbArch.AnchorRoomTagToSpace roomTagToSpace = aecObj as AecDbArch.AnchorRoomTagToSpace;
			if (roomTagToSpace != null) {
				Stream(snoopCollector.Data(), roomTagToSpace);
				return;
			}

            AecDbArch.CurtainWallLayoutStyle cwlStyle = aecObj as AecDbArch.CurtainWallLayoutStyle;
            if (cwlStyle != null) {
                Stream(snoopCollector.Data(), cwlStyle);
                return;
            }

            AecDbArch.CurtainWallUnitStyle cwuStyle = aecObj as AecDbArch.CurtainWallUnitStyle;
            if (cwuStyle != null) {
                Stream(snoopCollector.Data(), cwuStyle);
                return;
            }

            AecDbArch.WindowAssemblyStyle waStyle = aecObj as AecDbArch.WindowAssemblyStyle;
            if (waStyle != null) {
                Stream(snoopCollector.Data(), waStyle);
                return;
            }

            AecDbArch.WallCleanupGroupDefinition wcgDef = aecObj as AecDbArch.WallCleanupGroupDefinition;
            if (wcgDef != null) {
                Stream(snoopCollector.Data(), wcgDef);
                return;
            }

            AecDbArch.OpeningEndcapStyle oeStyle = aecObj as AecDbArch.OpeningEndcapStyle;
            if (oeStyle != null) {
                Stream(snoopCollector.Data(), oeStyle);
                return;
            }

            AecDbArch.SpaceModifier spaceMod = aecObj as AecDbArch.SpaceModifier;
            if (spaceMod != null) {
                Stream(snoopCollector.Data(), spaceMod);
                return;
            }

            AecDbArch.StandardSizeOpeningList ssoList = aecObj as AecDbArch.StandardSizeOpeningList;
            if (ssoList != null) {
                Stream(snoopCollector.Data(), ssoList);
                return;
            }

            AecArchAppSrvcs.ArchDBVariables archVars = aecObj as AecArchAppSrvcs.ArchDBVariables;
            if (archVars != null) {
                Stream(snoopCollector.Data(), archVars);
                return;
            }
        }
        
        private void
        Stream(ArrayList data, AecDbArch.DoorStyle doorStyle)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDbArch.DoorStyle)));
            
            data.Add(new MgdDbg.Snoop.Data.Distance("Frame width", doorStyle.FrameWidth));
            data.Add(new MgdDbg.Snoop.Data.Distance("Frame depth", doorStyle.FrameDepth));
            data.Add(new MgdDbg.Snoop.Data.Distance("Stop width", doorStyle.StopWidth));
            data.Add(new MgdDbg.Snoop.Data.Distance("Stop depth", doorStyle.StopDepth));
            data.Add(new MgdDbg.Snoop.Data.Bool("Use depth of wall", doorStyle.UseDepthOfWall));
            data.Add(new MgdDbg.Snoop.Data.Distance("Door thickness", doorStyle.DoorThickness));
            data.Add(new MgdDbg.Snoop.Data.Distance("Glass thickness", doorStyle.GlassThickness));
            data.Add(new MgdDbg.Snoop.Data.String("Door type", doorStyle.DoorType.ToString()));
            data.Add(new MgdDbg.Snoop.Data.String("Door shape", doorStyle.DoorShape.ToString()));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Custom shape ID", doorStyle.CustomShapeId));
            data.Add(new MgdDbg.Snoop.Data.Int("Number of leaves", doorStyle.NumberOfLeaves));
        }
        
        private void
        Stream(ArrayList data, AecDbArch.WindowStyle winStyle)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDbArch.WindowStyle)));
            
            data.Add(new MgdDbg.Snoop.Data.Distance("Frame width", winStyle.FrameWidth));
            data.Add(new MgdDbg.Snoop.Data.Distance("Frame depth", winStyle.FrameDepth));
            data.Add(new MgdDbg.Snoop.Data.Distance("Sash width", winStyle.SashWidth));
            data.Add(new MgdDbg.Snoop.Data.Distance("Sash depth", winStyle.SashDepth));
            data.Add(new MgdDbg.Snoop.Data.Bool("Use depth of wall", winStyle.UseDepthOfWall));
            data.Add(new MgdDbg.Snoop.Data.Distance("Glass thickness", winStyle.GlassThickness));
            data.Add(new MgdDbg.Snoop.Data.String("Window type", winStyle.WindowType.ToString()));
            data.Add(new MgdDbg.Snoop.Data.String("Window shape", winStyle.WindowShape.ToString()));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Custom shape ID", winStyle.CustomShapeId));
        }
        
        private void
        Stream(ArrayList data, AecDbArch.WallStyle wallStyle)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDbArch.WallStyle)));
            
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Components", wallStyle.Components));
            data.Add(new MgdDbg.Snoop.Data.Bool("Has instance width", wallStyle.HasInstanceWidth));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Shrinkwrap body material ID", wallStyle.ShrinkwrapBodyMaterialId));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Default door opening endcap style ID", wallStyle.DefaultDoorOpeningEndcapStyle));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Default opening opening endcap style ID", wallStyle.DefaultOpeningOpeningEndcapStyle));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Default wall endcap style ID", wallStyle.DefaultWallEndcapStyle));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Default window assembly endcap style ID", wallStyle.DefaultWindowAssemblyOpeningEndcapStyle));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Default window endcap style ID", wallStyle.DefaultWindowOpeningEndcapStyle));
        }

        private void
        Stream(ArrayList data, AecDbArch.StairStyle stairStyle)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDbArch.StairStyle)));

            data.Add(new MgdDbg.Snoop.Data.Distance("Additional width", stairStyle.AdditionalWidth));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Component definitions", stairStyle.ComponentDefinitions));
            data.Add(new MgdDbg.Snoop.Data.String("Flags", stairStyle.Flags.ToString()));
            data.Add(new MgdDbg.Snoop.Data.Distance("Landing length", stairStyle.LandingLength));
            data.Add(new MgdDbg.Snoop.Data.String("Landing location", stairStyle.LandingLocation.ToString()));
            data.Add(new MgdDbg.Snoop.Data.Distance("Landing thickness", stairStyle.LandingThickness));
            data.Add(new MgdDbg.Snoop.Data.Distance("Lower rule limit", stairStyle.LowerRuleLimit));
            data.Add(new MgdDbg.Snoop.Data.Distance("Maximum tread", stairStyle.MaximumTread));
            data.Add(new MgdDbg.Snoop.Data.Distance("Maximum rise", stairStyle.MaximumRise));
            data.Add(new MgdDbg.Snoop.Data.Distance("Minimum landing extension down", stairStyle.MinimumLandingExtensionDown));
            data.Add(new MgdDbg.Snoop.Data.Bool("Minimum landing extension down add tread", stairStyle.MinimumLandingExtensionDownAddTread));
            data.Add(new MgdDbg.Snoop.Data.Distance("Minimum landing extension up", stairStyle.MinimumLandingExtensionUp));
            data.Add(new MgdDbg.Snoop.Data.Bool("Minimum landing extension up add tread", stairStyle.MinimumLandingExtensionUpAddTread));
            data.Add(new MgdDbg.Snoop.Data.Distance("Minimum rise", stairStyle.MinimumRise));
            data.Add(new MgdDbg.Snoop.Data.Distance("Minimum tread", stairStyle.MinimumTread));
            data.Add(new MgdDbg.Snoop.Data.Distance("Nosing", stairStyle.Nosing));
            data.Add(new MgdDbg.Snoop.Data.Distance("Optimum rise", stairStyle.OptimumRise));
            data.Add(new MgdDbg.Snoop.Data.Distance("Optimum tread", stairStyle.OptimumTread));
            data.Add(new MgdDbg.Snoop.Data.Bool("Override calculator", stairStyle.OverrideCalculator));
            data.Add(new MgdDbg.Snoop.Data.Distance("Rise factor", stairStyle.RiseFactor));
            data.Add(new MgdDbg.Snoop.Data.Distance("Riser thickness", stairStyle.RiserThickness));
            data.Add(new MgdDbg.Snoop.Data.String("Riser type", stairStyle.RiserType.ToString()));
            data.Add(new MgdDbg.Snoop.Data.Bool("Slab", stairStyle.Slab));
            data.Add(new MgdDbg.Snoop.Data.Distance("Tread factor", stairStyle.TreadFactor));
            data.Add(new MgdDbg.Snoop.Data.Distance("Tread thickness", stairStyle.TreadThickness));
            data.Add(new MgdDbg.Snoop.Data.Distance("Tread factor", stairStyle.TreadFactor));
            data.Add(new MgdDbg.Snoop.Data.Distance("Upper rule limit", stairStyle.UpperRuleLimit));
        }

        private void
        Stream(ArrayList data, AecDbArch.StairWinderStyle swStyle)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDbArch.StairWinderStyle)));

            data.Add(new MgdDbg.Snoop.Data.Int("Number of treads in turn", swStyle.NumberOfTreadsInTurn));
            data.Add(new MgdDbg.Snoop.Data.Bool("Support fixed number of treads in turn", swStyle.SupportFixedNumberOfTreadsInTurn));
            data.Add(new MgdDbg.Snoop.Data.Bool("Use riser line", swStyle.UseRiserLine));
            data.Add(new MgdDbg.Snoop.Data.Bool("Use winder control point", swStyle.UseWinderControlPoint));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Winder type ID", swStyle.WinderTypeId));           
        }

        private void
        Stream(ArrayList data, AecDbArch.ZoneStyle zoneStyle)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDbArch.ZoneStyle)));

            data.Add(new MgdDbg.Snoop.Data.Bool("Exclusive space", zoneStyle.ExclusiveSpace));
            data.Add(new MgdDbg.Snoop.Data.Bool("Exclusive zone", zoneStyle.ExclusiveZone));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Name definition style ID", zoneStyle.NameDefinitionStyleId));
        }

        private void
        Stream(ArrayList data, AecDbArch.ZoneDefinition zoneDef)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDbArch.ZoneDefinition)));

            //data.Add(new MgdDbg.Snoop.Data.Object("Root", zoneDef.Root)); // TBD: crashes!
        }

        private void
        Stream(ArrayList data, AecDbArch.StairWinderType swType)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDbArch.StairWinderType)));

            data.Add(new MgdDbg.Snoop.Data.Bool("Allow fixed tread count in turn", swType.AllowFixedTreadCountInTurn));
            data.Add(new MgdDbg.Snoop.Data.Bool("Use winder control point", swType.UseWinderControlPoint));

            AecDbArch.StairWinderTypeBalanced swtBalanced = swType as AecDbArch.StairWinderTypeBalanced;
            if (swtBalanced != null) {
                Stream(data, swtBalanced);
                return;
            }

            AecDbArch.StairWinderTypeManual swtManual = swType as AecDbArch.StairWinderTypeManual;
            if (swtManual != null) {
                Stream(data, swtManual);
                return;
            }

            AecDbArch.StairWinderTypeSinglePoint swtSinglePt = swType as AecDbArch.StairWinderTypeSinglePoint;
            if (swtSinglePt != null) {
                Stream(data, swtSinglePt);
                return;
            }
        }

        private void
        Stream(ArrayList data, AecDbArch.StairWinderTypeBalanced swtBalanced)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDbArch.StairWinderTypeBalanced)));         
        }

        private void
        Stream(ArrayList data, AecDbArch.StairWinderTypeManual swtManual)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDbArch.StairWinderTypeManual))); 
        }

        private void
        Stream(ArrayList data, AecDbArch.StairWinderTypeSinglePoint swtSinglePt)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDbArch.StairWinderTypeSinglePoint))); 
        }


        private void
        Stream(ArrayList data, AecDbArch.SpaceStyle spStyle)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDbArch.SpaceStyle)));
            
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Name definition style ID", spStyle.NameDefinitionStyleId));
            data.Add(new MgdDbg.Snoop.Data.String("Category", spStyle.Category));
            data.Add(new MgdDbg.Snoop.Data.Int("Category ID", spStyle.CategoryId));
            data.Add(new MgdDbg.Snoop.Data.Int("Constraint ID", spStyle.ConstraintId));
            data.Add(new MgdDbg.Snoop.Data.Distance("Net to gross offset", spStyle.NetToGrossOffset));
            data.Add(new MgdDbg.Snoop.Data.Double("Area", spStyle.Area));
            data.Add(new MgdDbg.Snoop.Data.Double("Maximum area", spStyle.MaximumArea));
            data.Add(new MgdDbg.Snoop.Data.Distance("Maximum length", spStyle.MaximumLength));
            data.Add(new MgdDbg.Snoop.Data.Distance("Maximum width", spStyle.MaximumWidth));
            data.Add(new MgdDbg.Snoop.Data.Double("Minimum area", spStyle.MinimumArea));
            data.Add(new MgdDbg.Snoop.Data.Distance("Minimum length", spStyle.MinimumLength));
            data.Add(new MgdDbg.Snoop.Data.Distance("Minimum width", spStyle.MinimumWidth));
            data.Add(new MgdDbg.Snoop.Data.Distance("Length", spStyle.Length));
            data.Add(new MgdDbg.Snoop.Data.Distance("Width", spStyle.Width));
            data.Add(new MgdDbg.Snoop.Data.Distance("Ceiling height", spStyle.CeilingHeight));
            data.Add(new MgdDbg.Snoop.Data.Distance("Net offset", spStyle.NetOffset));
            data.Add(new MgdDbg.Snoop.Data.Distance("Net to gross offset", spStyle.NetToGrossOffset));
            data.Add(new MgdDbg.Snoop.Data.Distance("Net to usable offset", spStyle.NetToUsableOffset));

            // TBD: GetMaterialId()
        }

        private void
        Stream(ArrayList data, AecDbArch.RailingStyle railStyle)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDbArch.RailingStyle)));

            data.Add(new MgdDbg.Snoop.Data.Bool("Ballusters", railStyle.Balusters));
            data.Add(new MgdDbg.Snoop.Data.Distance("Balluster spacing", railStyle.BalusterSpacing));
            data.Add(new MgdDbg.Snoop.Data.Int("Ballusters per tread", railStyle.BalustersPerTread));
            data.Add(new MgdDbg.Snoop.Data.Bool("Balluster tread override", railStyle.BalusterTreadOverride));
            data.Add(new MgdDbg.Snoop.Data.Object("Balluster template", railStyle.BalusterTemplate));

            data.Add(new MgdDbg.Snoop.Data.Bool("Bottomrail", railStyle.Bottomrail));
            data.Add(new MgdDbg.Snoop.Data.Int("Bottomrail count", railStyle.BottomrailCount));
            data.Add(new MgdDbg.Snoop.Data.Distance("Bottomrail height", railStyle.BottomrailHeight));
            data.Add(new MgdDbg.Snoop.Data.Distance("Bottomrail offset", railStyle.BottomrailOffset));
            data.Add(new MgdDbg.Snoop.Data.String("Bottomrail placement", railStyle.BottomrailPlacement.ToString()));
            data.Add(new MgdDbg.Snoop.Data.Distance("Bottomrail sloping height", railStyle.BottomrailSlopingHeight));
            data.Add(new MgdDbg.Snoop.Data.Distance("Bottomrail spacing", railStyle.BottomrailSpacing));
            data.Add(new MgdDbg.Snoop.Data.Object("Bottomrail template", railStyle.BottomrailTemplate));

            data.Add(new MgdDbg.Snoop.Data.Bool("Corner posts", railStyle.CornerPosts));

            data.Add(new MgdDbg.Snoop.Data.Bool("Dynamic posts", railStyle.DynamicPosts));
            data.Add(new MgdDbg.Snoop.Data.Distance("Dynamic post spacing", railStyle.DynamicPostSpacing));
            data.Add(new MgdDbg.Snoop.Data.Object("Dynamic post template", railStyle.DynamicPostTemplate));
           
            data.Add(new MgdDbg.Snoop.Data.Bool("Extensions fixed", railStyle.ExtensionsFixed));

            data.Add(new MgdDbg.Snoop.Data.Bool("Guardrail", railStyle.Guardrail));
            data.Add(new MgdDbg.Snoop.Data.Distance("Guardrail height", railStyle.GuardrailHeight));
            data.Add(new MgdDbg.Snoop.Data.Distance("Guardrail offset", railStyle.GuardrailOffset));
            data.Add(new MgdDbg.Snoop.Data.String("Guardrail placement", railStyle.GuardrailPlacement.ToString()));
            data.Add(new MgdDbg.Snoop.Data.Distance("Guardrail sloping height", railStyle.GuardrailSlopingHeight));
            data.Add(new MgdDbg.Snoop.Data.Object("Guardrail template", railStyle.GuardrailTemplate));

            data.Add(new MgdDbg.Snoop.Data.Bool("Handrail", railStyle.Handrail));
            data.Add(new MgdDbg.Snoop.Data.Distance("Handrail height", railStyle.HandrailHeight));
            data.Add(new MgdDbg.Snoop.Data.Distance("Handrail offset", railStyle.HandrailOffset));
            data.Add(new MgdDbg.Snoop.Data.String("Handrail placement", railStyle.HandrailPlacement.ToString()));
            data.Add(new MgdDbg.Snoop.Data.Distance("Handrail sloping height", railStyle.HandrailSlopingHeight));
            data.Add(new MgdDbg.Snoop.Data.Object("Handrail template", railStyle.HandrailTemplate));
            data.Add(new MgdDbg.Snoop.Data.Object("Last post template", railStyle.LastPostTemplate));

            data.Add(new MgdDbg.Snoop.Data.Bool("Post dimensions fixed", railStyle.PostDimensionsFixed));
            data.Add(new MgdDbg.Snoop.Data.Bool("Posts", railStyle.Posts));
            data.Add(new MgdDbg.Snoop.Data.Object("Post template", railStyle.PostTemplate));

            data.Add(new MgdDbg.Snoop.Data.Bool("Rail dimensions fixed", railStyle.RailDimensionsFixed));
            data.Add(new MgdDbg.Snoop.Data.Bool("Use stair landing extents at floor", railStyle.UseStairLandingExtensionAtFloor));
            data.Add(new MgdDbg.Snoop.Data.Bool("Use stair landing extents at landing", railStyle.UseStairLandingExtensionAtLanding));

            // TBD:
            // GetMaterialId()
            // GetExtension()
            // GetExtensionAddTread()
        }

        private void
        Stream(ArrayList data, AecDbArch.SlabEdgeStyle seStyle)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDbArch.SlabEdgeStyle)));

            data.Add(new MgdDbg.Snoop.Data.Angle("Fascia angle", seStyle.FasciaAngle));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Fascia material ID", seStyle.FasciaMaterialId));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Fascia profile ID", seStyle.FasciaProfileId));
            data.Add(new MgdDbg.Snoop.Data.Bool("Has fascia", seStyle.HasFascia));
            data.Add(new MgdDbg.Snoop.Data.Bool("Has soffit", seStyle.HasSoffit));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is fit fascia", seStyle.IsFitFascia));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is fit soffit", seStyle.IsFitSoffit));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is sqare cut", seStyle.IsSquareCut));
            data.Add(new MgdDbg.Snoop.Data.Distance("Overhang", seStyle.Overhang));
            data.Add(new MgdDbg.Snoop.Data.Angle("Soffit angle", seStyle.SoffitAngle));
            data.Add(new MgdDbg.Snoop.Data.Distance("Soffit baseline offset", seStyle.SoffitBaselineOffset));
            data.Add(new MgdDbg.Snoop.Data.Distance("Soffit horizontal offset", seStyle.SoffitHorizontalOffset));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Soffit material ID", seStyle.SoffitMaterialId));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Soffit profile ID", seStyle.SoffitProfileId));
            data.Add(new MgdDbg.Snoop.Data.Double("Soffit vertical offset", seStyle.SoffitVerticalOffset));
            data.Add(new MgdDbg.Snoop.Data.Bool("Use angle", seStyle.UseAngle));
            data.Add(new MgdDbg.Snoop.Data.Bool("Use orientation", seStyle.UseOrientation));
            data.Add(new MgdDbg.Snoop.Data.Bool("Use overhang", seStyle.UseOverhang));            

            AecDbArch.RoofSlabEdgeStyle rseStyle = seStyle as AecDbArch.RoofSlabEdgeStyle;
            if (rseStyle != null) {
                Stream(data, rseStyle);
                return;
            }
        }

        private void
        Stream(ArrayList data, AecDbArch.RoofSlabEdgeStyle rseStyle)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDbArch.RoofSlabEdgeStyle)));
           
            // no data at this level
        }

        private void
        Stream(ArrayList data, AecDbArch.SlabStyle sStyle)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDbArch.SlabStyle)));

            data.Add(new MgdDbg.Snoop.Data.ObjectId("Default base edge style ID", sStyle.DefaultBaseEdgeStyleId));
            data.Add(new MgdDbg.Snoop.Data.Distance("Default horizontal offset", sStyle.DefaultHorizontalOffset));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Default perimeter edge style ID", sStyle.DefaultPerimeterEdgeStyleId));
            data.Add(new MgdDbg.Snoop.Data.Angle("Default slope", sStyle.DefaultSlope));
            data.Add(new MgdDbg.Snoop.Data.Distance("Default thickness", sStyle.DefaultThickness));
            data.Add(new MgdDbg.Snoop.Data.Distance("Default vertical offset", sStyle.DefaultVerticalOffset));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is variable thickness", sStyle.IsVariableThickness));
            data.Add(new MgdDbg.Snoop.Data.String("Register section key", sStyle.RegisterSectionKey));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Shrinkwrap body material ID", sStyle.ShrinkwrapBodyMaterialId));
            data.Add(new MgdDbg.Snoop.Data.Bool("Use baseline edge style", sStyle.UseBaselineEdgeStyle));
            data.Add(new MgdDbg.Snoop.Data.Bool("Use perimeter edge style", sStyle.UsePerimeterEdgeStyle));
            data.Add(new MgdDbg.Snoop.Data.Bool("Use slope", sStyle.UseSlope));
            data.Add(new MgdDbg.Snoop.Data.Bool("Use thickness", sStyle.UseThickness));
            data.Add(new MgdDbg.Snoop.Data.Bool("Use vertical ofset", sStyle.UseVerticalOffset));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Components", sStyle.Components));

            // TBD: GetMaterialId()

            AecDbArch.RoofSlabStyle rsStyle = sStyle as AecDbArch.RoofSlabStyle;
            if (rsStyle != null) {
                Stream(data, rsStyle);
                return;
            }
        }        

        private void
        Stream(ArrayList data, AecDbArch.RoofSlabStyle rsStyle)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDbArch.RoofSlabStyle)));

            // no data at this level
        }

        private void
        Stream(ArrayList data, AecDbArch.WallEndcapStyle weStyle)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDbArch.WallEndcapStyle)));

            data.Add(new MgdDbg.Snoop.Data.Enumerable("Endcaps", weStyle.Endcaps));              
            data.Add(new MgdDbg.Snoop.Data.Distance("Return offset", weStyle.ReturnOffset)); 
        }

        private void
        Stream(ArrayList data, AecDbArch.WallModifierStyle wmStyle)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDbArch.WallModifierStyle)));

            data.Add(new MgdDbg.Snoop.Data.Object("Component modifier definition", wmStyle.ComponentModifierDefinition));
        }
      
        private void
        Stream(ArrayList data, AecDbArch.AnchorEntityToStairLanding anchorLanding)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDbArch.AnchorEntityToStairLanding)));

            data.Add(new MgdDbg.Snoop.Data.Bool("End at same height", anchorLanding.EndAtSameHeight));
            data.Add(new MgdDbg.Snoop.Data.Int("Stair component index", anchorLanding.StairComponentIndex));            
        }

        private void
        Stream(ArrayList data, AecDbArch.AnchorRailing anchorRailing)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDbArch.AnchorRailing)));

            data.Add(new MgdDbg.Snoop.Data.Bool("Automatic post cleanup", anchorRailing.AutomaticPostCleanup));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Railing ID", anchorRailing.RailingId));

            AecDbArch.AnchorFreeRailingToStair afrToStair = anchorRailing as AecDbArch.AnchorFreeRailingToStair;
            if (afrToStair != null) {
                Stream(data, afrToStair);
                return;
            }

            AecDbArch.AnchorRailingToObjects arToObjects = anchorRailing as AecDbArch.AnchorRailingToObjects;
            if (arToObjects != null) {
                Stream(data, arToObjects);
                return;
            }

            AecDbArch.AnchorRailingToStair arToStair = anchorRailing as AecDbArch.AnchorRailingToStair;
            if (arToStair != null) {
                Stream(data, arToStair);
                return;
            }
        }

        private void
        Stream(ArrayList data, AecDbArch.AnchorFreeRailingToStair afrToStair)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDbArch.AnchorFreeRailingToStair)));

            data.Add(new MgdDbg.Snoop.Data.ObjectId("Stair ID", afrToStair.StairId));
        }

        private void
        Stream(ArrayList data, AecDbArch.AnchorRailingToObjects arToObjects)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDbArch.AnchorRailingToObjects)));

            data.Add(new MgdDbg.Snoop.Data.Bool("Calculate elevation at posts only", arToObjects.CalculateElevationAtPostsOnly));
        }

        private void
        Stream(ArrayList data, AecDbArch.AnchorRailingToStair arToStair)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDbArch.AnchorRailingToStair)));

            data.Add(new MgdDbg.Snoop.Data.Int("Component index", arToStair.ComponentIndex));
            data.Add(new MgdDbg.Snoop.Data.Distance("End offset", arToStair.EndOffset));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is automatic placed", arToStair.IsAutomaticPlaced));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is top to bottom", arToStair.IsTopToBottom));
            data.Add(new MgdDbg.Snoop.Data.String("Justification", arToStair.Justification.ToString()));
            data.Add(new MgdDbg.Snoop.Data.Distance("Start offset", arToStair.StartOffset));
            data.Add(new MgdDbg.Snoop.Data.Distance("End offset", arToStair.EndOffset));
            data.Add(new MgdDbg.Snoop.Data.Distance("Offset", arToStair.Offset));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Stair ID", arToStair.StairId));
        }
    
        private void
        Stream(ArrayList data, AecDbArch.AnchorEntityToWall anchor2Wall)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDbArch.AnchorEntityToWall)));
          
            AecDbArch.AnchorOpeningBaseToWall anchorOpening = anchor2Wall as AecDbArch.AnchorOpeningBaseToWall;
			if (anchorOpening != null) {
				Stream(data, anchorOpening);
				return;
			}  
        }
        
        private void
        Stream(ArrayList data, AecDbArch.AnchorOpeningBaseToWall anchorOpening)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDbArch.AnchorOpeningBaseToWall)));
          
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Starting endcap style ID", anchorOpening.StartingEndcapStyleId));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Ending endcap style ID", anchorOpening.EndingEndcapStyleId));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Opening endcap style ID", anchorOpening.OpeningEndcapStyleId));

            AecDbArch.AnchorWindowAssemblyToWall anchorWinAssm = anchorOpening as AecDbArch.AnchorWindowAssemblyToWall;
			if (anchorWinAssm != null) {
				Stream(data, anchorWinAssm);
				return;
			}  
        }

        private void
        Stream(ArrayList data, AecDbArch.AnchorWindowAssemblyToWall anchorWinAssm)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDbArch.AnchorWindowAssemblyToWall)));
          
            // no data at this level
        }

        private void
        Stream(ArrayList data, AecDbArch.AnchorRoomTagToSpace roomTagToSpace)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDbArch.AnchorRoomTagToSpace)));
          
            // no data at this level
        }

        private void
        Stream(ArrayList data, AecDbArch.CurtainWallLayoutStyle cwlStyle)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDbArch.CurtainWallLayoutStyle)));
          
            data.Add(new MgdDbg.Snoop.Data.Distance("Default base height", cwlStyle.DefaultBaseHeight));
            data.Add(new MgdDbg.Snoop.Data.Int("Default flags", cwlStyle.DefaultFlags));
            data.Add(new MgdDbg.Snoop.Data.Distance("Default floorline offset", cwlStyle.DefaultFloorlineOffset));
            data.Add(new MgdDbg.Snoop.Data.Distance("Default roofline offset", cwlStyle.DefaultRooflineOffset));
        }

        private void
        Stream(ArrayList data, AecDbArch.CurtainWallUnitStyle cwuStyle)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDbArch.CurtainWallUnitStyle)));

            // no data at this level
        }

        private void
        Stream(ArrayList data, AecDbArch.WindowAssemblyStyle waStyle)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDbArch.WindowAssemblyStyle)));

            data.Add(new MgdDbg.Snoop.Data.String("Shape", waStyle.Shape.ToString()));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Custom shape ID", waStyle.CustomShapeId));
            data.Add(new MgdDbg.Snoop.Data.Int("Default flags", waStyle.DefaultFlags));
            data.Add(new MgdDbg.Snoop.Data.Distance("Default width", waStyle.DefaultWidth));
            data.Add(new MgdDbg.Snoop.Data.Distance("Default height", waStyle.DefaultHeight));
            data.Add(new MgdDbg.Snoop.Data.Distance("Default rise", waStyle.DefaultRise));
        }

        private void
        Stream(ArrayList data, AecDbArch.WallCleanupGroupDefinition wcgDef)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDbArch.WallCleanupGroupDefinition)));

            data.Add(new MgdDbg.Snoop.Data.Bool("Allow foreign anchored objects to enter", wcgDef.AllowForeignAnchoredObjectsToEnter));
            data.Add(new MgdDbg.Snoop.Data.Bool("Cross Xref boundaries", wcgDef.CrossXrefBoundaries));
        }

        private void
        Stream(ArrayList data, AecDbArch.OpeningEndcapStyle oeStyle)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDbArch.OpeningEndcapStyle)));

            data.Add(new MgdDbg.Snoop.Data.Int("Cap count", oeStyle.CapCount));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Starting endcap ID", oeStyle.StartingEndcapId));            
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Ending endcap ID", oeStyle.EndingEndcapId));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Head endcap ID", oeStyle.HeadEndcapId));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Sill endcap ID", oeStyle.SillEndcapId));
        }

        private void
        Stream(ArrayList data, AecDbArch.SpaceModifier spaceMod)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDbArch.SpaceModifier)));

            data.Add(new MgdDbg.Snoop.Data.String("Expression", spaceMod.Expression));
            data.Add(new MgdDbg.Snoop.Data.String("Get partial replaced xpression", spaceMod.GetPartialReplacedExpression()));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Algorithms", spaceMod.GetAlgorithms()));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Parameter pairs", spaceMod.GetParameterPairs()));

            data.Add(new MgdDbg.Snoop.Data.CategorySeparator("Static Members"));
            data.Add(new MgdDbg.Snoop.Data.String("Hard coded value name", AecDbArch.SpaceModifier.HardCodedValueName));
        }

        private void
        Stream(ArrayList data, AecDbArch.StandardSizeOpeningList ssoList)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDbArch.StandardSizeOpeningList)));

            data.Add(new MgdDbg.Snoop.Data.Enumerable("Openings", ssoList.Openings));

            data.Add(new MgdDbg.Snoop.Data.CategorySeparator("Static Member"));
            data.Add(new MgdDbg.Snoop.Data.String("Extension dictionary key name", AecDbArch.StandardSizeOpeningList.ExtensionDictionaryKeyName));
            data.Add(new MgdDbg.Snoop.Data.String("Extension dictionary name", AecDbArch.StandardSizeOpeningList.ExtensionDictionaryName));
        }

        private void
        Stream(ArrayList data, AecArchAppSrvcs.ArchDBVariables archVars)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecArchAppSrvcs.ArchDBVariables)));

            data.Add(new MgdDbg.Snoop.Data.Distance("Autosnap search radius", archVars.AutosnapSearchRadius));
            data.Add(new MgdDbg.Snoop.Data.Bool("Baseline adjust mode create", archVars.BaselineAdjustModeCreate));
            data.Add(new MgdDbg.Snoop.Data.Bool("Baseline adjust mode grips", archVars.BaselineAdjustModeGrips));
            data.Add(new MgdDbg.Snoop.Data.Bool("Cleanup mode justified", archVars.CleanupModeJustified));
            data.Add(new MgdDbg.Snoop.Data.Distance("Roof thickness", archVars.RoofThickness));
            data.Add(new MgdDbg.Snoop.Data.Bool("Use below and above property for member", archVars.UseBelowAndAbovePropertyForMember));
            data.Add(new MgdDbg.Snoop.Data.Distance("Wall cleanup radius", archVars.WallCleanupRadius));
            data.Add(new MgdDbg.Snoop.Data.Bool("Wall do not display opening endcap style", archVars.WallDoNotDisplayOpeningEndcapStyle));
            data.Add(new MgdDbg.Snoop.Data.String("Wall dimension to opening type", archVars.WallDimensionToOpeningType.ToString()));
        }
    }
}
