
// $Header: //Desktop/Source/TestTools/MgdDbgAec/Snoop/CollectorExts/ObjectBase.cs#3 $
// $Author: ayyara $ $DateTime: 2008/03/07 17:52:55 $ $Change: 94950 $
//
// (C) Copyright 2004 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//

using System;
using System.Diagnostics;
using System.Collections;
using System.Windows.Forms;
using System.Collections.Specialized;

using MgdDbg.Snoop;
using Autodesk.AutoCAD.Geometry;
using Autodesk.AutoCAD.DatabaseServices;
using AcDb        = Autodesk.AutoCAD.DatabaseServices;
using AecDb       = Autodesk.Aec.DatabaseServices;
using AecDbArch   = Autodesk.Aec.Arch.DatabaseServices;
using AecAppSrvcs = Autodesk.Aec.ApplicationServices;


namespace MgdDbgAec.Snoop.CollectorExts
{
	/// <summary>
	/// This is a Snoop Collector Extension object to collect data from AEC objects.
	/// </summary>
	public class ObjectBase : MgdDbg.Snoop.CollectorExts.CollectorExt
	{
		public ObjectBase()
		{
		}
		
        protected override void
        CollectEvent(object sender, MgdDbg.Snoop.Collectors.CollectorEventArgs e)
        {
                // bail early if this isn't an object we are responsible for
            AecDb.DBObject aecObj = e.ObjToSnoop as AecDb.DBObject;
            if (aecObj == null)
                return;

                // cast the sender object to the SnoopCollector we are expecting
            MgdDbg.Snoop.Collectors.Collector snoopCollector = sender as MgdDbg.Snoop.Collectors.Collector;
            if (snoopCollector == null) {
                Debug.Assert(false);    // why did someone else send us the message?
                return;
            }
            
            Stream(snoopCollector.Data(), aecObj);
        }
        

        private void
        Stream(ArrayList data, AecDb.DBObject obj)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDb.DBObject)));

            data.Add(new MgdDbg.Snoop.Data.String("Display name", obj.DisplayName));
            data.Add(new MgdDbg.Snoop.Data.Icon("Type icon", obj.TypeIcon));
            data.Add(new MgdDbg.Snoop.Data.String("Description", obj.Description));
            data.Add(new MgdDbg.Snoop.Data.Bool("Swapping references", obj.SwappingReferences));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Overrides", obj.Overrides));
            
            IntegerCollection compIds = null;
            StringCollection compNames = null;
            AcDb.ObjectIdCollection materialIds = null;
            obj.GetMaterialComponents(out compIds, out compNames, out materialIds);
            Debug.Assert((compIds.Count == compNames.Count) && (compIds.Count == materialIds.Count));
            
            int len = compIds.Count;
            for (int i=0; i<len; i++) {
                data.Add(new MgdDbg.Snoop.Data.ObjectId(string.Format("Component material ID ({0:d}, {1})", compIds[i], compNames[i]), materialIds[i]));
            }

           
            AecDb.Anchor anchor = obj as AecDb.Anchor;
            if (anchor != null) {
                Stream(data, anchor);
                return;
            }

            AecDb.TextNote tNote = obj as AecDb.TextNote;
            if (tNote != null) {
                Stream(data, tNote);
                return;
            }

            AecDb.ReferenceDocument refDoc = obj as AecDb.ReferenceDocument;
            if (refDoc != null) {
                Stream(data, refDoc);
                return;
            }
        }

        private void
        Stream(ArrayList data, AecDb.Anchor anchor)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDb.Anchor)));

            AecDb.AnchorToReference anchorRef = anchor as AecDb.AnchorToReference;
            if (anchorRef != null) {
                Stream(data, anchorRef);
                return;
            }
        }
        
        private void
        Stream(ArrayList data, AecDb.AnchorToReference anchorRef)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDb.AnchorToReference)));
            
            Autodesk.Aec.DatabaseServices.RelationType relType;
            ObjectId refId = anchorRef.GetSingleReference(out relType);
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Single reference ID", refId));
            data.Add(new MgdDbg.Snoop.Data.String("Single reference relation type", relType.ToString()));            
            
            AecDb.AnchorEntityToCurve anchorCurve = anchorRef as AecDb.AnchorEntityToCurve;
			if (anchorCurve != null) {
				Stream(data, anchorCurve);
				return;
			}
			
			AecDb.AnchorEntityToEntity anchorEnt = anchorRef as AecDb.AnchorEntityToEntity;
			if (anchorEnt != null) {
				Stream(data, anchorEnt);
				return;
			}
			
		    AecDb.AnchorEntityToLayoutNode anchorNode = anchorRef as AecDb.AnchorEntityToLayoutNode;
			if (anchorNode != null) {
				Stream(data, anchorNode);
				return;
			}
			
		    AecDb.AnchorEntityToGridAssembly anchorGridAssm = anchorRef as AecDb.AnchorEntityToGridAssembly;
			if (anchorGridAssm != null) {
				Stream(data, anchorGridAssm);
				return;
			}

        }

        private void
        Stream(ArrayList data, AecDb.AnchorEntityToCurve anchorCurve)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDb.AnchorEntityToCurve)));
            
            data.Add(new MgdDbg.Snoop.Data.Object("Anchor X", anchorCurve.AnchorX));
            data.Add(new MgdDbg.Snoop.Data.Object("Anchor Y", anchorCurve.AnchorY));
            data.Add(new MgdDbg.Snoop.Data.Object("Anchor Z", anchorCurve.AnchorZ));
            data.Add(new MgdDbg.Snoop.Data.Bool("Flip X", anchorCurve.FlipX));            
            data.Add(new MgdDbg.Snoop.Data.Bool("Flip Y", anchorCurve.FlipY));            
            data.Add(new MgdDbg.Snoop.Data.Bool("Flip Z", anchorCurve.FlipZ));            
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Curve ID", anchorCurve.CurveId));            
            data.Add(new MgdDbg.Snoop.Data.Angle("Rotation", anchorCurve.Rotation));            
            data.Add(new MgdDbg.Snoop.Data.Angle("Rotation around X", anchorCurve.RotationAroundX));            
            data.Add(new MgdDbg.Snoop.Data.Distance("Curve width", anchorCurve.CurveWidth));            
            data.Add(new MgdDbg.Snoop.Data.Distance("Curve thickness", anchorCurve.CurveThickness));            
            data.Add(new MgdDbg.Snoop.Data.Vector3d("Curve normal", anchorCurve.CurveNormal));            
        }
        
        private void
        Stream(ArrayList data, AecDb.AnchorEntityToLayoutNode anchorNode)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDb.AnchorEntityToLayoutNode)));
            
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Layout tool ID", anchorNode.LayoutToolId));
            data.Add(new MgdDbg.Snoop.Data.Int("Layout node ID", anchorNode.LayoutNodeId));
            data.Add(new MgdDbg.Snoop.Data.Int("Layout node index", anchorNode.LayoutNodeIndex));
            data.Add(new MgdDbg.Snoop.Data.Bool("Flip X", anchorNode.FlipX));            
            data.Add(new MgdDbg.Snoop.Data.Bool("Flip Y", anchorNode.FlipY));            
            data.Add(new MgdDbg.Snoop.Data.Bool("Flip Z", anchorNode.FlipZ));            
            data.Add(new MgdDbg.Snoop.Data.Angle("X Rotation", anchorNode.XRotation));            
            data.Add(new MgdDbg.Snoop.Data.Angle("Y Rotation", anchorNode.YRotation));            
            data.Add(new MgdDbg.Snoop.Data.Angle("Z Rotation", anchorNode.ZRotation));            
            data.Add(new MgdDbg.Snoop.Data.Vector3d("Offset", anchorNode.OffsetVector));            
            data.Add(new MgdDbg.Snoop.Data.Bool("Offset to center", anchorNode.OffsetToCenter));
            data.Add(new MgdDbg.Snoop.Data.Vector3d("Normal", anchorNode.NormalVector));
            data.Add(new MgdDbg.Snoop.Data.Bool("Use local coord system", anchorNode.UseLocalCoordinateSystem));
            
            AecDb.AnchorLeaderToNode anchorLdr = anchorNode as AecDb.AnchorLeaderToNode;
			if (anchorLdr != null) {
				Stream(data, anchorLdr);
				return;
			}
			
			AecDb.AnchorEntityToLayoutCell anchorCell = anchorNode as AecDb.AnchorEntityToLayoutCell;
			if (anchorCell != null) {
				Stream(data, anchorCell);
				return;
			}
        }
        
        private void
        Stream(ArrayList data, AecDb.AnchorLeaderToNode anchorLdr)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDb.AnchorLeaderToNode)));
            
            data.Add(new MgdDbg.Snoop.Data.Distance("First extension", anchorLdr.FirstExtension));
            data.Add(new MgdDbg.Snoop.Data.Distance("Second extension", anchorLdr.SecondExtension));
            data.Add(new MgdDbg.Snoop.Data.Angle("Angle", anchorLdr.Angle));
            
            int i=0;
            foreach (Vector2d vec in anchorLdr.Vectors)
                data.Add(new MgdDbg.Snoop.Data.Vector2d(string.Format("Vector [{0}]", i++), vec));
        }
        
        private void
        Stream(ArrayList data, AecDb.AnchorEntityToLayoutCell anchorCell)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDb.AnchorEntityToLayoutCell)));
            
            data.Add(new MgdDbg.Snoop.Data.Distance("Resize offset", anchorCell.ResizeOffset));
            data.Add(new MgdDbg.Snoop.Data.Bool("Apply resize", anchorCell.ApplyResize));
            
            AecDb.AnchorEntityToLayoutVolume anchorVol = anchorCell as AecDb.AnchorEntityToLayoutVolume;
			if (anchorVol != null) {
				Stream(data, anchorVol);
				return;
			}
        }
        
        private void
        Stream(ArrayList data, AecDb.AnchorEntityToLayoutVolume anchorCell)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDb.AnchorEntityToLayoutVolume)));
        }
        
        private void
        Stream(ArrayList data, AecDb.AnchorEntityToGridAssembly anchorGridAssm)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDb.AnchorEntityToGridAssembly)));
            
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Grid assembly ID", anchorGridAssm.GridAssemblyId));
            data.Add(new MgdDbg.Snoop.Data.Int("Grid assembly cell ID", anchorGridAssm.GridAssemblyCellId));
            
            bool flipX, flipY, flipZ;
            anchorGridAssm.GetOrientation(out flipX, out flipY, out flipZ);
            data.Add(new MgdDbg.Snoop.Data.Bool("Flip X", flipX));            
            data.Add(new MgdDbg.Snoop.Data.Bool("Flip Y", flipY));            
            data.Add(new MgdDbg.Snoop.Data.Bool("Flip Z", flipZ));
            
            data.Add(new MgdDbg.Snoop.Data.Bool("Has sizing offsets", anchorGridAssm.HasSizingOffsets));            
            data.Add(new MgdDbg.Snoop.Data.Distance("Bottom offset", anchorGridAssm.BottomOffset));            
            data.Add(new MgdDbg.Snoop.Data.Distance("Top offset", anchorGridAssm.TopOffset));            
            data.Add(new MgdDbg.Snoop.Data.Distance("Left offset", anchorGridAssm.LeftOffset));            
            data.Add(new MgdDbg.Snoop.Data.Distance("Right offset", anchorGridAssm.RightOffset));            
            data.Add(new MgdDbg.Snoop.Data.Bool("Allow varies from infill definition", anchorGridAssm.AllowVariesFromInfillDefinition));            
            data.Add(new MgdDbg.Snoop.Data.Distance("Y offset", anchorGridAssm.YOffset));            
            data.Add(new MgdDbg.Snoop.Data.String("Y alignment", anchorGridAssm.YAlignment.ToString()));            
        }
        
        private void
        Stream(ArrayList data, AecDb.AnchorEntityToEntity anchorEnt)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDb.AnchorEntityToEntity)));
            
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Referenced entity ID", anchorEnt.ReferencedEntityId));
        }

        private void
        Stream(ArrayList data, AecDb.TextNote tNote)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDb.TextNote)));

            data.Add(new MgdDbg.Snoop.Data.String("Note", tNote.Note));
            data.Add(new MgdDbg.Snoop.Data.String("Extension dictionary name", AecDb.TextNote.ExtensionDictionaryName));          
        }

        private void
        Stream(ArrayList data, AecDb.ReferenceDocument refDoc)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(AecDb.ReferenceDocument)));

            data.Add(new MgdDbg.Snoop.Data.String("File full path", refDoc.FileFullPath));
            data.Add(new MgdDbg.Snoop.Data.String("File path", refDoc.FilePath));
            data.Add(new MgdDbg.Snoop.Data.String("Extension dictionary name", AecDb.ReferenceDocument.ExtensionDictionaryName));          
        }        
	}
}
