using System;
using System.Text;
using System.Collections;
using System.Diagnostics;
using System.Collections.Generic;

using Autodesk.Aec.DatabaseServices;

using MgdDbg.Snoop;

namespace MgdDbgAec.Snoop.CollectorExts {
    class RxObject : MgdDbg.Snoop.CollectorExts.CollectorExt {

        public RxObject()
		{
		}

        protected override void
        CollectEvent(object sender, MgdDbg.Snoop.Collectors.CollectorEventArgs e)
        {
            // bail early if this isn't an object we are responsible for
            Autodesk.AutoCAD.Runtime.RXObject rxObj = e.ObjToSnoop as Autodesk.AutoCAD.Runtime.RXObject;            
            if (rxObj == null)
                return;

            // cast the sender object to the SnoopCollector we are expecting
            MgdDbg.Snoop.Collectors.Collector snoopCollector = sender as MgdDbg.Snoop.Collectors.Collector;
            if (snoopCollector == null) {
                Debug.Assert(false);    // why did someone else send us the message?
                return;
            }           

            EntitySelectedComponentInformation entSelCompInfo = e.ObjToSnoop as EntitySelectedComponentInformation;
            if (entSelCompInfo != null) {
                Stream(snoopCollector.Data(), entSelCompInfo);
                return;
            }

            GsMarkerInformationNode gsMarkerInfoNode = e.ObjToSnoop as GsMarkerInformationNode;
            if (gsMarkerInfoNode != null) {
                Stream(snoopCollector.Data(), gsMarkerInfoNode);
                return;
            }

            StreamAcad streamAcad = e.ObjToSnoop as StreamAcad;
            if (streamAcad != null) {
                Stream(snoopCollector.Data(), streamAcad);
                return;
            }
        }

        private void
        Stream(ArrayList data, EntitySelectedComponentInformation entSelCompInfo)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(EntitySelectedComponentInformation)));

            data.Add(new MgdDbg.Snoop.Data.String("Component name", entSelCompInfo.ComponentName));
            data.Add(new MgdDbg.Snoop.Data.Bool("Current by material status", entSelCompInfo.CurrentByMaterialStatus));
            data.Add(new MgdDbg.Snoop.Data.String("Display component name", entSelCompInfo.DisplayComponentName));
            data.Add(new MgdDbg.Snoop.Data.Bool("Displayed drawing default warning message", entSelCompInfo.DisplayedDrawingDefaultWarningMessage));
            data.Add(new MgdDbg.Snoop.Data.Bool("Displayed material warning message", entSelCompInfo.DisplayedMaterialWarningMessage));
            data.Add(new MgdDbg.Snoop.Data.Bool("Displayed style override warning message", entSelCompInfo.DisplayedStyleOverrideWarningMessage));
            data.Add(new MgdDbg.Snoop.Data.Bool("Displayed system override message", entSelCompInfo.DisplayedSystemOverrideWarningMessage));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Display properties", entSelCompInfo.DisplayProperties));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Display properties ID", entSelCompInfo.DisplayPropertiesId));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Display representation ID", entSelCompInfo.DisplayRepresentationId));
            data.Add(new MgdDbg.Snoop.Data.Bool("Has material information", entSelCompInfo.HasMaterialInformation));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Material display properties ID", entSelCompInfo.MaterialDisplayPropertiesId));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Material display representation ID", entSelCompInfo.MaterialDisplayRepresentationId));
            data.Add(new MgdDbg.Snoop.Data.Object("Material information", entSelCompInfo.MaterialInformation));
        }

        private void
        Stream(ArrayList data, GsMarkerInformationNode gsMarkerInfoNode)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(GsMarkerInformationNode)));

            data.Add(new MgdDbg.Snoop.Data.Int("Gs marker", gsMarkerInfoNode.GsMarker));
            data.Add(new MgdDbg.Snoop.Data.Object("Parent node", gsMarkerInfoNode.ParentNode));
            data.Add(new MgdDbg.Snoop.Data.Object("Parent tree", gsMarkerInfoNode.ParentTree));

            GsMarkerInformationTree gsMarkerInfoTree = gsMarkerInfoNode as GsMarkerInformationTree;
            if (gsMarkerInfoTree != null) {
                Stream(data, gsMarkerInfoTree);
                return;
            }

            GsMarkerDisplayComponentInformationNode gsMarkerDispCompInfoNode = gsMarkerInfoNode as GsMarkerDisplayComponentInformationNode;
            if (gsMarkerDispCompInfoNode != null) {
                Stream(data, gsMarkerDispCompInfoNode);
                return;
            }

            GsMarkerDisplayRepresentationInformationNode gsMarkerDispRepInfoNode = gsMarkerInfoNode as GsMarkerDisplayRepresentationInformationNode;
            if (gsMarkerDispRepInfoNode != null) {
                Stream(data, gsMarkerDispRepInfoNode);
                return;
            }
        }

        private void
        Stream(ArrayList data, GsMarkerInformationTree gsMarkerInfoTree)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(GsMarkerInformationTree)));

            data.Add(new MgdDbg.Snoop.Data.Bool("Are components selected", gsMarkerInfoTree.AreComponentsSelected));
            data.Add(new MgdDbg.Snoop.Data.Int("Current gs marker", gsMarkerInfoTree.CurrentGsMarker));
            data.Add(new MgdDbg.Snoop.Data.Object("Current gs marker info node", gsMarkerInfoTree.CurrentGsMarkerInformationNode));
            data.Add(new MgdDbg.Snoop.Data.Int("Custom block gs marker", gsMarkerInfoTree.CustomBlockGsMarker));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Current block owner entity id", gsMarkerInfoTree.CustomBlockOwnerEntityId));
            data.Add(new MgdDbg.Snoop.Data.Bool("Entity is from custom block", gsMarkerInfoTree.EntityIsFromCustomBlock));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Gs markers for hover highlighting", gsMarkerInfoTree.GsMarkersForHoverHighlighting));
            data.Add(new MgdDbg.Snoop.Data.Bool("Has Gs markers for hover highlighting", gsMarkerInfoTree.HasGsMarkersForHoverHighlighting));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is drawing custom block", gsMarkerInfoTree.IsDrawingCustomBlock));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is drawing with displat theme", gsMarkerInfoTree.IsDrawingWithDisplayTheme));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is entity 2D section type", gsMarkerInfoTree.IsEntity2dSectionType));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is entity 2D section type", gsMarkerInfoTree.IsEntity2dSectionType));
            data.Add(new MgdDbg.Snoop.Data.Int("Next gs marker", gsMarkerInfoTree.NextGsMarker));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Object ID", gsMarkerInfoTree.ObjectId));
            data.Add(new MgdDbg.Snoop.Data.Bool("Populate tree", gsMarkerInfoTree.PopulateTree));
            data.Add(new MgdDbg.Snoop.Data.Bool("Push custom override node", gsMarkerInfoTree.PushCustomOverrideNode));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Selected object display representation id", gsMarkerInfoTree.SelectedObjectDisplayRepresentationId));
            data.Add(new MgdDbg.Snoop.Data.Int("Start Gs marker", gsMarkerInfoTree.StartGsMarker));
        }

        private void
        Stream(ArrayList data, GsMarkerDisplayComponentInformationNode gsMarkerDispCompInfoNode)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(GsMarkerDisplayComponentInformationNode)));

            data.Add(new MgdDbg.Snoop.Data.String("Display component name", gsMarkerDispCompInfoNode.DisplayComponentName));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Display properties ID", gsMarkerDispCompInfoNode.DisplayPropertiesId));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Display representation ID", gsMarkerDispCompInfoNode.DisplayRepresentationId));
            data.Add(new MgdDbg.Snoop.Data.Object("Parent display representation information node", gsMarkerDispCompInfoNode.ParentDisplayRepresentationInformationNode));
        }

        private void
        Stream(ArrayList data, GsMarkerDisplayRepresentationInformationNode gsMarkerDispRepInfoNode)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(GsMarkerDisplayRepresentationInformationNode)));

            data.Add(new MgdDbg.Snoop.Data.ObjectId("Display properties ID", gsMarkerDispRepInfoNode.DisplayPropertiesId));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Display representation ID", gsMarkerDispRepInfoNode.DisplayRepresentationId));
        }

        private void
        Stream(ArrayList data, StreamAcad streamAcad)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(StreamAcad)));

            data.Add(new MgdDbg.Snoop.Data.Bool("Always resolve by layer plot styles", streamAcad.AlwaysResolveByLayerPlotStyles));
            data.Add(new MgdDbg.Snoop.Data.Object("Annotation scale", streamAcad.AnnotationScale));
            data.Add(new MgdDbg.Snoop.Data.Bool("Apply surface hatching", streamAcad.ApplySurfaceHatching));
            data.Add(new MgdDbg.Snoop.Data.Bool("Attenuate color", streamAcad.AttenuateColor));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("AutoCAD render material", streamAcad.AutoCADRenderMaterial));
            data.Add(new MgdDbg.Snoop.Data.Bool("Backface cull surface hatching", streamAcad.BackfaceCullSurfaceHatching));
            data.Add(new MgdDbg.Snoop.Data.String("Body edge color mode", streamAcad.BodyEdgeColorMode.ToString()));
            data.Add(new MgdDbg.Snoop.Data.String("Body face color mode", streamAcad.BodyFaceColorMode.ToString()));
            data.Add(new MgdDbg.Snoop.Data.Int("Color index", streamAcad.ColorIndex));
            data.Add(new MgdDbg.Snoop.Data.Object("Current entity draw data", streamAcad.CurrentEntityDrawData));
            data.Add(new MgdDbg.Snoop.Data.Object("Current Gs marker info tree", streamAcad.CurrentGsMarkerInfoTree));
            data.Add(new MgdDbg.Snoop.Data.Object("Current insert Xform", streamAcad.CurrentInsertXform));
            data.Add(new MgdDbg.Snoop.Data.Object("Current sub object Xform", streamAcad.CurrentSubObjectXform));
            data.Add(new MgdDbg.Snoop.Data.Object("Current Xform", streamAcad.CurrentXform));
            data.Add(new MgdDbg.Snoop.Data.Vector3d("Display direction", streamAcad.DisplayDirection));
            data.Add(new MgdDbg.Snoop.Data.Object("Entity", streamAcad.Entity));
            data.Add(new MgdDbg.Snoop.Data.String("Fill type", streamAcad.FillType.ToString()));
            data.Add(new MgdDbg.Snoop.Data.Bool("Filter frozen layers", streamAcad.FilterFrozenLayers));
            data.Add(new MgdDbg.Snoop.Data.Bool("Filter off layers", streamAcad.FilterOffLayers));
            data.Add(new MgdDbg.Snoop.Data.Int("Gs marker", streamAcad.GsMarker));
            data.Add(new MgdDbg.Snoop.Data.Bool("Gs markers valid", streamAcad.GsMarkersValid));
            data.Add(new MgdDbg.Snoop.Data.Bool("Ignore live sections", streamAcad.IgnoreLiveSections));
            data.Add(new MgdDbg.Snoop.Data.Bool("Independent layer off control", streamAcad.IndependentLayerOffControl));
            data.Add(new MgdDbg.Snoop.Data.Object("Inverse insert Xform", streamAcad.InverseInsertXform));
            data.Add(new MgdDbg.Snoop.Data.Object("Inverse sub object Xform", streamAcad.InverseSubObjectXform));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is perspective view", streamAcad.IsPerspectiveView));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is scaled", streamAcad.IsScaled));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is uni scaled", streamAcad.IsUniScaled));
            data.Add(new MgdDbg.Snoop.Data.Bool("Is Xform", streamAcad.IsXform));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Layer", streamAcad.Layer));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Linetype", streamAcad.Linetype));
            data.Add(new MgdDbg.Snoop.Data.Double("Linetype scale", streamAcad.LineTypeScale));
            data.Add(new MgdDbg.Snoop.Data.String("Line weight", streamAcad.LineWeight.ToString()));
            data.Add(new MgdDbg.Snoop.Data.Bool("Line section required", streamAcad.LiveSectionRequired));
            data.Add(new MgdDbg.Snoop.Data.Bool("Masking required", streamAcad.MaskingRequired));
            data.Add(new MgdDbg.Snoop.Data.Object("Material", streamAcad.Material));
            data.Add(new MgdDbg.Snoop.Data.String("Render mode", streamAcad.RenderMode.ToString()));
            data.Add(new MgdDbg.Snoop.Data.Bool("Resolve by block traits", streamAcad.ResolveByBlockTraits));
            data.Add(new MgdDbg.Snoop.Data.Bool("Resolve by layer traits", streamAcad.ResolveByLayerTraits));
            data.Add(new MgdDbg.Snoop.Data.Scale3d("Scale", streamAcad.Scale));
            data.Add(new MgdDbg.Snoop.Data.Int("Skip push pop display properties flags", streamAcad.SkipPushPopDisplayPropertiesFlags));
            data.Add(new MgdDbg.Snoop.Data.Bool("Suppress shell and mesh extended data", streamAcad.SuppressShellAndMeshExtendedData));
            data.Add(new MgdDbg.Snoop.Data.Object("True color", streamAcad.TrueColor));
            data.Add(new MgdDbg.Snoop.Data.Bool("Use layer property override per viewport if exists", streamAcad.UseLayerPropertyOverridePerViewportIfExists));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Viewport Id", streamAcad.ViewportId));
            data.Add(new MgdDbg.Snoop.Data.ObjectId("Visual style", streamAcad.VisualStyle));

            StreamCollectBodies streamCollBodies = streamAcad as StreamCollectBodies;
            if (streamCollBodies != null) {
                Stream(data, streamCollBodies);
                return;
            }

            StreamCollectClipBodies streamCollClipBodies = streamAcad as StreamCollectClipBodies;
            if (streamCollClipBodies != null) {
                Stream(data, streamCollClipBodies);
                return;
            }

            StreamCollectMaterials streamCollMaterials = streamAcad as StreamCollectMaterials;
            if (streamCollMaterials != null) {
                Stream(data, streamCollMaterials);
                return;
            }

            StreamCurves streamCurves = streamAcad as StreamCurves;
            if (streamCurves != null) {
                Stream(data, streamCurves);
                return;
            }

            StreamExplode streamExplode = streamAcad as StreamExplode;
            if (streamExplode != null) {
                Stream(data, streamExplode);
                return;
            }

            StreamExtent streamExtent = streamAcad as StreamExtent;
            if (streamExtent != null) {
                Stream(data, streamExtent);
                return;
            }

            StreamIntersect streamIntersect = streamAcad as StreamIntersect;
            if (streamIntersect != null) {
                Stream(data, streamIntersect);
                return;
            }

            StreamVector streamVector = streamAcad as StreamVector;
            if (streamVector != null) {
                Stream(data, streamVector);
                return;
            }
        }

        private void
        Stream(ArrayList data, StreamCollectBodies streamCollBodies)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(StreamCollectBodies)));

            // no data at this level

            StreamSlice streamSlice = streamCollBodies as StreamSlice;
            if (streamSlice != null) {
                Stream(data, streamSlice);
                return;
            }
        }

        private void
        Stream(ArrayList data, StreamSlice streamSlice)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(StreamSlice)));

            data.Add(new MgdDbg.Snoop.Data.Object("Profile", streamSlice.GetProfile()));
        }

        private void
        Stream(ArrayList data, StreamCollectClipBodies streamCollClipBodies)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(StreamCollectClipBodies)));

            data.Add(new MgdDbg.Snoop.Data.Bool("Backface cull surface hatching", streamCollClipBodies.BackfaceCullSurfaceHatching));
            data.Add(new MgdDbg.Snoop.Data.Bool("Calculate shrink wrap", streamCollClipBodies.CalculateShrinkwrap));
            data.Add(new MgdDbg.Snoop.Data.Object("Shrink wrap profile", streamCollClipBodies.ShrinkwrapProfile));
        }

        private void
        Stream(ArrayList data, StreamCollectMaterials streamCollMaterials)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(StreamCollectMaterials)));

            data.Add(new MgdDbg.Snoop.Data.Bool("Combine bodies", streamCollMaterials.CombineBodies));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Material filter", streamCollMaterials.MaterialFilter));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Material ID's", streamCollMaterials.MaterialIds));
        }

        private void
        Stream(ArrayList data, StreamCurves streamCurves)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(StreamCurves)));

            data.Add(new MgdDbg.Snoop.Data.Bool("Apply surface hatching", streamCurves.ApplySurfaceHatching));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Arcs", streamCurves.Arcs));
            data.Add(new MgdDbg.Snoop.Data.Bool("Backface cull surface hatching", streamCurves.BackfaceCullSurfaceHatching));
            data.Add(new MgdDbg.Snoop.Data.Bool("Collect arcs", streamCurves.CollectArcs));
            data.Add(new MgdDbg.Snoop.Data.Bool("Collect display props info for 2d section", streamCurves.CollectDisplayPropertiesInfomationFor2dSection));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Display component entity arcs", streamCurves.DisplayComponentEntityArcs));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Display component entity lines", streamCurves.DisplayComponentEntityLines));
            data.Add(new MgdDbg.Snoop.Data.Bool("Ignore hatch", streamCurves.IgnoreHatch));
            data.Add(new MgdDbg.Snoop.Data.Bool("Ignore text", streamCurves.IgnoreText));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Lines", streamCurves.Lines));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Nested entity filter", streamCurves.NestedEntityFilter));
        }

        private void
        Stream(ArrayList data, StreamExplode streamExplode)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(StreamExplode)));

            data.Add(new MgdDbg.Snoop.Data.Bool("Is visual explode", streamExplode.IsVisualExplode));
            data.Add(new MgdDbg.Snoop.Data.Bool("Use face and edge params while drawing mesh", streamExplode.UseFaceAndEdgeParamsWhileDrawingMesh));
        }

        private void
        Stream(ArrayList data, StreamExtent streamExtent)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(StreamExtent)));

            data.Add(new MgdDbg.Snoop.Data.Bool("Allow mtext", streamExtent.AllowMText));
            data.Add(new MgdDbg.Snoop.Data.Object("Extents", streamExtent.Extents));
            data.Add(new MgdDbg.Snoop.Data.Bool("Text stroking status", streamExtent.TextStrokingStatus));
        }

        private void
        Stream(ArrayList data, StreamIntersect streamIntersect)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(StreamIntersect)));

            data.Add(new MgdDbg.Snoop.Data.String("Gs selection marker", streamIntersect.GsSelectionMarker.ToString()));
        }

        private void
        Stream(ArrayList data, StreamVector streamVector)
        {
            data.Add(new MgdDbg.Snoop.Data.ClassSeparator(typeof(StreamVector)));

            data.Add(new MgdDbg.Snoop.Data.Bool("Apply surface hatching", streamVector.ApplySurfaceHatching));
            data.Add(new MgdDbg.Snoop.Data.Bool("Backface cull surface hatching", streamVector.BackfaceCullSurfaceHatching));
            data.Add(new MgdDbg.Snoop.Data.Enumerable("Lines", streamVector.Lines));
        }  
    }
}
