
// $Header: //Desktop/Source/TestTools/MgdDbgAec/Utils/Db.cs#2 $
// $Author: ayyara $ $DateTime: 2007/03/28 11:46:10 $ $Change: 69505 $
//
// (C) Copyright 2004 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//

using System;
using System.Diagnostics;

using Autodesk.AutoCAD.Geometry;
using Autodesk.AutoCAD.DatabaseServices;
using Autodesk.Aec.Geometry;
using Autodesk.Aec.Arch.DatabaseServices;

using AcadApp     = Autodesk.AutoCAD.ApplicationServices.Application;

namespace MgdDbgAec.Utils
{
	/// <summary>
	/// Summary description for Db.
	/// </summary>
	public class Db
	{
		public Db()
		{
		}
		
		public static Polyline
        CompCurve2dToPolyline(CompoundCurve2d compCurve)
        {
            int segCount = compCurve.SegmentCount;
            if (segCount == 0) {
                Debug.Assert(false);
                return null;
            }

            Polyline pline = new Polyline();

            for (int j=0; j<segCount; j++) {
                Segment2d seg = compCurve.Segment(j);

                pline.AddVertexAt(j, seg.StartPoint, seg.Bulge, 0.0, 0.0);

                if ((j == segCount-1) && (compCurve.IsClosed == false)) {
                    pline.AddVertexAt(j+1, seg.EndPoint, 0.0, 0.0, 0.0);
                }
            }

            pline.Closed = compCurve.IsClosed;

            return pline;
        }
        
        public static void
        MakePointEnt(Point3d pt, int colorIndex, Transaction tr, Database db)
        {
            short mode = (short)AcadApp.GetSystemVariable("pdmode");
            if (mode == 0)
                AcadApp.SetSystemVariable("pdmode", 99);

            using (DBPoint dbPt = new DBPoint(pt)) {
                dbPt.ColorIndex = colorIndex;
                MgdDbg.Utils.SymTbl.AddToCurrentSpace(dbPt, db, tr);
            }
        }

        
        public static void
        MakeExtentsBlock(BoundBlock3d ext, int colorIndex, Database db)
        {
            Point3d minPt = ext.GetMinimumPoint();
            Point3d maxPt = ext.GetMaximumPoint();
            
            double deltaX = Math.Abs(maxPt.X - minPt.X);
            double deltaY = Math.Abs(maxPt.Y - minPt.Y);
            double deltaZ = Math.Abs(maxPt.Z - minPt.Z);

            Point3d[] pts = new Point3d[8];
            
            pts[0] = minPt;
            pts[6] = maxPt;

                // make bottom face
            pts[1] = new Point3d(pts[0].X + deltaX, pts[0].Y, pts[0].Z);
            pts[2] = new Point3d(pts[1].X, pts[1].Y + deltaY, pts[1].Z);
            pts[3] = new Point3d(pts[0].X, pts[0].Y + deltaY, pts[0].Z);

                // project up by Z
            pts[4] = new Point3d(pts[0].X, pts[0].Y, pts[0].Z + deltaZ);
            pts[5] = new Point3d(pts[1].X, pts[1].Y, pts[1].Z + deltaZ);
            pts[7] = new Point3d(pts[3].X, pts[3].Y, pts[3].Z + deltaZ);

            Vector3d offset = minPt.GetAsVector();

                // move points so that they are centered at WCS origin
                // for block creation.  Express everything in WCS since
                // that is what Entity.Extents works in.
            for (int i=0; i<pts.Length; i++) {
                pts[i] -= offset;
            }

            DBObjectCollection faceEnts = new DBObjectCollection();
            faceEnts.Add(new Face(pts[0], pts[1], pts[2], pts[3], true, true, true, true));  // bottom face
            faceEnts.Add(new Face(pts[4], pts[5], pts[6], pts[7], true, true, true, true));  // top face
            faceEnts.Add(new Face(pts[0], pts[1], pts[5], pts[4], true, true, true, true));  // front face
            faceEnts.Add(new Face(pts[1], pts[2], pts[6], pts[5], true, true, true, true));  // right side face
            faceEnts.Add(new Face(pts[2], pts[3], pts[7], pts[6], true, true, true, true));  // back side face
            faceEnts.Add(new Face(pts[3], pts[0], pts[4], pts[7], true, true, true, true));  // left side face
            
            MgdDbg.CompBldrAnonBlkDef compBldr = new MgdDbg.CompBldrAnonBlkDef(db);
            compBldr.Start();
            
            foreach (Entity ent in faceEnts) {
                compBldr.SetToDefaultProps(ent);
                compBldr.AddToDb(ent);
            }
            
            compBldr.Commit();
            
            BlockReference blkRef = new BlockReference(minPt, compBldr.BlockDefId);
            blkRef.ColorIndex = colorIndex;
            MgdDbg.Utils.SymTbl.AddToCurrentSpaceAndClose(blkRef, compBldr.Database);
        }             
	}
}
